set define '^' verify off
prompt ...wwv_flow_copy_metadata
create or replace package wwv_flow_copy_metadata as
--------------------------------------------------------------------------------
--
--  Copyright (c) 2000, 2023, Oracle and/or its affiliates.
--
--    NAME
--      wwv_flow_copy_metadata.sql
--
--    DESCRIPTION
--      Used to subscribe and pull flow objects.
--
--    RUNTIME DEPLOYMENT: YES
--
--    MODIFIED (MM/DD/YYYY)
--     cbcho   01/22/2002 - Created
--
--------------------------------------------------------------------------------
--==============================================================================
-- Global types and constants
--==============================================================================
type templ_opt_checksum_type is record (templ_opt_name varchar2(255), tmpl_opt_checksum varchar2(32767));
type templ_opt_checksum_tbl is table of templ_opt_checksum_type index by binary_integer;

type t_component is record(
    table_name         wwv_flow_global.t_dbms_id,
    name_col           wwv_flow_global.t_dbms_id,
    additional_col     wwv_flow_global.t_dbms_id,
    build_option_col   wwv_flow_global.t_dbms_id,
    authorization_col  wwv_flow_global.t_dbms_id,
    unique_cols        varchar2(500),
    unique_err_message wwv_flow_messages$.name%type );

subtype t_component_type is varchar2(25);
--
-- constants for shared component types
-- these values must be in sync with type_name used in wwv_flow_appl_component_types view
--
c_app_computation   constant t_component_type := 'APP_COMPUTATION';
c_app_item          constant t_component_type := 'APP_ITEM';
c_app_process       constant t_component_type := 'APP_PROCESS';
c_app_setting       constant t_component_type := 'APP_SETTING';
c_authentication    constant t_component_type := 'AUTHENTICATION';
c_authorization     constant t_component_type := 'AUTHORIZATION';
c_build_option      constant t_component_type := 'BUILD_OPTION';
c_component_group   constant t_component_type := 'COMPONENT_GROUP';
c_component_setting constant t_component_type := 'PLUGIN_SETTINGS';
c_custom_map_bkg    constant t_component_type := 'MAP_BACKGROUND';
c_data_load         constant t_component_type := 'DATA_LOAD';
c_email_template    constant t_component_type := 'EMAIL_TEMPLATE';
c_list              constant t_component_type := 'LIST';
c_lov               constant t_component_type := 'LOV';
c_message           constant t_component_type := 'MESSAGE';
c_page              constant t_component_type := 'PAGE';
c_plugin            constant t_component_type := 'PLUGIN';
c_report_layout     constant t_component_type := 'REPORT_LAYOUT';
c_search_config     constant t_component_type := 'SEARCH_CONFIG';
c_shortcut          constant t_component_type := 'SHORTCUT';
c_user_group        constant t_component_type := 'APP_ACL';
c_web_src_module    constant t_component_type := 'WEB_SOURCE';

procedure remove_page_reference_id (
    --
    -- This procedure removes reference id from page templates.
    --
    p_id in number
    );

procedure remove_region_reference_id (
    --
    -- This procedure removes reference id from region templates.
    --
    p_id in number
    );

procedure remove_list_reference_id (
    --
    -- This procedure removes reference id from list templates.
    --
    p_id in number
    );

procedure remove_report_reference_id (
    --
    -- This procedure removes reference id from report templates.
    --
    p_id in number
    );

procedure remove_field_reference_id (
    --
    -- This procedure removes reference id from field templates.
    --
    p_id in number
    );

procedure remove_popuplov_reference_id (
    --
    -- This procedure removes reference id from a popup LOV template.
    --
    p_id in number
    );

procedure remove_menu_reference_id (
    --
    -- This procedure removes reference id from a menu template.
    --
    p_id in number
    );

procedure remove_button_reference_id (
    --
    -- This procedure removes reference id from a button template.
    --
    p_id in number
    );

procedure remove_calendar_reference_id (
    --
    -- This procedure removes reference id from a calendar template.
    --
    p_id in number
    );
procedure remove_scheme_reference_id (
    --
    -- This procedure removes reference id from a security scheme.
    --
    p_id in number
    );

procedure remove_shortcut_reference_id (
    --
    -- This procedure removes reference id from a shortcut.
    --
    p_id in number
    );

procedure remove_navbar_reference_id (
    --
    -- This procedure removes reference id from a navbar.
    --
    p_id in number
    );

procedure remove_authentication_ref_id (
    --
    -- This procedure removes reference id from an authentication.
    --
    p_id in number );

procedure remove_item_help_reference_id (
    --
    -- This procedure removes reference id from an item help
    --
    p_id in number
    );

procedure subscribe_theme_options (
    --
    -- This procedure pulls theme level options from the master theme and
    -- copies it to the subscribing theme.
    --
    p_master_theme_id      in number,
    p_subscribing_theme_id in number
);

procedure subscribe_page_templates (
    --
    -- This procedure pulls content from all subscribed master page templates of a theme and
    -- copies it to the subscribing page templates.
    --
    p_master_theme_flow_id  in number,
    p_master_theme_theme_id in number,
    p_sub_theme_flow_id     in number,
    p_sub_theme_theme_id    in number );

procedure subscribe_page_template (
    --
    -- This procedure pulls content from the master template and
    -- copies it to the subscribing template.
    --
    p_master_template_id         in number,
    p_subscribing_template_id    in number
);

procedure subscribe_region_templates (
    --
    -- This procedure pulls content from all subscribed master region templates of a theme and
    -- copies it to the subscribing region templates.
    --
    p_master_theme_flow_id  number,
    p_master_theme_theme_id number,
    p_sub_theme_flow_id     number,
    p_sub_theme_theme_id    number );

procedure subscribe_region_template (
    --
    -- This procedure pulls content from the master region template and
    -- copies it to the subscribing region template.
    --
    p_master_template_id         in number,
    p_subscribing_template_id    in number
);

procedure subscribe_list_templates (
    --
    -- This procedure pulls content from all subscribed master list templates of a theme and
    -- copies it to the subscribing list templates.
    --
    p_master_theme_flow_id  in number,
    p_master_theme_theme_id in number,
    p_sub_theme_flow_id     in number,
    p_sub_theme_theme_id    in number );

procedure subscribe_list_template (
    --
    -- This procedure pulls content from the master list template and
    -- copies it to the subscribing list template.
    --
    p_master_template_id         in number,
    p_subscribing_template_id    in number
);

procedure subscribe_report_templates (
    --
    -- This procedure pulls content from all subscribed master report templates of a theme and
    -- copies it to the subscribing report templates.
    --
    p_master_theme_flow_id  in number,
    p_master_theme_theme_id in number,
    p_sub_theme_flow_id     in number,
    p_sub_theme_theme_id    in number );

procedure subscribe_report_template (
    --
    -- This procedure pulls content from the master report template and
    -- copies it to the subscribing report template.
    --
    p_master_template_id         in number,
    p_subscribing_template_id    in number
);

procedure subscribe_field_templates (
    --
    -- This procedure pulls content from all subscribed master field templates of a theme and
    -- copies it to the subscribing fierld templates.
    --
    p_master_theme_flow_id  in number,
    p_master_theme_theme_id in number,
    p_sub_theme_flow_id     in number,
    p_sub_theme_theme_id    in number );

procedure subscribe_field_template (
    --
    -- This procedure pulls content from the master field template and
    -- copies it to the subscribing field template.
    --
    p_master_template_id         in number,
    p_subscribing_template_id    in number
);

procedure subscribe_popuplov_templates (
    --
    -- This procedure pulls content from all subscribed master popup templates of a theme and
    -- copies it to the subscribing popup templates.
    --
    p_master_theme_flow_id  in number,
    p_master_theme_theme_id in number,
    p_sub_theme_flow_id     in number,
    p_sub_theme_theme_id    in number );

procedure subscribe_popuplov_template (
    --
    -- This procedure pulls content from the master popup lov template and
    -- copies it to the subscribing popup lov template.
    --
    p_master_template_id         in number,
    p_subscribing_template_id    in number
);

procedure subscribe_menu_templates (
    --
    -- This procedure pulls content from all subscribed master menu templates of a theme and
    -- copies it to the subscribing menu templates.
    --
    p_master_theme_flow_id  in number,
    p_master_theme_theme_id in number,
    p_sub_theme_flow_id     in number,
    p_sub_theme_theme_id    in number );

procedure subscribe_menu_template (
    --
    -- This procedure pulls content from the master menu template and
    -- copies it to the subscribing menu template.
    --
    p_master_template_id         in number,
    p_subscribing_template_id    in number
);

procedure subscribe_button_templates (
    --
    -- This procedure pulls content from all subscribed master button templates of a theme and
    -- copies it to the subscribing button templates.
    --
    p_master_theme_flow_id  in number,
    p_master_theme_theme_id in number,
    p_sub_theme_flow_id     in number,
    p_sub_theme_theme_id    in number );

procedure subscribe_button_template (
    --
    -- This procedure pulls content from the master button template and
    -- copies it to the subscribing button template.
    --
    p_master_template_id         in number,
    p_subscribing_template_id    in number
);

procedure subscribe_calendar_templates (
    --
    -- This procedure pulls content from all subscribed master calendar templates of a theme and
    -- copies it to the subscribing calendar templates.
    --
    p_master_theme_flow_id  in number,
    p_master_theme_theme_id in number,
    p_sub_theme_flow_id     in number,
    p_sub_theme_theme_id    in number );

procedure subscribe_calendar_template (
    --
    -- This procedure pulls content from the master calendar template and
    -- copies it to the subscribing calendar template.
    --
    p_master_template_id         in number,
    p_subscribing_template_id    in number
);

procedure subscribe_navbar (
    --
    -- This procedure pulls content from the master navbar and
    -- copies it to the subscribing navbar.
    --
    p_master_navbar_id         in number,
    p_subscribing_navbar_id    in number
    );

procedure subscribe_authentication (
    --
    -- This procedure pulls content from the master authentication and
    -- copies it to the subscribing authentication.
    p_master_auth_id         in number,
    p_subscribing_auth_id    in number,
    p_subscribing_app_id     in number default null);

procedure subscribe_item_help (
    --
    -- This procedure pulls content from the master item help and
    -- copies it to the subscribing item help.
    p_master_item_id         in number,
    p_subscribing_item_id    in number
    );

--
-- This procedure pulls content from the master theme style and
-- copies it to the subscribing theme style.
--
procedure subscribe_theme_style (
    p_master_theme_style_id      in number,
    p_subscribing_theme_style_id in number );

procedure push_page_template (
    --
    -- This procedure pushes content of the page template to
    -- all templates that reference this page template.
    --
    p_template_id in number
    );

procedure push_region_template (
    --
    -- This procedure pushes content of the region template to
    -- all templates that reference this region template.
    --
    p_template_id in number
    );

procedure push_list_template (
    --
    -- This procedure pushes content of the list template to
    -- all templates that reference this list template.
    --
    p_template_id in number
    );

procedure push_report_template (
    --
    -- This procedure pushes content of the report template to
    -- all templates that reference this report template.
    --
    p_template_id in number
    );

procedure push_field_template (
    --
    -- This procedure pushes content of the field template to
    -- all templates that reference this field template.
    --
    p_template_id in number
    );

procedure push_popuplov_template (
    --
    -- This procedure pushes content of the popup LOV template to
    -- all templates that reference this popup LOV template.
    --
    p_template_id in number
    );

procedure push_template_component (
    --
    -- This procedure pushes content of the popup LOV template to
    -- all templates that reference this popup LOV template.
    --
    p_template_id in number
    );

procedure push_menu_template (
    --
    -- This procedure pushes content of the menu template to
    -- all templates that reference this menu template.
    --
    p_template_id in number
    );

procedure push_button_template (
    --
    -- This procedure pushes content of the button template to
    -- all templates that reference this button template.
    --
    p_template_id in number
    );

procedure push_calendar_template (
    --
    -- This procedure pushes content of the calendar template to
    -- all templates that reference this calendar template.
    --
    p_template_id in number
    );



procedure push_shortcut (
    --
    -- This procedure pushes content of the short cut to
    -- all short cuts that reference this short cut.
    --
    p_shortcut_id in number
    );

procedure push_navbar (
    --
    -- This procedure pushes content of the navbar to
    -- all navbars that reference this navbar.
    --
    p_navbar_id in number
    );

procedure push_authentication (
    --
    -- This procedure pushes content of the authentication to
    -- all authentications that reference this authentication.
    --
    p_authentication_id in number );

procedure push_item_help (
    --
    -- This procedure pushes content of the item help to
    -- all item helps that reference this item help.
    --
    p_item_id in number
    );

--
-- This procedure pushes content of the theme style to
-- all theme styles that reference this theme style.
--
procedure push_theme_style (
    p_theme_style_id in number );

procedure refresh_page_template (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_region_template (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_report_template (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_list_template (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_field_template (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_popuplov_template (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_menu_template (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_button_template (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_calendar_template (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_template_component (
    --
    -- If the master template ID is not passed, get the master template ID
    -- and refresh subscribing template.
    --
    p_subscribing_template_id    in number
    );

procedure refresh_navbar (
    --
    -- If the master navbar ID is not passed, get the master navbar ID
    -- and refresh subscribing navbar.
    p_subscribing_navbar_id    in number
    );

procedure refresh_item_help (
    --
    -- If the master item help ITEM ID is not passed, get the master item help ITEM ID
    -- and refresh subscribing item help.
    --
    p_subscribing_item_id    in number
    );

procedure refresh_theme_style (
    --
    -- If the master theme style ID is not passed, get the master theme style ID
    -- and refresh subscribing theme style.
    --
    p_subscribing_theme_style_id in number );

function template_options_match (
    --
    -- compares template options of template with those of master template
    --
    p_template_id    in number,
    p_reference_id   in number
) return boolean;

function compare_themes (
    --
    -- compares all templates of the subscribing theme with the references templates in the master themes
    --
    p_subscribing_theme_id in number
) return number;

procedure unsubscribe_theme (
    --
    -- This procedure removes all reference IDs from the theme and
    -- associated templates
    --
    p_subscribing_theme_id in number
);

procedure resubscribe_theme (
    --
    -- This procedure re-subscribes templates within a theme to corresponding templates in a master
    -- theme based on matching template name
    p_master_theme_id      in number,
    p_subscribing_theme_id in number,
    p_match_by_name        in varchar2 default 'Y'
);

procedure subscribe_theme (
    --
    -- This procedure pulls all templates and related components from the master theme and
    -- copies it to the subscribing theme.
    --
    p_master_theme_id      in number,
    p_subscribing_theme_id in number
);

procedure set_internal_names (
    -- set internal theme and template names
    p_theme_id  in number
);

-----------------------------------------------------------------------
-- This procedure copies the list data from master list to subscribing list
--
-- Parameters:
--
-- * p_master_list_id: Master list id
-- * p_subscribing_list_id: Subscribing list id
----------------------------------------------------------------------
procedure subscribe_list (
    p_master_list_id      in number,
    p_subscribing_list_id in number );

-----------------------------------------------------------------------
-- This procedure pushes the list data from master list to all subscribing lists
--
-- Parameters:
--
-- * p_list_id: Master list id
----------------------------------------------------------------------
procedure push_list (
    p_list_id in number );

-----------------------------------------------------------------------
-- This procedure refreshes the list data from master list to the subscribing list
--
-- Parameters:
--
-- * p_list_id: list id to refresh
----------------------------------------------------------------------
procedure refresh_list (
    p_list_id in number );

-----------------------------------------------------------------------
-- This procedure copies a list
--
-- Parameters:
-- * p_copy_from_app_id:     app ID to copy the list from
-- * p_copy_from_list_id:    list ID to copy
-- * p_new_list_name:        name for the new list
-- * p_new_application_id:   app ID to copy the list to
-- * p_include_subscription: also copy the subscription status of the list
-- * p_copy_subscription:    Flag to determine whether to copy reference_id as is or not from p_copy_from_app_id
--                           This flag is applicable only if p_include_subscription is true
--
-- Returns:
-- * p_new_list_id:          ID of the new list
----------------------------------------------------------------------
procedure copy_list_from_app (
    p_copy_from_app_id      in number,
    p_copy_from_list_id     in number,
    p_new_list_name         in varchar2,
    p_new_application_id    in number,
    p_include_subscription  in boolean default false,
    p_copy_subscription     in boolean default false,
    --
    p_new_list_id          out number );

--==============================================================================
-- This function returns meta data like table_name, name_col etc of given component type
--
-- Parameters:
-- * p_component_type:       t_component_type
--
-- Returns:
-- * Component meta data as t_component type
--==============================================================================
function get_component(
    p_component_type in t_component_type )
    return t_component;

function copy_authentication (
    p_copy_from_flow_id      in number,
    p_flow_id                in number,
    p_from_authentication_id in number,
    p_to_authentication_name in varchar2 default null,
    p_subscribe              in boolean default false )
return number;

--==============================================================================
-- Common function to copy a component
-- Calls corresponding copy function or uses generic copy function based on p_component_type value
-- Throws exception / error message if a component with same name already exists
--
-- Parameters:
-- * p_component_type:       t_component_type
-- * p_from_id:              Component ID to copy
-- * p_from_app_id:          App ID to copy the component from
-- * p_to_app_id:            App ID to copy the component to
-- * p_new_name:             Name of the new component, when null source name is used
-- * p_additional_col:       Additional column info that should be copied to new shared component
-- * p_subscribe:            Flag to determine whether to subscribe to the shared component or not
-- * p_copy_subscription:    Flag to determine whether to copy reference_id as is or not from p_from_app_id
--                           This flag is applicable only if p_subscribe is true
--
-- Returns:
-- * ID of the new copied component
--==============================================================================
function copy_component (
    p_component_type       in t_component_type,
    p_from_id              in number,
    p_from_app_id          in number,
    p_to_app_id            in number,
    p_new_name             in varchar2 default null,
    p_additional_col       in varchar2 default null,
    p_subscribe            in boolean  default false,
    p_copy_subscription    in boolean  default false )
    return number;

/*
--==============================================================================
-- Function to check if the given component name is unique or not
--   Can be used only if name column is unique column for the component
--
-- Parameters:
-- * p_component_type:       t_component
-- * p_app_id:               App ID to check
-- * p_name:                 Component Name to check
-- * p_additional_col_val:   Additional column to check
--
-- Returns:
-- * returns true, if name is unique, returns false otherwise
--==============================================================================
function is_component_unique (
    p_component            in t_component,
    p_app_id               in number,
    p_name                 in varchar2,
    p_additional_col_val   in varchar2 default null )
    return boolean;

--==============================================================================
-- Function to check if the given component name is unique or not
--   Can be used only if name column is unique column for the component
--
-- Parameters:
-- * p_component_type:       t_component_type
-- * p_app_id:               App ID to check
-- * p_name:                 Component Name to check
-- * p_additional_col_val:   Additional column to check
--
-- Returns:
-- * returns true, if name is unique, returns false otherwise
--==============================================================================
function is_component_unique (
    p_component_type       in t_component_type,
    p_app_id               in number,
    p_name                 in varchar2,
    p_additional_col_val   in varchar2 default null )
    return boolean; */

--==============================================================================
-- Common procedure to refresh a component
-- Calls corresponding refresh procedure or uses generic refresh procedure based on p_component_type value
--
-- Parameters:
-- * p_component_type:       t_component_type
-- * p_master_id:            Master component ID
-- * p_subscribing_id:       Subscribing component ID
-- * p_subscribing_app_id:   Subscribing app ID
-- * p_initial_refresh:      Flag to determine if the refresh is happening right after subscription
--                           or during subsequent refresh or publish operations
--                           this parameter is not applicable to all shared components
-- * p_master_app_id:        Master app ID, this is only required for PAGE,
--                           because ID is not unique in wwv_flow_steps table
--                           not used or applicable for other components
--
--==============================================================================
procedure refresh_component (
    p_component_type     in t_component_type,
    p_master_id          in number,
    p_subscribing_id     in number,
    p_subscribing_app_id in number,
    p_initial_refresh    in boolean default false,
    p_master_app_id      in number  default null );

--==============================================================================
-- Generic procedure to unsubscribe a shared component
--
-- Parameters:
-- * p_component_type:       t_component_type
-- * p_component_id:         Shared component ID to unsubscribe
-- * p_app_id:               Application ID
--==============================================================================
procedure unsubscribe_component (
    p_component_type  in t_component_type,
    p_component_id    in number,
    p_app_id          in number );

--==============================================================================
-- Generic procedure to clear reference_id column in subscribing app
-- It's called when master shared component is deleted
--
-- Parameters:
-- * p_component_type:       t_component_type
-- * p_master_id:            Shared component ID that's being deleted
--
--==============================================================================
procedure clear_subscriptions (
    p_component_type  in t_component_type,
    p_master_id       in number );

--==============================================================================
-- This procedure looks for the build option by name in p_to_application_id app
-- if build option is not found, then build option is copied from p_from_application_id to p_to_application_id
--
-- Parameters:
--
-- * p_from_patch_id:        Build option to search for
-- * p_from_application_id:  App ID where p_from_patch_id build option exists
-- * p_to_application_id:    App ID to search or copy the build option to
-- * p_include_subscription: Flag to determine whether to subscribe to the Build Option or not
-- * p_copy_subscription:    Flag to determine whether to copy reference_id as is or not from p_copy_from_app_id
--                           This flag is applicable only if p_include_subscription is true
--
-- Returns:
-- * Build option id
--==============================================================================
function get_patch_in_other_app(
    p_from_patch_id        in number,
    p_from_application_id  in number,
    p_to_application_id    in number,
    p_include_subscription in boolean default false,
    p_copy_subscription    in boolean default false )
    return number;

--==============================================================================
-- This procedure publishes the build option data from master to all subscribing build options
--
-- Parameters:
--
-- * p_patch_id: Master build option id
--==============================================================================
procedure push_patch (
    p_patch_id in number );

--==============================================================================
-- This procedure publishes the application item data from master to all subscribing items
--
-- Parameters:
--
-- * p_item_id: Master application item id
--==============================================================================
procedure push_app_item (
    p_item_id in number );

--==============================================================================
-- This procedure publishes the application process data from master to all subscribing processes
--
-- Parameters:
-- * p_process_id: Master application process id
--==============================================================================
procedure push_app_process (
    p_process_id in number );

--==============================================================================
-- This function copies an application computation
--
-- Parameters:
-- * p_copy_from_computation_id:  Application computation ID to copy
-- * p_copy_from_app_id:          App ID to copy the Application computation from
-- * p_new_computation_item:      New application computation item
-- * p_copy_to_app_id:            App ID to copy the Application computation to
-- * p_include_subscription:      Flag to determine whether to subscribe to the Application computation or not
--
-- Returns:
-- * ID of the new Application computation
--==============================================================================
function copy_app_computation (
    p_copy_from_computation_id  in number,
    p_copy_from_app_id          in number,
    p_new_computation_item      in varchar2,
    p_copy_to_app_id            in number,
    p_include_subscription      in boolean default false )
    return number;

--==============================================================================
-- This procedure refreshes the application computation data from master to the subscribing computation
--
-- Parameters:
-- * p_master_computation_id:      Master application computation id
-- * p_subscribing_computation_id: Subscribing application computation id
-- * p_subscribing_app_id:         Subscribing application id
--==============================================================================
procedure refresh_app_computation (
    p_master_computation_id      in number,
    p_subscribing_computation_id in number,
    p_subscribing_app_id         in number );

--==============================================================================
-- This procedure publishes the application computation data from master to all subscribing computations
--
-- Parameters:
-- * p_computation_id: Master application computation id
--==============================================================================
procedure push_app_computation (
    p_computation_id in number );

--==============================================================================
-- This procedure publishes the text message data from master to all subscribing messages
--
-- Parameters:
-- * p_message_id: Master text message id
--==============================================================================
procedure push_message (
    p_message_id in number );

--==============================================================================
-- This procedure publishes the ACL Role data from master to all subscribing roles
--
-- Parameters:
-- * p_user_group_id: Master ACL role id
--==============================================================================
procedure push_user_group (
    p_user_group_id in number );

--==============================================================================
-- This procedure publishes the report layout data from master to all subscribing report layouts
--
-- Parameters:
--
-- * p_report_layout_id: Master report layout id
--==============================================================================
procedure push_report_layout (
    p_report_layout_id in number );

--==============================================================================
-- This procedure publishes the application setting data from master to all subscribing settings
--
-- Parameters:
--
-- * p_setting_id: Master application setting id
--==============================================================================
procedure push_app_setting (
    p_setting_id in number );

--==============================================================================
-- This procedure publishes the component settings data from master to all subscribing components
--
-- Parameters:
--
-- * p_setting_id: Master component settings id
--==============================================================================
procedure push_component_setting (
    p_setting_id in number );

--==============================================================================
-- This procedure refreshes all the components that are in the status "Needs Refresh"
--  for a given shared component type
--
-- Parameters:
--
-- * p_app_id:         Application ID
-- * p_component_type: Shared component type to refresh
--==============================================================================
procedure refresh_all_components (
    p_app_id         in number,
    p_component_type in varchar2 );

--==============================================================================
-- This function gets publish text message for a given shared component
--  it is used in page 4000:149
--
-- Parameters:
--
-- * p_id:             Shared Component ID
-- * p_component_type: Shared component type
--==============================================================================
function get_publish_text (
    p_id             in number,
    p_component_type in t_component_type )
    return varchar2;

--==============================================================================
-- This function gets SQL query to list subscribed components of a shared component
--  it is used in page 4000:149
--
-- Parameters:
--
-- * p_component_type: Shared component type
--==============================================================================
function get_subscribed_components_sql (
    p_component_type in t_component_type )
    return varchar2;

end wwv_flow_copy_metadata;
/
show errors