set define '^'
set verify off
prompt ...create flows_files


Rem  Copyright (c) 2001, 2023, Oracle and/or its affiliates.
Rem
Rem    NAME
Rem      flows_files_new.sql
Rem
Rem    DESCRIPTION
Rem      Creates database table required to store uploaded files.
Rem
Rem    NOTES
Rem
Rem    ARGUMENTS
Rem       1. TNS connect information
Rem       2.
Rem       3. Flows Schema Owner
Rem       4.
Rem       5. Flows Password
Rem       6.
Rem       7.
Rem       8.
Rem       9. Application Tablespace
Rem
Rem    RUNTIME DEPLOYMENT: YES
Rem
Rem    MODIFIED   (MM/DD/YYYY)
Rem       jstraub  10/04/2001 - Created
Rem


prompt ... create wwv_flow_file_objects

create table wwv_flow_file_objects$ (
    id                               number
                                     constraint wwv_flow_file_obj_pk
                                     primary key,
    flow_id                          number not null,
    name                             varchar2(400) not null unique,
    pathid                           number,
    filename                         varchar2(400),
    title                            varchar2(255),
    mime_type                        varchar2(255),
    doc_size                         number,
    dad_charset                      varchar2(128),
    created_by                       varchar2(255),
    created_on                       date,
    updated_by                       varchar2(255),
    updated_on                       date,
    deleted_as_of                    date default to_date('01-01-0001','DD-MM-YYYY') not null,
    last_updated                     date,
    content_type                     varchar2(128),
    blob_content                     blob,
    language                         varchar2(30),
    description                      varchar2(4000),
    security_group_id                number,
    file_type                        varchar2(255),
    file_charset                     varchar2(128),
    session_id                       number
    )
/


comment on table wwv_flow_file_objects$ is
'General file repository table for all flows applications, used to store SQL Scripts, BLOBs, etc.'
/

create index wwv_flow_files_file_idx on wwv_flow_file_objects$ (security_group_id, filename, flow_id)
/

create index wwv_flow_files_user_idx on wwv_flow_file_objects$ (created_by)
/

create index wwv_flow_files_session_idx on flows_files.wwv_flow_file_objects$ (session_id)
/

create or replace public synonym wwv_flow_file_objects$ sharing=NONE for wwv_flow_file_objects$
/
