set define '^' verify off
prompt ...wwv_util_APEX_240100
create or replace package sys.wwv_util_APEX_240100 as
--------------------------------------------------------------------------------
--
--  Copyright (c) Oracle Corporation 1999 - 2022. All Rights Reserved.
--
--    NAME
--      wwv_util_apex.sql
--
--    DESCRIPTION
--      This package is run during installation and creates a package in the
--      SYS schema named wwv_util_APEX_220200. It is used to create another package in SYS 
--      WWV_FLOW_VAL. It also contains a procedure to adjust the start_date of the APEX 
--      upgrade job on ADB.
--
--
--    MODIFIED   (MM/DD/YYYY)
--     jstraub    02/20/2020 - Created
--
--------------------------------------------------------------------------------

--==============================================================================
--    PARAMETERS
--      * p_overwrite_key: If true, always overwrite an existing key. If null,
--                         keep an old key unless it's version is too old.
--==============================================================================
procedure create_key_package (
    p_overwrite_key in boolean default null );

--==============================================================================
procedure adb_upgrade_now( 
	p_start_date in timestamp with time zone default systimestamp );

--==============================================================================
-- Grants the object privileges of role APEX_GRANTS_FOR_NEW_USERS_ROLE
-- to a user identified by p_to_user.
-- To be called by APEX when provisioning a new schema.
-- Oject privileges need to be granted by SYS so they are not
-- dropped when APEX is upgraded or uninstalled.
--==============================================================================
procedure grant_object_privileges (
    p_to_user in varchar2 );

--==============================================================================
-- Returns whether the given p_schema is an APEX repository schema. Used to
-- distinguish upgraded APEX schemas from imposter schemas.
--==============================================================================
function is_apex_schema (
    p_schema in varchar2 )
    return boolean;

--==============================================================================
-- calls backported LIBRARY SYS.CRYPTO_TOOLKIT_LIBRARY
--==============================================================================
procedure ecdh_genkeypair (
    curveid      in  pls_integer,
    pubkey       out raw,
    privkey      out raw );

--==============================================================================
-- calls backported LIBRARY SYS.CRYPTO_TOOLKIT_LIBRARY
--==============================================================================
procedure ecdhderive_sharedsecret (
    curveid      in  pls_integer,
    peerpubkey   in  raw,
    privkey      in  raw,
    sharedsecret out raw );

--==============================================================================
-- calls backported LIBRARY SYS.CRYPTO_TOOLKIT_LIBRARY
--==============================================================================
function sign (
    src        in raw,
    prv_key    in raw,
    pubkey_alg in pls_integer,
    sign_alg   in pls_integer )
    return raw;

--==============================================================================
-- calls backported LIBRARY SYS.CRYPTO_TOOLKIT_LIBRARY
--==============================================================================
function encrypt (
    dat in            raw,
    typ in            pls_integer,
    key in            raw,
    iv  in            raw,
    aad in            raw,
    tag out           raw )
    return  raw;

function decrypt (
    dat in            raw,
    typ in            pls_integer,
    key in            raw,
    iv  in            raw,
    aad in            raw,
    tag in            raw )
    return  raw;

end wwv_util_APEX_240100;
/
show errors
