/*!
 * Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 */

/*!
 *  sql.js - Monarch impementation of Oracle SQL & PL/SQL for specific use in Oracle APEX
 */

/* global define */

define(['require', 'exports'], function (require, exports) {
    'use strict';
    Object.defineProperty(exports, '__esModule', { value: true });
    exports.conf = {
        comments: {
            lineComment: '--',
            blockComment: ['/*', '*/'],
        },
        brackets: [
            ['{', '}'],
            ['[', ']'],
            ['(', ')']
        ],
        autoClosingPairs: [
            { open: '{', close: '}' },
            { open: '[', close: ']' },
            { open: '(', close: ')' },
            { open: '"', close: '"' },
            { open: '\'', close: '\'' },
        ],
        surroundingPairs: [
            { open: '{', close: '}' },
            { open: '[', close: ']' },
            { open: '(', close: ')' },
            { open: '"', close: '"' },
            { open: '\'', close: '\'' },
        ]
    };
    exports.language = {
        defaultToken: '',
        tokenPostfix: '.sql',
        ignoreCase: true,
        brackets: [
            { open: '[', close: ']', token: 'delimiter.square' },
            { open: '(', close: ')', token: 'delimiter.parenthesis' }
        ],
        // keywords without duplicated of atoms, datatypes or builtinFunctions
        keywords: [
            // all keywords from v$reserved_words with length > 1
            'CASE','SYS_OP_TRTB','INTERVAL','CALL','RULE','CSCONVERT','ENQUEUE','NORELOCATE','ADG_REDIRECT_PLSQL','ANALYZE','NO_GBY_PUSHDOWN','RANDOM','LOBS','QUEUE_CURR','PATH_PREFIX','UNIXTIME','APPROX_COUNT','OR','NOMINVALUE','GLOBAL_NAME','CUBE_AJ','QUORUM','XMLINDEX_REWRITE','SYSBACKUP','NEVER','STATS_F_TEST','CLASS','ARRAYS','CREATE','NO_DIST_AGG_PROLLUP_PUSHDOWN','SORT','AVRO','REDACTION','ENABLE_PRESET','FROM','CLONE','CONTINUE','SEQUENCED','SYS_XMLINSTR','NO_USE_CUBE','BACKUP','FORCE','TRANSITION','TO_ACLID','NOMINIMIZE','NLJ_PREFETCH','FETCH','NO_BIND_AWARE','SYS_XQERRH','REMOVE','DATA','SYS_XQFTCONTAIN','ACTIVE_DATA','MERGE_AJ','PUBLIC','SYS_OP_CSX_PATCH','AUTOALLOCATE','CPU_COUNT','TO_BLOB','PERCENT_RANK','SYS_OP_PARGID','TABLESPACE','SYS_XMLNODEID_GETCID','NO_SET_TO_JOIN','XID','REFTORAW','USE_HASH_GBY_FOR_PUSHDOWN','DATA_SECURITY_REWRITE_LIMIT','WITH_PLSQL','SYS_XQEXSTWRP','TRACE','LIFECYCLE','ACTIVE','PURGE','NO_PQ_CONCURRENT_UNION','STATS_T_TEST_ONE','SOURCE_FILE_DIRECTORY','UNION','XMLTOKENSET','CURRENT_SCHEMA','UTF16LE','CROSSEDITION','JSON_SERIALIZE','ENCRYPTION','NO_RESULT_CACHE','CASCADE','NO_PLACE_GROUP_BY','SYS_OP_ENFORCE_NOT_NULL$','GROUP_BY','BITMAP_AND','AUTHID','DEDUPLICATE','VERSIONS_STARTSCN','SYS_OP_VREF','UNTIL','LOG_READ_ONLY_VIOLATIONS','LIST','NO_PULL_PRED','NLJ_BATCHING','LINK','BUFFER_POOL','NO_USE_SCALABLE_GBY_INVDIST','DISABLE_CONTAINER_MAP','ACTIONS','LEAF','DML_UPDATE','SYS_OP_BLOOM_FILTER','NOSORT','RECOVERABLE','SYS_OP_VECBIT','DISCONNECT','V2','PATHS','TRUSTED','DATASTORE','TO_APPROX_PERCENTILE','SUBSTRC','DATAOBJ_TO_MAT_PARTITION','KEYSTORE','SYS_SC_2_XMLT','PHV','NATIVE','MINIMUM','NO_CPU_COSTING','MAXSIZE','AUTO_LOGIN','NO_SUBQUERY_PRUNING','SYS_OP_UNDESCEND','STANDBY_MAX_DATA_DELAY','JSONGET','STATS_T_TEST_INDEPU','PLSQL_WARNINGS','SAVE','TIMEZONE_REGION','SYS_OP_VECOR','LAST','SUBSTR4','SYS_OP_ROWIDTOOBJ','NO_USE_HASH_AGGREGATION','NOENTITYESCAPING','BYTE','CUBE_SJ','ELIM_GROUPBY','INDEX_SS','NOWAIT','FLOB','NOGUARANTEE','NO_BAND_JOIN','FEATURE','SYS_XQEXLOBVAL','SID','ENCRYPT','SYS_XQESHTMLURI','APPEND_VALUES','XMLTOJSON','SYS_OP_NII','RETRY_ON_ROW_CHANGE','MATERIALIZE','LOGMINING','DOMAIN_INDEX_NO_SORT','SYS_XQSEQDEEPEQ','FINE','FBTSCAN','MEMOPTIMIZE_WRITE','TO','SYS_OP_HCS_TABLE','POOL_16K','SYS_XQ_ATOMCNVCHK','SUBSTR2','STORAGE','VALIDATE','MERGE','CLUSTER_DISTANCE','BINARY_DOUBLE_NAN','RESOLVE','NO_USE_NL','COMMON','SYS_XQPOLYMOD','XMLISVALID','REFERENCED','ANYSCHEMA','OLD','QUARANTINE','PIV_SSF','APPROX_PERCENTILE_AGG','ORA_INVOKING_XS_USER_GUID','MODEL_DONTVERIFY_UNIQUENESS','PCTFREE','BEGIN','NO_DOMAIN_INDEX_FILTER','TIMEZONE_MINUTE','CON_ID','RESIZE','ANSI_REARCH','INDEXES','SYS_OP_PART_ID','RECONNECT','RDBA','SHARD','SYS_OP_RMTD','LINEAR','FAILGROUP','FTP','SYS_XQPOLYSUB','BINARY_DOUBLE_INFINITY','ANTIJOIN','VALUES','WRAPPER','VECTOR','ADD_COLUMN','PARALLEL','FOLLOWING','MAXIMIZE','DAY','NO_DST_UPGRADE_INSERT_CONV','EQUIPART','NEW','REJECT','INDICATOR','OFFSET','RANKM','DEFAULT','INDEX_COMBINE','DDL','SERVICES','UPDATEXML','NO_CLUSTERING','PLSQL_CODE_TYPE','XS','CRASH','NLJ_INDEX_FILTER','CONNECT_BY_FILTERING','JSON_OBJECTAGG','BITMAP_BIT_POSITION','HIGH','STARTUP','RBA','IMPACT','HIDE','REGR_AVGY','ERROR_ARGUMENT','SYS_XQRYVARGET','GRANT','LOB','WITH','ISOLATION','HOST','REDUNDANCY','NO_XML_DML_REWRITE','SYS_OP_MAKEOID','PARTITION_RANGE','NO_UNNEST','RECORDS_PER_BLOCK','VISIBILITY','USE_HASH','ESCAPE','REF','FEATURE_ID','NLS_SPECIAL_CHARS','OMIT','MISSING','SCRUB','SYS_PXQEXTRACT','COMMIT','MATCH_NUMBER',
            'DETERMINED','BATCHSIZE','ISOLATION_LEVEL','FREELIST','NOREVERSE','CAPTION','COARSE','SYS_OP_COUNTCHG','TABLE','MANAGEMENT','SYS_OP_GROUPING','HOUR','NO_FILTERING','ORDER_SUBQ','NO_BUSHY_JOIN','DENSE_RANKM','NOMAPPING','MODIFY','SYS_XMLTRANSLATE','ACCOUNT','CORRUPTION','CLAUSE','LOGICAL_READS_PER_CALL','LEAD_DIFF','IO_REQUESTS','CONFIRM','PUSH_HAVING_TO_GBY','OTHER','SD_SHOW','INDEX_STATS','RESTART','SYS_CHECK_PRIVILEGE','INDEX_ASC','COLUMNS','ROWDEPENDENCIES','ALTER','OR_PREDICATES','LOCALTIME','CHECK_ACL_REWRITE','CPU_COSTING','NL_AJ','STREAM','XMLDIFF','SERVERERROR','DEFERRED','AVCACHE_OP','COMPILE','PASSWORD_REUSE_MAX','DELETEXML','DATAMOVEMENT','ORA_CHECK_PRIVILEGE','APPROX_RANK','PRECOMPUTE_SUBQUERY','SELECT','NO_REF_CASCADE','DISABLE_PRESET','NO_SQL_TUNE','COLUMN_AUTHORIZATION_INDICATOR','SYS_FILTER_ACLS','VECTOR_READ_TRACE','CLUSTER_BY_ROWID','SHARDS','SYS_OP_PAR_1','NOPARALLEL_INDEX','DEALLOCATE','NO_MULTIMV_REWRITE','EXPORT','NLSSORT','SWITCH','APPLY','BEFORE','ORA_INVOKING_USER','SYS_XMLNODEID','IDLE_TIME','STATEMENT_QUEUING','FREELISTS','ROUND_TIES_TO_EVEN','WORK','OUTLINE','WEEK','JSON_OBJECT','FRESH','PREVIOUS','SYS_OP_NTCIMG$','OBJ_ID','INTERPRETED','PERCENTILE_DISC','UNDROP','ROLLUP','MODEL_PUSH_REF','COPY','FLASH_CACHE','MAXTRANS','SB4','SYS_XQDOC','TIES','STANDBY','PASSWORD_LIFE_TIME','SYS_OP_LBID','ANY','RESPECT','MAPPER','ROLESET','FORCE_XML_QUERY_REWRITE','HIER_WINDOW','CFILE','UNDER','SKIP_EXT_OPTIMIZER','MINUTE','UNNEST_NOSEMIJ_NODISTINCTVIEW','ROWS','BITMAP_TREE','OCCURRENCES','COLLECT','MIGRATE','LOBNVL','CPU_PER_SESSION','ORA_GET_ACLIDS','SMALLFILE','SYS_DL_CURSOR','ASSEMBLY','NO_ROOT_SW_FOR_LOCAL','PRIVILEGE','SKIP','DEGREE','ZONEMAP','ANALYTIC','STATE','REPAIR','SYS_OP_COMBINED_HASH','STRUCTURE','CRITICAL','NO_TEMP_TABLE','ENTERPRISE','STRIPE_COLUMNS','PATCH','LDAP_REGISTRATION_ENABLED','ORA_DM_PARTITION_NAME','VERIFY','NO_STATS_GSETS','XMLCONCAT','MERGE$ACTIONS','ELIMINATE_JOIN','SYS_XQFNDATIM','FUNCTIONS','FIRST_ROWS','GROUP','COMPONENT','APPLICATION','PACKAGES','CONNECT_BY_COMBINE_SW','DISASSOCIATE','UB2','PUSH_SUBQ','SITE','UNCONDITIONAL','ORA_RAWCONCAT','AUTOEXTEND','CONTROLFILE','DEREF_NO_REWRITE','COMPOUND','SYS_MAKE_XMLNODEID','COLD','DISABLE_PARALLEL_DML','CORRUPT_XID','UPGRADE','DISABLE_CONTAINERS_DEFAULT','TEST','SUBQUERY_PRUNING','EXPIRE','NAN','IDENTIFIED','CLUSTER_SET','TO_NCHAR','ELIMINATE_OUTER_JOIN','ENCODE','SYS_ET_RAW_TO_BFILE','COLUMN_AUTH_INDICATOR','MODIFICATION','MAX_PDB_SNAPSHOTS','IMPORT','USE_CONCAT','COOKIE','REGISTER','LABEL','SYS_XMLGEN','CLOSE','INITRANS','NO_VECTOR_TRANSFORM','SYS_OP_VECXOR','SYS_OP_XPTHIDX','PASSWORD_REUSE_TIME','SYS_ET_IMAGE_TO_BLOB','HELP','COMPUTE','KEEP_DUPLICATES','SIBLINGS','TO_BINARY_FLOAT','EXEMPT','INCLUDE_VERSION','NO_ELIMINATE_OUTER_JOIN','SYS_OP_DUMP','CURSOR','JSON_MERGEPATCH','SYS_XQSEQINSB','NO_INDEX','IGNORE_OPTIM_EMBEDDED_HINTS','FEATURE_VALUE','RAWTOREF','PROFILE','LIKEC','SYS_ET_BLOB_TO_IMAGE','CONST','EXCLUDING','NETWORK','ORA_DST_CONVERT','XML','REGR_AVGX','NO_BATCH_TABLE_ACCESS_BY_ROWID','UNLOAD','MAP','NLS_LANGUAGE','CONTAINER_MAP_OBJECT','FULL_OUTER_JOIN_TO_OUTER','JSONPARSE','PREV','SYS_CHECKACL','STATS_T_TEST_PAIRED','NO_ADAPTIVE_PLAN','SOME','HTTP','NO_REWRITE','NO_INMEMORY_PRUNING','REGR_R2','NO_LOAD','MODIFY_COLUMN_TYPE','CHOOSE','SYS_XQFORMATNUM','GATHER_OPTIMIZER_STATISTICS','NO_GATHER_OPTIMIZER_STATISTICS','SYS_TYPEID','WALLET','HINTSET_END','ELEMENT','NEXT','INDEX_DESC','INDENT','RESTRICT_ALL_REF_CONS','SCHEMA','ASYNCHRONOUS','OVERFLOW_NOMOVE',
            'CANCEL','SYS_RAW_TO_XSID','CO_AUTH_IND','WITHIN','NATIONAL','OID','SYS_XMLNODEID_GETSVAL','SYS_XQPOLYCST','BITMAPS','NO_BUFFER','SAVE_AS_INTERVALS','SYS_OP_PIVOT','CLEAN','RETENTION','CON_ID_TO_CON_NAME','UTF16BE','ORA_CONCAT_RWKEY','DEFERRABLE','MANAGER','TAG','PULL_PRED','SYS_XMLNODEID_GETLOCATOR','UNUSABLE','QUIESCE','REMOTE_MAPPED','SYS_XQSEQ2CON','CURSOR_SPECIFIC_SEGMENT','OUTLINE_LEAF','CONDITIONAL','CPU_TIME','USING','AUTO_REOPTIMIZE','DIRECT_LOAD','SYS_MKXTI','PACKAGE','SYS_OP_BL2R','SYS_OP_CONVERT','LEAD_CDB_URI','REFRESH','READ_OR_WRITE','NUM_INDEX_KEYS','OLAP','FIXED_VIEW_DATA','LAX','PQ_MAP','DML','NOLOGGING','INITIAL','SUBSTITUTABLE','SYS_XQFNNSURI','AV_CACHE','CONNECT','SYS_OP_PARGID_1','PRECEDES','PROGRAM','MEMBER','GET','OLD_PUSH_PRED','SUSPEND','NO_USE_PARTITION_WISE_DISTINCT','VIEW','SYS_CTX_CONTAINS2','INDEX_JOIN','PROXY','SEG_FILE','SYS_XQPFXFRMQNM','MAX_DIAG_SIZE','SYS_CTX_MKIVIDX','PRECEDING','NESTED_TABLE_ID','PQ_DISTRIBUTE','UNPLUG','USE','MASTER','VERIFIER','DISTINCT','ITERATION_NUMBER','PASSWORDFILE_METADATA_CACHE','UNPROTECTED','SPFILE','BAND_JOIN','SYS_XQFNPREDTRUTH','ALIAS','SIGNAL_COMPONENT','NOAUDIT','SYS_XMLTYPE2SQL','SET','USE_HIDDEN_PARTITIONS','SYS_XMLCONTAINS','NOARCHIVELOG','NO_SQL_TRANSLATION','SHUTDOWN','STATS_MODE','SDO_GEOM_MAX_X','ADG_REDIRECT_DML','AS_JSON','NO_FULL_OUTER_JOIN_TO_OUTER','SYS_XQRESVURI','END_OUTLINE_DATA','COMMON_DATA_MAP','HIER_ANCESTOR','SESSIONS_PER_USER','SOURCE','NAV','ESTIMATE','BY','WHITESPACE','NOCYCLE','NLS_NUMERIC_CHARACTERS','NLS_COLLATION_ID','CREATE_STORED_OUTLINES','FAILOVER','JSON','FORCE_SPATIAL','ALL_ROWS','MEASURES','END','SYS_PLSQL_IO','SYS_OP_MAP_NONNULL','ABSENT','DV','SYS_XQPOLYUPLS','PARTITION_CONTAINED','SDO_GEOM_MAX_Z','TEXT','SYS_DM_RXFORM_NUM','BUFFER_CACHE','MINING','SHARE_OF','MONTH','OIDINDEX','PRIMARY','EMPTY','NLS_TERRITORY','ERROR_ON_OVERLAP_TIME','KILL','TABLE_STATS','DEFINED','XMLTSET_DML_ENABLE','OVER','JSON_EXISTS','SIMPLE','RANDOM_LOCAL','RULES','CALLBACK','INSERTCHILDXMLAFTER','TENANT_ID','NO_EXPAND_TABLE','CURRENT_INSTANCE','REGR_SLOPE','STATS_CROSSTAB','SERIALIZABLE','OWN','TO_BINARY_DOUBLE','INHERIT','OPCODE','SYS_OP_KEY_VECTOR_FILTER','SEG_BLOCK','ENTRY','NO_DATA_SECURITY_REWRITE','SYS_XQPOLYVNE','SYS_OP_VVD','SYS_FNMATCHES','NO_COST_XML_QUERY_REWRITE','NONBLOCKING','AUDIT','ROW','TIME','CLUSTER_PROBABILITY','PORT','SUBSET','METHOD','COMPACT','CLUSTER_DETAILS','ORDER','XMLFOREST','TOKEN','SHRINK','SYS_XQNSP4PFX','NO_USE_VECTOR_AGGREGATION','SCHEMACHECK','PCTINCREASE','ADAPTIVE_PLAN','USE_PARTITION_WISE_GBY','GLOBAL_TOPIC_ENABLED','REWRITE_OR_ERROR','SECUREFILE_DBA','ARRAY','OPT_PARAM','ORA_CLUSTERING','SDO_GEOM_MIN_Y','MINUS','INSERTCHILDXML','FACTORIZE_JOIN','PRIOR','ASC','USERGROUP','NO_XML_QUERY_REWRITE','RESTORE','ALTERNATE','DOMAIN_INDEX_SORT','NLS_CALENDAR','SYS_ROW_DELTA','SEQUENTIAL','HIER_MEMBER_UNIQUE_NAME','GROUPS','INFINITE','INSERTXMLBEFORE','WRITE','INDEX_ROWS','CREATE_FILE_DEST','PCTTHRESHOLD','SYS_XQ_PKSQL2XML','NOSCHEMACHECK','VERSIONS','SYS_XQPOLYDIV','ORA_ROWSCN_RAW','NO_MONITOR','ROLLBACK','SYS_PATHID_LASTNMSPC','TIMEZONE_HOUR','SHELFLIFE','SYS_XQENDSWITH','INCLUDE','SYS_XQINSERT','FAR','BITMAP_COUNT','REGR_SXX','ALL','EXPLOSION','DOWNGRADE','RELOCATE','MONITORING','POWERMULTISET','ARCHIVELOG','LIFE','PROPERTY','GUARANTEED','RECYCLEBIN','MAXLOGFILES','DST_UPGRADE_INSERT_CONV','JSON_TEXTCONTAINS2','TRIGGERS','ORGANIZATION','XMLCOMMENT','MULTIDIMENSIONAL','TIER','INCREMENT','REQUIRED','RELY','FREE','PROCEDURAL','CLOSE_CACHED_OPEN_CURSORS',
            'DIRECTORY','NLS_CHARSET_DECL_LEN','SHARDSPACE','LOW','BINARY_FLOAT_INFINITY','SPLIT','DROP','DISABLE','UNQUIESCE','HIERARCHICAL','NO_SWAP_JOIN_INPUTS','SUBPARTITION','SYSASM','REKEY','SYS_XQSTRJOIN','EXPAND','WITH_EXPRESSION','RESTRICT','SYS_XQNSPFRMQNM','SYS_XQINDXOF','FACTOR','SYS_OP_REF','LOGICAL','SYS_OP_XMLCONS_FOR_CSX','INPLACE','ID','DATE_MODE','NULLS','DISALLOW','AND','SYS_FNREPLACE','SYS_XMLI_LOC_ISTEXT','TYPE','ADD','NO_OR_EXPAND','OLS','NO_INMEMORY','NO_TABLE_LOOKUP_BY_NL','NO_NLJ_BATCHING','LOCK','JAVA','!=','SYS_XQMKNODEREF','DIRECT_PATH','ENABLE','KEYS','BLOCK','BLOCKSIZE','XMLEXISTS2','MODEL_NO_ANALYSIS','HIER_MEMBER_NAME','VARRAY','XDB_FASTPATH_INSERT','INACTIVE_ACCOUNT_TIME','SYS_XMLCONV','COMPOSITE_LIMIT','ACTIVITY','NO_SUBSTRB_PAD','SEVERE','DATAPUMP','NO','SCOPE','COST','OR_EXPAND','COMPLIANCE','INTERSECT','ADD_GROUP','XMLPI','TIME_ZONE','MODEL','STATS_BINOMIAL_TEST','NLS_LOWER','DYNAMIC_SAMPLING_EST_CDN','SYS_XQBASEURI','HAVING','INDEX_RS_ASC','ON','ROLES','NO_COALESCE_SQ','SCHEDULER','PARTITIONS','SYS_OP_DV_CHECK','DELEGATE','LIKE2','INMEMORY','PAST','JSON_ARRAYAGG','INDEXED','LEVEL','SYS_OP_BLOOM_FILTER_LIST','UPDATABLE','SYS_ORDERKEY_DEPTH','CONTEXT','UPD_INDEXES','RETURNING','NO_ORDER_ROLLUPS','ALGORITHM','UPDATE','BETWEEN','NLS_INITCAP','PRESENT','USERS','SYS_OP_BITVEC','UNARCHIVED','SYS_XQXFORM','ADVANCED','EXISTS','REBALANCE','LOCAL','DECRYPT','LIBRARY','BUSHY_JOIN','NESTED_TABLE_GET_REFS','ENCODING','NO_PARTIAL_COMMIT','SYS_CONNECT_BY_PATH','NO_EXPAND','EDITIONING','SYS_OP_NOEXPAND','SYS_PATHID_IS_NMSPC','ORDINALITY','CURSOR_SHARING_EXACT','PIKEY','RATIO_TO_REPORT','FILEGROUP','PRECISION','LOGFILES','LESS','HALF_YEARS','PRESERVE_OID','STATIC','LOAD','NL_SJ','FILE_NAME_CONVERT','EXCHANGE','FACT','SYS_OP_CL2C','HYBRID','VECTOR_READ','AVMDX_OP','PX_FAULT_TOLERANCE','SQL_TRANSLATION_PROFILE','MINVALUE','ORA_INVOKING_USERID','DB_VERSION','UNUSED','HASH_AJ','OBJECT','INTERNAL_CONVERT','SYS_XQFNLNAME','EDITION','NO_PQ_SKEW','MEASURE','BEGIN_OUTLINE_DATA','LIFETIME','SECRET','TIMESTAMP_TO_NUMBER','SYSOPER','ADJ_DATE','ASSOCIATE','SYS_OP_RDTM','COLLATE','UNBOUNDED','EDITIONS','SYS_OP_BIN2ADT','DB_UNIQUE_NAME','CPU_PER_CALL','V1','STOP','LEAD_CDB','NO_ANSI_REARCH','XS_SYS_CONTEXT','KEY_LENGTH','LAG_DIFF','SUBQUERIES','ORA_PARTITION_VALIDATION','CHILD','NODELAY','UPD_JOININDEX','SUPPLEMENTAL','SYS_ORDERKEY_PARENT','BEQUEATH','NO_ANSWER_QUERY_USING_STATS','HASH','NO_USE_MERGE','PRIVILEGES','SYS_OP_DISTINCT','SYS_EXTPDTXT','WHEN','NO_OUTER_JOIN_TO_ANTI','NO_ACCESS','MODEL_DYNAMIC_SUBQUERY','NO_CROSS_CONTAINER','DEMAND','CAPACITY','ROLE','BROADCAST','NAMED','STATEMENTS','NO_VECTOR_TRANSFORM_FACT','ACL','XML2OBJECT','STANDARD_HASH','ORA_SEARCH_RWKEY','ABORT','SYS_XQRNDHALF2EVN','PERMANENT','UNISTR','SYS_OP_KEY_VECTOR_FILTER_LIST','GLOBALLY','FAILED','PFILE','SYS_XQIRI2URI','INSERT','CONFORMING','INDEX_SCAN','SWAP_JOIN_INPUTS','NONE','BINARY','LOGICAL_READS_PER_SESSION','PATH','USAGE','LIKE4','FAILED_LOGIN_ATTEMPTS','ORDERED_PREDICATES','SHARDED','QUARTERS','MATCHES','FOR','XMLELEMENT','SYS_OP_KEY_VECTOR_USE','XMLPATCH','CONDITION','NO_CONNECT_BY_COST_BASED','DETERMINES','NO_SEMIJOIN','INSTEAD','DATAOBJNO','CHANGE','FOREIGN','SYS_OP_DECOMP','FOLLOWS','INITIALLY','NESTED_TABLE_SET_SETID','SYS_GET_ACLIDS','IMMEDIATE','REGR_COUNT','GLOBAL','NOMONITORING','THROUGH','NLS_SORT','SYS_OP_VECAND','ORA_TABVERSION','SOURCE_FILE_NAME_CONVERT','MIGRATION','DG_GATHER_STATS','SELF','NO_CARTESIAN','UNSUBSCRIBE','TIV_GB','PARTIAL_ROLLUP_PUSHDOWN',
            'SERVICE_NAME_CONVERT','JSON_ARRAY','DECR','PQ_FILTER','DOMAIN_INDEX_FILTER','USE_SEMI','CURRENT','SYS_XMLNODEID_GETOKEY','NO_CHECK_ACL_REWRITE','ACTIVE_FUNCTION','SYS_XQ_NRNG','SYS_XMLEXNSURI','SYS_PATHID_IS_ATTR','PARAMETERS','SYS_XQPOLYCEL','NLS_CHARSET_ID','CON_DBID_TO_ID','EXTENTS','PQ_DISTRIBUTE_WINDOW','APPROX_COUNT_DISTINCT_DETAIL','TEMPORARY','SYS_XQREPLACE','REGR_INTERCEPT','MONTHS','UNPACKED','MEMORY','DECREMENT','MOVEMENT','OUTER_JOIN_TO_ANTI','CON_ID_TO_DBID','ACCESS','STANDBYS','SYS_XQCONCAT','SYS_XQNODENAME','DANGLING','MAXEXTENTS','VARRAYS','HASH_SJ','SYS_XQNORMSPACE','REDEFINE','NO_CONNECT_BY_FILTERING','MODULE','SYS_XQNORMUCODE','STREAMS','STANDALONE','XMLEXISTS','SYS_OP_RAWTONUM','ENFORCE','BEHALF','NO_ZONEMAP','NOOVERRIDE','DYNAMIC','DOCFIDELITY','SYS_XQRSLVQNM','DIST_AGG_PROLLUP_PUSHDOWN','OPTIMIZER_FEATURES_ENABLE','OPAQUE','SYS_OP_EXTRACT','DBSTR2UTF8','PCTUSED','PRETTY','OCCURENCES','EXTEND','REPLICATION','FACILITY','SUCCESSFUL','COMPONENTS','NO_FACT','SYS_PATHID_LASTNAME','ERROR','ONLY','SYS_OP_PAR','INSERTCHILDXMLBEFORE','COLLATION','PREDICTION_SET','SYS_DM_RXFORM_LAB','DICTIONARY','NO_AUTO_REOPTIMIZE','CACHE_CB','OWNER','SYS_OP_VECTOR_GROUP_BY','SYSTEM','MIRROR','SYS_XQSEQRVS','CORRUPT_XID_ALL','INDEX_FFS','SYSAUX','ACTION','UNITE','RIGHT','VALIDATION','HIDDEN','NO_CONNECT_BY_CB_WHR_ONLY','FRESH_MV','REDO','NO_MONITORING','SDO_GEOM_MBR','BITMAP_CONSTRUCT_AGG','CALCULATED','SYS_XQRENAME','SYSDG','CLEAR','NLS_LANG','LIMIT','INSERTXMLAFTER','OUTER_JOIN_TO_INNER','BSON','USE_NL','MODE','NO_USE_PARTITION_WISE_GBY','FAMILY','INCLUDES','SCAN_INSTANCES','USE_MERGE_CARTESIAN','TRAILING','CLOSEST','BODY','ROLLING','XMLISNODE','REVOKE','PERCENTILE_CONT','ONE','HASHKEYS','DISTINGUISHED','NO_USE_PARTITION_WISE_WIF','MISMATCH','DETERMINISTIC','NORESETLOGS','ITERATE','TABLE_LOOKUP_BY_NL','BITMAP_OR_AGG','FAST','ACROSS','FRAGMENT_NUMBER','METADATA','XMLPARSE','NLS_UPPER','SYS_OP_NICOMBINE','NOSEGMENT','SYS_XQSUBSTRAFT','USE_HASH_GBY_FOR_DAGGPSHD','SDO_GEOM_MBB','GRANULAR','AUTO','CELLMEMORY','CURRENT_TIME','INTERLEAVED','ELAPSED_TIME','NO_PUSH_PRED','OVERLAPS','APPROX_SUM','SYS_XQTREATAS','FEATURE_SET','ADMINISTER','ASIS','PROJECT','SYS_REMAP_XMLTYPE','SYS_XQFNNM','NO_BASETABLE_MULTIMV_REWRITE','MAXARCHLOGS','POLICY','SYS_OP_NIX','MBRC','MATCH_RECOGNIZE','CV','SYS_AUDIT','UNRECOVERABLE','SYS_OP_NUMTORAW','RECYCLE','MAKE_REF','TEMPFILE','INVALIDATION','TREAT','NO_USE_HASH_GBY_FOR_PUSHDOWN','NOSCALE','SECURITY','SYS_OP_ITR','SYS_OP_DSB_SERIALIZE','SYS_XQDOCURI','NO_USE_INVISIBLE_INDEXES','PROCESS','UBA','VERSIONS_ENDTIME','PLSQL_OPTIMIZE_LEVEL','SYS_DM_RXFORM_CHR','CACHING','FAILURE','THAN','IO_LOGICAL','SYS_SYNRCIREDO','READS','LOGON','PARTITION','VOLUME','SEMI_TO_INNER','SYS_XQDFLTCOLATION','ELSE','SCAN','SYS_OP_XTXT2SQLT','CONTAINERS_DEFAULT','INFORMATIONAL','IGNORE_WHERE_CLAUSE','GENERATED','NO_QKN_BUFF','UCS2','PREDICTION_COST','ENFORCED','HIERARCHIES','HINTSET_BEGIN','NESTED_TABLE_SET_REFS','DESCENDANT','ACTIVE_TAG','PERCENT_RANKM','UNLOCK','HIERARCHY','RESTRICTED','SYS_XQPOLYABS','NO_PX_JOIN_FILTER','NO_USE_HASH_GBY_FOR_DAGGPSHD','REPEAT','QUALIFY','SYS_XQRYENVPGET','EACH','OFF','NO_CONTAINERS','EXPLAIN','IN','INDEX_SS_DESC','STRIP','PARTITION_HASH','SCALE','SUCCESS','EXCLUDE','NLS_NCHAR_CONV_EXCP','XMLTRANSFORMBLOB','CROSS','TRUST','OBJNO_REUSE','POST_TRANSACTION','RESETLOGS','DIGEST','INDEX_RRS','DISKS','WIDTH_BUCKET','LOCKDOWN','SYS_XQPOLYFLR','REGULAR','USE_SCALABLE_GBY_INVDIST','ROW_NUMBER','SYS_XQFNQNM','RENAME','NLS_LENGTH_SEMANTICS',
            'PREBUILT','PIV_GB','SYS_CTXINFOPK','SETTINGS','SDO_TOLERANCE','BINDING','SYS_OP_DESCEND','ADVISOR','USE_ANTI','LOST','SYS_XQPOLYADD','DEREF','PARENT','ROW_LENGTH','TRIGGER','SNAPSHOT','TRACKING','IN_MEMORY_METADATA','ALLOCATE','EVENTS','QUOTA','DB_ROLE_CHANGE','INTRA_CDB','RAWTONHEX','EVAL','TUNING','INSTANCES','NLS_DATE_LANGUAGE','SYNONYM','INDEX_RS','BASIC','PARAM','DEFAULT_PDB_HINT','RESULT_CACHE','OLTP','SECONDS','NO_CLUSTER_BY_ROWID','COMPOSITE','COMMENT','SYS_XQRYWRP','SYS_OP_MSR','NO_ELIMINATE_JOIN','SYS_OP_ADTCONS','PARTIAL','NO_TRANSFORM_DISTINCT_AGG','NO_STAR_TRANSFORMATION','USABLE','NO_NLJ_PREFETCH','STORAGE_INDEX','SYSTEM_STATS','HIER_CHILD_COUNT','LDAP_REG_SYNC_INTERVAL','GROUPING','SYS_XQNILLED','SCN','NO_CONNECT_BY_ELIM_DUPS','SYS_PARALLEL_TXN','SYS_FBT_INSDEL','HIER_PARENT','INDEX_FILTER','NTILE','USE_PRIVATE_OUTLINES','ANCILLARY','SYNC','BECOME','BUILD','MATERIALIZED','REF_CASCADE_CURSOR','SYS_OP_TOSETID','NOREWRITE','SYS_XQFNBOOL','DATAOBJ_TO_PARTITION','DATA_VALIDATE','TEMPLATE','AFD_DISKSTRING','NO_USE_DAGG_UNION_ALL_GSETS','CREDENTIALS','NO_INDEX_FFS','DEBUG','ONLINE','ORA_DST_ERROR','BITS','INDEX_RS_DESC','PASSWORD_GRACE_TIME','INCREMENTAL','SUBPARTITIONING','BOUND','USE_INVISIBLE_INDEXES','HIER_CHILDREN','RESERVOIR_SAMPLING','VECTOR_ENCODE','UNNEST','IF','SYS_XQREF2VAL','PQ_SKEW','CLUSTER','SYS_OP_TPR','ACTIVE_COMPONENT','OPEN','PER','WEEKS','NLS_CURRENCY','EVERY','NOCOPY','USE_HASH_AGGREGATION','BEGINNING','JSON_TRANSFORM','SYS_OP_NIEXTRACT','IOTFRSPEED','ILM','USE_MERGE','LOCAL_INDEXES','PLUGGABLE','START','SYS_XQTOKENIZE','VERSIONS_STARTTIME','FLUSH','CERTIFICATE','PREDICTION_DETAILS','SPACE','ROWID_MAPPING_TABLE','MOUNTPATH','SET_TO_JOIN','MANAGE','NO_PARTIAL_ROLLUP_PUSHDOWN','SYS_MAKEXML','SYS_OP_LOBLOC2NCLOB','NO_PARTIAL_JOIN','OBJNO','HWM_BROKERED','FEATURE_COMPARE','SYS_OP_XPTHOP','PREDICATE_REORDERS','ENABLE_PARALLEL_DML','SYNCHRONOUS','NO_REORDER_WIF','SAVEPOINT','COLUMNAR','FLASHBACK','LAG_DIFF_PERCENT','SYS_XQEXVAL','ORA_SHARD_ID','SHARE','BULK','NO_XDB_FASTPATH_INSERT','XMLTABLE','CONTAINER_MAP','PX_GRANULE','PLSQL_DEBUG','SYS_XQSTARTSWITH','DAYS','UNLIMITED','OF','IGNORE','PQ_CONCURRENT_UNION','TIMEZONE_ABBR','NO_COMMON_DATA','=>','FREEPOOLS','OPTION','OPERATIONS','SYS_XQDURDIV','NOFORCE','ORADEBUG','SYS_XQPOLYVGE','SYS_OP_IIX','NO_PLACE_DISTINCT','SINGLE','XMLCAST','SYS_RID_ORDER','XMLCDATA','SYS_EXTRACT_UTC','SUPPRESS_LOAD','>=','THE','PRESENTNNV','TIV_SSF','CACHE_TEMP_TABLE','TO_TIME','OPERATOR','MOVE','READ','USE_PARTITION_WISE_DISTINCT','SYS_XQCASTABLEERRH','PIVOT','BITMAP','NCHAR_CS','JSON_LENGTH','HOT','NOCPU_COSTING','PRIVILEGED','WELLFORMED','SYS_OP_ADT2BIN','AUTOMATIC','OPTIMAL','NORMAL','INTO','LDAP_REGISTRATION','COMPRESS','UTF8','SYS_XQPOLYVLT','XMLINDEX_REWRITE_IN_SELECT','SEED','MERGE_CONST_ON','NO_PUSH_SUBQ','CURRENTV','BIND_AWARE','NO_ELIMINATE_OBY','OUT_OF_LINE','XMLATTRIBUTES','KEEP','ANCESTOR','APPROX_COUNT_DISTINCT','PERMISSION','CORR_S','UNDO','CONSISTENT','LOGGING','RANGE','QUEUE','BATCH','MAX_SHARED_TEMP_SIZE','SECOND','MIRRORCOLD','KEYSIZE','IGNORE_ROW_ON_DUPKEY_INDEX','PROPAGATE','GUARD','TYPENAME','TRANSACTION','NOMAXVALUE','MAIN','SDO_GEOM_KEY','SYS_XQED4URI','FILESYSTEM_LIKE_LOGGING','EXTRA','ELIMINATE_SQ','SETS','ACCESSED','WHERE','MINUTES','XMLSERIALIZE','MINIMIZE','UPDATED','KEY','SEMIJOIN','PUSH_PRED','VECTOR_TRANSFORM_DIMS','AUTHENTICATION','NONPARTITIONED','INNER','DIMENSION','NO_PX_FAULT_TOLERANCE','NO_OUTER_JOIN_TO_INNER','SEARCH','PARTITIONSET','CONSTRAINTS','GBY_CONC_ROLLUP','MAXDATAFILES',
            'USE_VECTOR_AGGREGATION','TIMEOUT','SYS_GETXTIVAL','NEG','MAPPING','USER_TABLESPACES','TRANSLATION','BYPASS_UJVC','NO_MERGE','TABNO','LEAD_DIFF_PERCENT','SYS_XQLANG','PLACE_GROUP_BY','WRAPPED','PERMUTE','BIGFILE','SDO_GEOM_MIN_X','DECLARE','NORELY','SYS_XQSTATBURI','INDEXTYPES','CYCLE','INLINE','ORA_CHECK_ACL','UNIFORM','REFERENCES','MAXINSTANCES','NOEXTEND','RESUME','SYS_OP_CAST','QUERY','HIER_LAG','PRIVATE','SCALE_ROWS','SHOW','POOL_8K','SYS_DST_PRIM2SEC','LOCATOR','OFFLINE','SYS_XQSTR2CODEP','SYS_OPTLOBPRBSC','IOSEEKTIM','SYSGUID','ORA_INVOKING_XS_USER','SYS_XQSEQTYPMATCH','EVALUATION','PERFORMANCE','STATS_KS_TEST','UNMATCHED','CDB_HTTP_HANDLER','SYS_XQFNROOT','FOREVER','DEFAULTS','SEQUENCE','DEFINER','OPAQUE_XCANONICAL','PQ_NOMAP','DBA_RECYCLEBIN','SYS_OP_R2O','EXTERNALLY','SYS_GET_COL_ACLIDS','PENDING','PASSWORD_VERIFY_FUNCTION','CHECK','INDEXTYPE','CONNECT_TIME','PASSWORD','RELATIONAL','ASYNC','TBL$OR$IDX$PART$NUM','USE_PARTITION_WISE_WIF','APPROX_PERCENTILE_DETAIL','NONEDITIONABLE','USE_DAGG_UNION_ALL_GSETS','PASSWORD_LOCK_TIME','SYS_GET_PRIVILEGES','SYS_XQSEQSUB','SYS_OP_CSR','CHUNK','ARCHIVAL','JOIN','INACTIVE','SELECTIVITY','SYS_OP_OPNSIZE','PROTECTED','ISOLATE','ASSIGN','SYS_OP_GUID','SYS_OP_KEY_VECTOR_CREATE','INVALIDATE','PLAN','LEVELS','NO_INDEX_SS','MATCH','MONITOR','SKIP_UNUSABLE_INDEXES','RAPIDLY','HEAP','DECORRELATE','SYS_XQFNCMP','CONSTRAINT','AT','SYS_OP_XPTHATG','METADATA_SOURCE_PDB','SAMPLE','DISK','HOURS','AGGREGATES','EDITIONABLE','THEN','CLUSTERING_FACTOR','GUARANTEE','PREDICTION','BLOCK_RANGE','NAME','JSON_EQUAL','AS','LEAVES','SYS_OP_C2C','INITIALIZED','SYS_XQPOLYRND','COALESCE_SQ','PROTOCOL','ANOMALY','UNIQUE','APPROX_MEDIAN','ORA_ROWVERSION','OBJECT2XML','UNINSTALL','TYPES','HEADER','MANAGED','INVISIBLE','VECTOR_TRANSFORM','ORDERED','SYS_PATH_REVERSE','SYS_OP_CSCONVTEST','UNNEST_SEMIJ_VIEW','REUSE','SYS_OP_COMP','ALLOW','BACKGROUND','REMOTE','VERSIONS_OPERATION','NO_XMLINDEX_REWRITE_IN_SELECT','TO_UTC_TIMESTAMP_TZ','COMMITTED','POWERMULTISET_BY_CARDINALITY','CON_GUID_TO_ID','SYS_XQFUNCR','INSTANTLY','XMLSCHEMA','EVALNAME','SYS_XQPOLYCSTBL','SYS_XQLLNMFRMQNM','LEFT','RUNNING','BREADTH','CONNECT_BY_ISCYCLE','SKIP_PROXY','INTERMEDIATE','SPECIFICATION','NLS_CHARSET_NAME','NOREPLAY','VARYING','EXTENDS','USE_WEAK_NAME_RESL','SYS_XQDELETE','TO_TIME_TZ','APPEND','BYPASS_RECURSIVE_CHECK','UNRESTRICTED','APPENDCHILDXML','OVERFLOW','FLEX','ORA_RAWCOMPARE','EXTERNAL','SYS_MKXMLATTR','NO_PQ_EXPAND_TABLE','CREDENTIAL','PCTVERSION','NO_XMLINDEX_REWRITE','JSON_QUERY','NESTED_TABLE_FAST_INSERT','USE_NL_WITH_INDEX','HIER_COLUMN','NOSTRICT','SYS_OP_ALSCRVAL','NOSWITCH','INSTANTIABLE','NOTHING','SQL','YES','STAR','SYS_XQINSPFX','RESET','STORE','REALM','EM','VALIDATE_CONVERSION','EXISTSNODE','SYS_ZMAP_FILTER','DATABASE','MODEL_PBY','MIGRATE_CROSS_CON','NOCACHE','SYS_XMLNODEID_GETPATHID','VECTOR_TRANSFORM_FACT','NO_NATIVE_FULL_OUTER_JOIN','RECOVER','QB_NAME','SUBPARTITION_REL','DISABLE_RPKE','USER_DATA','STATEMENT','LOGOFF','VERSION','NOPARALLEL','FINAL','MEDIUM','MAXLOGHISTORY','THREAD','ZONE','SYSKM','PROCEDURE','PBL_HS_END','HIER_CAPTION','NO_PARALLEL_INDEX','RLS_FORCE','MINEXTENTS','CLUSTERING','MINUS_NULL','APPROX_COUNT_DISTINCT_AGG','NOROWDEPENDENCIES','ARCHIVE','JSON_TABLE','SYS_XQPOLYMUL','OPAQUE_TRANSFORM','FORWARD','SYS_ORDERKEY_MAXCHILD','MEMCOMPRESS','PARTIAL_JOIN','PBL_HS_BEGIN','SYS_XQEXTRREF','SYS_OP_OIDVALUE','SYSDBA','EXCEPTIONS','ORA_HASH','SYS_DBURIGEN','PHYSICAL','INSTALL','SYS_XQ_UPKXML2SQL','DISTRIBUTED','SIZE','CELL_FLASH_CACHE',
            'UNNEST_INNERJ_DISTINCT_VIEW','GROUPING_ID','SYS_OP_CYCLED_SEQ','PQ_EXPAND_TABLE','NOVALIDATE','ENABLE_ALL','COST_XML_QUERY_REWRITE','CONTAINER_DATA','NESTED','OPTIMIZER_GOAL','YEARS','TOPLEVEL','AND_EQUAL','DELETE','NOAPPEND','CONTAINERS','FULL','BRANCH','STATS_MW_TEST','IDENTIFIER','POOL_32K','SUBPARTITIONS','WHENEVER','NOT','STDDEV_SAMP','USE_TTT_FOR_GSETS','XMLCOLATTVAL','TO_APPROX_COUNT_DISTINCT','VALID_TIME_END','ADMINISTRATOR','EXPAND_GSET_TO_UNION','DESC','QUOTAGROUP','ALWAYS','CONSIDER','NOLOCAL','AW','SIGNAL_FUNCTION','SQL_TRACE','PERIOD','DISABLE_ALL','PREDICTION_BOUNDS','SYS_XMLANALYZE','EXISTING','TABLESPACE_NO','BOTH','QUEUE_ROWP','CUME_DISTM','CON_NAME_TO_ID','SYS_GETTOKENID','CON_UID_TO_ID','PRINTBLOBTOCLOB','TIMES','SYS_OP_RPB','REGEXP_LIKE','BUFFER','DESCRIPTION','DISMOUNT','JSON_TEXTCONTAINS','PARITY','PRUNING','SHARED','CATEGORY','PROTECTION','SUBSTRB','DUPLICATED','REFERENCING','STRIPE_WIDTH','NO_USE_HASH','DEFAULT_COLLATION','DEQUEUE','REGR_SYY','HIER_ORDER','HIER_DESCRIPTION','PERSISTABLE','COLUMN_VALUE','MOUNT','NOORDER','SYS_OP_LSVI','AUTHENTICATED','XMLTRANSFORM','SYS_ET_BFILE_TO_RAW','SESSIONTZNAME','VERSIONING','MAX_AUDIT_SIZE','PQ_REPLICATE','NO_ELIMINATE_SQ','RECOVERY','USE_CUBE','STATISTICS','SYS_XQEXTRACT','DIMENSIONS','VERSIONS_ENDSCN','ANSWER_QUERY_USING_STATS','DBMS_STATS','<<','SUBSCRIBE','INCLUDING','CLASSIFIER','NLS_COMP','RESTORE_AS_INTERVALS','ADVISE','INMEMORY_PRUNING','IN_XQUERY','ORA_BRANCH','SESSION','DBA','SYS_OP_DSB_DESERIALIZE','DROP_COLUMN','RBO_OUTLINE','FEATURE_DETAILS','STATS_ONE_WAY_ANOVA','CACHE','CONTAINER_DATA_ADMIT_NULL','IDGENERATORS','RESOURCE','FILTER','MV_MERGE','HIER_LEVEL','UNPIVOT','CONVERSION','OSON','NO_PQ_MAP','HIER_DEPTH','SYS_XMLNODEID_GETTID','VISIBLE','SECUREFILE','TX','FLAGGER','CUBE','SYS_XMLLOCATOR_GETSVAL','STALE','SYS_PLSQL_COUNT','AFTER','NLS_COLLATION_NAME','SYS_OP_ATG','FIRST','REORDER_WIF','MAXVALUE','NO_OBJECT_LINK','INDEX_SKIP_SCAN','ARCHIVED','CONNECT_BY_CB_WHR_ONLY','NO_ELIM_GROUPBY','NOSHARD','DISKGROUP','MODEL_SV','JSON_EXISTS2','ORA_CHECK_SYS_PRIVILEGE','FEDERATION','NLJ_INDEX_SCAN','PATTERN','IO_MEGABYTES','CONNECT_BY_ELIM_DUPS','INTERNAL_USE','TIMEZONE_OFFSET','SYS_ZMAP_REFRESH','PREDICTION_PROBABILITY','ACTIVATE','COLUMN_STATS','REGR_SXY','FILE','CLASSIFICATION','STATEMENT_ID','DECOMPRESS','SKIP_UNQ_UNUSABLE_IDX','SYS_DST_SEC2PRIM','SYS_XQCODEPEQ','BATCH_TABLE_ACCESS_BY_ROWID','DATA_LINK_DML','FOLDER','SYS_XQPOLYUMUS','MERGE_SJ','IS','DEFINITION','INLINE_XT','UTF32','TRACING','USER_RECYCLEBIN','LANGUAGE','SPREADSHEET','POSITION','SYS_OP_VERSION','LATERAL','SYS_UMAKEXML','OWNERSHIP','CACHE_INSTANCES','OPTIMIZE','FUNCTION','PASSING','PARTIALLY','NO_CONNECT_BY_COMBINE_SW','LIKE','NO_QUERY_TRANSFORMATION','FINISH','XMLNAMESPACES','SD_ALL','ORA_ROWSCN','INDEXING','INSTANCE','REFERENCE','INCR','REFTOHEX','PX_JOIN_FILTER','ELIMINATE_OBY','EXCLUSIVE','PLACE_DISTINCT','JSONTOXML','SUBMULTISET','SYS_XMLNODEID_GETRID','HIER_LEAD','XMLROOT','SYS_OP_CEG','DATAFILES','SYS_OP_DRA','PRELOAD','DEFINE','SEMIJOIN_DRIVER','CONTENT','SINGLETASK','DELAY','IDENTITY','CONNECT_BY_COST_BASED','DESTROY','ATTRIBUTE','NO_PUSH_HAVING_TO_GBY','POOL_4K','ROWIDTOCHAR','NLS_DATE_FORMAT','SYS_OP_LOBLOC2CLOB','COLUMN','MULTISET','SYS_OP_LOBLOC2BLOB','AFFINITY','SYS_OP_ZONE_ID','INLINE_XMLTYPE_NT','DEPENDENT','DUPLICATE','SYSTEM_DEFINED','CHECKPOINT','SYS_XQCODEP2STR','BITMAP_BUCKET_NUMBER','EXPAND_TABLE','SYS_GUID','KERBEROS','TRANSITIONAL','MATCHED','ANY_VALUE','SYS_XQPOLYSQRT','SYS_XQSEQ2CON4XC','REVERSE',
            'MIRRORHOT','STRICT','SQL_SCOPE','EXPR_CORR_CHECK','SERVICE','SYS_PLSQL_CPU','SYS_XQSEQRM','TRUNCATE','CHANGE_DUPKEY_ERROR_INDEX','OUTER','USE_STORED_OUTLINES','NLS_ISO_CURRENCY','EXTENT','SYS_OPTXQCASTASNQ','SDO_GEOM_MIN_Z','SD_INHIBIT','GATHER_PLAN_STATISTICS','ENTITYESCAPING','COMPATIBILITY','LOCKED','LEADING','CHAR_CS','QUERY_BLOCK','NOTIFICATION','EXTENDED','NO_DECORRELATE','REDUCED','PRIORITY','BASICFILE','VERSIONS_XID','<=','CONTENTS','POOL_2K','ORA_GET_PRIVILEGES','ADMIN','COMPUTATION','LOCKING','EXECUTE','NEEDED','RETURN','XML_DML_RWT_STMT','APPROX_PERCENTILE','CONSTANT','INDEX_SS_ASC','GRANTED','DOCUMENT','UPSERT','SYS_OP_KEY_VECTOR_SUCCEEDED','SESSION_CACHED_CURSORS','SYS_OP_HASH','NO_VECTOR_TRANSFORM_DIMS','PRESENTV','EVALUATE','NO_STATEMENT_QUEUING','ROOT','EXTRACTVALUE','RESOLVER','SYSRAC','CUBE_GB','NOREPAIR','XMLQUERYVAL','YEAR','ROWIDTONCHAR','INSENSITIVE','NO_PRUNE_GSETS','MANUAL','DELETE_ALL','MAXLOGMEMBERS','SALT','NO_PQ_REPLICATE','FIRSTM','NO_FACTORIZE_JOIN','ORA_WRITE_TIME','DEPTH','PARTITION_LIST','TARGET','SERIAL','SUMMARY','XMLINDEX_SEL_IDX_TBL','AVAILABILITY','NO_EXPAND_GSET_TO_UNION','BINARY_FLOAT_NAN','PREPARE','AVERAGE_RANK','STDDEV_POP','DETACHED','PRIVATE_SGA','NO_MODEL_PUSH_REF','SYS_XQPOLYVLE','UNBOUND','CDB$DEFAULT','SYS_OP_LOBLOC2ID','VALUE','XPATHTABLE','PDB_LOCAL_ONLY','AGGREGATE','LOGFILE','CONTAINER','SYS_XQCON2SEQ','SHARING','PART$NUM$INST','NOCOMPRESS','SYS_XQGETCONTENT','WAIT','PARALLEL_INDEX','SQLLDR','RESUMABLE','SYS_XMLT_2_SC','STAR_TRANSFORMATION','SYS_XSID_TO_RAW','NOKEEP','CURRENT_USER','JSON_HASH','HIER_HAS_CHILDREN','JSON_VALUE','XMLQUERY','GBY_PUSHDOWN','SYS_XQSUBSTRBEF','DOUBLE','MATRIX','NO_PARALLEL','REWRITE','NATIVE_FULL_OUTER_JOIN','LAYER','COMPLETE','CLUSTER_ID','SEGMENT','SYS_OP_LOBLOC2TYP','TEMP_TABLE','VIRTUAL','PERCENT','NLS_CHARACTERSET','MODEL_NB','JSON_PATCH','HEXTOREF','DATAFILE','PARTITIONING','INDEX','DISTRIBUTE','ORA_DST_AFFECTED','NAMESPACE','VIOLATION','SCN_ASCENDING','DYNAMIC_SAMPLING','SYS_XMLNODEID_GETPTRID','USER_DEFINED','CONNECT_BY_ISLEAF','USING_NO_EXPAND','SYS_DOM_COMPARE','DRIVING_SITE','EXPRESS','SYSOBJ','X_DYN_PRUNE','PRESERVE','MEMOPTIMIZE','TRANSFORM_DISTINCT_AGG','DEBUGGER','FORMAT','SHARED_POOL','SYS_XMLI_LOC_ISNODE','LOCATION','MODEL_MIN_ANALYSIS','AUTHORIZATION','STATS_WSR_TEST','EXTRACTCLOBXML','STATS_T_TEST_INDEP','CHAINED','BLOCKS','PLSCOPE_SETTINGS','CON_ID_FILTER','SYS_XQFB2STR','SWITCHOVER','CREATION','PLSQL_CCFLAGS','NO_SEMI_TO_INNER','SDO_GEOM_MAX_Y','DROP_GROUP','SYS_OP_KEY_VECTOR_PAYLOAD','CLEANUP','MODEL_COMPILE_SUBQUERY','SYS_XQPOLYVGT','OPT_ESTIMATE','CLIENT','CONNECT_BY_ROOT','EXCEPT','SYS_OP_XTNN','SYS_OP_LVL','SYS_XQPOLYVEQ','NONSCHEMA','SYS_XQERR','SYS_OPTXICMP','SYS_OP_CSCONV','ERRORS','REBUILD','SYS_XQ_ASQLCNV','NOW','TABLES','CORR_K','WITHOUT','DISCARD','SIBLING',

            // all PL/SQL reserved words and keywords that are not present in above view
            'A','ACCESSIBLE','AGENT','BFILE_BASE','BLOB_BASE','C','CALLING','CHARSET','CHARSETFORM','CHARSETID','CHAR_BASE','CLOB_BASE','CLUSTERS','COLAUTH','COMPILED','CONSTRUCTOR','CUSTOMDATUM','DATE_BASE','DURATION','ELSIF','EXCEPTION','EXIT','FIXED','FORALL','GENERAL','GOTO','IMMUTABLE','INDICES','INTERFACE','LARGE','LIMITED','LOOP','MAXLEN','MUTABLE','NUMBER_BASE','OCICOLL','OCIDATE','OCIDATETIME','OCIDURATION','OCIINTERVAL','OCILOBLOCATOR','OCINUMBER','OCIRAW','OCIREF','OCIREFCURSOR','OCIROWID','OCISTRING','OCITYPE','ORACLE','ORADATA','ORLANY','ORLVARY','OTHERS','OUT','OVERLAPSPROCEDURE','OVERRIDING','PARALLEL_ENABLE','PARAMETER','PASCAL','PIPE','PIPELINED','POLYMORPHIC','PRAGMA','RAISE','RECORD','RELIES_ON','REM','RESULT','SB1','SB2','SEPARATE','SHORT','SIZE_T','SPARSE','SQLDATA','SQLNAME','SQLSTATE','STANDARD','STORED','STRUCT','STYLE','SUBTYPE','TABAUTH','TDO','TRANSACTIONAL','UB1','UB4','UNSIGNED','UNTRUSTED','VALIST','VARIABLE','VIEWS','VOID','WHILE'
        ],
        atoms: [
            'TRUE', 'FALSE', 'NULL'
        ],
        datatypes: [
            // manually added
            'BOOLEAN',
            // all valid keywords that are also datatypes
            // minus "EXCEPTION" as this is better suited to be highlighted as a keyword
            'BINARY_DOUBLE','DECIMAL','RAW','VARCHAR','CLOB','ROWID','NVARCHAR2','VARCHAR2','BINARY_FLOAT','INTEGER','NCLOB','CHAR','POINT','SMALLINT','BLOB','BFILE','NCHAR','TIMESTAMP','JOB','NUMBER','REAL','INT','NATURAL','UROWID','LONG','DEC','XMLTYPE','CHARACTER','DATE','FLOAT','NUMERIC','STRING','MLSLABEL',
            // apex-specific data types
            // select listagg('''' || synonym_name || ''',') from all_synonyms where owner = 'PUBLIC' and synonym_name like 'APEX_T\_%' escape '\' order by synonym_name;
            'APEX_T_APPROVAL_LOG_TABLE','APEX_T_APPROVAL_TASK','APEX_T_PARSER_WORKSHEET','APEX_T_PARSER_ROW','APEX_T_HUMAN_TASK','APEX_T_APPROVAL_LOG_ROW','APEX_T_CLOB','APEX_T_APPROVAL_TASKS','APEX_T_EXPORT_FILE','APEX_T_NUMBER','APEX_T_PARSER_COLUMN','APEX_T_VARCHAR2','APEX_T_HUMAN_TASKS','APEX_T_HUMAN_TASK_LOG_TABLE','APEX_T_PARSER_WORKSHEETS','APEX_T_LDAP_ATTRIBUTE','APEX_T_LDAP_ATTRIBUTES','APEX_T_HUMAN_TASK_LOG_ROW','APEX_T_EXPORT_FILES','APEX_T_PARSER_TABLE','APEX_T_PARSER_COLUMNS'
        ],
        builtinFunctions: [
            // source: techonthenet.com/oracle/functions/index.php
            // String/Char Functions
            'ASCII','ASCIISTR','CHR','COMPOSE','CONCAT','CONVERT','DECOMPOSE','DUMP','INITCAP','INSTR','INSTR2','INSTR4','INSTRB','INSTRC','LENGTH','LENGTH2','LENGTH4','LENGTHB','LENGTHC','LOWER','LPAD','LTRIM','NCHR','REGEXP_INSTR','REGEXP_REPLACE','REGEXP_SUBSTR','REPLACE','RPAD','RTRIM','SOUNDEX','SUBSTR','TRANSLATE','TRIM','UPPER','VSIZE',
            // Numeric/Math Functions
            'ABS','ACOS','ASIN','ATAN','ATAN2','AVG','BITAND','CEIL','COS','COSH','COUNT','EXP','FLOOR','GREATEST','LEAST','LN','LOG','MAX','MEDIAN','MIN','MOD','POWER','REGEXP_COUNT','REMAINDER','ROUND','ROWNUM','SIGN','SIN','SINH','SQRT','SUM','TAN','TANH','TRUNC',
            // Date/Time Functions
            'ADD_MONTHS','CURRENT_DATE','CURRENT_TIMESTAMP','DBTIMEZONE','EXTRACT','LAST_DAY','LOCALTIMESTAMP','MONTHS_BETWEEN','NEW_TIME','NEXT_DAY','ROUND','SESSIONTIMEZONE','SYSDATE','SYSTIMESTAMP','TRUNC','TZ_OFFSET',
            // Conversion Functions
            'BIN_TO_NUM','CAST','CHARTOROWID','FROM_TZ','HEXTORAW','NUMTODSINTERVAL','NUMTOYMINTERVAL','RAWTOHEX','TO_CHAR','TO_CLOB','TO_DATE','TO_DSINTERVAL','TO_LOB','TO_MULTI_BYTE','TO_NCLOB','TO_NUMBER','TO_SINGLE_BYTE','TO_TIMESTAMP','TO_TIMESTAMP_TZ','TO_YMINTERVAL',
            // Analytic Functions
            'CORR','COVAR_POP','COVAR_SAMP','CUME_DIST','DENSE_RANK','FIRST_VALUE','LAG','LAST_VALUE','LEAD','LISTAGG','NTH_VALUE','RANK','STDDEV','VAR_POP','VAR_SAMP','VARIANCE',
            // Advanced Functions
            'BFILENAME','CARDINALITY','COALESCE','DECODE','EMPTY_BLOB','EMPTY_CLOB','GROUP_ID','LNNVL','NANVL','NULLIF','NVL','NVL2','SYS_CONTEXT','UID','USER','USERENV','SQLCODE','SQLERRM'
        ],
        builtinVariables: [
        ],
        operators: [
        ],
        tokenizer: {
            root: [
                { include: '@comments' },
                { include: '@whitespace' },
                { include: '@numbers' },
                { include: '@strings' },
                [/[;,.]/, 'delimiter'],
                [/[()]/, '@brackets'],
                [/[\w@#$]+/, {
                        cases: {
                            '@atoms': 'atom',
                            '@datatypes': 'predefined',
                            '@builtinFunctions': 'function',
                            '@builtinVariables': 'predefined',
                            '@keywords': 'keyword',
                            '@operators': 'operator',
                            '@default': 'identifier'
                        }
                    }],
                [/[<>=!%&+\-*/|~^]/, 'operator'],
            ],
            whitespace: [
                [/\s+/, 'white']
            ],
            comments: [
                [/--+.*/, 'comment'],
                [/\/\*/, { token: 'comment.quote', next: '@comment' }]
            ],
            comment: [
                [/[^*/]+/, 'comment'],
                [/\/\*/, { token: 'comment.quote', next: '@push' }],
                [/\*\//, { token: 'comment.quote', next: '@pop' }],
                [/./, 'comment']
            ],
            numbers: [
                [/0[xX][0-9a-fA-F]*/, 'number'],
                [/[$][+-]*\d*(\.\d*)?/, 'number'],
                [/((\d+(\.\d*)?)|(\.\d+))([eE][\-+]?\d+)?/, 'number']
            ],
            strings: [
                [/N'/, { token: 'string', next: '@string' }],
                [/'/, { token: 'string', next: '@string' }],
                // caters for the Alternative Quoting Mechanism (q'[...]') for String Literals
                // the start and end tokens will be highlighted as keywords, to make it more clear what the actual string is
                [/q'\[/, { token: 'keyword', next: '@qStringSquareBracket'}],
                [/q'</, { token: 'keyword', next: '@qStringAngleBracket'}],
                [/q'{/, { token: 'keyword', next: '@qStringCurlyBracket'}],
                [/q'\(/, { token: 'keyword', next: '@qStringParentheses'}],
                // same character start and end delimiter
                [/q'(.)/, { token: 'keyword' , next: '@qStringCustom.$1'}]
            ],
            // basic string ending
            string: [
                [/[^']+/, 'string'],
                [/''/, 'string'],
                [/'/, { token: 'string', next: '@pop' }]
            ],
            // q'[foobar]'
            qStringSquareBracket: [
                [/[^\]']+/, 'string'],
                [/'/, 'string'],
                [/\](?!')/, 'string'],
                [/]'/, {token: 'keyword', next: '@pop'}]
            ],
            // q'<foobar>'
            qStringAngleBracket: [
                [/[^>']+/, 'string'],
                [/'/, 'string'],
                [/>(?!')/, 'string'],
                [/>'/, {token: 'keyword', next: '@pop'}]
            ],
            // q'{foobar}'
            qStringCurlyBracket: [
                [/[^\}']+/, 'string'],
                [/'/, 'string'],
                [/}(?!')/, 'string'],
                [/}'/, {token: 'keyword', next: '@pop'}]
            ],
            // q'(foobar)'
            qStringParentheses: [
                [/[^\)']+/, 'string'],
                [/'/, 'string'],
                [/\)(?!')/, 'string'],
                [/\)'/, {token: 'keyword', next: '@pop'}]
            ],
            // q'!foobar!'
            qStringCustom: [
                [/(.)(?!')/, 'string'],
                [/(.)'/, {
                    cases: {
                        '$1==$S2': {token: 'keyword', next: '@pop'},
                        '$1!=$S2': {token: 'string'}
                    }
                }]
            ]
        }
    };
});
