/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var hooks = require('preact/hooks');
var compat = require('preact/compat');
require('./TabbableModeContext-4fb7c910.js');
var useTabbableMode = require('./useTabbableMode-98443dc8.js');
var useActionable = require('./useActionable-604c4209.js');
var classNames = require('./classNames-3bb28eb9.js');
var dimensions = require('./dimensions-77289825.js');
var padding = require('./padding-f5c99049.js');
var colors = require('./colors-52f16e9d.js');
var aria = require('./aria-1c2e3b5d.js');
var mergeInterpolations = require('./mergeInterpolations-032e29ea.js');
var UNSAFE_ActionCard_themes_ActionCardStyles_css = require('./UNSAFE_ActionCard/themes/ActionCardStyles.css.js');
var useTestId = require('./useTestId-4fe30223.js');

const interpolations = [
    ...Object.values(dimensions.dimensionInterpolations),
    ...Object.values(colors.colorInterpolations),
    ...Object.values(padding.paddingInterpolations),
    ...Object.values(aria.ariaInterpolations)
];
const mergedInterpolations = mergeInterpolations.mergeInterpolations(interpolations);
/**
 * An action card is a container that renders a group of related information in a styled rectangular area, provides enhanced styling for hover and focus states, and invokes an action event when clicked.
 * Width, height, padding, and background color are set through props.
 */
const ActionCard = compat.forwardRef(({ children, onAction, testId, ...rest }, ref) => {
    const { isTabbable } = useTabbableMode.useTabbableMode();
    const pressHandler = (e) => {
        if (onAction) {
            onAction({
                reason: e.type == 'keyup' ? 'keyboard' : 'pointer'
            });
        }
    };
    const { actionableProps, isHover, isActive } = useActionable.useActionable(pressHandler);
    const { class: mergedClasses, ariaLabel, ariaLabelledBy, ...mergedStyles } = mergedInterpolations(rest);
    const actionStyles = classNames.classNames([
        UNSAFE_ActionCard_themes_ActionCardStyles_css.baseStyle,
        mergedClasses,
        isHover && UNSAFE_ActionCard_themes_ActionCardStyles_css.hoverStyle,
        isActive && UNSAFE_ActionCard_themes_ActionCardStyles_css.activeStyle
    ]);
    const rootRef = compat.useRef(null);
    hooks.useImperativeHandle(ref, () => ({
        focus: () => {
            rootRef.current?.focus();
        },
        blur: () => {
            rootRef.current?.blur();
        }
    }));
    const testIdProps = useTestId.useTestId(testId);
    return (jsxRuntime.jsx("div", { "aria-label": ariaLabel, "aria-labelledby": ariaLabelledBy, ref: rootRef, ...actionableProps, class: actionStyles, tabIndex: isTabbable ? 0 : -1, role: "button", style: mergedStyles, ...testIdProps, children: children }));
});

exports.ActionCard = ActionCard;
//# sourceMappingURL=ActionCard-9e2ed3f8.js.map
