/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var compat = require('preact/compat');
var usePress = require('./usePress-42f3876b.js');
var useHover = require('./useHover-8f282a4d.js');
var useActive = require('./useActive-5db5a3fb.js');
require('./TabbableModeContext-4fb7c910.js');
var useTabbableMode = require('./useTabbableMode-98443dc8.js');
var useColorScheme = require('./useColorScheme-53b27a87.js');
var classNames = require('./classNames-3bb28eb9.js');
var dimensions = require('./dimensions-77289825.js');
var mergeInterpolations = require('./mergeInterpolations-032e29ea.js');
var clientHints$1 = require('./clientHints-666b4780.js');
var mergeProps = require('./mergeProps-f6f4322f.js');
var UNSAFE_BaseButton_themes_BaseButtonStyles_css = require('./UNSAFE_BaseButton/themes/BaseButtonStyles.css.js');
var useTestId = require('./useTestId-4fe30223.js');

const interpolations = [...Object.values(dimensions.dimensionInterpolations)];
const styleInterpolations = mergeInterpolations.mergeInterpolations(interpolations);
const clientHints = clientHints$1.getClientHints();
// :active only works on IOS devices if a touch handler exists
const iosProps = clientHints.platform === 'ios' ? { ontouchstart: function () { } } : {};
const isHybrid = clientHints$1.getClientHints().hoverSupport === 'events';
const roleProps = (elType, hidden, isFocusable) => {
    switch (elType) {
        case 'link':
            return { role: 'link' };
        case 'button':
            return {};
        default:
            return hidden ? { role: 'none' } : isFocusable ? { role: 'button' } : { role: 'img' };
    }
};
const BaseButton = compat.forwardRef(({ variant, isDisabled = false, isRepeat = false, size, elementDetails = { type: 'button', buttonType: 'button' }, styling = ['default'], edge = 'none', 'aria-label': accessibleLabel, 'aria-hidden': ariaHidden, 'aria-expanded': ariaExpanded, 'aria-roledescription': ariaRoledescription, 'aria-haspopup': ariaHasPopup, 'aria-describedby': ariaDescribedBy, testId, onBlur, onFocus, onMouseEnter, onMouseLeave, onTouchEnd, onTouchStart, onKeyDown, ...props }, ref) => {
    const pressHandler = (e) => {
        if (props.onAction) {
            props.onAction({
                reason: e.type == 'keyup' ? 'keyboard' : 'pointer'
            });
        }
    };
    const { pressProps } = usePress.usePress(pressHandler, {
        isDisabled,
        isRepeat
    });
    const isStyled = (name) => {
        return styling.indexOf(name) > -1;
    };
    const { hoverProps, isHover } = useHover.useHover({ isDisabled: !isHybrid });
    const { activeProps, isActive } = useActive.useActive();
    const { class: cls, ...styles } = styleInterpolations(props);
    const { type: elementType = 'button', isFocusable: isElementFocusable = true, buttonType = 'button', ...elementProps } = { ...elementDetails };
    const typeProps = elementType === 'button' && buttonType === 'button' ? { type: buttonType } : {};
    // 'role' should be 'none' when aria-hidden is true or 'img' when false and not focusable
    const role = roleProps(elementType, ariaHidden ?? false, isElementFocusable);
    const ElementType = elementDetails.type;
    const isSpan = ElementType == 'span';
    const isBaseButton = ElementType == 'button';
    const { tabbableModeProps } = useTabbableMode.useTabbableMode(isDisabled);
    const colorScheme = useColorScheme.useColorScheme();
    const variantClasses = UNSAFE_BaseButton_themes_BaseButtonStyles_css.multiVariantStyles({
        size,
        variant,
        unstyled: isStyled('unstyled') ? 'isUnstyled' : 'notUnstyled',
        embedded: isStyled('embedded') ? 'isEmbedded' : 'notEmbedded',
        container: isStyled('container') ? 'isContainer' : 'notContainer',
        noBorderRadiusStart: isStyled('noBorderRadiusStart')
            ? 'isNoBorderRadiusStart'
            : 'notNoBorderRadiusStart',
        noBorderRadiusEnd: isStyled('noBorderRadiusEnd')
            ? 'isNoBorderRadiusEnd'
            : 'notNoBorderRadiusEnd',
        activeStyle: isStyled('active') ? 'isActiveStyle' : 'notActiveStyle',
        fill: isStyled('fill') ? 'isFill' : 'notFill',
        min: isStyled('min') ? 'isMin' : 'notMin',
        disabled: isDisabled ? 'isDisabled' : 'notDisabled',
        edge,
        hybrid: isHybrid ? 'isHybrid' : 'notHybrid',
        active: isActive ? 'isActive' : 'notActive',
        hover: isHover ? 'isHover' : 'notHover',
        span: isSpan ? 'isSpan' : 'notSpan',
        colorScheme
    });
    const globalEventProps = {
        onBlur,
        onFocus,
        onMouseEnter,
        onMouseLeave,
        onTouchEnd,
        onTouchStart,
        onKeyDown
    };
    const mergedProps = mergeProps.mergeProps(elementProps, pressProps, iosProps, hoverProps, isStyled('container') ? {} : activeProps, isBaseButton ? {} : { 'aria-disabled': isDisabled }, globalEventProps);
    const wrapperClasses = classNames.classNames([cls, variantClasses]);
    const testIdProps = useTestId.useTestId(testId);
    const getTabIndex = () => {
        if (isSpan && (isElementFocusable === false || isDisabled)) {
            // no tabIndex value should be defined for a span element when disabled or non-focusable
            return { tabIndex: undefined };
        }
        return tabbableModeProps;
    };
    return (jsxRuntime.jsx(ElementType
    // The ref casting was done to satisfy TS compiler, when the forwardRef had to be updated to ForwardedRef.
    , { 
        // The ref casting was done to satisfy TS compiler, when the forwardRef had to be updated to ForwardedRef.
        ref: ref, disabled: isDisabled, class: wrapperClasses, style: styles, autofocus: props['autofocus'], title: props.title, ...getTabIndex(), "aria-label": accessibleLabel, "aria-haspopup": ariaHasPopup ? 'menu' : undefined, "aria-describedby": ariaDescribedBy, "aria-hidden": ariaHidden, "aria-roledescription": ariaRoledescription, "aria-expanded": ariaExpanded, ...typeProps, ...testIdProps, ...mergedProps, ...role, children: props.children }));
});
BaseButton.displayName = 'BaseButton';

exports.BaseButton = BaseButton;
//# sourceMappingURL=BaseButton-1fd978ce.js.map
