/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var preact = require('@testing-library/preact');
var userEvent = require('@testing-library/user-event');
var chai = require('chai');
var Text = require('./Text-15925a9e.js');
var View = require('./View-65155455.js');
var Avatar = require('./Avatar-c489526c.js');
var Button = require('./Button-23e05530.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var userEvent__default = /*#__PURE__*/_interopDefaultLegacy(userEvent);

const CardContent = ({ context, showButton }) => {
    return (jsxRuntime.jsxs(View.View, { padding: "4x", children: [jsxRuntime.jsx(Avatar.Avatar, { initials: context.data.initials, size: "xl" }), jsxRuntime.jsx("div", { style: "margin-top: 15px;", children: jsxRuntime.jsx(Text.Text, { variant: "primary", size: "md", lineClamp: 1, children: context.data.name }) }), jsxRuntime.jsx(Text.Text, { variant: "secondary", size: "md", lineClamp: 1, children: context.data.title }), showButton && (jsxRuntime.jsxs("div", { style: "margin-top: 5px;", children: [jsxRuntime.jsx("div", { style: "margin-right: 5px; display: inline-block;", children: jsxRuntime.jsx(Button.Button, { variant: "outlined", size: "md", label: "A" }) }), jsxRuntime.jsx("div", { style: "display: inline-block;", children: jsxRuntime.jsx(Button.Button, { variant: "outlined", size: "md", label: "B" }) })] })), context.selector && (jsxRuntime.jsx("div", { style: "position: relative; left: -13px", children: context.selector?.() }))] }));
};
const populateData = (count, isNumberKey) => [...Array(count)].map((_, i) => ({
    id: isNumberKey ? i : `${i}`,
    name: `Employee ${i}`,
    title: `Employee title ${i}`,
    initials: `E${i}`
}));
const getRoot = async () => {
    return preact.screen.findByRole('grid');
};
const getItems = async (role = 'row') => {
    return preact.screen.findAllByRole(role);
};

const testRender = ({ GenericCardGrid, ActionCardGrid, SelectionCardGrid, ContentUpdatableCardGrid }) => {
    it('initial render', async function () {
        const data = populateData(100);
        preact.render(jsxRuntime.jsx(GenericCardGrid, { data: data }));
        const items = await getItems('gridcell');
        chai.expect(items.length).eq(25);
    });
    it('checking key type - string key', async () => {
        const data = populateData(3);
        preact.render(jsxRuntime.jsx(GenericCardGrid, { data: data }));
        const items = await getItems('gridcell');
        chai.expect(items[0].dataset['ojKeyType'] === undefined).eq(true);
        chai.expect(items[1].dataset['ojKeyType'] === undefined).eq(true);
        chai.expect(items[2].dataset['ojKeyType'] === undefined).eq(true);
    });
    it('checking key type - numeric key', async () => {
        const data = populateData(3, true);
        preact.render(jsxRuntime.jsx(GenericCardGrid, { data: data }));
        const items = await getItems('gridcell');
        chai.expect(items[0].dataset['ojKeyType']).eq('number');
        chai.expect(items[1].dataset['ojKeyType']).eq('number');
        chai.expect(items[2].dataset['ojKeyType']).eq('number');
    });
    it('aria attributes - multiselectable and selected, selectionMode is none', async () => {
        const data = populateData(2);
        preact.render(jsxRuntime.jsx(SelectionCardGrid, { data: data }));
        const items = await getItems('gridcell');
        const grid = await getRoot();
        chai.expect(grid.getAttribute('aria-multiselectable')).eq(null);
        chai.expect(items[0].getAttribute('aria-selected')).eq(null);
        chai.expect(items[1].getAttribute('aria-selected')).eq(null);
    });
    it('aria attributes - multiselectable and selected, selectionMode is single', async () => {
        const data = populateData(2);
        preact.render(jsxRuntime.jsx(SelectionCardGrid, { data: data, selectionMode: 'single', selectedKeys: new Set(['1']) }));
        const items = await getItems('gridcell');
        const grid = await getRoot();
        chai.expect(grid.getAttribute('aria-multiselectable')).eq('false');
        chai.expect(items[0].getAttribute('aria-selected')).eq('false');
        chai.expect(items[1].getAttribute('aria-selected')).eq('true');
    });
    it('aria attributes - multiselectable and selected, selectionMode is multiple', async () => {
        const data = populateData(3);
        preact.render(jsxRuntime.jsx(SelectionCardGrid, { data: data, selectionMode: 'multiple', selectedKeys: new Set(['1', '2']) }));
        const items = await getItems('gridcell');
        const grid = await getRoot();
        chai.expect(grid.getAttribute('aria-multiselectable')).eq('true');
        chai.expect(items[0].getAttribute('aria-selected')).eq('false');
        chai.expect(items[1].getAttribute('aria-selected')).eq('true');
        chai.expect(items[2].getAttribute('aria-selected')).eq('true');
    });
    // skip this test in jest, only run it in karma mocha
    if (typeof jest === typeof undefined) {
        // because jest doesn't show grid layout, the number of focusable elements is not calculated correctly
        it('content focusBehavior - valid case', async () => {
            const data = populateData(3);
            preact.render(jsxRuntime.jsx(ActionCardGrid, { data: data }));
            const items = await getItems('gridcell');
            await userEvent__default["default"].tab();
            await preact.waitFor(() => {
                chai.expect(items[0].matches(':focus-within')).to.be.true;
            });
        });
        it('content focusBehavior - invalid case', async () => {
            const data = populateData(3);
            preact.render(jsxRuntime.jsx(ActionCardGrid, { data: data, showButton: true }));
            const grid = await getRoot();
            await userEvent__default["default"].tab();
            await preact.waitFor(() => {
                chai.expect(grid.matches(':focus')).to.be.true;
            });
        });
    }
    // JET-61732
    it('memoization - update memoized children', async () => {
        const data = populateData(3);
        const { queryByTestId } = preact.render(jsxRuntime.jsx(ContentUpdatableCardGrid, { data: data }));
        const elem0 = queryByTestId(data[0].id);
        const elem1 = queryByTestId(data[1].id);
        const elem2 = queryByTestId(data[2].id);
        chai.expect(elem0?.innerHTML, 'the 1st item initial count should be 0').eq('0');
        chai.expect(elem1?.innerHTML, 'the 2nd item initial count should be 0').eq('0');
        chai.expect(elem2?.innerHTML, 'the 3rd item initial count should be 0').eq('0');
        const button = queryByTestId('mybutton');
        await userEvent__default["default"].click(button);
        chai.expect(elem0?.innerHTML, 'the 1st item new count should be 1').eq('1');
        chai.expect(elem1?.innerHTML, 'the 2nd item new count should be 1').eq('1');
        chai.expect(elem2?.innerHTML, 'the 3rd item new count should be 1').eq('1');
    });
};

exports.CardContent = CardContent;
exports.getRoot = getRoot;
exports.populateData = populateData;
exports.testRender = testRender;
//# sourceMappingURL=BaseCardViewTest-0e6bfde1.js.map
