/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var useTextDimensions = require('./useTextDimensions-ee1e9aa4.js');
var useCssVars = require('./useCssVars-f6f61981.js');
var cssUtils = require('./cssUtils-33e707ee.js');
var UNSAFE_Chart_themes_ChartStyles_css = require('./UNSAFE_Chart/themes/ChartStyles.css.js');
var hooks = require('preact/hooks');
var Gridlines = require('./Gridlines-f9b1a429.js');
var useId = require('./useId-42e8026e.js');
var Axis = require('./Axis-29b9b5d8.js');
var useChartViewport = require('./useChartViewport-37b2dfc3.js');
var layoutUtils = require('./layoutUtils-07bcee32.js');
var useChartNav = require('./useChartNav-3e92775b.js');
var mergeProps = require('./mergeProps-f6f4322f.js');
var useSelectDrill = require('./useSelectDrill-8058c001.js');
var useTestId = require('./useTestId-4fe30223.js');
var TrackResizeContainer = require('./TrackResizeContainer-75071c52.js');
var ScrollBar = require('./ScrollBar-8c1f5409.js');

function getNextGroup(info, viewportEndIndex, isGroupNavigable) {
    const groupIndex = info.groupIndex + 1;
    if (groupIndex >= viewportEndIndex) {
        return;
    }
    if (isGroupNavigable(groupIndex)) {
        return { groupIndex };
    }
    return getNextGroup({ groupIndex }, viewportEndIndex, isGroupNavigable);
}
function getPreviousGroup(info, isGroupNavigable) {
    const groupIndex = info.groupIndex - 1;
    if (groupIndex < 0) {
        return;
    }
    if (isGroupNavigable(groupIndex)) {
        return { groupIndex };
    }
    return getPreviousGroup({ groupIndex }, isGroupNavigable);
}
function getGroupNavUtil(groups, renderedGroups, xIndexExtent, drilling, isRtl) {
    const isGroupNavigable = (groupIndex) => {
        const groupsDrilling = groups[renderedGroups[groupIndex]].drilling;
        return (drilling === 'on' || drilling === 'groupsOnly') && groupsDrilling != 'off';
    };
    const getDefaultNavGroup = (info) => {
        if (xIndexExtent.startIndex > renderedGroups[info.groupIndex]) {
            info.groupIndex = 0;
            return info;
        }
        else if (xIndexExtent.endIndex < renderedGroups[info.groupIndex]) {
            info.groupIndex = renderedGroups.length - 1;
            return info;
        }
        return;
    };
    const getNextNavGroup = (info, arrow) => {
        let nextGroup = getDefaultNavGroup(info);
        if (nextGroup)
            return nextGroup;
        if (arrow === 'ArrowRight') {
            nextGroup = isRtl
                ? getPreviousGroup(info, isGroupNavigable)
                : getNextGroup(info, renderedGroups.length, isGroupNavigable);
        }
        else {
            nextGroup = isRtl
                ? getNextGroup(info, renderedGroups.length, isGroupNavigable)
                : getPreviousGroup(info, isGroupNavigable);
        }
        return nextGroup || info;
    };
    return { getDefaultNavGroup, getNextNavGroup };
}

function Chart({ width = '100%', height = '448px', selectedIds = [], hiddenIds, hideAndShowBehavior = 'none', orientation = 'vertical', xAxis, yAxis, plotArea, series, groups, isStacked = false, getDataItem, selectionMode = 'none', drilling = 'off', dataCursorPosition, onSelectionChange, isDataCursorEnabled, dataCursorStyle, onViewportChange, onDataCursorPositionChange, onItemDrill, onGroupDrill, onItemInput, datatip, zoomAndScroll = 'off', offset, getNavUtil, getDataItemPos, getGapRatio, valueFormats, findNearest, isRtl, children, testId, renderGridLinesInFront, ...props }) {
    // TODO: reevaluate how we handle empty data
    const chartVars = cssUtils.getChartVars();
    const testIdProps = useTestId.useTestId(testId);
    return (jsxRuntime.jsx(TrackResizeContainer.TrackResizeContainer, { width: width, height: height, children: function _(width, height) {
            const rootRef = hooks.useRef(null);
            const isHoriz = orientation === 'horizontal';
            const { xAxisPosition, yAxisPosition } = layoutUtils.getAxesPosition(yAxis, isHoriz, isRtl);
            const { resolvedVars, cssContent } = useCssVars.useCssVars(chartVars);
            const { textMeasureContent, getTextDimensions } = useTextDimensions.useTextDimensions();
            let availSpace = layoutUtils.getAvailSpace(width, height);
            const updatedSpace = layoutUtils.getScrollBarSpace(availSpace, zoomAndScroll, xAxisPosition);
            let scrollDims = updatedSpace.scrollDims;
            availSpace = updatedSpace.availSpace;
            const hiddenSet = new Set(hiddenIds);
            const { xProps, xScale, xViewportExtent, xAxisExtent, xIndexExtent, yViewportExtent, yAxisExtent, groupWidth, yProps, yScale, yMaxDim, yMajorTicks, yMinorTicks, plotAreaSpace } = layoutUtils.getLayoutInfo(series, groups, getDataItem, xAxis, yAxis, availSpace, hiddenSet, hideAndShowBehavior, isHoriz, isRtl, isStacked, resolvedVars['labelFontSize'], offset, getTextDimensions);
            scrollDims = layoutUtils.updateScrollBarSpace(scrollDims, xAxisPosition === 'bottom', yMaxDim);
            const gapRatio = getGapRatio(xIndexExtent.startIndex, xIndexExtent.endIndex);
            const getNextChartItem = getNavUtil(xIndexExtent.startIndex, xIndexExtent.endIndex);
            const renderedGroupsIndex = xProps.tickLabels.map(({ index }) => index + xIndexExtent.startIndex);
            const { getNextNavGroup, getDefaultNavGroup } = getGroupNavUtil(groups, renderedGroupsIndex, xIndexExtent, drilling, isRtl);
            const { activeId, focusedItemInfo, hoveredItemInfo, focusedGroupInfo, navProps } = useChartNav.useChartNav({
                rootRef,
                getNextChartItem,
                getNextNavGroup,
                getDefaultNavGroup,
                onItemInput,
                drilling,
                xIndexExtent
            });
            const selectDrillProps = useSelectDrill.useSelectDrill({
                selectionMode,
                selection: selectedIds,
                onChange: onSelectionChange,
                rootRef,
                drilling,
                onItemDrill,
                onGroupDrill,
                focusedGroupInfo,
                focusedItemInfo,
                groups,
                getDataItem
            });
            const viewportProps = useChartViewport.useChartViewport({
                width,
                height,
                xViewportExtent,
                yViewportExtent,
                xAxisExtent,
                yAxisExtent,
                xScale,
                yScale,
                plotAreaSpace,
                isRtl,
                isHoriz,
                zoomAndScroll,
                onViewportChange
            });
            const findNearestData = findNearest(xScale, yScale, groupWidth, gapRatio);
            const getDataItemPosition = getDataItemPos(xScale, yScale, groupWidth, gapRatio);
            const { datatipContent, dataCursorContent, datatipProps } = useSelectDrill.useChartDatatipAndCursor({
                series,
                groups,
                width,
                height,
                findNearest: findNearestData,
                plotAreaSpace,
                onDataCursorPositionChange,
                isDataCursorEnabled,
                dataCursorPosition,
                dataCursorStyle,
                hoveredItemInfo,
                focusedItemInfo,
                getDataItem,
                getDataItemPos: getDataItemPosition,
                datatip,
                valueFormats,
                isHoriz,
                xScale: xScale,
                yScale
            });
            const eventsProps = mergeProps.mergeProps(navProps, selectDrillProps, viewportProps, datatipProps);
            const isGroupInteractive = (index) => {
                if (xAxis?.timeAxisType != null) {
                    return false;
                }
                const groupDrill = groups[index].drilling;
                if (groupDrill === 'on') {
                    return true;
                }
                if (groupDrill != 'off' && (drilling === 'on' || drilling === 'groupsOnly')) {
                    return true;
                }
                return false;
            };
            const plotAreaClipPathId = useId.useId();
            const { viewportMin: xViewportMin, viewportMax: xViewportMax } = xViewportExtent;
            const { startIndex: xStartIndex, endIndex: xEndIndex } = xIndexExtent;
            const gridLines = (jsxRuntime.jsx(Gridlines.Gridlines, { plotArea: plotArea, yMinorTicks: yMinorTicks, xScale: xScale, yScale: yScale, isLog: yAxis?.scale === 'log', xViewportMin: xViewportMin, xViewportMax: xViewportMax, yTicks: yMajorTicks, yAxisPosition: yAxisPosition }));
            return (jsxRuntime.jsxs("div", { tabIndex: 0, ref: rootRef, "aria-activedescendant": activeId, "aria-label": props['aria-label'], "aria-labelledby": props['aria-labelledby'], "aria-describedby": props['aria-describedby'], style: { width, height }, role: "application", className: UNSAFE_Chart_themes_ChartStyles_css.styles.outline, ...testIdProps, ...eventsProps, children: [jsxRuntime.jsxs("svg", { width: "100%", height: "100%", children: [jsxRuntime.jsx("clipPath", { id: `${plotAreaClipPathId}`, children: jsxRuntime.jsx("rect", { ...plotAreaSpace }) }), jsxRuntime.jsx(Gridlines.PlotArea, { color: plotArea?.color, yScale: yScale, isLog: yAxis?.scale === 'log', availSpace: plotAreaSpace, xAxisPosition: xAxisPosition }), !renderGridLinesInFront && gridLines, jsxRuntime.jsx(Axis.Axis, { ...yProps, axisPosition: yAxisPosition }), jsxRuntime.jsx(Axis.Axis, { ...xProps, axisPosition: xAxisPosition, isTextInteractive: isGroupInteractive, activeId: activeId, focusedGroupIndex: focusedGroupInfo.isFocusVisible ? focusedGroupInfo.groupIndex : undefined }), jsxRuntime.jsxs("g", { "clip-path": `url(#${plotAreaClipPathId})`, children: [children({
                                        xStartIndex,
                                        xEndIndex,
                                        xScale: xScale,
                                        yScale,
                                        gapRatio,
                                        activeId,
                                        focusedItemInfo,
                                        hoveredItemInfo,
                                        groupWidth
                                    }), renderGridLinesInFront && gridLines, dataCursorContent] }), zoomAndScroll === 'live' && (jsxRuntime.jsx(ScrollBar.ScrollBar, { availSpace: scrollDims, viewport: xViewportExtent, axisExtent: xAxisExtent, isHoriz: xAxisPosition === 'bottom' }))] }), textMeasureContent, cssContent, datatipContent] }));
        } }));
}

exports.Chart = Chart;
//# sourceMappingURL=Chart-8cbc54b7.js.map
