/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var compat = require('preact/compat');
var useCollectionFocusRing = require('./useCollectionFocusRing-c5bb4a10.js');
var useTranslationBundle = require('./useTranslationBundle-9549e176.js');
var Flex = require('./Flex-299660c2.js');
var Label = require('./Label-7d743faa.js');
var InlineHelp = require('./InlineHelp-41dae2d1.js');
require('./logger-01b28029.js');
require('./TabbableModeContext-4fb7c910.js');
require('preact/hooks');


require('./UNSAFE_UserAssistance/themes/redwood/UserAssistanceVariants.css.js');
var InlineUserAssistance = require('./InlineUserAssistance-680e9a26.js');


require('./UNSAFE_MessageBanner/themes/redwood/MessageBannerVariants.css.js');
require('preact');
var utils = require('./utils-0015eceb.js');
require('./Floating-8ecf7694.js');
require('./TopLayerHost-b320c750.js');
var useTestId = require('./useTestId-4fe30223.js');


require('./hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentVariants.css.js');

require('./IconButton-df477106.js');

require('./MessageFormattingUtils-034ac55d.js');
require('./TransitionGroup-2e7eb24a.js');
require('./MessagesContext-bca47518.js');
var HiddenAccessible = require('./HiddenAccessible-ce5d15d7.js');


require('./UNSAFE_ComponentMessage/themes/redwood/ComponentMessageVariants.css.js');
require('./FormFieldContext-81c48156.js');
require('./Popup-a64bbf6a.js');
require('./UNSAFE_Separator/themes/SeparatorStyles.css.js');


require('./UNSAFE_Label/themes/redwood/LabelVariants.css.js');

require('./UNSAFE_Popup/themes/redwood/PopupVariants.css.js');
var useMessageSeverity = require('./useMessageSeverity-dd4dc041.js');
var stringUtils = require('./stringUtils-5ffab770.js');

var LabelValueLayout = require('./LabelValueLayout-1eaf50b9.js');
var CheckboxRadioContext = require('./CheckboxRadioContext-45cad1d9.js');
var LayoutStyles_css = require('./LayoutStyles.css-4d8455b4.js');

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
function useCheckboxRadioField({ ariaDescribedBy, baseId, hasValue, hasTopUserAssistance, hasBottomUserAssistance, isReadonly, isRequired = false, isDisabled, label, labelEdge, messages, parentComponentVariant }) {
    const optionsGroupId = `${baseId}-group`;
    const labelId = `${baseId}-label`;
    const topUaId = hasTopUserAssistance ? `${baseId}-top-ua` : undefined;
    const bottomUaId = hasBottomUserAssistance ? `${baseId}-bottom-ua` : undefined;
    const readonlyId = isReadonly ? `${baseId}-readonly` : undefined;
    const ariaLabel = label && label.length > 0 && labelEdge === 'none' ? label : undefined;
    const ariaLabelledBy = ariaLabel ? undefined : labelId;
    const isRadioSet = parentComponentVariant === 'radioSet';
    const highestMessageSeverity = useMessageSeverity.useMessageSeverity(messages);
    const hasErrorMessage = highestMessageSeverity === 'error';
    return {
        groupProps: {
            'aria-describedby': stringUtils.merge([topUaId, bottomUaId, readonlyId, ariaDescribedBy]),
            ...(isRadioSet && { 'aria-required': isRequired }),
            ...(ariaLabelledBy && { 'aria-labelledby': ariaLabelledBy }),
            ...(ariaLabel && { 'aria-label': ariaLabel }),
            ...(isReadonly && isRadioSet && { 'aria-readonly': true }),
            ...(hasErrorMessage && { 'aria-invalid': true }),
            ...(isDisabled && { 'aria-disabled': true }),
            id: optionsGroupId
        },
        labelProps: {
            forId: optionsGroupId,
            id: labelId
        },
        // TOP UA is assistive text and help link/text
        ...(hasTopUserAssistance && {
            topUserAssistanceProps: {
                id: topUaId,
                // Use reflow variant so that top UA doesn't have min height
                variant: 'reflow'
            }
        }),
        // Bottom UA is required indicator or messages
        ...(hasBottomUserAssistance && {
            bottomUserAssistanceProps: {
                id: bottomUaId,
                isRequiredShown: !!isRequired && !hasValue
            }
        }),
        ...(isReadonly && {
            readonlyId
        })
    };
}

var topUAContainerStyles = {top:'CheckboxRadioFieldStyles_topUAContainerStyles_top__j1xru60',start:'CheckboxRadioFieldStyles_topUAContainerStyles_start__j1xru61'};

const CheckboxRadioField = compat.forwardRef(({ 'aria-describedby': ariaDescribedBy, assistiveText, baseId, direction = 'column', columnSpan = 1, hasValue, helpSourceLink, helpSourceText, isDisabled, isReadonly, isRequired, label, labelEdge, labelStartWidth = '33%', parentComponentVariant, messages, userAssistanceDensity, role, children, testId }, ref = null) => {
    const testIdProps = useTestId.useTestId(testId);
    const hasTopUserAssistance = !isDisabled && !isReadonly && (!!assistiveText || (!!helpSourceLink && !!helpSourceText));
    const hasBottomUserAssistance = !isDisabled && !isReadonly;
    const groupRef = compat.useRef(null);
    const { groupProps, labelProps, topUserAssistanceProps, bottomUserAssistanceProps, readonlyId } = useCheckboxRadioField({
        ariaDescribedBy,
        baseId,
        isRequired,
        isReadonly,
        isDisabled,
        label,
        labelEdge,
        messages,
        hasValue,
        hasTopUserAssistance,
        hasBottomUserAssistance,
        parentComponentVariant
    });
    const translations = useTranslationBundle.useTranslationBundle('@oracle/oraclejet-preact');
    const readonlyLabel = translations.formControl_readOnly();
    const isRadioSet = parentComponentVariant === 'radioSet';
    compat.useImperativeHandle(ref, () => ({
        focus: () => {
            if (!isDisabled) {
                const inputChildren = groupRef.current?.querySelectorAll('input') ?? [];
                const selectedChild = Array.from(inputChildren).find((child) => child.checked);
                if (selectedChild) {
                    selectedChild.focus();
                }
                else {
                    inputChildren[0].focus();
                }
            }
        },
        blur: () => {
            if (groupRef.current?.contains(document.activeElement)) {
                document.activeElement.blur();
            }
        }
    }));
    // the props from the hook are used for focus management on the radioset variant to allow for arrow navigation
    // while checkboxset uses tabbing
    const [isFocusRingShown, focusRingProps] = useCollectionFocusRing.useCollectionFocusRing(groupRef, isRadioSet ? ['ArrowUp', 'ArrowRight', 'ArrowDown', 'ArrowLeft'] : ['Tab']);
    const content = (jsxRuntime.jsxs(jsxRuntime.Fragment, { children: [hasTopUserAssistance && (jsxRuntime.jsx("div", { id: `${baseId}-top-ua-container`, class: labelEdge === 'start' ? topUAContainerStyles.start : topUAContainerStyles.top, children: jsxRuntime.jsx(InlineUserAssistance.InlineUserAssistanceContainer, { ...topUserAssistanceProps, children: jsxRuntime.jsx(InlineHelp.InlineHelp, { assistiveText: assistiveText, sourceLink: helpSourceLink, sourceText: helpSourceText }) }) })), jsxRuntime.jsx(Flex.Flex, { direction: direction, ...(direction === 'row' && { gap: [0, utils.xUnits(8)], wrap: 'wrap' }), children: jsxRuntime.jsx(CheckboxRadioContext.CheckboxRadioContext.Provider, { value: { isFocusRingShown }, children: children }) }), hasBottomUserAssistance && !isReadonly && (jsxRuntime.jsx(InlineUserAssistance.InlineUserAssistance, { ...bottomUserAssistanceProps, requiredAlignment: "start", messages: messages, userAssistanceDensity: userAssistanceDensity === 'reflow' ? userAssistanceDensity : 'efficient' }))] }));
    // The top level element needs the column span class on it.
    const classes = LayoutStyles_css.layoutSpanStyles.layoutSpanColumn[columnSpan];
    return (jsxRuntime.jsxs("div", { role: role, ref: groupRef, class: classes, ...groupProps, ...focusRingProps, ...testIdProps, children: [labelEdge === 'none' && content, labelEdge !== 'none' && (jsxRuntime.jsx(LabelValueLayout.LabelValueLayout, { parentComponentVariant: parentComponentVariant, hasTopUserAssistance: hasTopUserAssistance, label: jsxRuntime.jsx(Label.Label, { ...labelProps, variant: labelEdge, userAssistanceDensity: "efficient", parentComponentVariant: parentComponentVariant, children: label }), labelEdge: labelEdge, labelStartWidth: labelStartWidth, children: content })), isReadonly && (jsxRuntime.jsx(HiddenAccessible.HiddenAccessible, { id: readonlyId, isHidden: true, children: readonlyLabel }))] }));
});

exports.CheckboxRadioField = CheckboxRadioField;
//# sourceMappingURL=CheckboxRadioField-13fa8c4d.js.map
