/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var compat = require('preact/compat');
var TopLayerHost = require('./TopLayerHost-b320c750.js');
var useThemeInterpolations = require('./useThemeInterpolations-64b0f552.js');

const _LOGICAL_PARENT = '__oj_logical_parent';
const Layer = (props) => {
    const context = compat.useContext(TopLayerHost.LayerContext);
    // the 'top' priority should always win regardless of whether it comes from context or prop
    const computedPriority = props.priority === 'top' || context.priority === 'top' ? 'top' : props.priority;
    const defaultHost = context.getHost?.(computedPriority);
    const host = compat.useMemo(() => defaultHost, [defaultHost]);
    const [layerContainer, setLayerContainer] = compat.useState(null);
    // because the children are being rendered into a portal that is out of the flow of parent dom,
    // we need to specify the colorScheme and scale class names here to pick up the correct CSS styles.
    const classes = useThemeInterpolations.useThemeInterpolations();
    // Really need useLayoutEffect instead of useEffect here. Otherwise the re-parented
    // content does not get re-rendered with valid ref's
    compat.useLayoutEffect(() => {
        if (!host)
            return;
        const doc = host.ownerDocument ?? document;
        const layer = doc.createElement('div');
        if (props.logicalParentRef) {
            layer[_LOGICAL_PARENT] = props.logicalParentRef.current;
        }
        host.appendChild(layer);
        setLayerContainer(layer);
        return () => {
            if (host && layer && host.contains(layer)) {
                delete layer[_LOGICAL_PARENT];
                host.removeChild(layer);
            }
            setLayerContainer(null);
        };
    }, [host, props.logicalParentRef]);
    // add the colorScheme and scale classes if there is a layerContainer
    if (layerContainer) {
        // use the theme interpolations to set the appropriate style classes for colorScheme and scale
        layerContainer.className = classes;
    }
    let newLayerContext = {};
    if (context.getHost) {
        newLayerContext = {
            // once 'priority' has been set to 'top', all children should render in the priority layer host
            priority: computedPriority,
            getHost: context.getHost
        };
    }
    return (jsxRuntime.jsx(TopLayerHost.LayerContext.Provider, { value: newLayerContext, children: layerContainer && compat.createPortal(props.children, layerContainer) }));
};

exports.Layer = Layer;
//# sourceMappingURL=Layer-c0f805fc.js.map
