/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var hooks = require('preact/hooks');
var useLegendLayout = require('./useLegendLayout-436b2dc1.js');
var LegendUtils = require('./LegendUtils-412c9829.js');
var mergeProps = require('./mergeProps-f6f4322f.js');
var useItemFocus = require('./useItemFocus-e9a52855.js');
var UNSAFE_Legend_themes_LegendStyles_css = require('./UNSAFE_Legend/themes/LegendStyles.css.js');
var useCssVars = require('./useCssVars-f6f61981.js');
var classNames = require('./classNames-3bb28eb9.js');
var useTestId = require('./useTestId-4fe30223.js');

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
function getItem(itemInfo, items) {
    const { itemIdx } = itemInfo;
    if (itemIdx == -1) {
        return items[items.length - 1];
    }
    return items[itemIdx];
}
function getNextItemInfo(itemInfo, items) {
    let itemIdx = itemInfo.itemIdx;
    if (itemInfo.itemIdx == -1) {
        itemIdx = items.length - 1;
    }
    itemIdx = Math.min(items.length - 1, itemIdx + 1);
    return { sectionIdx: 0, itemIdx };
}
function getPrevItemInfo(itemInfo, items) {
    let itemIdx = itemInfo.itemIdx;
    if (itemInfo.itemIdx == -1) {
        itemIdx = items.length - 1;
    }
    itemIdx = Math.max(0, itemIdx - 1);
    return { sectionIdx: 0, itemIdx };
}
/**
 * Returns utils functions for navigation in simple legend.
 * @param items
 * @returns
 */
function getLegendNavUtils(items) {
    return {
        getItem: (itemInfo) => {
            return getItem(itemInfo, items);
        },
        getDetailFromInfo: (itemInfo) => {
            if (!itemInfo) {
                return { itemId: undefined };
            }
            const item = getItem(itemInfo, items);
            return { itemId: item.id };
        },
        getPrevItemInfo: (itemInfo) => {
            return getPrevItemInfo(itemInfo, items);
        },
        getNextItemInfo: (itemInfo) => {
            return getNextItemInfo(itemInfo, items);
        }
    };
}

/**
 * A legend displays an interactive description of symbols, colors, etc., used in graphical information representations.
 */
function Legend({ orientation = 'horizontal', items, isReadOnly = true, onItemAction, onInput, testId, ...props }) {
    const testIdProps = useTestId.useTestId(testId);
    const rootRef = hooks.useRef(null);
    const rootDimsRef = hooks.useRef();
    const { getItem, getDetailFromInfo, getPrevItemInfo, getNextItemInfo } = getLegendNavUtils(items);
    const { resolvedVars, cssContent } = useCssVars.useCssVars(LegendUtils.legendVars);
    const legendFontStylesObj = LegendUtils.getLegendTextStyle({
        fontStyle: props.textFontStyle,
        fontSize: props.textFontSize?.toString(),
        fontWeight: props.textFontWeight?.toString(),
        fontFamily: props.textFontFamily
    }, resolvedVars);
    const { textMeasureContent, layoutInfo } = useLegendLayout.useLegendLayout({
        resolvedVars,
        items,
        rootRef,
        rootDimsRef,
        orientation,
        legendFontStylesObj,
        symbolWidth: props.symbolWidth
    });
    const { hoveredItemInfo, focusedItemInfo, ...eventsProps } = useLegendLayout.useEvents(!isReadOnly, props.hiddenIds != null, getDetailFromInfo, getPrevItemInfo, getNextItemInfo, onItemAction, onInput);
    const { focusedItemRef } = useItemFocus.useItemFocus(focusedItemInfo);
    const datatipText = LegendUtils.getDatatipText(items, getItem, focusedItemInfo, hoveredItemInfo);
    const { tooltipContent, tooltipProps } = useLegendLayout.useLegendTooltip({
        text: datatipText,
        rootDimsRef,
        focusedItemInfo,
        focusedItemRef
    });
    const ariaProps = LegendUtils.getAriaProps(isReadOnly, props['aria-label'], props.hiddenIds, props['aria-describedby'], props['aria-labelledby']);
    const mergedProps = mergeProps.mergeProps(eventsProps, tooltipProps);
    const { itemsPerCol, itemsPerRow } = LegendUtils.getGridNums(items.length, layoutInfo);
    const { legendStyle, renderedLegendStyle, preRenderedLegendStyle } = UNSAFE_Legend_themes_LegendStyles_css.styles;
    const canRender = orientation === 'vertical' || !!layoutInfo;
    return (jsxRuntime.jsxs("div", { ref: rootRef, tabIndex: isReadOnly ? undefined : 0, role: "application", class: classNames.classNames([legendStyle, canRender ? renderedLegendStyle : preRenderedLegendStyle]), ...testIdProps, ...ariaProps, ...mergedProps, children: [canRender ? (jsxRuntime.jsx(useLegendLayout.BaseLegend, { ...props, focusedItemRef: focusedItemRef, items: items, isReadOnly: isReadOnly, orientation: orientation, sectionIdx: 0, itemsPerRow: itemsPerRow, itemsPerCol: itemsPerCol, itemWidth: layoutInfo?.isWrapped ? layoutInfo?.maxItemWidth : undefined, focusedItemInfo: focusedItemInfo, hoveredItemInfo: hoveredItemInfo, activeId: eventsProps['aria-activedescendant'] })) : null, tooltipContent, textMeasureContent, cssContent] }));
}

exports.Legend = Legend;
//# sourceMappingURL=Legend-f284479d.js.map
