/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var Grid = require('./Grid-b0da0d92.js');
var classNames = require('./classNames-3bb28eb9.js');
var UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css = require('./UNSAFE_ListItemLayout/themes/ListItemLayoutStyles.css.js');
var Text = require('./Text-15925a9e.js');
require('./TabbableModeContext-4fb7c910.js');
var useTabbableMode = require('./useTabbableMode-98443dc8.js');

// TODO: revisit idea of negative margin when selector has a proper API for handling spacing
// Jira: JET-54543
function ListItemLayout({ children, selector, leading, primary, secondary, tertiary, overline, metadata, trailing, action, quaternary, navigation }) {
    // generate grid skeleton, adding cells based on which props are defined
    const { columnsTemplate, firstCell, secondCell, lastCell, hasSecondRow } = getColumnsTemplate(selector, leading, metadata, trailing, action, quaternary, navigation);
    // if ListItemLayout component has a child (ListItemTextLayout) then render that instead of individual props
    const textArea = children ? (jsxRuntime.jsx("div", { className: UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.childrenStyles, children: children })) : (jsxRuntime.jsxs("div", { className: UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.flexColumnStyles, children: [jsxRuntime.jsx("div", { children: overline }), jsxRuntime.jsx("div", { children: primary }), jsxRuntime.jsx("div", { children: secondary }), jsxRuntime.jsx("div", { children: tertiary })] }));
    // a second row is needed when quaternary or navigation are defined
    const secondRow = hasSecondRow
        ? [
            firstCell && jsxRuntime.jsx("span", { children: " " }),
            secondCell && jsxRuntime.jsx("span", { children: " " }),
            jsxRuntime.jsx("div", { className: UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.rowAlignStyles, children: quaternary }),
            lastCell && jsxRuntime.jsx("div", { className: UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.endJustifyStyles, children: navigation })
        ]
        : null;
    return (jsxRuntime.jsxs(Grid.Grid, { gap: "4x", gridTemplateColumns: columnsTemplate, children: [firstCell, secondCell, jsxRuntime.jsx("div", { className: UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.centerVerticalStyles, children: textArea }), lastCell, secondRow] }));
}
function getColumnsTemplate(selector, leading, metadata, trailing, action, quaternary, navigation) {
    const columnsTemplateBuilder = [];
    let firstCell = null;
    if (selector) {
        columnsTemplateBuilder.push('auto');
        firstCell = jsxRuntime.jsx("div", { className: UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.selectorStyles, children: selector });
    }
    let secondCell = null;
    if (leading) {
        columnsTemplateBuilder.push('auto');
        secondCell = jsxRuntime.jsx("div", { className: UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.centerVerticalStyles, children: leading });
    }
    columnsTemplateBuilder.push('1fr'); // Text area always gets 1 full fraction of allotted space
    let lastCell = null;
    if (metadata || trailing || action || navigation) {
        columnsTemplateBuilder.push('auto');
        lastCell = (jsxRuntime.jsxs("div", { className: UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.flexRowStyles, children: [jsxRuntime.jsx("div", { className: classNames.classNames([
                        UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.centerVerticalStyles,
                        metadata ? UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.leftPaddingStyles : ''
                    ]), children: metadata }), jsxRuntime.jsx("div", { className: classNames.classNames([
                        UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.centerVerticalStyles,
                        trailing ? UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.leftPaddingStyles : ''
                    ]), children: trailing }), jsxRuntime.jsx("div", { className: classNames.classNames([
                        UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.centerVerticalStyles,
                        action ? UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.leftPaddingStyles : ''
                    ]), children: action })] }));
    }
    const columnsTemplate = columnsTemplateBuilder.join(' ');
    const hasSecondRow = quaternary || navigation ? true : false;
    return { columnsTemplate, firstCell, secondCell, lastCell, hasSecondRow };
}

const textVariants = {
    primary: 'primary',
    secondary: 'primary',
    tertiary: 'secondary',
    quaternary: 'secondary',
    overline: 'secondary',
    metadata: 'secondary'
};
const textSizes = {
    primary: 'md',
    secondary: 'sm',
    tertiary: 'xs',
    quaternary: 'sm',
    overline: 'xs',
    metadata: 'sm'
};
const textLineClamps = {
    primary: undefined,
    secondary: 2,
    tertiary: 2,
    quaternary: 2,
    overline: 1,
    metadata: 1
};
function ListItemText({ children, variant }) {
    const { isTabbable } = useTabbableMode.useTabbableMode();
    const textContent = (jsxRuntime.jsx(Text.Text, { variant: textVariants[variant], size: textSizes[variant], lineClamp: textLineClamps[variant], children: children }));
    return isTabbable ? jsxRuntime.jsx("div", { tabIndex: 0, children: textContent }) : textContent;
}

function ListItemTextLayout({ primary, secondary, tertiary, overline }) {
    return (jsxRuntime.jsxs("div", { className: UNSAFE_ListItemLayout_themes_ListItemLayoutStyles_css.styles.flexColumnStyles, children: [jsxRuntime.jsx(ListItemText, { variant: "overline", children: overline }), jsxRuntime.jsx(ListItemText, { variant: "primary", children: primary }), jsxRuntime.jsx(ListItemText, { variant: "secondary", children: secondary }), jsxRuntime.jsx(ListItemText, { variant: "tertiary", children: tertiary })] }));
}

exports.ListItemLayout = ListItemLayout;
exports.ListItemText = ListItemText;
exports.ListItemTextLayout = ListItemTextLayout;
//# sourceMappingURL=ListItemTextLayout-aff1d86e.js.map
