/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var preact = require('preact');
var classNames = require('./classNames-3bb28eb9.js');
var PRIVATE_Message_themes_MessageStyles_css = require('./PRIVATE_Message/themes/MessageStyles.css.js');
var MessageFormattingUtils = require('./MessageFormattingUtils-034ac55d.js');

/**
 * Default renderer for rendering the detail content.
 *
 * @param item The data item object
 * @returns Rendered detail content
 */
function defaultDetailRenderer(item) {
    const { detail } = item.data;
    // If the detail is null or an empty string, do not render the
    // content row
    if (!MessageFormattingUtils.isValidValueForProp(detail)) {
        return null;
    }
    return jsxRuntime.jsx(preact.Fragment, { children: detail });
}
/**
 * Detail Component for rendering the detail content of the Message
 */
function MessageDetail({ item, renderer, variant = 'banner' }) {
    const isCustomRendered = renderer != null;
    const renderedContent = (renderer ?? defaultDetailRenderer)(item);
    if (renderedContent == null)
        return null;
    // If detail content is rendered, then wrap it in a div with specified style classes
    const classes = classNames.classNames([PRIVATE_Message_themes_MessageStyles_css.messageDetailStyles.base, PRIVATE_Message_themes_MessageStyles_css.messageDetailStyles[variant]]);
    return (jsxRuntime.jsx("div", { class: classes, ...(isCustomRendered ? { 'data-oj-message-custom-detail': '' } : {}), children: renderedContent }));
}

exports.MessageDetail = MessageDetail;
//# sourceMappingURL=MessageDetail-55843718.js.map
