/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var hooks = require('preact/hooks');
var useTranslationBundle = require('./useTranslationBundle-9549e176.js');
var clientHints = require('./clientHints-666b4780.js');
var mergeInterpolations = require('./mergeInterpolations-032e29ea.js');
var flexitem = require('./flexitem-403ef425.js');
var classNames = require('./classNames-3bb28eb9.js');
var Icon = require('./Icon-faf50cac.js');
var useTabBarContext = require('./useTabBarContext-a36c1b87.js');
var Sheet = require('./Sheet-8c21ddb8.js');
var TabBarItem = require('./TabBarItem-45a8d0c1.js');
var useComponentTheme = require('./useComponentTheme-b6e9b324.js');
var UNSAFE_TabBarCommon_themes_redwood_TabBarItemTheme = require('./UNSAFE_TabBarCommon/themes/redwood/TabBarItemTheme.js');
var NavigationListItem = require('./NavigationListItem-703baa8d.js');
var RemovableNavigationListItem = require('./RemovableNavigationListItem-d75c9a37.js');
var Dropdown = require('./Dropdown-ad221176.js');

const SvgOverflowH = (props) => (jsxRuntime.jsx(Icon.Icon, { viewBox: "0 0 24 24", ...props, children: jsxRuntime.jsx("g", { fill: "none", children: jsxRuntime.jsx("path", { d: "M6 12a2 2 0 1 1-4 0 2 2 0 0 1 4 0zm8 0a2 2 0 1 1-4 0 2 2 0 0 1 4 0zm8 0a2 2 0 1 1-4 0 2 2 0 0 1 4 0z", fill: "currentcolor" }) }) }));

/**
 * An OverflowTabBarItem renders TabBarItems in a device appropriate popup. OverflowTabBarItem is used internally in an OverflowTabBar.
 */
function OverflowTabBarItem({ overflowItemKey, icon, badge, overflowItems, isOpen, onClose }) {
    const { selection, showFocusRing, isEdgeBottom, onSelect, onRemove, layout } = useTabBarContext.useTabBarContext();
    const isPointerSelection = hooks.useRef(false);
    const overflowTabBarItemRef = hooks.useRef(null);
    const navigationListRef = hooks.useRef();
    const handleSelectionChange = hooks.useCallback((detail) => {
        if (detail.reason === 'pointer') {
            isPointerSelection.current = true;
        }
        onSelect?.({ value: detail.value });
        onClose?.({ reason: 'itemAction' });
    }, [onSelect, onClose]);
    const handleDropdownClose = hooks.useCallback((e) => {
        onClose?.({ reason: e.reason });
    }, [onClose]);
    const handleSheetClose = hooks.useCallback(() => {
        onClose?.({ reason: 'dismissed' });
    }, [onClose]);
    //When NavigationList steals focus the app resets it. Not always would it be reset to
    //parent of OverflowTabBarItem. Though currentKey would be correct, but focusRing would would not be visible
    //Hence we check for isPointerSelection.current
    const focusRingRequirement = showFocusRing && !isPointerSelection.current && !getIsMobile();
    //The focus should be on NavigationList only when it is open using keyboard
    hooks.useEffect(() => {
        if (isOpen && navigationListRef.current && focusRingRequirement) {
            navigationListRef.current.focus();
        }
        isPointerSelection.current = false;
    }, [isOpen, focusRingRequirement]);
    const overflowContainsKey = (searchKey) => {
        return overflowItems?.find((item) => item.itemKey === searchKey);
    };
    const translations = useTranslationBundle.useTranslationBundle('@oracle/oraclejet-preact');
    const isSelected = selection !== undefined && overflowContainsKey(selection);
    const { classes } = useComponentTheme.useComponentTheme(UNSAFE_TabBarCommon_themes_redwood_TabBarItemTheme.TabBarItemRedwoodTheme, {
        selection: isSelected ? 'isSelected' : 'notSelected',
        edgeBottom: isEdgeBottom ? 'isBottom' : 'notBottom'
    });
    const overflowTabBarItem = classNames.classNames([classes]);
    //instead of specifying flex: '1 0 auto' in cssProps var
    const styleInterpolations = mergeInterpolations.mergeInterpolations([
        ...Object.values(flexitem.flexitemInterpolations)
    ]);
    const flexDimensions = styleInterpolations({
        flex: layout === 'stretch' ? '1 0 auto' : '0 0 auto'
    });
    const getNavigationList = hooks.useCallback(() => {
        return (jsxRuntime.jsx(NavigationListItem.NavigationList, { ref: navigationListRef, selection: selection, onSelectionChange: handleSelectionChange, onRemove: onRemove, children: overflowItems?.map((item) => item.isRemovable ? (jsxRuntime.jsx(RemovableNavigationListItem.RemovableNavigationListItem, { itemKey: item.itemKey, label: item.label, badge: item.badge, metadata: item.metadata, severity: item.severity })) : (jsxRuntime.jsx(NavigationListItem.NavigationListItem, { itemKey: item.itemKey, label: item.label, badge: item.badge, metadata: item.metadata, severity: item.severity }))) }));
    }, [overflowItems, selection, handleSelectionChange, onRemove]);
    return (jsxRuntime.jsxs(jsxRuntime.Fragment, { children: [overflowItems && overflowItems.length > 0 && (jsxRuntime.jsx("div", { ref: overflowTabBarItemRef, class: overflowTabBarItem, style: flexDimensions, children: jsxRuntime.jsx(TabBarItem.BaseTabBarItem, { itemKey: overflowItemKey, label: " ", icon: icon ? icon : jsxRuntime.jsx(SvgOverflowH, { accessibleLabel: translations.overflowItemLabel() }), badge: badge, showFocusRingOverride: focusRingRequirement }) })), getIsMobile() ? (jsxRuntime.jsx(Sheet.Sheet, { isOpen: isOpen, onClose: handleSheetClose, children: getNavigationList() })) : (jsxRuntime.jsx(Dropdown.Dropdown, { isOpen: isOpen, onClose: handleDropdownClose, placement: isEdgeBottom ? 'top-end' : 'bottom-end', anchorRef: overflowTabBarItemRef, children: getNavigationList() }))] }));
}
const getIsMobile = () => {
    const { deviceType } = clientHints.getClientHints();
    return deviceType === 'phone';
};

exports.OverflowTabBarItem = OverflowTabBarItem;
//# sourceMappingURL=OverflowTabBarItem-7071d029.js.map
