/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var hooks = require('preact/hooks');
var useLegendLayout = require('./useLegendLayout-436b2dc1.js');
var mergeProps = require('./mergeProps-f6f4322f.js');
var LegendUtils = require('./LegendUtils-412c9829.js');
var useItemFocus = require('./useItemFocus-e9a52855.js');
var Flex = require('./Flex-299660c2.js');
var UNSAFE_Legend_themes_LegendStyles_css = require('./UNSAFE_Legend/themes/LegendStyles.css.js');
var useCssVars = require('./useCssVars-f6f61981.js');
var classNames = require('./classNames-3bb28eb9.js');
var useTestId = require('./useTestId-4fe30223.js');

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
function getItem(itemInfo, sections) {
    const { sectionIdx, itemIdx } = itemInfo;
    if (sectionIdx == -1 && itemIdx == -1) {
        const itemsLength = sections[sections.length - 1].items.length;
        return sections[sections.length - 1].items[itemsLength - 1];
    }
    return sections[sectionIdx].items[itemIdx];
}
function getNextItemInfo(itemInfo, sections) {
    let { itemIdx, sectionIdx } = itemInfo;
    if (sectionIdx == -1 && itemIdx == -1) {
        return {
            itemIdx: sections[sections.length - 1].items.length - 1,
            sectionIdx: sections.length - 1
        };
    }
    itemIdx += 1;
    const itemsLength = sections[sectionIdx].items.length;
    if (itemIdx >= itemsLength) {
        if (sectionIdx === sections.length - 1) {
            itemIdx = itemInfo.itemIdx;
        }
        else {
            sectionIdx += 1;
            itemIdx = 0;
        }
    }
    return { itemIdx, sectionIdx };
}
function getPrevItemInfo(itemInfo, sections) {
    let { itemIdx, sectionIdx } = itemInfo;
    if (sectionIdx == -1 && itemIdx == -1) {
        (itemIdx = sections[sections.length - 1].items.length - 1), (sectionIdx = sections.length - 1);
    }
    itemIdx -= 1;
    if (itemIdx < 0) {
        if (sectionIdx === 0) {
            itemIdx = 0;
        }
        else {
            itemIdx = sections[sectionIdx - 1].items.length - 1;
            sectionIdx -= 1;
        }
    }
    return { itemIdx, sectionIdx };
}
/**
 * Returns utils functions for navigation in simple sectional legend.
 * @param items
 * @returns
 */
function getSectionalNavUtils(sections) {
    return {
        getItem: (itemInfo) => {
            return getItem(itemInfo, sections);
        },
        getDetailFromInfo: (itemInfo) => {
            if (!itemInfo) {
                return { itemId: undefined, sectionId: undefined };
            }
            const item = getItem(itemInfo, sections);
            const sectionId = sections[itemInfo.sectionIdx].id;
            return { itemId: item.id, sectionId: sectionId };
        },
        getPrevItemInfo: (itemInfo) => {
            return getPrevItemInfo(itemInfo, sections);
        },
        getNextItemInfo: (itemInfo) => {
            return getNextItemInfo(itemInfo, sections);
        }
    };
}

/**
 * A Sectional Legend allows grouping of legend items in sections with a specific title.
 */
function SectionalLegend({ orientation = 'horizontal', sectionTitleHAlign = 'start', sections, isReadOnly = true, onItemAction, onInput, testId, ...props }) {
    const testIdProps = useTestId.useTestId(testId);
    const rootRef = hooks.useRef(null);
    const rootDimsRef = hooks.useRef();
    const { resolvedVars, cssContent } = useCssVars.useCssVars(LegendUtils.legendVars);
    const { getItem, getDetailFromInfo, getPrevItemInfo, getNextItemInfo } = getSectionalNavUtils(sections);
    const legendFontStylesObj = LegendUtils.getLegendTextStyle({
        fontStyle: props.textFontStyle,
        fontSize: props.textFontSize?.toString(),
        fontWeight: props.textFontWeight?.toString(),
        fontFamily: props.textFontFamily
    }, resolvedVars);
    const { textMeasureContent, layoutInfo } = useLegendLayout.useLegendLayout({
        rootRef,
        rootDimsRef,
        resolvedVars,
        sections,
        orientation,
        legendFontStylesObj,
        symbolWidth: props.symbolWidth
    });
    const { focusedItemInfo, hoveredItemInfo, ...eventsProps } = useLegendLayout.useEvents(!isReadOnly, props.hiddenIds != null, getDetailFromInfo, getPrevItemInfo, getNextItemInfo, onItemAction, onInput);
    const { focusedItemRef } = useItemFocus.useItemFocus(focusedItemInfo);
    const text = LegendUtils.getDatatipText(sections, getItem, focusedItemInfo, hoveredItemInfo);
    const { tooltipContent, tooltipProps } = useLegendLayout.useLegendTooltip({
        text,
        rootDimsRef,
        focusedItemInfo,
        focusedItemRef
    });
    const ariaProps = LegendUtils.getAriaProps(isReadOnly, props['aria-label'], props.hiddenIds);
    const mergedProps = mergeProps.mergeProps(eventsProps, tooltipProps);
    const titleStyle = {
        fontFamily: props.sectionTitleFontFamily,
        fontSize: props.sectionTitleFontSize,
        color: props.sectionTitleColor,
        fontStyle: props.sectionTitleFontStyle,
        fontWeight: props.sectionTitleFontWeight,
        textDecoration: props.sectionTitleTextDecoration
    };
    const { sectionalLegendBaseStyles, renderedLegendStyle, preRenderedLegendStyle } = UNSAFE_Legend_themes_LegendStyles_css.styles;
    const isHoriz = orientation === 'horizontal';
    const canRender = orientation === 'vertical' || !!layoutInfo;
    return (jsxRuntime.jsxs("div", { ref: rootRef, tabIndex: isReadOnly ? undefined : 0, role: "application", class: classNames.classNames([
            sectionalLegendBaseStyles,
            canRender ? renderedLegendStyle : preRenderedLegendStyle
        ]), ...testIdProps, ...ariaProps, ...mergedProps, children: [canRender ? (jsxRuntime.jsx(Flex.Flex, { direction: isHoriz ? 'row' : 'column', wrap: 'wrap', gap: '4x', children: sections.map((section, index) => {
                    const labelledById = !isReadOnly ? LegendUtils.getRandomId() : '';
                    const isWrappedSection = layoutInfo ? layoutInfo.isWrapped : false;
                    const flexDirection = !isHoriz || isWrappedSection ? 'column' : 'row';
                    const alignItems = !isHoriz || isWrappedSection ? 'start' : 'center';
                    const { itemsPerCol, itemsPerRow } = LegendUtils.getGridNums(section.items.length, layoutInfo);
                    return (jsxRuntime.jsxs(Flex.Flex, { align: alignItems, direction: flexDirection, maxWidth: '100%', justify: 'start', wrap: "wrap", children: [jsxRuntime.jsx(useLegendLayout.LegendText, { text: section.title, ...titleStyle, type: "title", id: labelledById, align: sectionTitleHAlign }), jsxRuntime.jsx(useLegendLayout.BaseLegend, { ...props, focusedItemRef: focusedItemRef, labelledBy: labelledById, sectionIdx: index, items: section.items, orientation: orientation, itemsPerRow: itemsPerRow, itemsPerCol: itemsPerCol, itemWidth: layoutInfo?.isWrapped ? layoutInfo?.maxItemWidth : undefined, isReadOnly: isReadOnly, focusedItemInfo: focusedItemInfo, hoveredItemInfo: hoveredItemInfo, activeId: eventsProps['aria-activedescendant'] })] }));
                }) })) : null, tooltipContent, textMeasureContent, cssContent] }));
}

exports.SectionalLegend = SectionalLegend;
//# sourceMappingURL=SectionalLegend-56742e61.js.map
