/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var classNames = require('./classNames-3bb28eb9.js');
var dimensions = require('./dimensions-77289825.js');
var padding = require('./padding-f5c99049.js');
var colors = require('./colors-52f16e9d.js');
var aria = require('./aria-1c2e3b5d.js');
var mergeInterpolations = require('./mergeInterpolations-032e29ea.js');
var useTestId = require('./useTestId-4fe30223.js');
var UNSAFE_SelectionCard_themes_SelectionCardStyles_css = require('./UNSAFE_SelectionCard/themes/SelectionCardStyles.css.js');
var useHover = require('./useHover-8f282a4d.js');

const interpolations = [
    ...Object.values(dimensions.dimensionInterpolations),
    ...Object.values(colors.colorInterpolations),
    ...Object.values(aria.ariaInterpolations),
    ...Object.values(padding.paddingInterpolations)
];
const mergedInterpolations = mergeInterpolations.mergeInterpolations(interpolations);
/**
 * A selection card is a container element that is a part of a collection of elements, and renders a group of related information in a styled rectangular area.
 * Width, height, and backgroundColor are set through props.
 */
const SelectionCard = ({ children, isSelected = false, testId, ...rest }) => {
    const { hoverProps, isHover } = useHover.useHover();
    const { class: mergedClasses, ...mergedStyles } = mergedInterpolations(rest);
    const selectionStyles = classNames.classNames([
        UNSAFE_SelectionCard_themes_SelectionCardStyles_css.baseStyle,
        mergedClasses,
        isHover && UNSAFE_SelectionCard_themes_SelectionCardStyles_css.hoverStyle,
        isSelected && (isHover ? UNSAFE_SelectionCard_themes_SelectionCardStyles_css.selectedHoverStyle : UNSAFE_SelectionCard_themes_SelectionCardStyles_css.selectedStyle)
    ]);
    const testIdProps = useTestId.useTestId(testId);
    return (jsxRuntime.jsx("div", { role: "region", ...hoverProps, class: selectionStyles, style: mergedStyles, ...testIdProps, children: children }));
};

exports.SelectionCard = SelectionCard;
//# sourceMappingURL=SelectionCard-27da3a94.js.map
