/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var compat = require('preact/compat');
var Check = require('./Check-559fd546.js');
var CheckboxOff = require('./CheckboxOff-695fb5e1.js');
var CheckboxOn = require('./CheckboxOn-83eda3ff.js');
var CheckboxMixed = require('./CheckboxMixed-06cb4e7f.js');
var HiddenAccessible = require('./HiddenAccessible-ce5d15d7.js');
var useComponentTheme = require('./useComponentTheme-b6e9b324.js');
var UNSAFE_Checkbox_themes_CheckboxIconContract_css = require('./UNSAFE_Checkbox/themes/CheckboxIconContract.css.js');
var UNSAFE_Checkbox_themes_redwood_CheckboxIconTheme = require('./UNSAFE_Checkbox/themes/redwood/CheckboxIconTheme.js');
require('./TabbableModeContext-4fb7c910.js');
var useTabbableMode = require('./useTabbableMode-98443dc8.js');

/**
 * StyledCheckbox is a controlled component that displays a stylized checkbox for 'checked',
 * 'unchecked', and 'partial'. It passes intrinsic props to the native input element. This should be used
 * in conjunction with a label and be passed an onChange handler. If used within a collection use
 * 'useCollectionFocusRing', or use 'useCheckboxRadioContext' within RadioSet/CheckboxSet, to determine if you
 * should display the focus ring using keyboard navigation.
 */
const StyledCheckbox = compat.forwardRef(({ onClick, isActive, isFocusRingShown, isDisabled, isReadonly, isRequired, isChecked: propIsChecked = 'unchecked', 
// Remaining props should be intrinsic input props only. If you need additional input props exposed,
// add to the picked items from InputProps.
id, name, value, onKeyUp, onKeyDown, onChange, onFocus, onBlur, 'aria-label': ariaLabel, 'aria-labelledby': ariaLabelledBy, 'aria-describedby': ariaDescribedBy }, ref = null) => {
    const { isTabbable } = useTabbableMode.useTabbableMode();
    const isPartial = propIsChecked === 'partial';
    const isChecked = propIsChecked === 'checked' || isPartial;
    const ariaChecked = isChecked ? (isPartial ? 'mixed' : true) : false;
    const { classes } = useComponentTheme.useComponentTheme(UNSAFE_Checkbox_themes_redwood_CheckboxIconTheme.CheckboxIconRedwoodTheme, {
        disabled: isDisabled ? 'isDisabled' : 'notDisabled',
        checked: isChecked ? 'isChecked' : 'notChecked',
        readonly: isReadonly ? 'isReadonly' : 'notReadonly',
        active: isActive ? 'isActive' : 'notActive',
        focus: isFocusRingShown ? 'isFocused' : 'notFocused',
        iconSize: '4xUnits'
    });
    const IconComp = isReadonly
        ? isChecked
            ? Check.SvgCheck
            : CheckboxOff.SvgCheckboxOff
        : isChecked
            ? isPartial
                ? CheckboxMixed.SvgCheckboxMixed
                : CheckboxOn.SvgCheckboxOn
            : CheckboxOff.SvgCheckboxOff;
    return (jsxRuntime.jsxs(jsxRuntime.Fragment, { children: [jsxRuntime.jsx(HiddenAccessible.HiddenAccessible, { children: jsxRuntime.jsx("input", { ...(isReadonly ? { 'aria-readonly': true } : {}), "aria-checked": ariaChecked, "aria-label": ariaLabel, "aria-labelledby": ariaLabelledBy, "aria-describedby": ariaDescribedBy, "aria-required": isRequired, ref: ref, name: name, value: value, onKeyUp: onKeyUp, onKeyDown: onKeyDown, onBlur: onBlur, onChange: onChange, onFocus: onFocus, id: id, type: "checkbox", disabled: isDisabled, checked: isChecked, tabIndex: isTabbable ? 0 : -1 }) }), jsxRuntime.jsx("span", { className: classes, "aria-hidden": "true", onClick: onClick, role: "img", children: jsxRuntime.jsx(IconComp, { size: UNSAFE_Checkbox_themes_CheckboxIconContract_css.checkboxIconVars.iconSize, color: "currentColor" }) })] }));
});

exports.StyledCheckbox = StyledCheckbox;
//# sourceMappingURL=StyledCheckbox-762a6108.js.map
