/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var hooks = require('preact/hooks');
var classNames = require('./classNames-3bb28eb9.js');
var usePress = require('./usePress-42f3876b.js');
var useUser = require('./useUser-171bdb32.js');
var Text = require('./Text-15925a9e.js');
var Flex = require('./Flex-299660c2.js');
var useTabBarContext = require('./useTabBarContext-a36c1b87.js');
var mergeInterpolations = require('./mergeInterpolations-032e29ea.js');
var flexitem = require('./flexitem-403ef425.js');
var useComponentTheme = require('./useComponentTheme-b6e9b324.js');
var UNSAFE_TabBarCommon_themes_redwood_TabBarItemTheme = require('./UNSAFE_TabBarCommon/themes/redwood/TabBarItemTheme.js');
var WarningS = require('./WarningS-5dd358a1.js');
var ErrorS = require('./ErrorS-6012f9f3.js');
var InformationS = require('./InformationS-c18dd806.js');
var SuccessS = require('./SuccessS-3a2c6dd4.js');
var Badge = require('./Badge-c5cd8ee5.js');

function BaseTabBarItem({ itemKey, label, icon, badge, metadata, severity = 'none', removeIcon, showFocusRingOverride }) {
    const { onSelect, selection, currentKey, showFocusRing, layout, display, size, isEdgeBottom, tabItemPrefix } = useTabBarContext.useTabBarContext();
    const isCurrent = currentKey === itemKey;
    const ref = hooks.useRef(null);
    /**
     * FIXME: 
     *
     * This is a temporary workaround until [JET-63308]() is resolved.
     *
     * This will optimistically attempt to scroll tab items into view as the tab
     * item becomes "current" regardless of whether this tab item is nested
     * inside of a conveyor/scrollable container.
     *
     * After JET-63308 (or equivalent) is implemented, we can remove this with a
     * proper utility that only invokes inside of a scrolling context.
     */
    hooks.useEffect(() => {
        if (ref.current && isCurrent) {
            ref.current.scrollIntoView({ behavior: 'smooth', block: 'nearest', inline: 'nearest' });
        }
    }, [isCurrent]);
    //Selection
    //Dependency on currentKey because upon removal selectionChange does not change hence no re render
    //and it is still holding the deleted key as selection
    const handlePress = hooks.useCallback(() => {
        onSelect?.({ value: itemKey });
    }, [itemKey, onSelect]);
    const { pressProps } = usePress.usePress(handlePress);
    const isSelected = selection === itemKey;
    const isStacked = display === 'stacked' ? true : false;
    const isIconOnly = display === 'icons' ? true : false;
    const isTall = size === 'lg' ? true : false;
    const labelRequirement = label && !(label === ' ' || label === undefined);
    const severityRequirement = !(severity === 'none' || severity === undefined);
    const IconComponent = (severityValue) => {
        switch (severityValue) {
            case 'error':
                return jsxRuntime.jsx(ErrorS.SvgErrorS, { color: "danger" });
            case 'warning':
                return jsxRuntime.jsx(WarningS.SvgWarningS, { color: "warning" });
            case 'info':
                return jsxRuntime.jsx(InformationS.SvgInformationS, { color: "info" });
            case 'confirmation':
                return jsxRuntime.jsx(SuccessS.SvgSuccessS, { color: "success" });
            default:
                return null;
        }
    };
    const { direction } = useUser.useUser();
    const makeFocusRingVisible = showFocusRingOverride !== undefined ? showFocusRingOverride : showFocusRing;
    const { classes, styles: { tabBarItemIcon, tabBarItemIconStacked, tabBarItemMetadata, tabBarItemStackedBadge, tabBarItemStackedBadgeRtl, tabBarItemRemoveIcon } } = useComponentTheme.useComponentTheme(UNSAFE_TabBarCommon_themes_redwood_TabBarItemTheme.TabBarItemRedwoodTheme, {
        selection: isSelected ? 'isSelected' : 'notSelected',
        current: isCurrent ? 'isCurrent' : 'notCurrent',
        focusRing: makeFocusRingVisible ? 'showFocusRing' : 'noFocusRing',
        edgeBottom: isEdgeBottom ? 'isBottom' : 'notBottom'
    });
    const tabBarItem = classNames.classNames([classes]);
    const tabBarItemIconClasses = classNames.classNames([
        tabBarItemIcon,
        (isStacked || isIconOnly || !labelRequirement) && tabBarItemIconStacked
    ]);
    const tabBarItemMetadataClasses = classNames.classNames([tabBarItemMetadata]);
    const tabBarItemStackedBadgeClasses = classNames.classNames([
        tabBarItemStackedBadge,
        direction === 'rtl' && tabBarItemStackedBadgeRtl
    ]);
    const removeClass = classNames.classNames([tabBarItemRemoveIcon]);
    //instead of specifying flex: '1 0 auto' in cssProps var
    const styleInterpolations = mergeInterpolations.mergeInterpolations([
        ...Object.values(flexitem.flexitemInterpolations)
    ]);
    const flexDimensions = styleInterpolations({
        flex: layout === 'stretch' ? '1 0 auto' : '0 0 auto'
    });
    return (jsxRuntime.jsx("div", { "data-oj-key": itemKey, "data-oj-removable": removeIcon ? 'true' : undefined, id: tabItemPrefix + itemKey, class: tabBarItem, ref: ref, role: "tab", "aria-selected": isSelected, ...pressProps, style: flexDimensions, children: jsxRuntime.jsxs(Flex.Flex, { minWidth: "10.5x", minHeight: isTall ? '14.5x' : '100%', height: isStacked && labelRequirement ? '18.5x' : '10.5x', gap: "2x", flex: "1 1 auto", children: [jsxRuntime.jsxs(Flex.Flex, { direction: isStacked && labelRequirement ? 'column' : 'row', gap: "2x", align: "center", justify: "center", flex: "1 1 auto", children: [(isStacked || isIconOnly || (!labelRequirement && icon)) && (jsxRuntime.jsxs("div", { class: tabBarItemIconClasses, children: [icon, jsxRuntime.jsx("span", { class: tabBarItemStackedBadgeClasses, children: badge ? (jsxRuntime.jsx(Badge.Badge, { size: "xs", variant: "neutral", children: badge })) : null })] })), ((!isIconOnly && labelRequirement) ||
                            metadata ||
                            severityRequirement ||
                            removeIcon ||
                            (icon && labelRequirement && !isStacked && !isIconOnly)) && (jsxRuntime.jsxs(Flex.Flex, { gap: "2x", align: "center", justify: "center", children: [!isIconOnly && !isStacked && icon && labelRequirement && (jsxRuntime.jsx("span", { class: tabBarItemIconClasses, children: icon })), !isIconOnly && labelRequirement && (jsxRuntime.jsx(Text.Text, { size: "md", weight: "semiBold", children: label })), !isStacked && !isIconOnly && badge && labelRequirement && (jsxRuntime.jsx(Badge.Badge, { size: "md", variant: "neutralSubtle", children: badge })), metadata && (jsxRuntime.jsx("span", { class: tabBarItemMetadataClasses, children: jsxRuntime.jsx(Text.Text, { children: '( ' + metadata + ' )' }) })), severityRequirement && (jsxRuntime.jsx("span", { class: tabBarItemIconClasses, children: IconComponent(severity) }))] }))] }), removeIcon && jsxRuntime.jsx("span", { class: removeClass, children: removeIcon })] }) }));
}

function TabBarItem({ itemKey, label, icon, badge, metadata, severity }) {
    return (jsxRuntime.jsx(BaseTabBarItem, { itemKey: itemKey, label: label, icon: icon, badge: badge, metadata: metadata, severity: severity }));
}

exports.BaseTabBarItem = BaseTabBarItem;
exports.TabBarItem = TabBarItem;
//# sourceMappingURL=TabBarItem-45a8d0c1.js.map
