/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var hooks = require('preact/hooks');
var compat = require('preact/compat');
var mergeProps = require('./mergeProps-f6f4322f.js');
var collectionUtils = require('./collectionUtils-e624b53a.js');
var useCollectionFocusRing = require('./useCollectionFocusRing-c5bb4a10.js');
var useCurrentKey = require('./useCurrentKey-0803c983.js');
var useId = require('./useId-42e8026e.js');
var UNSAFE_TabBarMixed_themes_TabBarMixedStyles_css = require('./UNSAFE_TabBarMixed/themes/TabBarMixedStyles.css.js');
var useTabBarContext = require('./useTabBarContext-a36c1b87.js');

const ITEM_SELECTOR = '[role="tab"]';
const REMOVAL_ITEM_SELECTOR = '[data-oj-tabbar-item-remove-icon]';
/**
 * Re-implements UNSAFE_TabBar/TabBar behavior for focus and keyboad handling
 */
function useTabBar(options) {
    const { children, class: className, display, edge = 'top', selection, onSelect, onRemove, ref: rootRef = hooks.useRef(null), size } = options;
    const tabId = useId.useId();
    const [currentKey, setCurrentKey] = hooks.useState(selection);
    const tabItemPrefix = tabId + '_';
    /**
     * A reference to the previous set of tab keys that this tablist
     * contains before children were re-rendered. In the event the "current" tab
     * is removed and the "currentKey" reference is broken, we can redirect
     * the "currentKey" to another tab based on the index of the removed tab.
     */
    const prevTabKeys = hooks.useRef();
    hooks.useEffect(() => {
        if (rootRef.current) {
            const tabKeys = Array.from(rootRef.current.querySelectorAll(ITEM_SELECTOR)).map((elem) => collectionUtils.getKey(elem));
            if (currentKey && !tabKeys.includes(currentKey) && prevTabKeys.current) {
                setCurrentKey(findNextCurrentKey(currentKey, tabKeys, prevTabKeys.current));
            }
            prevTabKeys.current = tabKeys;
        }
    }, [children, currentKey]);
    const [showFocusRing, focusRingProps] = useCollectionFocusRing.useCollectionFocusRing(rootRef, [
        'ArrowRight',
        'ArrowLeft'
    ]);
    const { currentKeyProps } = useCurrentKey.useCurrentKey((element) => onRemove
        ? extractOnlyItemKey(element, ITEM_SELECTOR, REMOVAL_ITEM_SELECTOR)
        : collectionUtils.keyExtractor(element, ITEM_SELECTOR), false, undefined, undefined, collectionUtils.getPrevNextKey(rootRef.current, currentKey, true, ITEM_SELECTOR), collectionUtils.getPrevNextKey(rootRef.current, currentKey, false, ITEM_SELECTOR), currentKey, (detail) => setCurrentKey(detail.value));
    const onKeyDown = (event) => {
        if (currentKey && event.key === 'Enter') {
            onSelect?.({ value: currentKey });
        }
        if (currentKey && event.key === 'Delete') {
            onRemove?.({ value: currentKey });
        }
    };
    const onFocus = () => {
        if (rootRef.current && currentKey === undefined) {
            const key = collectionUtils.getFirstVisibleKey(rootRef.current, ITEM_SELECTOR);
            if (key) {
                setCurrentKey(key);
            }
        }
    };
    const eventProps = { onKeyDown, onFocus };
    return {
        rootProps: mergeProps.mergeProps({
            'aria-activedescendant': currentKey ? tabItemPrefix + currentKey : '',
            'aria-multiselectable': false,
            class: className,
            ref: rootRef,
            role: 'tablist',
            tabIndex: 0
        }, currentKeyProps, focusRingProps, eventProps),
        tabBarContext: {
            currentKey,
            display,
            isEdgeBottom: edge === 'bottom',
            layout: 'condense',
            onRemove,
            onSelect,
            showFocusRing,
            selection,
            size,
            tabItemPrefix
        }
    };
}
//useCurrentKey use click capture that captures first click, which is on remove button
//when we remove and sets the item being removed to currentKey. So we should not allow
//this if click is on remove button
const extractOnlyItemKey = (element, itemSelector, itemEliminator) => {
    const tabBarItem = element.closest(itemSelector);
    const removeButton = element.closest(itemEliminator);
    return tabBarItem?.contains(removeButton) ? null : collectionUtils.keyExtractor(element, itemSelector);
};
const findNextCurrentKey = (currentKey, currTabs, prevTabs) => {
    const index = prevTabs.indexOf(currentKey);
    if (index > 0) {
        const nextIndex = index === prevTabs.length - 1 ? index - 1 : index + 1;
        const nextKey = prevTabs[nextIndex];
        if (currTabs.indexOf(nextKey) !== -1) {
            return nextKey;
        }
    }
    // update current key to be the first one if we can't find a suitable next key
    return currTabs[0];
};

/**
 * A navigation component that enables horizontal navigation between distinct content with a mixture of static and dynamic tabs.
 *
 * @param {TabBarMixedProps} props TabBarMixed component props.
 * @returns {JSX.Element} TabBarMixed component element.
 */
const TabBarMixed = compat.forwardRef((props, focusHandleRef) => {
    const { children, onRemove, onSelect, selection, size } = props;
    const rootRef = hooks.useRef(null);
    const { rootProps, tabBarContext } = useTabBar({
        class: UNSAFE_TabBarMixed_themes_TabBarMixedStyles_css.styles.tabBarBase,
        edge: 'bottom',
        onRemove,
        onSelect,
        ref: rootRef,
        selection,
        size,
        children
    });
    //Allows to call focus on rootRef without having to expose it
    compat.useImperativeHandle(focusHandleRef, () => ({
        focus: () => {
            if (rootRef.current) {
                rootRef.current.focus();
            }
        },
        blur: () => {
            if (rootRef.current) {
                rootRef.current.blur();
            }
        }
    }));
    return (jsxRuntime.jsx(useTabBarContext.TabBarContext.Provider, { value: tabBarContext, children: jsxRuntime.jsx("div", { ...rootProps, children: children }) }));
});

exports.TabBarMixed = TabBarMixed;
//# sourceMappingURL=TabBarMixed-ae36e3ef.js.map
