/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var mergeProps = require('./mergeProps-f6f4322f.js');
var useHover = require('./useHover-8f282a4d.js');
var useActive = require('./useActive-5db5a3fb.js');
var useFocus = require('./useFocus-e84e6c2e.js');
var usePress = require('./usePress-42f3876b.js');

const DefaultActionableOptions = {
    isDisabled: false,
    isRepeat: false
};
/**
 * A hook that can add actionable support to a target element, turning it into a
 * clickable button, div, card, etc.
 * If isHover, isFocus, and isActive are only used for changing visual rendering, it would be
 * faster to not use this hook, and instead use :hover, :focus-visible, :active and usePress.
 * @param onActionHandler
 * @returns
 */
function useActionable(onActionHandler, settings = DefaultActionableOptions) {
    const mergedSettings = { ...DefaultActionableOptions, ...settings };
    const { pressProps } = usePress.usePress(onActionHandler, mergedSettings);
    const { hoverProps, isHover } = useHover.useHover({ isDisabled: mergedSettings.isDisabled });
    const { focusProps, isFocus } = useFocus.useFocus({ isDisabled: mergedSettings.isDisabled });
    const { activeProps, isActive } = useActive.useActive({ isDisabled: mergedSettings.isDisabled });
    const actionableProps = mergeProps.mergeProps(pressProps, hoverProps, activeProps, focusProps);
    return { actionableProps, isActive, isHover, isFocus };
}

exports.useActionable = useActionable;
//# sourceMappingURL=useActionable-604c4209.js.map
