/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var useDatatip = require('./useDatatip-21b6ba1a.js');
var util = require('./util-1e3729d6.js');
var jsxRuntime = require('preact/jsx-runtime');
var VisTabularDatatip = require('./VisTabularDatatip-77e52ec0.js');
var colorUtils = require('./colorUtils-8190afc7.js');
require('./accUtils-c4465ee9.js');
var useUser = require('./useUser-171bdb32.js');

/**
 * Returns the data required for the VisTabularDatatip to render.
 * @param activeInfo active item info.
 * @param series The array of chart series.
 * @param groups The array of chart groups.
 * @param valueFormats The array of chart series.
 * @param getDataItem function which returns chartItem
 */
function getChartTabularData(data, seriesValue, groupValue, valueFormats) {
    //TODO: Use translation strings
    const value = data.value;
    const itemValue = valueFormats?.value?.format ? valueFormats?.value?.format?.(value) : value;
    const tableData = [];
    if (valueFormats?.series?.isDisplayed != false) {
        tableData.push({ key: valueFormats?.series?.label || 'Series', value: seriesValue });
    }
    if (valueFormats?.group?.isDisplayed != false) {
        tableData.push({ key: valueFormats?.group?.label || 'Group', value: groupValue });
    }
    if (valueFormats?.value?.isDisplayed != false) {
        tableData.push({ key: valueFormats?.value?.label || 'Value', value: itemValue });
    }
    return tableData;
}
function getDefaultDatatipRenderer(options) {
    return (context) => ({
        content: (jsxRuntime.jsx(VisTabularDatatip.VisTabularDatatip, { tableData: getChartTabularData(context.data, options.series.name || options.series.id.toString(), options.groups.name || options.groups.id, options.valueFormats) })),
        borderColor: context.data.color || options.series.color
    });
}

const DATA_CURSOR_OFFSET = 10;
const useChartDatatip = ({ isDataCursorEnabled, dataCursorPosition, isHoriz, datatip, xScale, getDataItem, getDataItemPos, hoveredItemInfo, focusedItemInfo, series, valueFormats, groups, width, height, findNearest }) => {
    let activeInfo = focusedItemInfo?.isCurrent
        ? focusedItemInfo
        : hoveredItemInfo?.isCurrent &&
            hoveredItemInfo.seriesIndex !== undefined &&
            hoveredItemInfo.groupIndex !== undefined
            ? hoveredItemInfo
            : undefined;
    const anchor = focusedItemInfo?.isCurrent || isDataCursorEnabled ? 'element' : 'pointer';
    const elementBounds = focusedItemInfo?.isCurrent
        ? getDataItemPos(focusedItemInfo.seriesIndex, focusedItemInfo.groupIndex)
        : undefined;
    const { direction } = useUser.useUser();
    const isRtl = direction === 'rtl';
    let placement = 'top-start';
    let offset = util.calculateOffset(isRtl, width, elementBounds);
    if (isDataCursorEnabled && dataCursorPosition?.x != null && dataCursorPosition?.y != null) {
        // TOD0: The offset for data cursor should be calculated according to
        // datatip content dimensions as well.
        activeInfo = activeInfo || findNearest?.(dataCursorPosition.x, dataCursorPosition.y);
        const xPos = xScale.transform(dataCursorPosition.x);
        offset = { mainAxis: -1, crossAxis: -1 };
        if (isHoriz) {
            placement = 'end';
            offset.mainAxis = -1 * DATA_CURSOR_OFFSET;
            offset.crossAxis = xPos - height / 2;
        }
        else {
            placement = 'top';
            offset.crossAxis = isRtl ? width / 2 - xPos : xPos - width / 2;
            offset.mainAxis = -1 * DATA_CURSOR_OFFSET;
        }
    }
    let dtContent;
    let item;
    if (activeInfo?.seriesIndex != null && activeInfo?.groupIndex != null) {
        item = getDataItem(activeInfo.seriesIndex, activeInfo.groupIndex);
        const datatipContext = item
            ? {
                data: item,
                seriesIndex: activeInfo.seriesIndex,
                groupIndex: activeInfo.groupIndex,
                id: item.id
            }
            : undefined;
        if (datatipContext) {
            const datatipRenderer = datatip ||
                getDefaultDatatipRenderer({
                    series: series[activeInfo.seriesIndex],
                    groups: groups[activeInfo.groupIndex],
                    valueFormats
                });
            dtContent = datatipRenderer(datatipContext);
        }
    }
    const colors = colorUtils.getColorRamp();
    const borderColor = dtContent?.borderColor ||
        (activeInfo &&
            (series[activeInfo.seriesIndex].color ||
                series[activeInfo.seriesIndex].lineColor ||
                series[activeInfo.seriesIndex].areaColor)) ||
        (activeInfo && colors[activeInfo.seriesIndex % colors.length]);
    const { datatipContent, datatipProps } = useDatatip.useDatatip({
        content: dtContent?.content,
        anchor,
        placement,
        offset,
        borderColor
    });
    return { datatipContent, datatipProps };
};

exports.useChartDatatip = useChartDatatip;
//# sourceMappingURL=useChartDatatip-30b2a4e8.js.map
