/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var eventsUtils = require('./eventsUtils-086f0e32.js');
require('./accUtils-c4465ee9.js');

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
const isDrillable = (info, drilling, groups, getDataItem) => {
    const { seriesIndex, groupIndex } = info;
    if (groupIndex === undefined) {
        return false;
    }
    if (seriesIndex != null) {
        const itemDrilling = getDataItem(seriesIndex, groupIndex)?.drilling != 'off';
        return drilling === 'on' && itemDrilling;
    }
    else {
        return groups[groupIndex].drilling != 'off';
    }
};
/**
 * Returns the event handlers for drill support.
 */
function useChartDrill({ rootRef, focusedItemInfo, focusedGroupInfo, groups, selectionMode, drilling, getDataItem, onItemDrill, onGroupDrill }) {
    const processDrill = (info) => {
        if (!info ||
            info.groupIndex == undefined ||
            !isDrillable(info, drilling, groups, getDataItem)) {
            return;
        }
        const { seriesIndex, groupIndex } = info;
        if (seriesIndex != null) {
            const data = getDataItem(seriesIndex, groupIndex);
            onItemDrill?.({
                groupIndex,
                seriesIndex,
                data,
                id: data?.id
            });
        }
        else {
            onGroupDrill?.({
                groupIndex,
                group: groups[groupIndex],
                id: groups[groupIndex].id
            });
        }
    };
    const onPointerUp = (event) => {
        if (selectionMode != 'none') {
            return;
        }
        processDrill(eventsUtils.getInfo(rootRef, event));
    };
    const onDblClick = (event) => {
        if (selectionMode === 'none') {
            return;
        }
        processDrill(eventsUtils.getInfo(rootRef, event));
    };
    const onKeyDown = (event) => {
        if (event.key != 'Enter') {
            return;
        }
        const focusedInfo = focusedItemInfo.isCurrent ? focusedItemInfo : focusedGroupInfo;
        processDrill(focusedInfo);
    };
    return {
        onPointerUp,
        onDblClick,
        onKeyDown
    };
}

exports.useChartDrill = useChartDrill;
//# sourceMappingURL=useChartDrill-7364663e.js.map
