/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var jsxRuntime = require('preact/jsx-runtime');
var Layer = require('./Layer-c0f805fc.js');
require('preact/compat');
require('./TopLayerHost-b320c750.js');
var Floating = require('./Floating-8ecf7694.js');
var hooks = require('preact/hooks');
var hooks_PRIVATE_useDatatip_themes_useDatatip_css = require('./hooks/PRIVATE_useDatatip/themes/useDatatip.css.js');
var useId = require('./useId-42e8026e.js');

/**
 * The default offset for datatip mainAxis.
 */
const DEFAULT_OFFSET = 8;
/**
 * The hook to support custom datatip in visualization components.
 */
function useDatatip({ content, placement = 'top-end-corner', offset, borderColor, anchor }) {
    const uniqueIdRef = hooks.useRef(useId.useId());
    const [coords, setCoords] = hooks.useState({ x: 0, y: 0 });
    const offsetValue = offset || { mainAxis: DEFAULT_OFFSET, crossAxis: -1 * DEFAULT_OFFSET };
    const [isVisible, setIsVisible] = hooks.useState(false);
    const targetRef = hooks.useRef(null);
    const toggleTimeoutRef = hooks.useRef();
    const onPointerMove = (event) => {
        setCoords({ x: event.clientX, y: event.clientY });
        setIsVisible(true);
    };
    const onFocus = (event) => {
        if (event.eventPhase === Event.AT_TARGET || event.eventPhase === Event.BUBBLING_PHASE) {
            targetRef.current = event.target;
            setIsVisible(true);
        }
    };
    const onPointerEnter = (event) => {
        if (event.eventPhase === Event.AT_TARGET || event.eventPhase === Event.BUBBLING_PHASE) {
            targetRef.current = event.currentTarget;
            setIsVisible(true);
        }
    };
    const onBlur = () => {
        setIsVisible(false);
    };
    const onContentPointerEnter = () => {
        clearTimeout(toggleTimeoutRef.current);
    };
    const onPointerLeave = () => {
        toggleTimeoutRef.current = setTimeout(() => {
            setIsVisible(false);
        }, 20);
    };
    const onKeyUp = (event) => {
        if (event.code === 'Escape') {
            event.preventDefault();
            setIsVisible(false);
        }
        else if (event.code === 'ArrowUp' ||
            event.code === 'ArrowDown' ||
            event.code === 'ArrowRight' ||
            event.code === 'ArrowLeft' ||
            event.code === 'Tab') {
            setIsVisible(true);
        }
    };
    const anchorRef = anchor === 'element' ? targetRef : { current: coords };
    const datatipContent = isVisible && content != null ? (jsxRuntime.jsx(Layer.Layer, { children: jsxRuntime.jsx(Floating.Floating, { anchorRef: anchorRef, placement: placement, offsetValue: offsetValue, flipOptions: { crossAxis: false, mainAxis: false }, shiftOptions: { crossAxis: true, mainAxis: true }, children: jsxRuntime.jsx("div", { role: "tooltip", id: uniqueIdRef.current, className: hooks_PRIVATE_useDatatip_themes_useDatatip_css.styles.dvtDatatipStyle, style: { borderColor }, onPointerEnter: onContentPointerEnter, onPointerLeave: onPointerLeave, children: content }) }) })) : null;
    return {
        datatipContent,
        datatipProps: {
            'aria-describedby': uniqueIdRef.current,
            onKeyUp,
            onPointerLeave,
            onPointerEnter,
            onPointerMove,
            onFocus,
            onBlur
        }
    };
}

exports.useDatatip = useDatatip;
//# sourceMappingURL=useDatatip-21b6ba1a.js.map
