/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var hooks = require('preact/hooks');
var LayoutUtils = require('./LayoutUtils-d2e22f42.js');
var eventsUtils = require('./eventsUtils-086f0e32.js');
require('./accUtils-c4465ee9.js');

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
const getItemInfo = (event) => {
    const itemIndex = event.target.dataset['ojItemIndex'];
    if (itemIndex) {
        return {
            itemIndex: parseInt(itemIndex, 10),
        };
    }
    return;
};
/**
 * Hook to handle chart navigation. This can be shared between the pie, funnel and pyramid chart
 * as they follow similar navigation pattern.
 * TODO: rename
 */
function usePieChartNav({ getNextChartItem }) {
    const [focusedItemInfo, setfocusedItemInfo] = hooks.useState({
        itemIndex: 0
    });
    const [hoveredItemInfo, setHoveredItemInfo] = hooks.useState();
    const activeId = hooks.useRef();
    const updateFocusedItemInfo = (itemInfo) => {
        if (itemInfo.itemIndex !== focusedItemInfo.itemIndex ||
            itemInfo.isFocusVisible !== focusedItemInfo.isFocusVisible) {
            setfocusedItemInfo(itemInfo);
            setHoveredItemInfo(undefined);
        }
    };
    const onPointerUp = (event) => {
        const info = getItemInfo(event);
        if (info) {
            updateFocusedItemInfo({ ...info, isFocusVisible: false });
        }
    };
    const onPointerMove = (event) => {
        const info = getItemInfo(event);
        if (info?.itemIndex != hoveredItemInfo?.itemIndex) {
            setHoveredItemInfo(info);
            activeId.current = LayoutUtils.getRandomId();
        }
    };
    const onKeyDown = (event) => {
        const key = event.key;
        switch (key) {
            case 'Tab':
                return;
            case 'ArrowDown': {
                const info = getNextChartItem(focusedItemInfo, 'ArrowDown');
                updateFocusedItemInfo({ ...info, isFocusVisible: true });
                eventsUtils.cancelEvent(event);
                break;
            }
            case 'ArrowUp': {
                const info = getNextChartItem(focusedItemInfo, 'ArrowUp');
                updateFocusedItemInfo({ ...info, isFocusVisible: true });
                eventsUtils.cancelEvent(event);
                break;
            }
            case 'ArrowLeft': {
                const info = getNextChartItem(focusedItemInfo, 'ArrowLeft');
                updateFocusedItemInfo({ ...info, isFocusVisible: true });
                eventsUtils.cancelEvent(event);
                break;
            }
            case 'ArrowRight': {
                const info = getNextChartItem(focusedItemInfo, 'ArrowRight');
                updateFocusedItemInfo({ ...info, isFocusVisible: true });
                eventsUtils.cancelEvent(event);
                break;
            }
        }
        activeId.current = LayoutUtils.getRandomId();
    };
    const onKeyUp = (event) => {
        const key = event.key;
        if (key === 'Tab') {
            updateFocusedItemInfo({ ...focusedItemInfo, isFocusVisible: true });
        }
    };
    const onBlur = () => {
        updateFocusedItemInfo({
            ...focusedItemInfo,
            isFocusVisible: false
        });
    };
    return {
        focusedItemInfo,
        hoveredItemInfo,
        activeId: activeId.current,
        navProps: {
            onKeyDown,
            onKeyUp,
            onPointerUp,
            onBlur,
            onPointerMove
        }
    };
}

exports.usePieChartNav = usePieChartNav;
//# sourceMappingURL=usePieChartNav-6a919148.js.map
