/* @oracle/oraclejet-preact: 16.0.1 */
'use strict';

var hooks = require('preact/hooks');

class SharedResizeObserver {
    constructor() {
        this.targetCallbacks = new Map();
        this.resizeObserver = new ResizeObserver((entries) => {
            for (const entry of entries) {
                const callbacks = this.targetCallbacks.get(entry.target);
                if (callbacks !== undefined) {
                    callbacks.forEach((cb) => cb(entry));
                }
            }
        });
    }
    subscribe(target, callback, options) {
        const callbacks = this.targetCallbacks.get(target);
        if (callbacks === undefined) {
            this.targetCallbacks.set(target, new Set([callback]));
            this.resizeObserver.observe(target, options);
        }
        else {
            callbacks.add(callback);
        }
    }
    unsubscribe(target, callback) {
        const callbacks = this.targetCallbacks.get(target);
        if (callbacks === undefined)
            return;
        callbacks.delete(callback);
        if (callbacks.size === 0) {
            this.resizeObserver.unobserve(target);
            this.targetCallbacks.delete(target);
        }
    }
}
const sharedResizeObserver = new SharedResizeObserver();
/**
 * A hook for observing and responding to changes to the size of a DOM element.
 * It utilizes a ResizeObserver to observe changes to the size of the specified element
 * and accepts a callback function to respond to these changes.
 */
const useResizeObserver = ({ ref, callback, resizeObserverOptions }) => {
    hooks.useLayoutEffect(() => {
        // Note ref may be undefined in the hooks storybook demo
        if (!ref?.current)
            return;
        const target = ref.current;
        sharedResizeObserver.subscribe(target, callback, resizeObserverOptions);
        return () => sharedResizeObserver.unsubscribe(target, callback);
    }, [ref, callback, resizeObserverOptions]);
};

exports.useResizeObserver = useResizeObserver;
//# sourceMappingURL=useResizeObserver-1c0c68c4.js.map
