/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { useImperativeHandle } from 'preact/hooks';
import { forwardRef, useRef } from 'preact/compat';
import './TabbableModeContext-1e03f32d.js';
import { u as useTabbableMode } from './useTabbableMode-9c5d5a64.js';
import { u as useActionable } from './useActionable-e45f7161.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { d as dimensionInterpolations } from './dimensions-d9512d9e.js';
import { p as paddingInterpolations } from './padding-532a6a4d.js';
import { c as colorInterpolations } from './colors-8cd7269e.js';
import { a as ariaInterpolations } from './aria-20ecc4ae.js';
import { m as mergeInterpolations } from './mergeInterpolations-f71c7446.js';
import { baseStyle, hoverStyle, activeStyle } from './UNSAFE_ActionCard/themes/ActionCardStyles.css.js';
import { u as useTestId } from './useTestId-5050f73e.js';

const interpolations = [
    ...Object.values(dimensionInterpolations),
    ...Object.values(colorInterpolations),
    ...Object.values(paddingInterpolations),
    ...Object.values(ariaInterpolations)
];
const mergedInterpolations = mergeInterpolations(interpolations);
/**
 * An action card is a container that renders a group of related information in a styled rectangular area, provides enhanced styling for hover and focus states, and invokes an action event when clicked.
 * Width, height, padding, and background color are set through props.
 */
const ActionCard = forwardRef(({ children, onAction, testId, ...rest }, ref) => {
    const { isTabbable } = useTabbableMode();
    const pressHandler = (e) => {
        if (onAction) {
            onAction({
                reason: e.type == 'keyup' ? 'keyboard' : 'pointer'
            });
        }
    };
    const { actionableProps, isHover, isActive } = useActionable(pressHandler);
    const { class: mergedClasses, ariaLabel, ariaLabelledBy, ...mergedStyles } = mergedInterpolations(rest);
    const actionStyles = classNames([
        baseStyle,
        mergedClasses,
        isHover && hoverStyle,
        isActive && activeStyle
    ]);
    const rootRef = useRef(null);
    useImperativeHandle(ref, () => ({
        focus: () => {
            rootRef.current?.focus();
        },
        blur: () => {
            rootRef.current?.blur();
        }
    }));
    const testIdProps = useTestId(testId);
    return (jsx("div", { "aria-label": ariaLabel, "aria-labelledby": ariaLabelledBy, ref: rootRef, ...actionableProps, class: actionStyles, tabIndex: isTabbable ? 0 : -1, role: "button", style: mergedStyles, ...testIdProps, children: children }));
});

export { ActionCard as A };
//# sourceMappingURL=ActionCard-f58e7cae.js.map
