/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, jsx, Fragment } from 'preact/jsx-runtime';
import { g as getEnclosingRectDims } from './labelUtils-cdd6f389.js';
import { styles } from './UNSAFE_Axis/themes/AxisStyles.css.js';
import { AxisRedwoodTheme } from './UNSAFE_Axis/themes/redwood/AxisTheme.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { g as getClientHints } from './clientHints-ee1c8473.js';

/**
 * Renders the svg text with given style and svg attributes.
 */
function SVGText({ text, style, textProps, isTitle, isInteractive, id }) {
    const browser = getClientHints().browser;
    return (jsx("text", { style: style, ...textProps, class: classNames([
            isInteractive ? styles.isInteractive : '',
            isTitle ? styles.axisTitle : styles.axisLabel,
            !isTitle && browser === 'safari' ? styles.middleBaseline : ''
        ]), id: id, children: text }));
}
/**
 * Renders the chart axis.
 */
function Axis({ title, titleStyle, titleProps, tickLabels, tickLabelStyle, focusedGroupIndex, axisPosition, activeId, isTextInteractive }) {
    const { classes } = useComponentTheme(AxisRedwoodTheme);
    return (jsxs("g", { class: classes, children: [title && jsx(SVGText, { text: title, textProps: titleProps, style: titleStyle, isTitle: true }), tickLabels.length > 0 && (jsx("g", { style: tickLabelStyle, children: tickLabels.map(({ label, labelProps, index }, renderedIndex) => {
                    const isFocused = focusedGroupIndex === renderedIndex;
                    return (jsxs(Fragment, { children: [isFocused ? (jsx("rect", { ...getEnclosingRectDims(labelProps, axisPosition), class: styles.outlineRing })) : (''), jsx(SVGText, { textProps: {
                                    ...labelProps,
                                    'data-oj-object': 'group',
                                    'data-oj-group-index': index
                                }, text: label, isInteractive: isTextInteractive?.(index), id: isFocused ? activeId : '' })] }));
                }) }))] }));
}

export { Axis as A };
//# sourceMappingURL=Axis-a6108d0d.js.map
