/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { forwardRef } from 'preact/compat';
import { u as usePress } from './usePress-6e2cde1b.js';
import { u as useHover } from './useHover-693303bd.js';
import { u as useActive } from './useActive-ac391e31.js';
import './TabbableModeContext-1e03f32d.js';
import { u as useTabbableMode } from './useTabbableMode-9c5d5a64.js';
import { u as useColorScheme } from './useColorScheme-daa151e2.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { d as dimensionInterpolations } from './dimensions-d9512d9e.js';
import { m as mergeInterpolations } from './mergeInterpolations-f71c7446.js';
import { g as getClientHints } from './clientHints-ee1c8473.js';
import { m as mergeProps } from './mergeProps-dfdba053.js';
import { multiVariantStyles } from './UNSAFE_BaseButton/themes/BaseButtonStyles.css.js';
import { u as useTestId } from './useTestId-5050f73e.js';

const interpolations = [...Object.values(dimensionInterpolations)];
const styleInterpolations = mergeInterpolations(interpolations);
const clientHints = getClientHints();
// :active only works on IOS devices if a touch handler exists
const iosProps = clientHints.platform === 'ios' ? { ontouchstart: function () { } } : {};
const isHybrid = getClientHints().hoverSupport === 'events';
const roleProps = (elType, hidden, isFocusable) => {
    switch (elType) {
        case 'link':
            return { role: 'link' };
        case 'button':
            return {};
        default:
            return hidden ? { role: 'none' } : isFocusable ? { role: 'button' } : { role: 'img' };
    }
};
const BaseButton = forwardRef(({ variant, isDisabled = false, isRepeat = false, size, elementDetails = { type: 'button', buttonType: 'button' }, styling = ['default'], edge = 'none', 'aria-label': accessibleLabel, 'aria-hidden': ariaHidden, 'aria-expanded': ariaExpanded, 'aria-roledescription': ariaRoledescription, 'aria-haspopup': ariaHasPopup, 'aria-describedby': ariaDescribedBy, testId, onBlur, onFocus, onMouseEnter, onMouseLeave, onTouchEnd, onTouchStart, onKeyDown, ...props }, ref) => {
    const pressHandler = (e) => {
        if (props.onAction) {
            props.onAction({
                reason: e.type == 'keyup' ? 'keyboard' : 'pointer'
            });
        }
    };
    const { pressProps } = usePress(pressHandler, {
        isDisabled,
        isRepeat
    });
    const isStyled = (name) => {
        return styling.indexOf(name) > -1;
    };
    const { hoverProps, isHover } = useHover({ isDisabled: !isHybrid });
    const { activeProps, isActive } = useActive();
    const { class: cls, ...styles } = styleInterpolations(props);
    const { type: elementType = 'button', isFocusable: isElementFocusable = true, buttonType = 'button', ...elementProps } = { ...elementDetails };
    const typeProps = elementType === 'button' && buttonType === 'button' ? { type: buttonType } : {};
    // 'role' should be 'none' when aria-hidden is true or 'img' when false and not focusable
    const role = roleProps(elementType, ariaHidden ?? false, isElementFocusable);
    const ElementType = elementDetails.type;
    const isSpan = ElementType == 'span';
    const isBaseButton = ElementType == 'button';
    const { tabbableModeProps } = useTabbableMode(isDisabled);
    const colorScheme = useColorScheme();
    const variantClasses = multiVariantStyles({
        size,
        variant,
        unstyled: isStyled('unstyled') ? 'isUnstyled' : 'notUnstyled',
        embedded: isStyled('embedded') ? 'isEmbedded' : 'notEmbedded',
        container: isStyled('container') ? 'isContainer' : 'notContainer',
        noBorderRadiusStart: isStyled('noBorderRadiusStart')
            ? 'isNoBorderRadiusStart'
            : 'notNoBorderRadiusStart',
        noBorderRadiusEnd: isStyled('noBorderRadiusEnd')
            ? 'isNoBorderRadiusEnd'
            : 'notNoBorderRadiusEnd',
        activeStyle: isStyled('active') ? 'isActiveStyle' : 'notActiveStyle',
        fill: isStyled('fill') ? 'isFill' : 'notFill',
        min: isStyled('min') ? 'isMin' : 'notMin',
        disabled: isDisabled ? 'isDisabled' : 'notDisabled',
        edge,
        hybrid: isHybrid ? 'isHybrid' : 'notHybrid',
        active: isActive ? 'isActive' : 'notActive',
        hover: isHover ? 'isHover' : 'notHover',
        span: isSpan ? 'isSpan' : 'notSpan',
        colorScheme
    });
    const globalEventProps = {
        onBlur,
        onFocus,
        onMouseEnter,
        onMouseLeave,
        onTouchEnd,
        onTouchStart,
        onKeyDown
    };
    const mergedProps = mergeProps(elementProps, pressProps, iosProps, hoverProps, isStyled('container') ? {} : activeProps, isBaseButton ? {} : { 'aria-disabled': isDisabled }, globalEventProps);
    const wrapperClasses = classNames([cls, variantClasses]);
    const testIdProps = useTestId(testId);
    const getTabIndex = () => {
        if (isSpan && (isElementFocusable === false || isDisabled)) {
            // no tabIndex value should be defined for a span element when disabled or non-focusable
            return { tabIndex: undefined };
        }
        return tabbableModeProps;
    };
    return (jsx(ElementType
    // The ref casting was done to satisfy TS compiler, when the forwardRef had to be updated to ForwardedRef.
    , { 
        // The ref casting was done to satisfy TS compiler, when the forwardRef had to be updated to ForwardedRef.
        ref: ref, disabled: isDisabled, class: wrapperClasses, style: styles, autofocus: props['autofocus'], title: props.title, ...getTabIndex(), "aria-label": accessibleLabel, "aria-haspopup": ariaHasPopup ? 'menu' : undefined, "aria-describedby": ariaDescribedBy, "aria-hidden": ariaHidden, "aria-roledescription": ariaRoledescription, "aria-expanded": ariaExpanded, ...typeProps, ...testIdProps, ...mergedProps, ...role, children: props.children }));
});
BaseButton.displayName = 'BaseButton';

export { BaseButton as B };
//# sourceMappingURL=BaseButton-f9e8c783.js.map
