/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, jsx } from 'preact/jsx-runtime';
import { screen, render, waitFor } from '@testing-library/preact';
import userEvent from '@testing-library/user-event';
import { expect } from 'chai';
import { T as Text } from './Text-ec0e2d47.js';
import { V as View } from './View-e580294d.js';
import { A as Avatar } from './Avatar-8a03db80.js';
import { B as Button } from './Button-70c707d8.js';

const CardContent = ({ context, showButton }) => {
    return (jsxs(View, { padding: "4x", children: [jsx(Avatar, { initials: context.data.initials, size: "xl" }), jsx("div", { style: "margin-top: 15px;", children: jsx(Text, { variant: "primary", size: "md", lineClamp: 1, children: context.data.name }) }), jsx(Text, { variant: "secondary", size: "md", lineClamp: 1, children: context.data.title }), showButton && (jsxs("div", { style: "margin-top: 5px;", children: [jsx("div", { style: "margin-right: 5px; display: inline-block;", children: jsx(Button, { variant: "outlined", size: "md", label: "A" }) }), jsx("div", { style: "display: inline-block;", children: jsx(Button, { variant: "outlined", size: "md", label: "B" }) })] })), context.selector && (jsx("div", { style: "position: relative; left: -13px", children: context.selector?.() }))] }));
};
const populateData = (count, isNumberKey) => [...Array(count)].map((_, i) => ({
    id: isNumberKey ? i : `${i}`,
    name: `Employee ${i}`,
    title: `Employee title ${i}`,
    initials: `E${i}`
}));
const getRoot = async () => {
    return screen.findByRole('grid');
};
const getItems = async (role = 'row') => {
    return screen.findAllByRole(role);
};

const testRender = ({ GenericCardGrid, ActionCardGrid, SelectionCardGrid, ContentUpdatableCardGrid }) => {
    it('initial render', async function () {
        const data = populateData(100);
        render(jsx(GenericCardGrid, { data: data }));
        const items = await getItems('gridcell');
        expect(items.length).eq(25);
    });
    it('checking key type - string key', async () => {
        const data = populateData(3);
        render(jsx(GenericCardGrid, { data: data }));
        const items = await getItems('gridcell');
        expect(items[0].dataset['ojKeyType'] === undefined).eq(true);
        expect(items[1].dataset['ojKeyType'] === undefined).eq(true);
        expect(items[2].dataset['ojKeyType'] === undefined).eq(true);
    });
    it('checking key type - numeric key', async () => {
        const data = populateData(3, true);
        render(jsx(GenericCardGrid, { data: data }));
        const items = await getItems('gridcell');
        expect(items[0].dataset['ojKeyType']).eq('number');
        expect(items[1].dataset['ojKeyType']).eq('number');
        expect(items[2].dataset['ojKeyType']).eq('number');
    });
    it('aria attributes - multiselectable and selected, selectionMode is none', async () => {
        const data = populateData(2);
        render(jsx(SelectionCardGrid, { data: data }));
        const items = await getItems('gridcell');
        const grid = await getRoot();
        expect(grid.getAttribute('aria-multiselectable')).eq(null);
        expect(items[0].getAttribute('aria-selected')).eq(null);
        expect(items[1].getAttribute('aria-selected')).eq(null);
    });
    it('aria attributes - multiselectable and selected, selectionMode is single', async () => {
        const data = populateData(2);
        render(jsx(SelectionCardGrid, { data: data, selectionMode: 'single', selectedKeys: new Set(['1']) }));
        const items = await getItems('gridcell');
        const grid = await getRoot();
        expect(grid.getAttribute('aria-multiselectable')).eq('false');
        expect(items[0].getAttribute('aria-selected')).eq('false');
        expect(items[1].getAttribute('aria-selected')).eq('true');
    });
    it('aria attributes - multiselectable and selected, selectionMode is multiple', async () => {
        const data = populateData(3);
        render(jsx(SelectionCardGrid, { data: data, selectionMode: 'multiple', selectedKeys: new Set(['1', '2']) }));
        const items = await getItems('gridcell');
        const grid = await getRoot();
        expect(grid.getAttribute('aria-multiselectable')).eq('true');
        expect(items[0].getAttribute('aria-selected')).eq('false');
        expect(items[1].getAttribute('aria-selected')).eq('true');
        expect(items[2].getAttribute('aria-selected')).eq('true');
    });
    // skip this test in jest, only run it in karma mocha
    if (typeof jest === typeof undefined) {
        // because jest doesn't show grid layout, the number of focusable elements is not calculated correctly
        it('content focusBehavior - valid case', async () => {
            const data = populateData(3);
            render(jsx(ActionCardGrid, { data: data }));
            const items = await getItems('gridcell');
            await userEvent.tab();
            await waitFor(() => {
                expect(items[0].matches(':focus-within')).to.be.true;
            });
        });
        it('content focusBehavior - invalid case', async () => {
            const data = populateData(3);
            render(jsx(ActionCardGrid, { data: data, showButton: true }));
            const grid = await getRoot();
            await userEvent.tab();
            await waitFor(() => {
                expect(grid.matches(':focus')).to.be.true;
            });
        });
    }
    // JET-61732
    it('memoization - update memoized children', async () => {
        const data = populateData(3);
        const { queryByTestId } = render(jsx(ContentUpdatableCardGrid, { data: data }));
        const elem0 = queryByTestId(data[0].id);
        const elem1 = queryByTestId(data[1].id);
        const elem2 = queryByTestId(data[2].id);
        expect(elem0?.innerHTML, 'the 1st item initial count should be 0').eq('0');
        expect(elem1?.innerHTML, 'the 2nd item initial count should be 0').eq('0');
        expect(elem2?.innerHTML, 'the 3rd item initial count should be 0').eq('0');
        const button = queryByTestId('mybutton');
        await userEvent.click(button);
        expect(elem0?.innerHTML, 'the 1st item new count should be 1').eq('1');
        expect(elem1?.innerHTML, 'the 2nd item new count should be 1').eq('1');
        expect(elem2?.innerHTML, 'the 3rd item new count should be 1').eq('1');
    });
};

export { CardContent as C, getRoot as g, populateData as p, testRender as t };
//# sourceMappingURL=BaseCardViewTest-eaaf2132.js.map
