/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { d as dimensionInterpolations } from './dimensions-d9512d9e.js';
import { p as paddingInterpolations } from './padding-532a6a4d.js';
import { c as colorInterpolations } from './colors-8cd7269e.js';
import { m as mergeInterpolations } from './mergeInterpolations-f71c7446.js';
import { a as ariaInterpolations } from './aria-20ecc4ae.js';
import { baseStyle } from './UNSAFE_Card/themes/CardStyles.css.js';
import { c as classNames } from './classNames-2e96f1d4.js';

const interpolations = [
    ...Object.values(dimensionInterpolations),
    ...Object.values(colorInterpolations),
    ...Object.values(paddingInterpolations),
    ...Object.values(ariaInterpolations)
];
const mergedInterpolations = mergeInterpolations(interpolations);
/**
 * A card is a simple container that renders a group of related information in a styled rectangular area.
 * Width, height, padding, and background color are set through props.
 * The aria role for card is normally "group" but this changes to "button" for action cards.
 */
const Card = ({ children, ...rest }) => {
    const { class: mergedClasses, ariaLabel, ariaLabelledBy, ...mergedStyles } = mergedInterpolations(rest);
    const styles = classNames([baseStyle, mergedClasses]);
    return (jsx("div", { "aria-label": ariaLabel, "aria-labelledby": ariaLabelledBy, role: "group", className: styles, style: mergedStyles, children: children }));
};

export { Card as C };
//# sourceMappingURL=Card-b25c49d5.js.map
