/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx, jsxs } from 'preact/jsx-runtime';
import { u as useTextDimensions } from './useTextDimensions-16f92049.js';
import { u as useCssVars } from './useCssVars-4fd0769c.js';
import { g as getChartVars } from './cssUtils-aabc313b.js';
import { styles } from './UNSAFE_Chart/themes/ChartStyles.css.js';
import { useRef } from 'preact/hooks';
import { G as Gridlines, P as PlotArea } from './Gridlines-960c9cb7.js';
import { u as useId } from './useId-1bde71ab.js';
import { A as Axis } from './Axis-a6108d0d.js';
import { T as TrackResizeContainer } from './TrackResizeContainer-24a9d2bc.js';
import { u as useChartViewport } from './useChartViewport-bf5f57db.js';
import { a as getAxesPosition, b as getScrollBarSpace, c as getLayoutInfo, u as updateScrollBarSpace, d as getAvailSpace } from './layoutUtils-765327c6.js';
import { u as useChartNav } from './useChartNav-a5cc7137.js';
import { m as mergeProps } from './mergeProps-dfdba053.js';
import { a as useSelectDrill, u as useChartDatatipAndCursor } from './useSelectDrill-25bce4e4.js';
import { u as useTestId } from './useTestId-5050f73e.js';
import { S as ScrollBar } from './ScrollBar-df772753.js';

function getNextGroup(info, viewportEndIndex, isGroupNavigable) {
    const groupIndex = info.groupIndex + 1;
    if (groupIndex >= viewportEndIndex) {
        return;
    }
    if (isGroupNavigable(groupIndex)) {
        return { groupIndex };
    }
    return getNextGroup({ groupIndex }, viewportEndIndex, isGroupNavigable);
}
function getPreviousGroup(info, isGroupNavigable) {
    const groupIndex = info.groupIndex - 1;
    if (groupIndex < 0) {
        return;
    }
    if (isGroupNavigable(groupIndex)) {
        return { groupIndex };
    }
    return getPreviousGroup({ groupIndex }, isGroupNavigable);
}
function getGroupNavUtil(groups, renderedGroups, xIndexExtent, drilling, isRtl) {
    const isGroupNavigable = (groupIndex) => {
        const groupsDrilling = groups[renderedGroups[groupIndex]].drilling;
        return (drilling === 'on' || drilling === 'groupsOnly') && groupsDrilling != 'off';
    };
    const getDefaultNavGroup = (info) => {
        if (xIndexExtent.startIndex > renderedGroups[info.groupIndex]) {
            info.groupIndex = 0;
            return info;
        }
        else if (xIndexExtent.endIndex < renderedGroups[info.groupIndex]) {
            info.groupIndex = renderedGroups.length - 1;
            return info;
        }
        return;
    };
    const getNextNavGroup = (info, arrow) => {
        let nextGroup = getDefaultNavGroup(info);
        if (nextGroup)
            return nextGroup;
        if (arrow === 'ArrowRight') {
            nextGroup = isRtl
                ? getPreviousGroup(info, isGroupNavigable)
                : getNextGroup(info, renderedGroups.length, isGroupNavigable);
        }
        else {
            nextGroup = isRtl
                ? getNextGroup(info, renderedGroups.length, isGroupNavigable)
                : getPreviousGroup(info, isGroupNavigable);
        }
        return nextGroup || info;
    };
    return { getDefaultNavGroup, getNextNavGroup };
}

function Chart({ width = '100%', height = '448px', selectedIds = [], hiddenIds, hideAndShowBehavior = 'none', orientation = 'vertical', xAxis, yAxis, plotArea, series, groups, isStacked = false, getDataItem, selectionMode = 'none', drilling = 'off', dataCursorPosition, onSelectionChange, isDataCursorEnabled, dataCursorStyle, onViewportChange, onDataCursorPositionChange, onItemDrill, onGroupDrill, onItemInput, datatip, zoomAndScroll = 'off', offset, getNavUtil, getDataItemPos, getGapRatio, valueFormats, findNearest, isRtl, children, testId, renderGridLinesInFront, ...props }) {
    // TODO: reevaluate how we handle empty data
    const chartVars = getChartVars();
    const testIdProps = useTestId(testId);
    return (jsx(TrackResizeContainer, { width: width, height: height, children: function _(width, height) {
            const rootRef = useRef(null);
            const isHoriz = orientation === 'horizontal';
            const { xAxisPosition, yAxisPosition } = getAxesPosition(yAxis, isHoriz, isRtl);
            const { resolvedVars, cssContent } = useCssVars(chartVars);
            const { textMeasureContent, getTextDimensions } = useTextDimensions();
            let availSpace = getAvailSpace(width, height);
            const updatedSpace = getScrollBarSpace(availSpace, zoomAndScroll, xAxisPosition);
            let scrollDims = updatedSpace.scrollDims;
            availSpace = updatedSpace.availSpace;
            const hiddenSet = new Set(hiddenIds);
            const { xProps, xScale, xViewportExtent, xAxisExtent, xIndexExtent, yViewportExtent, yAxisExtent, groupWidth, yProps, yScale, yMaxDim, yMajorTicks, yMinorTicks, plotAreaSpace } = getLayoutInfo(series, groups, getDataItem, xAxis, yAxis, availSpace, hiddenSet, hideAndShowBehavior, isHoriz, isRtl, isStacked, resolvedVars['labelFontSize'], offset, getTextDimensions);
            scrollDims = updateScrollBarSpace(scrollDims, xAxisPosition === 'bottom', yMaxDim);
            const gapRatio = getGapRatio(xIndexExtent.startIndex, xIndexExtent.endIndex);
            const getNextChartItem = getNavUtil(xIndexExtent.startIndex, xIndexExtent.endIndex);
            const renderedGroupsIndex = xProps.tickLabels.map(({ index }) => index + xIndexExtent.startIndex);
            const { getNextNavGroup, getDefaultNavGroup } = getGroupNavUtil(groups, renderedGroupsIndex, xIndexExtent, drilling, isRtl);
            const { activeId, focusedItemInfo, hoveredItemInfo, focusedGroupInfo, navProps } = useChartNav({
                rootRef,
                getNextChartItem,
                getNextNavGroup,
                getDefaultNavGroup,
                onItemInput,
                drilling,
                xIndexExtent
            });
            const selectDrillProps = useSelectDrill({
                selectionMode,
                selection: selectedIds,
                onChange: onSelectionChange,
                rootRef,
                drilling,
                onItemDrill,
                onGroupDrill,
                focusedGroupInfo,
                focusedItemInfo,
                groups,
                getDataItem
            });
            const viewportProps = useChartViewport({
                width,
                height,
                xViewportExtent,
                yViewportExtent,
                xAxisExtent,
                yAxisExtent,
                xScale,
                yScale,
                plotAreaSpace,
                isRtl,
                isHoriz,
                zoomAndScroll,
                onViewportChange
            });
            const findNearestData = findNearest(xScale, yScale, groupWidth, gapRatio);
            const getDataItemPosition = getDataItemPos(xScale, yScale, groupWidth, gapRatio);
            const { datatipContent, dataCursorContent, datatipProps } = useChartDatatipAndCursor({
                series,
                groups,
                width,
                height,
                findNearest: findNearestData,
                plotAreaSpace,
                onDataCursorPositionChange,
                isDataCursorEnabled,
                dataCursorPosition,
                dataCursorStyle,
                hoveredItemInfo,
                focusedItemInfo,
                getDataItem,
                getDataItemPos: getDataItemPosition,
                datatip,
                valueFormats,
                isHoriz,
                xScale: xScale,
                yScale
            });
            const eventsProps = mergeProps(navProps, selectDrillProps, viewportProps, datatipProps);
            const isGroupInteractive = (index) => {
                if (xAxis?.timeAxisType != null) {
                    return false;
                }
                const groupDrill = groups[index].drilling;
                if (groupDrill === 'on') {
                    return true;
                }
                if (groupDrill != 'off' && (drilling === 'on' || drilling === 'groupsOnly')) {
                    return true;
                }
                return false;
            };
            const plotAreaClipPathId = useId();
            const { viewportMin: xViewportMin, viewportMax: xViewportMax } = xViewportExtent;
            const { startIndex: xStartIndex, endIndex: xEndIndex } = xIndexExtent;
            const gridLines = (jsx(Gridlines, { plotArea: plotArea, yMinorTicks: yMinorTicks, xScale: xScale, yScale: yScale, isLog: yAxis?.scale === 'log', xViewportMin: xViewportMin, xViewportMax: xViewportMax, yTicks: yMajorTicks, yAxisPosition: yAxisPosition }));
            return (jsxs("div", { tabIndex: 0, ref: rootRef, "aria-activedescendant": activeId, "aria-label": props['aria-label'], "aria-labelledby": props['aria-labelledby'], "aria-describedby": props['aria-describedby'], style: { width, height }, role: "application", className: styles.outline, ...testIdProps, ...eventsProps, children: [jsxs("svg", { width: "100%", height: "100%", children: [jsx("clipPath", { id: `${plotAreaClipPathId}`, children: jsx("rect", { ...plotAreaSpace }) }), jsx(PlotArea, { color: plotArea?.color, yScale: yScale, isLog: yAxis?.scale === 'log', availSpace: plotAreaSpace, xAxisPosition: xAxisPosition }), !renderGridLinesInFront && gridLines, jsx(Axis, { ...yProps, axisPosition: yAxisPosition }), jsx(Axis, { ...xProps, axisPosition: xAxisPosition, isTextInteractive: isGroupInteractive, activeId: activeId, focusedGroupIndex: focusedGroupInfo.isFocusVisible ? focusedGroupInfo.groupIndex : undefined }), jsxs("g", { "clip-path": `url(#${plotAreaClipPathId})`, children: [children({
                                        xStartIndex,
                                        xEndIndex,
                                        xScale: xScale,
                                        yScale,
                                        gapRatio,
                                        activeId,
                                        focusedItemInfo,
                                        hoveredItemInfo,
                                        groupWidth
                                    }), renderGridLinesInFront && gridLines, dataCursorContent] }), zoomAndScroll === 'live' && (jsx(ScrollBar, { availSpace: scrollDims, viewport: xViewportExtent, axisExtent: xAxisExtent, isHoriz: xAxisPosition === 'bottom' }))] }), textMeasureContent, cssContent, datatipContent] }));
        } }));
}

export { Chart as C };
//# sourceMappingURL=Chart-61d61471.js.map
