/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx, jsxs } from 'preact/jsx-runtime';
import { forwardRef, useRef } from 'preact/compat';
import { u as useCheckboxRadioContext } from './CheckboxRadioContext-660f410d.js';
import { H as HiddenAccessible } from './HiddenAccessible-653c0211.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { u as useFocus } from './useFocus-1c5b1863.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import './TabbableModeContext-1e03f32d.js';
import { u as useTabbableMode } from './useTabbableMode-9c5d5a64.js';
import { u as useTranslationBundle } from './useTranslationBundle-95f6039e.js';
import './UserAssistanceStyles.styles.css';
import './UNSAFE_UserAssistance/themes/redwood/UserAssistanceBaseTheme.styles.css';
import './UNSAFE_UserAssistance/themes/redwood/UserAssistanceVariants.css.js';
import 'preact/hooks';
import './Flex-47f90e61.js';
import './MessageBannerStyles.styles.css';
import './UNSAFE_MessageBanner/themes/redwood/MessageBannerBaseTheme.styles.css';
import './UNSAFE_MessageBanner/themes/redwood/MessageBannerVariants.css.js';
import 'preact';
import { u as useId } from './useId-1bde71ab.js';
import './Floating-9ce291ba.js';
import './TopLayerHost-d50e5624.js';
import './TooltipContentStyles.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentBaseTheme.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentVariants.css.js';
import './IconStyle.styles.css';
import './IconButton-c8fafbab.js';
import './MessageStyles.styles.css';
import './MessageFormattingUtils-56752721.js';
import './logger-c710d0a3.js';
import './TransitionGroup-a51c7df7.js';
import './MessagesContext-d5efd74c.js';
import './ComponentMessageStyles.styles.css';
import './UNSAFE_ComponentMessage/themes/redwood/ComponentMessageBaseTheme.styles.css';
import './UNSAFE_ComponentMessage/themes/redwood/ComponentMessageVariants.css.js';
import './FormContext-f5e6dd2a.js';
import './FormFieldContext-dea4d4d7.js';
import './InputGroupContext-e6856c27.js';
import './Popup-05f6a710.js';
import './UNSAFE_Separator/themes/SeparatorStyles.css.js';
import { u as useFocusWithin } from './useFocusWithin-96e9acb0.js';
import './LabelStyles.styles.css';
import './UNSAFE_Label/themes/redwood/LabelBaseTheme.styles.css';
import './UNSAFE_Label/themes/redwood/LabelVariants.css.js';
import { I as IconUserAssistance } from './IconUserAssistance-92d12ea5.js';
import { S as StyledCheckbox } from './StyledCheckbox-68bddaad.js';
import { CheckboxRedwoodTheme } from './UNSAFE_Checkbox/themes/redwood/CheckboxTheme.js';
import { u as useActive } from './useActive-ac391e31.js';
import { m as merge } from './stringUtils-8521231d.js';
import { l as layoutSpanStyles } from './LayoutStyles.css-f4783df4.js';

/**
 * CheckboxControl is an internal subcomponent used by both Checkbox and CheckboxItem.
 * It handles styling, placement of label, assistive text, and accessibility text for both parent components.
 * onCommit handling is handled in the parent components.
 */
const CheckboxControl = forwardRef(({ 'aria-describedby': ariaDescribedBy, name, value, children, isChecked, isDisabled, isReadonly, isRequired, onChange, onKeyUp, assistiveText, helpSourceLink, helpSourceText, columnSpan = 1 }, ref = null) => {
    // hooks
    const id = useId();
    const childWrapperRef = useRef(null);
    const { isTabbable } = useTabbableMode();
    const { isActive, activeProps } = useActive();
    const { isFocusRingShown } = useCheckboxRadioContext();
    const { isFocus, focusProps: { onFocus, onBlur } } = useFocus();
    // styling
    const { classes: checkboxClasses, styles: { valueLabelStyle } } = useComponentTheme(CheckboxRedwoodTheme, {
        disabled: isDisabled ? 'isDisabled' : 'notDisabled',
        focused: isFocus && isFocusRingShown ? 'isFocused' : 'notFocused'
    });
    // help available
    const hasUserAssistive = !isDisabled && !isReadonly && !!(assistiveText || helpSourceLink);
    const helpAvailableId = hasUserAssistive ? `${id}-help-available` : undefined;
    const uaId = hasUserAssistive ? `${id}-icon-ua` : undefined;
    const { formControl_helpAvailable } = useTranslationBundle('@oracle/oraclejet-preact');
    const helpAvailableString = formControl_helpAvailable();
    const userAssistiveHelp = hasUserAssistive ? (jsx(HiddenAccessible, { id: helpAvailableId, children: helpAvailableString })) : undefined;
    // prevent auto scrolling
    const onKeyDown = (event) => {
        const { key } = event;
        if (!isReadonly && key !== undefined && key === ' ') {
            event.preventDefault();
        }
    };
    // we want to prevent mouseDown on Links to set active
    const onMouseDownHandler = (e) => {
        // check to see if the target is within an <a> tag and the tag is within the label
        const closestAnchor = e.target.closest('a');
        closestAnchor && childWrapperRef.current?.contains(closestAnchor)
            ? e.stopPropagation()
            : activeProps['onMouseDown']?.(e);
    };
    // don't allow key down to set active state
    delete activeProps['onKeyDown'];
    const { focusProps, isFocused: isFocusedWithin } = useFocusWithin();
    // The top level element needs the column span class on it.
    const classes = layoutSpanStyles.layoutSpanColumn[columnSpan];
    return (jsxs("label", { class: classNames([checkboxClasses, classes]), children: [jsx("span", { ...activeProps, children: jsx(StyledCheckbox, { "aria-describedby": merge([ariaDescribedBy, helpAvailableId]), name: name, value: value, onChange: onChange, onBlur: onBlur, onFocus: onFocus, onKeyUp: onKeyUp, onKeyDown: onKeyDown, isActive: isActive, isChecked: isChecked ? 'checked' : 'unchecked', isDisabled: isDisabled, isReadonly: isReadonly, isRequired: isRequired, ref: ref }) }), userAssistiveHelp, jsxs("span", { class: valueLabelStyle, children: [jsx("span", { ...activeProps, ...focusProps, onMouseDown: onMouseDownHandler, ref: childWrapperRef, children: children }), hasUserAssistive && (jsx(IconUserAssistance, { assistiveText: assistiveText, helpSourceLink: helpSourceLink, helpSourceText: helpSourceText, isTabbable: isTabbable && (isFocus || isFocusedWithin), id: uaId }))] })] }));
});

export { CheckboxControl as C };
//# sourceMappingURL=CheckboxControl-b927cd2a.js.map
