/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { createContext, toChildArray } from 'preact';
import { useContext, useMemo } from 'preact/hooks';
import { forwardRef } from 'preact/compat';
import './FormContext-f5e6dd2a.js';
import { u as useFormContext } from './useFormContext-dcae8120.js';
import { F as FormFieldContext } from './FormFieldContext-dea4d4d7.js';
import { u as useFormFieldContextProps } from './useFormFieldContextProps-a74f741a.js';
import './CheckboxSetStyles.styles.css';
import { u as useId } from './useId-1bde71ab.js';
import { C as CheckboxRadioField } from './CheckboxRadioField-36b1cffa.js';

const CheckboxSetContext = createContext({});
const useCheckboxSetContext = () => useContext(CheckboxSetContext);

var styles = {ul:'CheckboxSetStyles_styles_ul__68p6630'};

const CheckboxSet = forwardRef(({ 'aria-describedby': ariaDescribedBy, assistiveText, columnSpan, direction = 'column', helpSourceLink, helpSourceText, isDisabled: propIsDisabled, isReadonly: propIsReadonly, isRequired = false, label, labelEdge: propLabelEdge, labelStartWidth: propLabelStartWidth, messages, onCommit, userAssistanceDensity: propUserAssistanceDensity, value, children }, ref) => {
    const hasValue = value !== undefined ? value.size > 0 : false;
    const baseId = useId();
    const { isDisabled: isFormDisabled, isReadonly: isFormReadonly, labelEdge: formLabelEdge, labelStartWidth: formLabelStartWidth, userAssistanceDensity: formUserAssistanceDensity } = useFormContext();
    // default to FormContext values if component properties are not specified
    const isDisabled = propIsDisabled ?? isFormDisabled;
    const isReadonly = propIsReadonly ?? isFormReadonly;
    const labelEdge = propLabelEdge ?? formLabelEdge;
    const labelStartWidth = propLabelStartWidth ?? formLabelStartWidth;
    const userAssistanceDensity = propUserAssistanceDensity ?? formUserAssistanceDensity;
    const formFieldContext = useFormFieldContextProps({
        hasValue,
        isDisabled,
        isReadonly
    });
    const checkboxSetContext = useMemo(() => ({
        name: baseId,
        value,
        onCommit
    }), [baseId, onCommit, value]);
    // wrap each CheckboxItem in an <li> tag and the entire set with <ul>
    const childrenArray = toChildArray(children).filter(Boolean);
    const wrappedChildren = childrenArray.map((item) => jsx("li", { children: item }));
    return (jsx(FormFieldContext.Provider, { value: formFieldContext, children: jsx(CheckboxRadioField, { "aria-describedby": ariaDescribedBy, assistiveText: assistiveText, baseId: baseId, columnSpan: columnSpan, direction: direction, helpSourceLink: helpSourceLink, helpSourceText: helpSourceText, isDisabled: isDisabled, isReadonly: isReadonly, isRequired: isRequired, label: label, labelEdge: labelEdge, labelStartWidth: labelStartWidth, parentComponentVariant: "checkboxSet", messages: messages, userAssistanceDensity: userAssistanceDensity, hasValue: hasValue, role: "group", ref: ref, children: jsx(CheckboxSetContext.Provider, { value: checkboxSetContext, children: jsx("ul", { className: styles.ul, children: wrappedChildren }) }) }) }));
});

export { CheckboxSet as C, CheckboxSetContext as a, useCheckboxSetContext as u };
//# sourceMappingURL=CheckboxSet-da1fbfd5.js.map
