/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { u as useActionable } from './useActionable-e45f7161.js';
import { useCallback } from 'preact/hooks';
import './TabbableModeContext-1e03f32d.js';
import { u as useTabbableMode } from './useTabbableMode-9c5d5a64.js';
import { multiVariantStyles } from './UNSAFE_Chip/themes/ChipStyles.css.js';
import { u as useInteractionStyle } from './useInteractionStyle-43b76a9a.js';
import { m as mergeProps } from './mergeProps-dfdba053.js';
import { u as useTestId } from './useTestId-5050f73e.js';

function Chip({ isSelected, isDisabled, 'aria-label': accessibleLabel, children, testId, onToggle }) {
    const handleAction = useCallback(() => {
        onToggle?.({ previousValue: isSelected, value: !isSelected });
    }, [isSelected, onToggle]);
    const { actionableProps } = useActionable(handleAction);
    const { interactionProps, applyActiveStyle, applyHoverStyle, applyPseudoHoverStyle } = useInteractionStyle();
    const mergedProps = mergeProps(actionableProps, interactionProps);
    const classes = multiVariantStyles({
        disabled: isDisabled ? 'isDisabled' : 'notDisabled',
        hover: applyHoverStyle && !isDisabled && !isSelected ? 'isHover' : 'notHover',
        pseudoHover: applyPseudoHoverStyle && !isDisabled && !isSelected ? 'isPseudoHover' : 'notPseudoHover',
        active: isSelected || applyActiveStyle ? 'isActive' : 'notActive'
    });
    const { isTabbable } = useTabbableMode();
    const testIdProps = useTestId(testId);
    return isDisabled ? (jsx("div", { class: classes, ...testIdProps, children: children })) : (jsx("div", { ...mergedProps, class: classes, ...testIdProps, tabIndex: isTabbable ? 0 : -1, "aria-label": accessibleLabel, role: "switch", "aria-checked": isSelected ? 'true' : 'false', children: children }));
}

export { Chip as C };
//# sourceMappingURL=Chip-6212be77.js.map
