/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx, jsxs } from 'preact/jsx-runtime';
import { B as ButtonLayout } from './ButtonLayout-af96cb96.js';
import { S as SvgChevronDown } from './ChevronDown-132b6bbf.js';
import { M as Menu } from './Menu-98442964.js';
import './MenuItem-b7f51744.js';
import { useRef, useImperativeHandle } from 'preact/hooks';
import { Fragment } from 'preact';
import { u as useTooltip } from './useTooltip-cb4418ec.js';
import './Floating-9ce291ba.js';
import { forwardRef } from 'preact/compat';
import './TopLayerHost-d50e5624.js';
import { m as mergeProps } from './mergeProps-dfdba053.js';
import './logger-c710d0a3.js';
import './TooltipContentStyles.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentBaseTheme.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentVariants.css.js';
import './IconStyle.styles.css';
import './SelectMenuGroupContext-b0fd7f84.js';
import './UNSAFE_Separator/themes/SeparatorStyles.css.js';
import './MenuSeparatorStyles.styles.css';
import './UNSAFE_Menu/themes/redwood/MenuSeparatorBaseTheme.styles.css';
import './UNSAFE_Menu/themes/redwood/MenuSeparatorVariants.css.js';
import { u as useMenuAction } from './useMenuAction-fd387386.js';
import { B as BaseButton } from './BaseButton-f9e8c783.js';

/**
 * A IconMenuButton is a combined action and menu button.
 */
const IconMenuButton = forwardRef(({ children, tooltip, accessibleLabel, variant = 'outlined', isDisabled = false, isMenuOpen = false, isIconOnly = false, onToggleMenu, size = 'md', type: buttonType = 'button', testId, width, icon, 'aria-describedby': ariaDescribedBy }, ref) => {
    const buttonRef = useRef(null);
    useImperativeHandle(ref, () => buttonRef.current, [buttonRef]);
    const { triggerProps, menuProps } = useMenuAction({
        isDisabled: isDisabled,
        onToggleMenu: onToggleMenu,
        isMenuOpen: isMenuOpen,
        anchorRef: buttonRef
    });
    const { tooltipContent, tooltipProps } = useTooltip({
        text: tooltip ? tooltip : accessibleLabel,
        isDisabled: isMenuOpen || isDisabled
    });
    const elementDetails = {
        type: 'button',
        buttonType: buttonType
    };
    const rootButton = (jsx(Fragment, { children: jsxs(BaseButton, { ...mergeProps(tooltipProps, triggerProps), styling: isMenuOpen ? ['min', 'active'] : ['min'], elementDetails: elementDetails, isDisabled: isDisabled, variant: variant, width: width, size: size, ref: buttonRef, "aria-describedby": ariaDescribedBy, "aria-label": accessibleLabel ?? tooltip, testId: testId, children: [jsx(ButtonLayout, { display: 'icons', startIcon: icon, endIcon: isIconOnly ? null : jsx(SvgChevronDown, {}), size: size }), tooltipContent] }) }));
    /*TODO: Remove this hardcoded offsetValue so we have dropdown offset only set on one single place.
     Could be done via menu props or having a dropdownMenu component. JET-60595*/
    return isDisabled ? (rootButton) : (jsxs(Fragment, { children: [rootButton, jsx(Menu, { ...menuProps, offsetValue: 4, "aria-label": accessibleLabel ?? tooltip, children: children })] }));
});
IconMenuButton.displayName = 'IconMenuButton';

export { IconMenuButton as I };
//# sourceMappingURL=IconMenuButton-e3cc47f9.js.map
