/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, Fragment, jsx } from 'preact/jsx-runtime';
import { useRef, useCallback } from 'preact/hooks';
import { u as useTranslationBundle } from './useTranslationBundle-95f6039e.js';
import { u as usePress } from './usePress-6e2cde1b.js';
import { S as SvgHelp } from './Help-2f0be11a.js';
import { P as Popup } from './Popup-05f6a710.js';
import { H as HiddenAccessible } from './HiddenAccessible-653c0211.js';
import { I as InlineHelp } from './InlineHelp-c2b08216.js';
import { iconUserAssistancePopupFocus, iconUserAssistanceIcon } from './UNSAFE_UserAssistance/themes/UserAssistanceStyles.css.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { PopupRedwoodTheme } from './UNSAFE_Popup/themes/redwood/PopupTheme.js';
import { u as useToggle } from './useToggle-7589399a.js';

/**
 * Created to match icon user assistance spec for use with radio option, pending design review
 * of inline UA assistance.
 */
const IconUserAssistance = ({ assistiveText, helpSourceLink, helpSourceText, isTabbable = true, id }) => {
    const { bool: isOpen, setFalse: setIsOpenFalse, toggle } = useToggle(false);
    const translations = useTranslationBundle('@oracle/oraclejet-preact');
    const userAssistance_learnMoreStr = translations.userAssistance_learnMore();
    const sourceText = helpSourceLink && (helpSourceText || userAssistance_learnMoreStr);
    const helpForRadioString = translations.radio_helpForRadio();
    const helpStringId = `${id}-help-string`;
    const anchorRef = useRef(null);
    const handleOutsideClick = useCallback((event) => {
        const target = event?.target;
        // We need to filter out anchor click events, because anchor is responsible for the toggling
        if ((target instanceof Element || target instanceof Document) &&
            !anchorRef.current?.contains(target) &&
            isOpen) {
            setIsOpenFalse();
        }
    }, [isOpen, setIsOpenFalse]);
    const handleOnKeyDown = useCallback((e) => {
        // allow user to close popup without having to focus it
        if (['Tab', 'Escape'].includes(e.key) && isOpen) {
            setIsOpenFalse();
        }
        // prevent scroll jump
        if (e.code === 'Space') {
            e.preventDefault();
        }
    }, [isOpen, setIsOpenFalse]);
    const { pressProps } = usePress((e) => {
        e.preventDefault();
        toggle();
    });
    const handleLinkEvent = useCallback((e) => {
        // if it's a key event that is not "enter", return early
        const { key } = e;
        if (key && key !== 'Enter') {
            return;
        }
        // We want to close the pop up if a link is triggered with mouse or keyboard
        const { tagName } = e.target;
        if (tagName?.toLowerCase() === 'a') {
            setIsOpenFalse();
            // we want to refocus to the help icon after the browser has launched the link
            // prematurely doing so prevents keyboard events from firing
            setTimeout(() => anchorRef.current?.focus());
        }
    }, [setIsOpenFalse]);
    // We are referencing popup vars, so we need to bring in the baseTheme for popup
    const { baseTheme: popupBaseTheme } = useComponentTheme(PopupRedwoodTheme);
    const iconUserAssistanceClasses = classNames([popupBaseTheme, iconUserAssistancePopupFocus]);
    return helpSourceLink || assistiveText ? (jsxs(Fragment, { children: [jsx("span", { ...pressProps, role: "link", "aria-describedby": helpStringId, "aria-haspopup": "dialog", ref: anchorRef, tabIndex: isTabbable ? 0 : -1, class: iconUserAssistanceIcon, onKeyDown: handleOnKeyDown, id: `${id}-help-icon`, children: jsx(SvgHelp, {}) }), jsx(HiddenAccessible, { id: helpStringId, isHidden: true, children: helpForRadioString }), jsx(Popup, { placement: "top", isOpen: isOpen, anchorRef: anchorRef, onClose: setIsOpenFalse, onClickOutside: handleOutsideClick, shiftOptions: { mainAxis: true, crossAxis: false }, children: jsx("div", { id: id, tabIndex: 0, class: iconUserAssistanceClasses, onMouseUp: handleLinkEvent, onKeyDown: handleLinkEvent, children: jsx(InlineHelp, { assistiveText: assistiveText, sourceLink: helpSourceLink, sourceText: sourceText }) }) })] })) : null;
};

export { IconUserAssistance as I };
//# sourceMappingURL=IconUserAssistance-92d12ea5.js.map
