/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { forwardRef } from 'preact/compat';
import { useCallback, useRef } from 'preact/hooks';
import { u as useFocusableTextField } from './useFocusableTextField-6f0e7632.js';
import './FormContext-f5e6dd2a.js';
import { u as useFormContext } from './useFormContext-dcae8120.js';
import { F as FormFieldContext } from './FormFieldContext-dea4d4d7.js';
import { u as useTextField } from './useTextField-852e3d17.js';
import { u as useHover } from './useHover-693303bd.js';
import { L as Label } from './Label-0d816f6b.js';
import { R as ReadonlyTextFieldInput } from './ReadonlyTextFieldInput-f97c62c7.js';
import { R as ReadonlyTextField } from './StyledTextField-6dcd9ff0.js';
import { T as TextField } from './TextField-f0a081f4.js';
import { T as TextFieldInput } from './TextFieldInput-e919c820.js';
import 'preact';
import './TopLayerHost-d50e5624.js';
import './LiveRegionStyles.styles.css';
import './logger-c710d0a3.js';
import './TabbableModeContext-1e03f32d.js';
import './UserAssistanceStyles.styles.css';
import './UNSAFE_UserAssistance/themes/redwood/UserAssistanceBaseTheme.styles.css';
import './UNSAFE_UserAssistance/themes/redwood/UserAssistanceVariants.css.js';
import { a as InlineUserAssistance } from './InlineUserAssistance-a843bead.js';
import { C as CompactUserAssistance } from './CompactUserAssistance-dc6b9c29.js';
import './Floating-9ce291ba.js';
import { u as useToggle } from './useToggle-7589399a.js';
import './TooltipContentStyles.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentBaseTheme.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentVariants.css.js';
import './IconStyle.styles.css';
import './Popup-05f6a710.js';
import './LabelStyles.styles.css';
import './UNSAFE_Label/themes/redwood/LabelBaseTheme.styles.css';
import './UNSAFE_Label/themes/redwood/LabelVariants.css.js';
import './InputGroupContext-e6856c27.js';
import './HiddenAccessibleStyles.styles.css';
import './UNSAFE_Popup/themes/redwood/PopupBaseTheme.styles.css';
import './UNSAFE_Popup/themes/redwood/PopupVariants.css.js';
import { u as useTranslationBundle } from './useTranslationBundle-95f6039e.js';
import { u as usePress } from './usePress-6e2cde1b.js';
import { S as SvgViewHide } from './ViewHide-bb60c820.js';
import { S as SvgView } from './View-5f6257d9.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { ButtonLayoutRedwoodTheme } from './UNSAFE_ButtonLayout/themes/redwood/ButtonLayoutTheme.js';
import './RevealToggleIconStyles.styles.css';
import { u as useClearIcon } from './useClearIcon-f24dd3af.js';
import { b as beforeVNode } from './componentUtils-6b214dc3.js';
import { C as ClearIcon } from './ClearIcon-b9ef3537.js';
import { u as useCurrentValueReducer } from './useCurrentValueReducer-4e0f7127.js';

var revealToggleIconBaseStyles = 'RevealToggleIconStyles_revealToggleIconBaseStyles__ghwny10';

function RevealToggleIcon({ isRevealed, onPress }) {
    const { pressProps } = usePress(onPress);
    const translations = useTranslationBundle('@oracle/oraclejet-preact');
    // title attribute. We removed these because title is taking precedence over aria-label and this
    // is messing up the screen readers. To be accessible we are removing title and will implement
    // this as a <Tooltip> component when we have one.
    // TODO JET-51517 implement reveal icon title as a Tooltip
    // const hidePasswordStr = translations.inputPassword_hide();
    // const showPasswordStr = translations.inputPassword_show();
    const passwordMaskedStr = translations.inputPassword_hidden();
    // Because the RevealToggleIconStyles.css.ts uses buttonLayoutBaseTheme styles, we
    // need their vars to be loaded as well. This is why we are adding the buttonLayoutBaseTheme
    // style class.
    const { baseTheme: buttonLayoutBaseTheme } = useComponentTheme(ButtonLayoutRedwoodTheme);
    const styleClasses = classNames([buttonLayoutBaseTheme, revealToggleIconBaseStyles]);
    // TODO: Use toggle <Button> when that is available, JET-49207
    return (jsx("button", { "aria-label": passwordMaskedStr, role: "switch", "aria-checked": isRevealed ? false : true, class: styleClasses, tabIndex: 0, ...pressProps, children: isRevealed ? jsx(SvgViewHide, {}) : jsx(SvgView, {}) }));
}

// One way InputPassword differs from InputText is InputPassword's readonly
// is implemented with an <input> and not a <div>.
const InputPassword = forwardRef(({ 'aria-describedby': ariaDescribedBy, assistiveText, autoComplete = 'off', autoFocus = false, columnSpan, hasClearIcon, hasRevealToggle = 'always', helpSourceLink, helpSourceText, isDisabled: propIsDisabled, isReadonly: propIsReadonly, isRequired = false, isRequiredShown, label, labelEdge: propLabelEdge, labelStartWidth: propLabelStartWidth, messages, placeholder, textAlign: propTextAlign, userAssistanceDensity: propUserAssistanceDensity, value, variant = 'default', onInput, onCommit, testId }, ref) => {
    const { currentCommitValue, dispatch } = useCurrentValueReducer({ value });
    const onInputAndDispatch = useCallback((detail) => {
        // Should dispatch happen first? This will queue up a re-render, ordering should not cause issues (this is async)
        dispatch({ type: 'input', payload: detail.value });
        onInput?.(detail);
    }, [onInput, dispatch]);
    const onCommitAndDispatch = useCallback((detail) => {
        // Should dispatch happen first? This will queue up a re-render, ordering should not cause issues (this is async)
        dispatch({ type: 'commit', payload: detail.value });
        onCommit?.(detail);
    }, [onCommit, dispatch]);
    const { isDisabled: isFormDisabled, isReadonly: isFormReadonly, labelEdge: formLabelEdge, labelStartWidth: formLabelStartWidth, textAlign: formTextAlign, userAssistanceDensity: formUserAssistanceDensity } = useFormContext();
    // default to FormContext values if component properties are not specified
    const isDisabled = propIsDisabled ?? isFormDisabled;
    const isReadonly = propIsReadonly ?? isFormReadonly;
    const labelEdge = propLabelEdge ?? formLabelEdge;
    const labelStartWidth = propLabelStartWidth ?? formLabelStartWidth;
    const textAlign = propTextAlign ?? formTextAlign;
    const userAssistanceDensity = propUserAssistanceDensity ?? formUserAssistanceDensity;
    const { bool: isRevealed, setFalse: setRevealedFalse, setTrue: setRevealedTrue } = useToggle(false);
    const { enabledElementRef, focusProps, isFocused, readonlyElementRef } = useFocusableTextField({ isDisabled, isReadonly, ref, onBlurWithin: setRevealedFalse });
    const { hoverProps, isHover } = useHover({ isDisabled: isReadonly || isDisabled || false });
    const { formFieldContext, inputProps, labelProps, textFieldProps, userAssistanceProps } = useTextField({
        ariaDescribedBy,
        helpSourceLink,
        helpSourceText,
        isDisabled,
        isFocused,
        isReadonly,
        isRequiredShown,
        labelEdge,
        messages,
        styleVariant: variant,
        userAssistanceDensity,
        value
    });
    // Callback on the RevealToggleButton.
    // When the user clicks on the reveal buttton,
    // toggle revealing/masking password
    const onRevealIconPress = useCallback(() => {
        isRevealed ? setRevealedFalse() : setRevealedTrue();
    }, [isRevealed, setRevealedFalse, setRevealedTrue]);
    const revealToggleIcon = !isDisabled && hasRevealToggle === 'always' ? (jsx(RevealToggleIcon, { onPress: onRevealIconPress, isRevealed: isRevealed })) : null;
    const onClickClearIcon = useCallback(() => {
        // Clicking the clear icon should put the focus on the input field
        enabledElementRef.current?.focus();
        // Send an event to clear the field's value
        onInputAndDispatch?.({ previousValue: value, value: '' });
    }, [onInputAndDispatch, value, enabledElementRef]);
    const maybeClearIcon = useClearIcon({
        clearIcon: jsx(ClearIcon, { onClick: onClickClearIcon }),
        display: hasClearIcon,
        hasValue: formFieldContext.hasValue,
        isFocused,
        isEnabled: !isReadonly && !isDisabled,
        isHover
    });
    const endContentCombined = beforeVNode(revealToggleIcon, maybeClearIcon);
    const labelComp = labelEdge !== 'none' ? jsx(Label, { ...labelProps, children: label }) : undefined;
    const fieldLabelProps = {
        label: labelEdge !== 'none' ? labelComp : undefined,
        labelEdge: labelEdge !== 'none' ? labelEdge : undefined,
        labelStartWidth: labelEdge !== 'none' ? labelStartWidth : undefined
    };
    const ariaLabel = labelEdge === 'none' ? label : undefined;
    const isInlineDensity = userAssistanceDensity === 'efficient' || userAssistanceDensity === 'reflow';
    const inlineUserAssistance = isInlineDensity ? (isDisabled || isReadonly ? (
    // save space for user assistance if density is 'efficient', even though we don't
    // render user assistance for disabled or readonly fields
    userAssistanceDensity !== 'efficient' ? undefined : (jsx(InlineUserAssistance, { userAssistanceDensity: userAssistanceDensity, ...userAssistanceProps }))) : (jsx(InlineUserAssistance, { assistiveText: assistiveText, helpSourceLink: helpSourceLink, helpSourceText: helpSourceText, messages: messages, isRequiredShown: isRequiredShown, userAssistanceDensity: userAssistanceDensity, ...userAssistanceProps }))) : undefined;
    const anchorRef = useRef(null);
    const compactUserAssistance = userAssistanceDensity === 'compact' ? (jsx(CompactUserAssistance, { anchorRef: anchorRef, messages: messages, assistiveText: assistiveText, ...userAssistanceProps })) : undefined;
    if (isReadonly) {
        return (jsx(FormFieldContext.Provider, { value: formFieldContext, children: jsx(ReadonlyTextField, { role: "presentation", columnSpan: columnSpan, compactUserAssistance: compactUserAssistance, inlineUserAssistance: inlineUserAssistance, ref: anchorRef, testId: testId, ...fieldLabelProps, children: jsx(ReadonlyTextFieldInput, { "aria-describedby": inputProps['aria-describedby'], "aria-label": ariaLabel, "aria-labelledby": labelProps.id, as: "input", autoFocus: autoFocus, elementRef: readonlyElementRef, textAlign: textAlign, type: "password", value: value, hasEmptyLabel: label === '' && labelEdge === 'none', hasInsideLabel: label !== undefined && labelEdge === 'inside' }) }) }));
    }
    const mainContent = (jsx(TextFieldInput, { "aria-label": ariaLabel, autoComplete: autoComplete, autoFocus: autoFocus, currentCommitValue: currentCommitValue, hasEmptyLabel: label === '' && labelEdge === 'none', hasInsideLabel: labelComp !== undefined && labelEdge === 'inside', inputRef: enabledElementRef, isRequired: isRequired, onInput: onInputAndDispatch, onCommit: onCommitAndDispatch, placeholder: placeholder, textAlign: textAlign, value: value, type: isRevealed ? 'text' : 'password', ...inputProps }));
    return (jsx(FormFieldContext.Provider, { value: formFieldContext, children: jsx(TextField, { columnSpan: columnSpan, endContent: endContentCombined, inlineUserAssistance: inlineUserAssistance, compactUserAssistance: compactUserAssistance, mainContent: mainContent, onBlur: focusProps.onfocusout, onFocus: focusProps.onfocusin, mainFieldRef: anchorRef, testId: testId, ...textFieldProps, ...fieldLabelProps, ...hoverProps }) }));
});

export { InputPassword as I };
//# sourceMappingURL=InputPassword-4f7c25ba.js.map
