/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, jsx } from 'preact/jsx-runtime';
import { useRef } from 'preact/hooks';
import { u as useLegendLayout, a as useEvents, b as useLegendTooltip, B as BaseLegend } from './useLegendLayout-16fdac28.js';
import { l as legendVars, a as getLegendTextStyle, b as getDatatipText, c as getAriaProps, d as getGridNums } from './LegendUtils-5b720c38.js';
import { m as mergeProps } from './mergeProps-dfdba053.js';
import { u as useItemFocus } from './useItemFocus-b0b6bc51.js';
import { styles } from './UNSAFE_Legend/themes/LegendStyles.css.js';
import { u as useCssVars } from './useCssVars-4fd0769c.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { u as useTestId } from './useTestId-5050f73e.js';

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
function getItem(itemInfo, items) {
    const { itemIdx } = itemInfo;
    if (itemIdx == -1) {
        return items[items.length - 1];
    }
    return items[itemIdx];
}
function getNextItemInfo(itemInfo, items) {
    let itemIdx = itemInfo.itemIdx;
    if (itemInfo.itemIdx == -1) {
        itemIdx = items.length - 1;
    }
    itemIdx = Math.min(items.length - 1, itemIdx + 1);
    return { sectionIdx: 0, itemIdx };
}
function getPrevItemInfo(itemInfo, items) {
    let itemIdx = itemInfo.itemIdx;
    if (itemInfo.itemIdx == -1) {
        itemIdx = items.length - 1;
    }
    itemIdx = Math.max(0, itemIdx - 1);
    return { sectionIdx: 0, itemIdx };
}
/**
 * Returns utils functions for navigation in simple legend.
 * @param items
 * @returns
 */
function getLegendNavUtils(items) {
    return {
        getItem: (itemInfo) => {
            return getItem(itemInfo, items);
        },
        getDetailFromInfo: (itemInfo) => {
            if (!itemInfo) {
                return { itemId: undefined };
            }
            const item = getItem(itemInfo, items);
            return { itemId: item.id };
        },
        getPrevItemInfo: (itemInfo) => {
            return getPrevItemInfo(itemInfo, items);
        },
        getNextItemInfo: (itemInfo) => {
            return getNextItemInfo(itemInfo, items);
        }
    };
}

/**
 * A legend displays an interactive description of symbols, colors, etc., used in graphical information representations.
 */
function Legend({ orientation = 'horizontal', items, isReadOnly = true, onItemAction, onInput, testId, ...props }) {
    const testIdProps = useTestId(testId);
    const rootRef = useRef(null);
    const rootDimsRef = useRef();
    const { getItem, getDetailFromInfo, getPrevItemInfo, getNextItemInfo } = getLegendNavUtils(items);
    const { resolvedVars, cssContent } = useCssVars(legendVars);
    const legendFontStylesObj = getLegendTextStyle({
        fontStyle: props.textFontStyle,
        fontSize: props.textFontSize?.toString(),
        fontWeight: props.textFontWeight?.toString(),
        fontFamily: props.textFontFamily
    }, resolvedVars);
    const { textMeasureContent, layoutInfo } = useLegendLayout({
        resolvedVars,
        items,
        rootRef,
        rootDimsRef,
        orientation,
        legendFontStylesObj,
        symbolWidth: props.symbolWidth
    });
    const { hoveredItemInfo, focusedItemInfo, ...eventsProps } = useEvents(!isReadOnly, props.hiddenIds != null, getDetailFromInfo, getPrevItemInfo, getNextItemInfo, onItemAction, onInput);
    const { focusedItemRef } = useItemFocus(focusedItemInfo);
    const datatipText = getDatatipText(items, getItem, focusedItemInfo, hoveredItemInfo);
    const { tooltipContent, tooltipProps } = useLegendTooltip({
        text: datatipText,
        rootDimsRef,
        focusedItemInfo,
        focusedItemRef
    });
    const ariaProps = getAriaProps(isReadOnly, props['aria-label'], props.hiddenIds, props['aria-describedby'], props['aria-labelledby']);
    const mergedProps = mergeProps(eventsProps, tooltipProps);
    const { itemsPerCol, itemsPerRow } = getGridNums(items.length, layoutInfo);
    const { legendStyle, renderedLegendStyle, preRenderedLegendStyle } = styles;
    const canRender = orientation === 'vertical' || !!layoutInfo;
    return (jsxs("div", { ref: rootRef, tabIndex: isReadOnly ? undefined : 0, role: "application", class: classNames([legendStyle, canRender ? renderedLegendStyle : preRenderedLegendStyle]), ...testIdProps, ...ariaProps, ...mergedProps, children: [canRender ? (jsx(BaseLegend, { ...props, focusedItemRef: focusedItemRef, items: items, isReadOnly: isReadOnly, orientation: orientation, sectionIdx: 0, itemsPerRow: itemsPerRow, itemsPerCol: itemsPerCol, itemWidth: layoutInfo?.isWrapped ? layoutInfo?.maxItemWidth : undefined, focusedItemInfo: focusedItemInfo, hoveredItemInfo: hoveredItemInfo, activeId: eventsProps['aria-activedescendant'] })) : null, tooltipContent, textMeasureContent, cssContent] }));
}

export { Legend as L };
//# sourceMappingURL=Legend-80348145.js.map
