/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx, jsxs } from 'preact/jsx-runtime';
import { G as Grid } from './Grid-ffd44650.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { styles } from './UNSAFE_ListItemLayout/themes/ListItemLayoutStyles.css.js';
import { T as Text } from './Text-ec0e2d47.js';
import './TabbableModeContext-1e03f32d.js';
import { u as useTabbableMode } from './useTabbableMode-9c5d5a64.js';

// TODO: revisit idea of negative margin when selector has a proper API for handling spacing
// Jira: JET-54543
function ListItemLayout({ children, selector, leading, primary, secondary, tertiary, overline, metadata, trailing, action, quaternary, navigation }) {
    // generate grid skeleton, adding cells based on which props are defined
    const { columnsTemplate, firstCell, secondCell, lastCell, hasSecondRow } = getColumnsTemplate(selector, leading, metadata, trailing, action, quaternary, navigation);
    // if ListItemLayout component has a child (ListItemTextLayout) then render that instead of individual props
    const textArea = children ? (jsx("div", { className: styles.childrenStyles, children: children })) : (jsxs("div", { className: styles.flexColumnStyles, children: [jsx("div", { children: overline }), jsx("div", { children: primary }), jsx("div", { children: secondary }), jsx("div", { children: tertiary })] }));
    // a second row is needed when quaternary or navigation are defined
    const secondRow = hasSecondRow
        ? [
            firstCell && jsx("span", { children: " " }),
            secondCell && jsx("span", { children: " " }),
            jsx("div", { className: styles.rowAlignStyles, children: quaternary }),
            lastCell && jsx("div", { className: styles.endJustifyStyles, children: navigation })
        ]
        : null;
    return (jsxs(Grid, { gap: "4x", gridTemplateColumns: columnsTemplate, children: [firstCell, secondCell, jsx("div", { className: styles.centerVerticalStyles, children: textArea }), lastCell, secondRow] }));
}
function getColumnsTemplate(selector, leading, metadata, trailing, action, quaternary, navigation) {
    const columnsTemplateBuilder = [];
    let firstCell = null;
    if (selector) {
        columnsTemplateBuilder.push('auto');
        firstCell = jsx("div", { className: styles.selectorStyles, children: selector });
    }
    let secondCell = null;
    if (leading) {
        columnsTemplateBuilder.push('auto');
        secondCell = jsx("div", { className: styles.centerVerticalStyles, children: leading });
    }
    columnsTemplateBuilder.push('1fr'); // Text area always gets 1 full fraction of allotted space
    let lastCell = null;
    if (metadata || trailing || action || navigation) {
        columnsTemplateBuilder.push('auto');
        lastCell = (jsxs("div", { className: styles.flexRowStyles, children: [jsx("div", { className: classNames([
                        styles.centerVerticalStyles,
                        metadata ? styles.leftPaddingStyles : ''
                    ]), children: metadata }), jsx("div", { className: classNames([
                        styles.centerVerticalStyles,
                        trailing ? styles.leftPaddingStyles : ''
                    ]), children: trailing }), jsx("div", { className: classNames([
                        styles.centerVerticalStyles,
                        action ? styles.leftPaddingStyles : ''
                    ]), children: action })] }));
    }
    const columnsTemplate = columnsTemplateBuilder.join(' ');
    const hasSecondRow = quaternary || navigation ? true : false;
    return { columnsTemplate, firstCell, secondCell, lastCell, hasSecondRow };
}

const textVariants = {
    primary: 'primary',
    secondary: 'primary',
    tertiary: 'secondary',
    quaternary: 'secondary',
    overline: 'secondary',
    metadata: 'secondary'
};
const textSizes = {
    primary: 'md',
    secondary: 'sm',
    tertiary: 'xs',
    quaternary: 'sm',
    overline: 'xs',
    metadata: 'sm'
};
const textLineClamps = {
    primary: undefined,
    secondary: 2,
    tertiary: 2,
    quaternary: 2,
    overline: 1,
    metadata: 1
};
function ListItemText({ children, variant }) {
    const { isTabbable } = useTabbableMode();
    const textContent = (jsx(Text, { variant: textVariants[variant], size: textSizes[variant], lineClamp: textLineClamps[variant], children: children }));
    return isTabbable ? jsx("div", { tabIndex: 0, children: textContent }) : textContent;
}

function ListItemTextLayout({ primary, secondary, tertiary, overline }) {
    return (jsxs("div", { className: styles.flexColumnStyles, children: [jsx(ListItemText, { variant: "overline", children: overline }), jsx(ListItemText, { variant: "primary", children: primary }), jsx(ListItemText, { variant: "secondary", children: secondary }), jsx(ListItemText, { variant: "tertiary", children: tertiary })] }));
}

export { ListItemLayout as L, ListItemText as a, ListItemTextLayout as b };
//# sourceMappingURL=ListItemTextLayout-2930aed8.js.map
