/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { Fragment } from 'preact';
import { c as classNames } from './classNames-2e96f1d4.js';
import { messageDetailStyles } from './PRIVATE_Message/themes/MessageStyles.css.js';
import { i as isValidValueForProp } from './MessageFormattingUtils-56752721.js';

/**
 * Default renderer for rendering the detail content.
 *
 * @param item The data item object
 * @returns Rendered detail content
 */
function defaultDetailRenderer(item) {
    const { detail } = item.data;
    // If the detail is null or an empty string, do not render the
    // content row
    if (!isValidValueForProp(detail)) {
        return null;
    }
    return jsx(Fragment, { children: detail });
}
/**
 * Detail Component for rendering the detail content of the Message
 */
function MessageDetail({ item, renderer, variant = 'banner' }) {
    const isCustomRendered = renderer != null;
    const renderedContent = (renderer ?? defaultDetailRenderer)(item);
    if (renderedContent == null)
        return null;
    // If detail content is rendered, then wrap it in a div with specified style classes
    const classes = classNames([messageDetailStyles.base, messageDetailStyles[variant]]);
    return (jsx("div", { class: classes, ...(isCustomRendered ? { 'data-oj-message-custom-detail': '' } : {}), children: renderedContent }));
}

export { MessageDetail as M };
//# sourceMappingURL=MessageDetail-705c7312.js.map
