/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { c as classNames } from './classNames-2e96f1d4.js';
import { S as SvgSuccessS } from './SuccessS-fd0eaab1.js';
import { S as SvgErrorS } from './ErrorS-a4c64c74.js';
import { S as SvgInformationS } from './InformationS-219d8de8.js';
import { S as SvgWarningS } from './WarningS-542fff95.js';
import { F as Flex } from './Flex-47f90e61.js';
import { startIconContainerStyles, messageStartIconStyles, severityIconStyles } from './PRIVATE_Message/themes/MessageStyles.css.js';
import { u as useTranslationBundle } from './useTranslationBundle-95f6039e.js';

const severityIcons = {
    confirmation: SvgSuccessS,
    error: SvgErrorS,
    info: SvgInformationS,
    warning: SvgWarningS
};
/**
 * StartIcon Component for rendering the severity based icon in Message
 */
function MessageStartIcon({ severity, variant = 'banner' }) {
    const IconComponent = severityIcons[severity];
    const translations = useTranslationBundle('@oracle/oraclejet-preact');
    const labelMap = {
        confirmation: translations.message_confirmation(),
        error: translations.message_error(),
        info: translations.message_info(),
        warning: translations.message_warning()
    };
    // TODO: JET-50793
    const iconContent = variant !== 'inline' ? (jsx(Flex, { align: "center", height: "100%", children: jsx("span", { class: startIconContainerStyles[variant], role: "img", "aria-label": labelMap[severity], children: jsx(IconComponent, {}) }) })) : (jsx("span", { class: startIconContainerStyles.inline, role: "img", "aria-label": labelMap[severity], children: jsx(IconComponent, {}) }));
    // If detail content is rendered, then wrap it in a div with specified style classes
    return (jsx("div", { class: classNames([
            messageStartIconStyles.base,
            messageStartIconStyles[variant],
            severityIconStyles[variant]
        ]), role: "presentation", children: iconContent }));
}

export { MessageStartIcon as M };
//# sourceMappingURL=MessageStartIcon-245107bf.js.map
