/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx, jsxs, Fragment } from 'preact/jsx-runtime';
import { useRef, useCallback, useEffect } from 'preact/hooks';
import { u as useTranslationBundle } from './useTranslationBundle-95f6039e.js';
import { g as getClientHints } from './clientHints-ee1c8473.js';
import { m as mergeInterpolations } from './mergeInterpolations-f71c7446.js';
import { f as flexitemInterpolations } from './flexitem-139efd0e.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { I as Icon } from './Icon-715e5a9e.js';
import { u as useTabBarContext } from './useTabBarContext-3f91c830.js';
import { S as Sheet } from './Sheet-1c2cc312.js';
import { B as BaseTabBarItem } from './TabBarItem-cb805a59.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { TabBarItemRedwoodTheme } from './UNSAFE_TabBarCommon/themes/redwood/TabBarItemTheme.js';
import { N as NavigationList, a as NavigationListItem } from './NavigationListItem-68a0d6ec.js';
import { R as RemovableNavigationListItem } from './RemovableNavigationListItem-24f293b6.js';
import { D as Dropdown } from './Dropdown-4501737c.js';

const SvgOverflowH = (props) => (jsx(Icon, { viewBox: "0 0 24 24", ...props, children: jsx("g", { fill: "none", children: jsx("path", { d: "M6 12a2 2 0 1 1-4 0 2 2 0 0 1 4 0zm8 0a2 2 0 1 1-4 0 2 2 0 0 1 4 0zm8 0a2 2 0 1 1-4 0 2 2 0 0 1 4 0z", fill: "currentcolor" }) }) }));

/**
 * An OverflowTabBarItem renders TabBarItems in a device appropriate popup. OverflowTabBarItem is used internally in an OverflowTabBar.
 */
function OverflowTabBarItem({ overflowItemKey, icon, badge, overflowItems, isOpen, onClose }) {
    const { selection, showFocusRing, isEdgeBottom, onSelect, onRemove, layout } = useTabBarContext();
    const isPointerSelection = useRef(false);
    const overflowTabBarItemRef = useRef(null);
    const navigationListRef = useRef();
    const handleSelectionChange = useCallback((detail) => {
        if (detail.reason === 'pointer') {
            isPointerSelection.current = true;
        }
        onSelect?.({ value: detail.value });
        onClose?.({ reason: 'itemAction' });
    }, [onSelect, onClose]);
    const handleDropdownClose = useCallback((e) => {
        onClose?.({ reason: e.reason });
    }, [onClose]);
    const handleSheetClose = useCallback(() => {
        onClose?.({ reason: 'dismissed' });
    }, [onClose]);
    //When NavigationList steals focus the app resets it. Not always would it be reset to
    //parent of OverflowTabBarItem. Though currentKey would be correct, but focusRing would would not be visible
    //Hence we check for isPointerSelection.current
    const focusRingRequirement = showFocusRing && !isPointerSelection.current && !getIsMobile();
    //The focus should be on NavigationList only when it is open using keyboard
    useEffect(() => {
        if (isOpen && navigationListRef.current && focusRingRequirement) {
            navigationListRef.current.focus();
        }
        isPointerSelection.current = false;
    }, [isOpen, focusRingRequirement]);
    const overflowContainsKey = (searchKey) => {
        return overflowItems?.find((item) => item.itemKey === searchKey);
    };
    const translations = useTranslationBundle('@oracle/oraclejet-preact');
    const isSelected = selection !== undefined && overflowContainsKey(selection);
    const { classes } = useComponentTheme(TabBarItemRedwoodTheme, {
        selection: isSelected ? 'isSelected' : 'notSelected',
        edgeBottom: isEdgeBottom ? 'isBottom' : 'notBottom'
    });
    const overflowTabBarItem = classNames([classes]);
    //instead of specifying flex: '1 0 auto' in cssProps var
    const styleInterpolations = mergeInterpolations([
        ...Object.values(flexitemInterpolations)
    ]);
    const flexDimensions = styleInterpolations({
        flex: layout === 'stretch' ? '1 0 auto' : '0 0 auto'
    });
    const getNavigationList = useCallback(() => {
        return (jsx(NavigationList, { ref: navigationListRef, selection: selection, onSelectionChange: handleSelectionChange, onRemove: onRemove, children: overflowItems?.map((item) => item.isRemovable ? (jsx(RemovableNavigationListItem, { itemKey: item.itemKey, label: item.label, badge: item.badge, metadata: item.metadata, severity: item.severity })) : (jsx(NavigationListItem, { itemKey: item.itemKey, label: item.label, badge: item.badge, metadata: item.metadata, severity: item.severity }))) }));
    }, [overflowItems, selection, handleSelectionChange, onRemove]);
    return (jsxs(Fragment, { children: [overflowItems && overflowItems.length > 0 && (jsx("div", { ref: overflowTabBarItemRef, class: overflowTabBarItem, style: flexDimensions, children: jsx(BaseTabBarItem, { itemKey: overflowItemKey, label: " ", icon: icon ? icon : jsx(SvgOverflowH, { accessibleLabel: translations.overflowItemLabel() }), badge: badge, showFocusRingOverride: focusRingRequirement }) })), getIsMobile() ? (jsx(Sheet, { isOpen: isOpen, onClose: handleSheetClose, children: getNavigationList() })) : (jsx(Dropdown, { isOpen: isOpen, onClose: handleDropdownClose, placement: isEdgeBottom ? 'top-end' : 'bottom-end', anchorRef: overflowTabBarItemRef, children: getNavigationList() }))] }));
}
const getIsMobile = () => {
    const { deviceType } = getClientHints();
    return deviceType === 'phone';
};

export { OverflowTabBarItem as O };
//# sourceMappingURL=OverflowTabBarItem-af85005a.js.map
