/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { multiVariantStyles } from './PRIVATE_StyledDayButton/themes/StyledDayButtonStyles.css.js';
import { styles } from './UNSAFE_BaseButton/themes/BaseButtonStyles.css.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import './BaseButtonStyles.styles.css';
import './StyledDayButtonStyles.styles.css';
import './vanilla-extract-recipes-createRuntimeFn.esm-8c3e2104.js';

/**
 * StyledDayButton is a controlled component that displays a stylized span for a day that goes into a Calendar grid.
 */
const StyledDayButton = ({ isActivable = false, isAdjacentMonth = false, isDimmed = false, isHoverable = false, isRestricted = false, isSelected = false, isToday = false, label }) => {
    // The multiVariantStyles function is defined in StyledDayButtonStyles.css.ts,
    // using the recipe utility provided by @vanilla-extract/recipes.
    // This function is used to merge the base styles, variants, and compound variants
    // into a single set of styles for text components based on the component's properties.
    const variantClasses = multiVariantStyles({
        active: isActivable ? 'isActivable' : 'notActiveable',
        dimmed: isDimmed ? 'isDimmed' : 'notDimmed',
        hover: isHoverable ? 'isHoverable' : 'notHoverable',
        adjacentMonth: isAdjacentMonth ? 'isAdjacentMonth' : 'notAdjacentMonth',
        restricted: isRestricted ? 'isRestricted' : 'notRestricted',
        selected: isSelected ? 'isSelected' : 'notSelected',
        today: isToday ? 'isToday' : 'notToday'
    });
    const classes = classNames([
        styles.base,
        styles.sizes.sm,
        styles.styled,
        variantClasses
    ]);
    return (jsx("span", { class: classes, children: label }));
};

export { StyledDayButton };
//# sourceMappingURL=PRIVATE_StyledDayButton.js.map
