/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, jsx } from 'preact/jsx-runtime';
import { useRef } from 'preact/hooks';
import { u as useLegendLayout, a as useEvents, b as useLegendTooltip, L as LegendText, B as BaseLegend } from './useLegendLayout-16fdac28.js';
import { m as mergeProps } from './mergeProps-dfdba053.js';
import { l as legendVars, a as getLegendTextStyle, b as getDatatipText, c as getAriaProps, g as getRandomId, d as getGridNums } from './LegendUtils-5b720c38.js';
import { u as useItemFocus } from './useItemFocus-b0b6bc51.js';
import { F as Flex } from './Flex-47f90e61.js';
import { styles } from './UNSAFE_Legend/themes/LegendStyles.css.js';
import { u as useCssVars } from './useCssVars-4fd0769c.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { u as useTestId } from './useTestId-5050f73e.js';

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
function getItem(itemInfo, sections) {
    const { sectionIdx, itemIdx } = itemInfo;
    if (sectionIdx == -1 && itemIdx == -1) {
        const itemsLength = sections[sections.length - 1].items.length;
        return sections[sections.length - 1].items[itemsLength - 1];
    }
    return sections[sectionIdx].items[itemIdx];
}
function getNextItemInfo(itemInfo, sections) {
    let { itemIdx, sectionIdx } = itemInfo;
    if (sectionIdx == -1 && itemIdx == -1) {
        return {
            itemIdx: sections[sections.length - 1].items.length - 1,
            sectionIdx: sections.length - 1
        };
    }
    itemIdx += 1;
    const itemsLength = sections[sectionIdx].items.length;
    if (itemIdx >= itemsLength) {
        if (sectionIdx === sections.length - 1) {
            itemIdx = itemInfo.itemIdx;
        }
        else {
            sectionIdx += 1;
            itemIdx = 0;
        }
    }
    return { itemIdx, sectionIdx };
}
function getPrevItemInfo(itemInfo, sections) {
    let { itemIdx, sectionIdx } = itemInfo;
    if (sectionIdx == -1 && itemIdx == -1) {
        (itemIdx = sections[sections.length - 1].items.length - 1), (sectionIdx = sections.length - 1);
    }
    itemIdx -= 1;
    if (itemIdx < 0) {
        if (sectionIdx === 0) {
            itemIdx = 0;
        }
        else {
            itemIdx = sections[sectionIdx - 1].items.length - 1;
            sectionIdx -= 1;
        }
    }
    return { itemIdx, sectionIdx };
}
/**
 * Returns utils functions for navigation in simple sectional legend.
 * @param items
 * @returns
 */
function getSectionalNavUtils(sections) {
    return {
        getItem: (itemInfo) => {
            return getItem(itemInfo, sections);
        },
        getDetailFromInfo: (itemInfo) => {
            if (!itemInfo) {
                return { itemId: undefined, sectionId: undefined };
            }
            const item = getItem(itemInfo, sections);
            const sectionId = sections[itemInfo.sectionIdx].id;
            return { itemId: item.id, sectionId: sectionId };
        },
        getPrevItemInfo: (itemInfo) => {
            return getPrevItemInfo(itemInfo, sections);
        },
        getNextItemInfo: (itemInfo) => {
            return getNextItemInfo(itemInfo, sections);
        }
    };
}

/**
 * A Sectional Legend allows grouping of legend items in sections with a specific title.
 */
function SectionalLegend({ orientation = 'horizontal', sectionTitleHAlign = 'start', sections, isReadOnly = true, onItemAction, onInput, testId, ...props }) {
    const testIdProps = useTestId(testId);
    const rootRef = useRef(null);
    const rootDimsRef = useRef();
    const { resolvedVars, cssContent } = useCssVars(legendVars);
    const { getItem, getDetailFromInfo, getPrevItemInfo, getNextItemInfo } = getSectionalNavUtils(sections);
    const legendFontStylesObj = getLegendTextStyle({
        fontStyle: props.textFontStyle,
        fontSize: props.textFontSize?.toString(),
        fontWeight: props.textFontWeight?.toString(),
        fontFamily: props.textFontFamily
    }, resolvedVars);
    const { textMeasureContent, layoutInfo } = useLegendLayout({
        rootRef,
        rootDimsRef,
        resolvedVars,
        sections,
        orientation,
        legendFontStylesObj,
        symbolWidth: props.symbolWidth
    });
    const { focusedItemInfo, hoveredItemInfo, ...eventsProps } = useEvents(!isReadOnly, props.hiddenIds != null, getDetailFromInfo, getPrevItemInfo, getNextItemInfo, onItemAction, onInput);
    const { focusedItemRef } = useItemFocus(focusedItemInfo);
    const text = getDatatipText(sections, getItem, focusedItemInfo, hoveredItemInfo);
    const { tooltipContent, tooltipProps } = useLegendTooltip({
        text,
        rootDimsRef,
        focusedItemInfo,
        focusedItemRef
    });
    const ariaProps = getAriaProps(isReadOnly, props['aria-label'], props.hiddenIds);
    const mergedProps = mergeProps(eventsProps, tooltipProps);
    const titleStyle = {
        fontFamily: props.sectionTitleFontFamily,
        fontSize: props.sectionTitleFontSize,
        color: props.sectionTitleColor,
        fontStyle: props.sectionTitleFontStyle,
        fontWeight: props.sectionTitleFontWeight,
        textDecoration: props.sectionTitleTextDecoration
    };
    const { sectionalLegendBaseStyles, renderedLegendStyle, preRenderedLegendStyle } = styles;
    const isHoriz = orientation === 'horizontal';
    const canRender = orientation === 'vertical' || !!layoutInfo;
    return (jsxs("div", { ref: rootRef, tabIndex: isReadOnly ? undefined : 0, role: "application", class: classNames([
            sectionalLegendBaseStyles,
            canRender ? renderedLegendStyle : preRenderedLegendStyle
        ]), ...testIdProps, ...ariaProps, ...mergedProps, children: [canRender ? (jsx(Flex, { direction: isHoriz ? 'row' : 'column', wrap: 'wrap', gap: '4x', children: sections.map((section, index) => {
                    const labelledById = !isReadOnly ? getRandomId() : '';
                    const isWrappedSection = layoutInfo ? layoutInfo.isWrapped : false;
                    const flexDirection = !isHoriz || isWrappedSection ? 'column' : 'row';
                    const alignItems = !isHoriz || isWrappedSection ? 'start' : 'center';
                    const { itemsPerCol, itemsPerRow } = getGridNums(section.items.length, layoutInfo);
                    return (jsxs(Flex, { align: alignItems, direction: flexDirection, maxWidth: '100%', justify: 'start', wrap: "wrap", children: [jsx(LegendText, { text: section.title, ...titleStyle, type: "title", id: labelledById, align: sectionTitleHAlign }), jsx(BaseLegend, { ...props, focusedItemRef: focusedItemRef, labelledBy: labelledById, sectionIdx: index, items: section.items, orientation: orientation, itemsPerRow: itemsPerRow, itemsPerCol: itemsPerCol, itemWidth: layoutInfo?.isWrapped ? layoutInfo?.maxItemWidth : undefined, isReadOnly: isReadOnly, focusedItemInfo: focusedItemInfo, hoveredItemInfo: hoveredItemInfo, activeId: eventsProps['aria-activedescendant'] })] }));
                }) })) : null, tooltipContent, textMeasureContent, cssContent] }));
}

export { SectionalLegend as S };
//# sourceMappingURL=SectionalLegend-58b07ef7.js.map
