/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { c as classNames } from './classNames-2e96f1d4.js';
import { d as dimensionInterpolations } from './dimensions-d9512d9e.js';
import { p as paddingInterpolations } from './padding-532a6a4d.js';
import { c as colorInterpolations } from './colors-8cd7269e.js';
import { a as ariaInterpolations } from './aria-20ecc4ae.js';
import { m as mergeInterpolations } from './mergeInterpolations-f71c7446.js';
import { u as useTestId } from './useTestId-5050f73e.js';
import { baseStyle, hoverStyle, selectedHoverStyle, selectedStyle } from './UNSAFE_SelectionCard/themes/SelectionCardStyles.css.js';
import { u as useHover } from './useHover-693303bd.js';

const interpolations = [
    ...Object.values(dimensionInterpolations),
    ...Object.values(colorInterpolations),
    ...Object.values(ariaInterpolations),
    ...Object.values(paddingInterpolations)
];
const mergedInterpolations = mergeInterpolations(interpolations);
/**
 * A selection card is a container element that is a part of a collection of elements, and renders a group of related information in a styled rectangular area.
 * Width, height, and backgroundColor are set through props.
 */
const SelectionCard = ({ children, isSelected = false, testId, ...rest }) => {
    const { hoverProps, isHover } = useHover();
    const { class: mergedClasses, ...mergedStyles } = mergedInterpolations(rest);
    const selectionStyles = classNames([
        baseStyle,
        mergedClasses,
        isHover && hoverStyle,
        isSelected && (isHover ? selectedHoverStyle : selectedStyle)
    ]);
    const testIdProps = useTestId(testId);
    return (jsx("div", { role: "region", ...hoverProps, class: selectionStyles, style: mergedStyles, ...testIdProps, children: children }));
};

export { SelectionCard as S };
//# sourceMappingURL=SelectionCard-dae878da.js.map
