/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { useRef } from 'preact/hooks';
import { F as Flex } from './Flex-47f90e61.js';
import { u as usePress } from './usePress-6e2cde1b.js';
import { styles } from './UNSAFE_Selector/themes/SelectorStyles.css.js';
import { S as StyledCheckbox } from './StyledCheckbox-68bddaad.js';
import { m as mergeProps } from './mergeProps-dfdba053.js';
import { u as useActive } from './useActive-ac391e31.js';
import { u as useFocus } from './useFocus-1c5b1863.js';
import { u as useTestId } from './useTestId-5050f73e.js';

/**
 * The SelectorAll component renders checkboxes for select all case.
 */
function SelectorAll({ 'aria-label': ariaLabel, 'aria-labelledby': ariaLabelledby, selected, onChange, testId }) {
    const pressHandler = (event) => {
        if (selected == 'none' || selected == 'partial') {
            onChange?.({ value: { all: true, deletedKeys: new Set() }, target: event.target });
        }
        else {
            onChange?.({ value: { all: false, keys: new Set() }, target: event.target });
        }
    };
    const inputRef = useRef(null);
    const { isActive, activeProps } = useActive();
    const { isFocus, focusProps } = useFocus();
    const { pressProps } = usePress(pressHandler);
    const isChecked = selected === 'partial' || selected === 'partial-all'
        ? 'partial'
        : selected === 'all'
            ? 'checked'
            : 'unchecked';
    const testIdProps = useTestId(testId);
    const onClick = () => {
        // ensure the checkbox is focused after click
        inputRef.current?.focus();
    };
    return (jsx("div", { class: styles.container, ...mergeProps({ onClick }, pressProps), ...testIdProps, children: jsx("div", { class: styles.base, ...activeProps, children: jsx(Flex, { align: "center", justify: "center", width: "11x", height: "11x", children: jsx(StyledCheckbox, { ref: inputRef, isChecked: isChecked, isActive: isActive, isFocusRingShown: isFocus, "aria-label": ariaLabel, "aria-labelledby": ariaLabelledby, onKeyDown: handleKeyboardEvent, onKeyUp: handleKeyboardEvent, ...focusProps }) }) }) }));
}
const handleKeyboardEvent = (event) => {
    if (event.key == 'Enter') {
        // prevent active styles applied to checkbox
        event.stopPropagation();
    }
};

export { SelectorAll as S };
//# sourceMappingURL=SelectorAll-3cb7c45a.js.map
