/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { useCallback } from 'preact/hooks';
import { M as Modal } from './Modal-006abcf9.js';
import { W as WindowOverlay } from './WindowOverlay-9fe9d83d.js';
import { u as useSheetSwipe } from './useSheetSwipe-743b81a4.js';
import { sheetMenuStyles } from './UNSAFE_Sheet/themes/SheetStyles.css.js';

const Sheet = ({ children, onClose, isOpen = false }) => {
    const handlePointerDismissed = useCallback(() => {
        onClose?.({ reason: 'pointerDismissed' });
    }, [onClose]);
    // TODO: Use useSwipe when it works on components that could scroll. Important to notice all components that have a
    // sheet will use this comp and this comp is going to use useSwipe. So comps that have their own impl (menu right now),
    // will stop having its own implementation
    const { swipeProps } = useSheetSwipe(handlePointerDismissed);
    const handleKeyDown = useCallback((event) => {
        if (event.defaultPrevented) {
            return;
        }
        switch (event.code) {
            case 'Tab':
            case 'Escape':
                onClose?.({ reason: 'keyboardDismissed' });
        }
    }, [onClose]);
    return (jsx(Modal, { isOpen: isOpen, onBackdropClick: handlePointerDismissed, children: jsx(WindowOverlay, { placement: "bottom", children: jsx("div", { ...swipeProps, class: sheetMenuStyles.sheetMenuContainerWrapper, style: { maxHeight: `calc(100vh - ${TOP_OFFSET}px)` }, onKeyDown: handleKeyDown, children: children }) }) }));
};
const TOP_OFFSET = 96;

export { Sheet as S };
//# sourceMappingURL=Sheet-1c2cc312.js.map
