/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, Fragment, jsx } from 'preact/jsx-runtime';
import { forwardRef } from 'preact/compat';
import { S as SvgCheck } from './Check-07ab1aa9.js';
import { S as SvgCheckboxOff } from './CheckboxOff-ef4c6fec.js';
import { S as SvgCheckboxOn } from './CheckboxOn-de808e18.js';
import { S as SvgCheckboxMixed } from './CheckboxMixed-df4cf0ec.js';
import { H as HiddenAccessible } from './HiddenAccessible-653c0211.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { checkboxIconVars } from './UNSAFE_Checkbox/themes/CheckboxIconContract.css.js';
import { CheckboxIconRedwoodTheme } from './UNSAFE_Checkbox/themes/redwood/CheckboxIconTheme.js';
import './TabbableModeContext-1e03f32d.js';
import { u as useTabbableMode } from './useTabbableMode-9c5d5a64.js';

/**
 * StyledCheckbox is a controlled component that displays a stylized checkbox for 'checked',
 * 'unchecked', and 'partial'. It passes intrinsic props to the native input element. This should be used
 * in conjunction with a label and be passed an onChange handler. If used within a collection use
 * 'useCollectionFocusRing', or use 'useCheckboxRadioContext' within RadioSet/CheckboxSet, to determine if you
 * should display the focus ring using keyboard navigation.
 */
const StyledCheckbox = forwardRef(({ onClick, isActive, isFocusRingShown, isDisabled, isReadonly, isRequired, isChecked: propIsChecked = 'unchecked', 
// Remaining props should be intrinsic input props only. If you need additional input props exposed,
// add to the picked items from InputProps.
id, name, value, onKeyUp, onKeyDown, onChange, onFocus, onBlur, 'aria-label': ariaLabel, 'aria-labelledby': ariaLabelledBy, 'aria-describedby': ariaDescribedBy }, ref = null) => {
    const { isTabbable } = useTabbableMode();
    const isPartial = propIsChecked === 'partial';
    const isChecked = propIsChecked === 'checked' || isPartial;
    const ariaChecked = isChecked ? (isPartial ? 'mixed' : true) : false;
    const { classes } = useComponentTheme(CheckboxIconRedwoodTheme, {
        disabled: isDisabled ? 'isDisabled' : 'notDisabled',
        checked: isChecked ? 'isChecked' : 'notChecked',
        readonly: isReadonly ? 'isReadonly' : 'notReadonly',
        active: isActive ? 'isActive' : 'notActive',
        focus: isFocusRingShown ? 'isFocused' : 'notFocused',
        iconSize: '4xUnits'
    });
    const IconComp = isReadonly
        ? isChecked
            ? SvgCheck
            : SvgCheckboxOff
        : isChecked
            ? isPartial
                ? SvgCheckboxMixed
                : SvgCheckboxOn
            : SvgCheckboxOff;
    return (jsxs(Fragment, { children: [jsx(HiddenAccessible, { children: jsx("input", { ...(isReadonly ? { 'aria-readonly': true } : {}), "aria-checked": ariaChecked, "aria-label": ariaLabel, "aria-labelledby": ariaLabelledBy, "aria-describedby": ariaDescribedBy, "aria-required": isRequired, ref: ref, name: name, value: value, onKeyUp: onKeyUp, onKeyDown: onKeyDown, onBlur: onBlur, onChange: onChange, onFocus: onFocus, id: id, type: "checkbox", disabled: isDisabled, checked: isChecked, tabIndex: isTabbable ? 0 : -1 }) }), jsx("span", { className: classes, "aria-hidden": "true", onClick: onClick, role: "img", children: jsx(IconComp, { size: checkboxIconVars.iconSize, color: "currentColor" }) })] }));
});

export { StyledCheckbox as S };
//# sourceMappingURL=StyledCheckbox-68bddaad.js.map
