/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, jsx } from 'preact/jsx-runtime';
import { forwardRef } from 'preact/compat';
import './FormContext-f5e6dd2a.js';
import { u as useFormContext } from './useFormContext-dcae8120.js';
import { L as LabelValueLayout } from './LabelValueLayout-f8d166a2.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import './FormFieldContext-dea4d4d7.js';
import { u as useFormFieldContext } from './useFormFieldContext-bc2595da.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import './InputGroupContext-e6856c27.js';
import { u as useInputGroupContext } from './useInputGroupContext-8405ef6c.js';
import { u as useUser } from './useUser-6e653d84.js';
import { TextFieldRedwoodTheme } from './UNSAFE_TextField/themes/redwood/TextFieldTheme.js';
import { LabelRedwoodTheme } from './UNSAFE_Label/themes/redwood/LabelTheme.js';
import { FormLayoutRedwoodTheme } from './UNSAFE_TextField/themes/redwood/FormLayoutTheme.js';
import { ReadonlyTextFieldRedwoodTheme } from './UNSAFE_TextField/themes/redwood/ReadonlyTextFieldTheme.js';
import { u as useTestId } from './useTestId-5050f73e.js';
import { l as layoutSpanStyles } from './LayoutStyles.css-f4783df4.js';
import { SkeletonRedwoodTheme } from './UNSAFE_Skeleton/themes/redwood/SkeletonTheme.js';

const StartTopLabelReadonlyTextField = forwardRef(({ outerClassNames, innerClassNames, label, labelEdge, labelStartWidth, compactUserAssistance, inlineUserAssistance, children, ...props }, ref) => {
    return (jsxs("div", { ...props, ref: ref, class: outerClassNames, children: [compactUserAssistance, jsx(LabelValueLayout, { label: label, labelEdge: labelEdge, labelStartWidth: labelStartWidth, children: jsx("div", { class: innerClassNames, children: children }) }), inlineUserAssistance] }));
});
const InsideLabelReadonlyTextField = forwardRef(({ outerClassNames, innerClassNames, compactUserAssistance, inlineUserAssistance, children, ...props }, ref) => {
    return (jsxs("div", { ...props, ref: ref, class: outerClassNames, children: [compactUserAssistance, jsx("div", { class: innerClassNames, children: children }), inlineUserAssistance] }));
});
const ReadonlyTextField = forwardRef(({ columnSpan = 1, label, labelEdge, children, variant, testId, ...props }, ref) => {
    const testIdProps = useTestId(testId);
    const { isFormLayout, isReadonly: isReadonlyForm } = useFormContext();
    const { isLoading } = useFormFieldContext();
    const hasInsideLabel = label !== undefined && labelEdge === 'inside';
    const isInsideNonReadonlyForm = isFormLayout && !isReadonlyForm;
    const { direction } = useUser();
    const isLtr = direction === 'ltr';
    const inputGroupContext = useInputGroupContext();
    const hasGroupContext = inputGroupContext !== null;
    // depending on browser direction, apply the appropriate styling.
    // when 'rtl' the flex row direction is reversed.
    const inputGroupPosition = inputGroupContext
        ? inputGroupContext.last
            ? isLtr
                ? 'groupRight'
                : 'groupLeft'
            : inputGroupContext.index === 0
                ? isLtr
                    ? 'groupLeft'
                    : 'groupRight'
                : 'groupMiddle'
        : undefined;
    const { classes, styles } = useComponentTheme(TextFieldRedwoodTheme, {
        readonly: 'isReadonly'
    });
    const { baseTheme: labelBaseTheme } = useComponentTheme(LabelRedwoodTheme);
    const { baseTheme: formLayoutTheme } = useComponentTheme(FormLayoutRedwoodTheme);
    const { classes: readonlyClassNames } = useComponentTheme(ReadonlyTextFieldRedwoodTheme, {
        insideNonReadonlyForm: isInsideNonReadonlyForm
            ? 'isInsideNonReadonlyForm'
            : 'notInsideNonReadonlyForm',
        insideLabel: hasInsideLabel ? 'hasInsideLabel' : 'noInsideLabel',
        textarea: variant === 'textarea' ? 'isTextArea' : 'notTextArea',
        loading: isLoading ? 'isLoading' : 'notLoading',
        withinGroup: hasGroupContext ? 'isWithinGroup' : undefined,
        inputGroupPosition
    });
    const { formControlBase } = styles;
    const outerClassNames = classNames([
        classes,
        formControlBase,
        formLayoutTheme,
        labelBaseTheme,
        layoutSpanStyles.layoutSpanColumn[columnSpan]
    ]);
    if (label !== undefined && (labelEdge === 'start' || labelEdge === 'top')) {
        return (jsx(StartTopLabelReadonlyTextField, { outerClassNames: outerClassNames, innerClassNames: readonlyClassNames, label: label, labelEdge: labelEdge, ref: ref, ...props, ...testIdProps, children: children }));
    }
    return (jsxs(InsideLabelReadonlyTextField, { outerClassNames: outerClassNames, innerClassNames: readonlyClassNames, ref: ref, ...props, ...testIdProps, children: [label, children] }));
});

const StyledTextField = forwardRef(({ columnSpan = 1, hasInsideLabel = false, variant = 'default', ...props }, ref) => {
    // because of the mismatch of height for inputs with inside labels and no labels,
    // we'll have to set everything to the same min-height while within an input group context.
    // Here we'll look to see if it's present and apply the style as needed
    const groupContext = useInputGroupContext();
    const { isReadonly } = useFormFieldContext();
    const { baseTheme: formLayoutTheme } = useComponentTheme(FormLayoutRedwoodTheme);
    const { baseTheme: labelBaseTheme } = useComponentTheme(LabelRedwoodTheme);
    const { classes: textFieldClasses, styles } = useComponentTheme(TextFieldRedwoodTheme, {
        readonly: isReadonly ? 'isReadonly' : 'notReadonly',
        formControlInsideLabel: hasInsideLabel ? 'hasInsideLabel' : 'noInsideLabel',
        withinGroup: groupContext !== null ? 'isWithinGroup' : undefined
    });
    const { formControlBase, textFieldBase, textFieldEmbedded } = styles;
    // because TextField components reference Skeleton scoped vars, we need to bring in the Skeleton base theme
    // to pick up the scoped var definitions.
    const { baseTheme: skeletonBaseTheme } = useComponentTheme(SkeletonRedwoodTheme);
    const classes = classNames([
        skeletonBaseTheme,
        formControlBase,
        textFieldBase,
        variant === 'embedded' && textFieldEmbedded,
        textFieldClasses,
        formLayoutTheme,
        labelBaseTheme,
        layoutSpanStyles.layoutSpanColumn[columnSpan]
    ]);
    return jsx("div", { ...props, class: classes, ref: ref });
});

export { ReadonlyTextField as R, StyledTextField as S };
//# sourceMappingURL=StyledTextField-6dcd9ff0.js.map
