/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx, jsxs } from 'preact/jsx-runtime';
import { useRef, useEffect, useCallback } from 'preact/hooks';
import { c as classNames } from './classNames-2e96f1d4.js';
import { u as usePress } from './usePress-6e2cde1b.js';
import { u as useUser } from './useUser-6e653d84.js';
import { T as Text } from './Text-ec0e2d47.js';
import { F as Flex } from './Flex-47f90e61.js';
import { u as useTabBarContext } from './useTabBarContext-3f91c830.js';
import { m as mergeInterpolations } from './mergeInterpolations-f71c7446.js';
import { f as flexitemInterpolations } from './flexitem-139efd0e.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { TabBarItemRedwoodTheme } from './UNSAFE_TabBarCommon/themes/redwood/TabBarItemTheme.js';
import { S as SvgWarningS } from './WarningS-542fff95.js';
import { S as SvgErrorS } from './ErrorS-a4c64c74.js';
import { S as SvgInformationS } from './InformationS-219d8de8.js';
import { S as SvgSuccessS } from './SuccessS-fd0eaab1.js';
import { B as Badge } from './Badge-d42090be.js';

function BaseTabBarItem({ itemKey, label, icon, badge, metadata, severity = 'none', removeIcon, showFocusRingOverride }) {
    const { onSelect, selection, currentKey, showFocusRing, layout, display, size, isEdgeBottom, tabItemPrefix } = useTabBarContext();
    const isCurrent = currentKey === itemKey;
    const ref = useRef(null);
    /**
     * FIXME: 
     *
     * This is a temporary workaround until [JET-63308]() is resolved.
     *
     * This will optimistically attempt to scroll tab items into view as the tab
     * item becomes "current" regardless of whether this tab item is nested
     * inside of a conveyor/scrollable container.
     *
     * After JET-63308 (or equivalent) is implemented, we can remove this with a
     * proper utility that only invokes inside of a scrolling context.
     */
    useEffect(() => {
        if (ref.current && isCurrent) {
            ref.current.scrollIntoView({ behavior: 'smooth', block: 'nearest', inline: 'nearest' });
        }
    }, [isCurrent]);
    //Selection
    //Dependency on currentKey because upon removal selectionChange does not change hence no re render
    //and it is still holding the deleted key as selection
    const handlePress = useCallback(() => {
        onSelect?.({ value: itemKey });
    }, [itemKey, onSelect]);
    const { pressProps } = usePress(handlePress);
    const isSelected = selection === itemKey;
    const isStacked = display === 'stacked' ? true : false;
    const isIconOnly = display === 'icons' ? true : false;
    const isTall = size === 'lg' ? true : false;
    const labelRequirement = label && !(label === ' ' || label === undefined);
    const severityRequirement = !(severity === 'none' || severity === undefined);
    const IconComponent = (severityValue) => {
        switch (severityValue) {
            case 'error':
                return jsx(SvgErrorS, { color: "danger" });
            case 'warning':
                return jsx(SvgWarningS, { color: "warning" });
            case 'info':
                return jsx(SvgInformationS, { color: "info" });
            case 'confirmation':
                return jsx(SvgSuccessS, { color: "success" });
            default:
                return null;
        }
    };
    const { direction } = useUser();
    const makeFocusRingVisible = showFocusRingOverride !== undefined ? showFocusRingOverride : showFocusRing;
    const { classes, styles: { tabBarItemIcon, tabBarItemIconStacked, tabBarItemMetadata, tabBarItemStackedBadge, tabBarItemStackedBadgeRtl, tabBarItemRemoveIcon } } = useComponentTheme(TabBarItemRedwoodTheme, {
        selection: isSelected ? 'isSelected' : 'notSelected',
        current: isCurrent ? 'isCurrent' : 'notCurrent',
        focusRing: makeFocusRingVisible ? 'showFocusRing' : 'noFocusRing',
        edgeBottom: isEdgeBottom ? 'isBottom' : 'notBottom'
    });
    const tabBarItem = classNames([classes]);
    const tabBarItemIconClasses = classNames([
        tabBarItemIcon,
        (isStacked || isIconOnly || !labelRequirement) && tabBarItemIconStacked
    ]);
    const tabBarItemMetadataClasses = classNames([tabBarItemMetadata]);
    const tabBarItemStackedBadgeClasses = classNames([
        tabBarItemStackedBadge,
        direction === 'rtl' && tabBarItemStackedBadgeRtl
    ]);
    const removeClass = classNames([tabBarItemRemoveIcon]);
    //instead of specifying flex: '1 0 auto' in cssProps var
    const styleInterpolations = mergeInterpolations([
        ...Object.values(flexitemInterpolations)
    ]);
    const flexDimensions = styleInterpolations({
        flex: layout === 'stretch' ? '1 0 auto' : '0 0 auto'
    });
    return (jsx("div", { "data-oj-key": itemKey, "data-oj-removable": removeIcon ? 'true' : undefined, id: tabItemPrefix + itemKey, class: tabBarItem, ref: ref, role: "tab", "aria-selected": isSelected, ...pressProps, style: flexDimensions, children: jsxs(Flex, { minWidth: "10.5x", minHeight: isTall ? '14.5x' : '100%', height: isStacked && labelRequirement ? '18.5x' : '10.5x', gap: "2x", flex: "1 1 auto", children: [jsxs(Flex, { direction: isStacked && labelRequirement ? 'column' : 'row', gap: "2x", align: "center", justify: "center", flex: "1 1 auto", children: [(isStacked || isIconOnly || (!labelRequirement && icon)) && (jsxs("div", { class: tabBarItemIconClasses, children: [icon, jsx("span", { class: tabBarItemStackedBadgeClasses, children: badge ? (jsx(Badge, { size: "xs", variant: "neutral", children: badge })) : null })] })), ((!isIconOnly && labelRequirement) ||
                            metadata ||
                            severityRequirement ||
                            removeIcon ||
                            (icon && labelRequirement && !isStacked && !isIconOnly)) && (jsxs(Flex, { gap: "2x", align: "center", justify: "center", children: [!isIconOnly && !isStacked && icon && labelRequirement && (jsx("span", { class: tabBarItemIconClasses, children: icon })), !isIconOnly && labelRequirement && (jsx(Text, { size: "md", weight: "semiBold", children: label })), !isStacked && !isIconOnly && badge && labelRequirement && (jsx(Badge, { size: "md", variant: "neutralSubtle", children: badge })), metadata && (jsx("span", { class: tabBarItemMetadataClasses, children: jsx(Text, { children: '( ' + metadata + ' )' }) })), severityRequirement && (jsx("span", { class: tabBarItemIconClasses, children: IconComponent(severity) }))] }))] }), removeIcon && jsx("span", { class: removeClass, children: removeIcon })] }) }));
}

function TabBarItem({ itemKey, label, icon, badge, metadata, severity }) {
    return (jsx(BaseTabBarItem, { itemKey: itemKey, label: label, icon: icon, badge: badge, metadata: metadata, severity: severity }));
}

export { BaseTabBarItem as B, TabBarItem as T };
//# sourceMappingURL=TabBarItem-cb805a59.js.map
