/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { useRef, useState, useEffect } from 'preact/hooks';
import { forwardRef, useImperativeHandle } from 'preact/compat';
import './TopLayerHost-d50e5624.js';
import './UNSAFE_Text/themes/TextStyles.css.js';
import './Flex-47f90e61.js';
import { T as TabBarContext } from './useTabBarContext-3f91c830.js';
import './flexitem-139efd0e.js';
import './logger-c710d0a3.js';
import './TabBarItemStyles.styles.css';
import './UNSAFE_TabBarCommon/themes/redwood/TabBarItemBaseTheme.styles.css';
import './UNSAFE_TabBarCommon/themes/redwood/TabBarItemVariants.css.js';
import 'preact';
import './Floating-9ce291ba.js';
import './TooltipContentStyles.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentBaseTheme.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentVariants.css.js';
import './IconStyle.styles.css';
import './Modal-006abcf9.js';
import './WindowOverlayStyles.styles.css';
import './UNSAFE_WindowOverlay/themes/redwood/WindowOverlayBaseTheme.styles.css';
import './UNSAFE_WindowOverlay/themes/redwood/WindowOverlayVariants.css.js';
import './SheetStyles.styles.css';
import { m as mergeProps } from './mergeProps-dfdba053.js';
import { a as getKey, j as getPrevNextKey, k as keyExtractor, c as getFirstVisibleKey } from './collectionUtils-ed8b8b4c.js';
import { u as useCollectionFocusRing } from './useCollectionFocusRing-2fdcd077.js';
import { u as useCurrentKey } from './useCurrentKey-7e2e0e68.js';
import { u as useId } from './useId-1bde71ab.js';
import { styles } from './UNSAFE_TabBarMixed/themes/TabBarMixedStyles.css.js';

const ITEM_SELECTOR = '[role="tab"]';
const REMOVAL_ITEM_SELECTOR = '[data-oj-tabbar-item-remove-icon]';
/**
 * Re-implements UNSAFE_TabBar/TabBar behavior for focus and keyboad handling
 */
function useTabBar(options) {
    const { children, class: className, display, edge = 'top', selection, onSelect, onRemove, ref: rootRef = useRef(null), size } = options;
    const tabId = useId();
    const [currentKey, setCurrentKey] = useState(selection);
    const tabItemPrefix = tabId + '_';
    /**
     * A reference to the previous set of tab keys that this tablist
     * contains before children were re-rendered. In the event the "current" tab
     * is removed and the "currentKey" reference is broken, we can redirect
     * the "currentKey" to another tab based on the index of the removed tab.
     */
    const prevTabKeys = useRef();
    useEffect(() => {
        if (rootRef.current) {
            const tabKeys = Array.from(rootRef.current.querySelectorAll(ITEM_SELECTOR)).map((elem) => getKey(elem));
            if (currentKey && !tabKeys.includes(currentKey) && prevTabKeys.current) {
                setCurrentKey(findNextCurrentKey(currentKey, tabKeys, prevTabKeys.current));
            }
            prevTabKeys.current = tabKeys;
        }
    }, [children, currentKey]);
    const [showFocusRing, focusRingProps] = useCollectionFocusRing(rootRef, [
        'ArrowRight',
        'ArrowLeft'
    ]);
    const { currentKeyProps } = useCurrentKey((element) => onRemove
        ? extractOnlyItemKey(element, ITEM_SELECTOR, REMOVAL_ITEM_SELECTOR)
        : keyExtractor(element, ITEM_SELECTOR), false, undefined, undefined, getPrevNextKey(rootRef.current, currentKey, true, ITEM_SELECTOR), getPrevNextKey(rootRef.current, currentKey, false, ITEM_SELECTOR), currentKey, (detail) => setCurrentKey(detail.value));
    const onKeyDown = (event) => {
        if (currentKey && event.key === 'Enter') {
            onSelect?.({ value: currentKey });
        }
        if (currentKey && event.key === 'Delete') {
            onRemove?.({ value: currentKey });
        }
    };
    const onFocus = () => {
        if (rootRef.current && currentKey === undefined) {
            const key = getFirstVisibleKey(rootRef.current, ITEM_SELECTOR);
            if (key) {
                setCurrentKey(key);
            }
        }
    };
    const eventProps = { onKeyDown, onFocus };
    return {
        rootProps: mergeProps({
            'aria-activedescendant': currentKey ? tabItemPrefix + currentKey : '',
            'aria-multiselectable': false,
            class: className,
            ref: rootRef,
            role: 'tablist',
            tabIndex: 0
        }, currentKeyProps, focusRingProps, eventProps),
        tabBarContext: {
            currentKey,
            display,
            isEdgeBottom: edge === 'bottom',
            layout: 'condense',
            onRemove,
            onSelect,
            showFocusRing,
            selection,
            size,
            tabItemPrefix
        }
    };
}
//useCurrentKey use click capture that captures first click, which is on remove button
//when we remove and sets the item being removed to currentKey. So we should not allow
//this if click is on remove button
const extractOnlyItemKey = (element, itemSelector, itemEliminator) => {
    const tabBarItem = element.closest(itemSelector);
    const removeButton = element.closest(itemEliminator);
    return tabBarItem?.contains(removeButton) ? null : keyExtractor(element, itemSelector);
};
const findNextCurrentKey = (currentKey, currTabs, prevTabs) => {
    const index = prevTabs.indexOf(currentKey);
    if (index > 0) {
        const nextIndex = index === prevTabs.length - 1 ? index - 1 : index + 1;
        const nextKey = prevTabs[nextIndex];
        if (currTabs.indexOf(nextKey) !== -1) {
            return nextKey;
        }
    }
    // update current key to be the first one if we can't find a suitable next key
    return currTabs[0];
};

/**
 * A navigation component that enables horizontal navigation between distinct content with a mixture of static and dynamic tabs.
 *
 * @param {TabBarMixedProps} props TabBarMixed component props.
 * @returns {JSX.Element} TabBarMixed component element.
 */
const TabBarMixed = forwardRef((props, focusHandleRef) => {
    const { children, onRemove, onSelect, selection, size } = props;
    const rootRef = useRef(null);
    const { rootProps, tabBarContext } = useTabBar({
        class: styles.tabBarBase,
        edge: 'bottom',
        onRemove,
        onSelect,
        ref: rootRef,
        selection,
        size,
        children
    });
    //Allows to call focus on rootRef without having to expose it
    useImperativeHandle(focusHandleRef, () => ({
        focus: () => {
            if (rootRef.current) {
                rootRef.current.focus();
            }
        },
        blur: () => {
            if (rootRef.current) {
                rootRef.current.blur();
            }
        }
    }));
    return (jsx(TabBarContext.Provider, { value: tabBarContext, children: jsx("div", { ...rootProps, children: children }) }));
});

export { TabBarMixed as T };
//# sourceMappingURL=TabBarMixed-8edb2d9e.js.map
