/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, jsx } from 'preact/jsx-runtime';
import { F as Flex } from './Flex-47f90e61.js';
import './dimensions-d9512d9e.js';
import './size-aa934024.js';
import './utils-4aa3c8ad.js';
import './Common/themes/themeContract.css.js';
import './colorUtils-30dc7d3f.js';
import './_curry1-b1d25277.js';
import './mergeInterpolations-f71c7446.js';
import './classNames-2e96f1d4.js';
import './_curry3-110d1c8b.js';
import './_curry2-3213c7b9.js';
import './_isObject-6f2350ae.js';
import './boxalignment-3404f84f.js';
import './arrayUtils-bda0b35d.js';
import './boxalignment.styles.css';
import './vanilla-extract-sprinkles-createRuntimeSprinkles.esm-b8958164.js';
import './flexbox-6adb0f8d.js';
import './flexbox.styles.css';
import './flexitem-139efd0e.js';
import './flexitem.styles.css';
import './FlexStyles.styles.css';

/**
 * Returns the flex order of the legend and chart in the flex container.
 * @param legendPos The legend position.
 * @returns
 */
function getFlexOrder(legendPos) {
    const isLegendFirst = legendPos === 'start' || legendPos === 'top';
    return {
        chartOrder: isLegendFirst ? 2 : 1,
        legendOrder: isLegendFirst ? 1 : 2
    };
}
/**
 * Returns the legend flex align and justify prop.
 */
function getLegendFlexProps(legendPos) {
    let justifyContent;
    let alignItem;
    if (legendPos === 'top') {
        justifyContent = 'center';
        alignItem = 'end';
    }
    else if (legendPos === 'bottom') {
        justifyContent = 'center';
        alignItem = 'start';
    }
    else if (legendPos === 'start') {
        justifyContent = 'end';
        alignItem = 'center';
    }
    else {
        justifyContent = 'start';
        alignItem = 'center';
    }
    return { justifyContent, alignItem };
}

/**
 * Component responsible for laying out the legend around a chart.
 */
function ChartWithLegend({ chart, legend, position, maxSize, size }) {
    const flexDirection = position === 'start' || position === 'end' ? 'row' : 'column';
    const { chartOrder, legendOrder } = getFlexOrder(position);
    const { justifyContent, alignItem } = getLegendFlexProps(position);
    return (jsxs(Flex, { width: '100%', height: '100%', direction: flexDirection, children: [jsx("div", { style: {
                    order: chartOrder,
                    flexGrow: 1
                }, children: chart }), legend && (jsx(Flex, { order: legendOrder, justify: justifyContent, align: alignItem, maxHeight: flexDirection === 'column' ? maxSize : undefined, maxWidth: flexDirection === 'row' ? maxSize : undefined, height: flexDirection === 'column' ? size : undefined, width: flexDirection === 'row' ? size : undefined, children: legend }))] }));
}

export { ChartWithLegend };
//# sourceMappingURL=UNSAFE_ChartWithLegend.js.map
