/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, jsx } from 'preact/jsx-runtime';
import { useState, useRef, useCallback, useEffect } from 'preact/hooks';
import { S as SvgChevronLeft, a as SvgChevronRight } from './ChevronLeft-9edee749.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { styles } from './UNSAFE_ConveyorBelt/themes/ConveyorBeltStyles.css.js';
import { u as useUser } from './useUser-6e653d84.js';
import { T as TabbableModeContext } from './TabbableModeContext-1e03f32d.js';
import { g as getClientHints } from './clientHints-ee1c8473.js';
import { x as xUnits } from './utils-4aa3c8ad.js';
import { B as Button } from './Button-70c707d8.js';
import './Icon-715e5a9e.js';
import 'preact';
import './size-aa934024.js';
import './useTooltip-cb4418ec.js';
import './useTooltipControlled-fe816388.js';
import './useId-1bde71ab.js';
import './Floating-9ce291ba.js';
import 'preact/compat';
import './useFloating-cec1aeb7.js';
import './positionUtils-c728c841.js';
import './refUtils-54b2b0a5.js';
import './useOutsideClick-113d9979.js';
import './arrayUtils-bda0b35d.js';
import './useComponentTheme-7202d4c1.js';
import './logger-c710d0a3.js';
import './Common/themes/themeContract.css.js';
import './UNSAFE_Floating/themes/redwood/FloatingTheme.js';
import './UNSAFE_Floating/themes/FloatingStyles.css.js';
import './FloatingStyles.styles.css';
import './UNSAFE_Floating/themes/redwood/FloatingBaseTheme.css.js';
import './UNSAFE_Floating/themes/redwood/FloatingBaseTheme.styles.css';
import './UNSAFE_Floating/themes/redwood/FloatingVariants.css.js';
import './vanilla-extract-recipes-createRuntimeFn.esm-8c3e2104.js';
import './vanilla-extract-dynamic.esm-4479b60e.js';
import './UNSAFE_Floating/themes/FloatingContract.css.js';
import './Layer-c8a9c60e.js';
import './TopLayerHost-d50e5624.js';
import './Common/themes/redwood/theme.js';
import './useThemeInterpolations-f5a207dc.js';
import './useColorScheme-daa151e2.js';
import './useScale-578a7ec2.js';
import './theme-651551cd.js';
import './Theme-99a9e50d.js';
import './mergeInterpolations-f71c7446.js';
import './_curry3-110d1c8b.js';
import './_curry1-b1d25277.js';
import './_curry2-3213c7b9.js';
import './_isObject-6f2350ae.js';
import './useHover-693303bd.js';
import './useToggle-7589399a.js';
import './useFocus-1c5b1863.js';
import './useTouch-c052c62f.js';
import './mergeProps-dfdba053.js';
import './useAnimation-7d188187.js';
import './useTestId-5050f73e.js';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentTheme.js';
import './hooks/UNSAFE_useTooltip/themes/TooltipContentStyles.css.js';
import './TooltipContentStyles.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentBaseTheme.css.js';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentBaseTheme.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentVariants.css.js';
import './TooltipContentVariants.styles.css';
import './EnvironmentProvider-12bc1f90.js';
import './LayerManager-587f5086.js';
import './UNSAFE_Icon/themes/IconStyle.css.js';
import './IconStyle.styles.css';
import './ConveyorBeltStyles.styles.css';
import './colorUtils-30dc7d3f.js';
import './BaseButton-f9e8c783.js';
import './usePress-6e2cde1b.js';
import './useActive-ac391e31.js';
import './useTabbableMode-9c5d5a64.js';
import './dimensions-d9512d9e.js';
import './UNSAFE_BaseButton/themes/BaseButtonStyles.css.js';
import './BaseButtonStyles.styles.css';
import './ButtonLayout-af96cb96.js';
import './Text-ec0e2d47.js';
import './UNSAFE_Text/themes/TextStyles.css.js';
import './TextStyles.styles.css';
import './UNSAFE_ButtonLayout/themes/redwood/ButtonLayoutTheme.js';
import './UNSAFE_ButtonLayout/themes/ButtonLayoutStyles.css.js';
import './ButtonLayoutStyles.styles.css';
import './UNSAFE_ButtonLayout/themes/redwood/ButtonLayoutBaseTheme.css.js';
import './UNSAFE_ButtonLayout/themes/redwood/ButtonLayoutBaseTheme.styles.css';
import './UNSAFE_ButtonLayout/themes/redwood/ButtonLayoutVariants.css.js';
import './ButtonLayoutVariants.styles.css';

/**
 * Helper function to determine whether the current device is a mobile device
 * @returns true if runnning on a mobile device, false otherwise
 */
function isMobile() {
    const deviceType = getClientHints().deviceType;
    return deviceType === 'phone' || deviceType === 'tablet';
}
function scrollPrevious(direction, overflowContainer, scrollAmount) {
    if (overflowContainer && scrollAmount) {
        if (direction === 'ltr') {
            overflowContainer.scrollTo({
                left: overflowContainer.scrollLeft - scrollAmount
            });
        }
        else {
            overflowContainer.scrollTo({
                left: overflowContainer.scrollLeft + scrollAmount
            });
        }
    }
}
function scrollNext(direction, overflowContainer, scrollAmount) {
    if (overflowContainer && scrollAmount) {
        if (direction === 'ltr') {
            overflowContainer.scrollTo({
                left: overflowContainer.scrollLeft + scrollAmount,
                behavior: 'smooth'
            });
        }
        else {
            overflowContainer.scrollTo({
                left: overflowContainer.scrollLeft - scrollAmount,
                behavior: 'smooth'
            });
        }
    }
}
/**
 * The Conveyor belt component is a container element that manages
 * overflow for its child elements and allows scrolling among them
 */
const ConveyorBelt = ({ children, scrollPosition, onScrollPositionChanged, arrowVisibility = 'auto' }) => {
    const [canPaginateNext, setCanPaginateNext] = useState(false);
    const [canPaginatePrevious, setCanPaginatePrevious] = useState(false);
    const overflowContainerRef = useRef(null);
    const contentContainerRef = useRef(null);
    const nextButtonContainerRef = useRef(null);
    const prevButtonContainerRef = useRef(null);
    const baseRef = useRef(null);
    const leftRef = useRef(null);
    const rightRef = useRef(null);
    const observerRef = useRef(null);
    const { direction } = useUser();
    const [buttonsHidden, setButtonsHidden] = useState((isMobile() && arrowVisibility === 'auto') || arrowVisibility === 'hidden');
    const scrollEndHandler = useCallback(() => {
        if (onScrollPositionChanged)
            onScrollPositionChanged(overflowContainerRef.current?.scrollLeft);
    }, [onScrollPositionChanged]);
    useEffect(() => {
        setButtonsHidden((isMobile() && arrowVisibility === 'auto') || arrowVisibility === 'hidden');
    }, [arrowVisibility]);
    useEffect(() => {
        if (!overflowContainerRef.current)
            return;
        overflowContainerRef.current.scrollTo({
            left: scrollPosition
        });
    }, [scrollPosition]);
    useEffect(() => {
        const overflowContainer = overflowContainerRef.current;
        const leftGuard = leftRef.current;
        const rightGuard = rightRef.current;
        if (overflowContainer && leftGuard && rightGuard) {
            if (!buttonsHidden) {
                observerRef.current = new IntersectionObserver((entries) => {
                    entries.forEach((entry) => {
                        if (entry.target.getAttribute('data-intersection-id') === 'left') {
                            setCanPaginatePrevious(!entry.isIntersecting);
                        }
                        else {
                            setCanPaginateNext(!entry.isIntersecting);
                        }
                    });
                }, {
                    root: overflowContainer
                });
                observerRef.current.observe(leftGuard);
                observerRef.current.observe(rightGuard);
            }
            overflowContainer.addEventListener('scrollend', scrollEndHandler);
        }
        // Clean-up
        return () => {
            if (!overflowContainer)
                return;
            overflowContainer.removeEventListener('scrollend', scrollEndHandler);
            observerRef.current.unobserve(leftGuard);
            observerRef.current.unobserve(rightGuard);
        };
    }, [buttonsHidden, scrollEndHandler]);
    return (jsxs("div", { ref: baseRef, className: classNames([styles.baseStyle]), children: [canPaginatePrevious && !buttonsHidden && (jsx("div", { "aria-hidden": true, ref: prevButtonContainerRef, class: classNames([
                    styles.buttonContainer,
                    direction === 'ltr' ? styles.previousButton : styles.nextButton
                ]), children: jsx(TabbableModeContext.Provider, { value: { isTabbable: false }, children: jsx(Button, { startIcon: direction === 'ltr' ? jsx(SvgChevronLeft, {}) : jsx(SvgChevronRight, {}), variant: "borderless", onAction: () => scrollPrevious(direction, overflowContainerRef.current, overflowContainerRef.current?.clientWidth) }) }) })), jsx("div", { style: {
                    clipPath: direction === 'ltr' ? `inset(0rem ${canPaginateNext ? xUnits(12) : '0rem'} 0rem ${canPaginatePrevious ? xUnits(12) : 0})` : `inset(0rem ${canPaginatePrevious ? xUnits(12) : '0rem'} 0rem ${canPaginateNext ? xUnits(12) : 0})`
                }, className: styles.overflowContainer, ref: overflowContainerRef, children: jsxs("div", { className: styles.contentContainer, ref: contentContainerRef, children: [jsx("div", { "data-intersection-id": "left", ref: leftRef, style: { minWidth: 1 } }), children, jsx("div", { "data-intersection-id": "right", ref: rightRef, style: { minWidth: 1 } })] }) }), canPaginateNext && !buttonsHidden && (jsx("div", { "aria-hidden": true, ref: nextButtonContainerRef, class: classNames([
                    styles.buttonContainer,
                    direction === 'ltr' ? styles.nextButton : styles.previousButton
                ]), children: jsx(TabbableModeContext.Provider, { value: { isTabbable: false }, children: jsx(Button, { startIcon: direction === 'ltr' ? jsx(SvgChevronRight, {}) : jsx(SvgChevronLeft, {}), variant: "borderless", onAction: () => scrollNext(direction, overflowContainerRef.current, overflowContainerRef.current?.clientWidth) }) }) }))] }));
};

export { ConveyorBelt };
//# sourceMappingURL=UNSAFE_ConveyorBelt.js.map
