/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx, jsxs } from 'preact/jsx-runtime';
import { forwardRef } from 'preact/compat';
import { W as WindowOverlay } from './WindowOverlay-9fe9d83d.js';
import { u as useTestId } from './useTestId-5050f73e.js';
import { u as useAnimation } from './useAnimation-7d188187.js';
import { useState, useEffect } from 'preact/hooks';
import { DialogRedwoodTheme } from './UNSAFE_Dialog/themes/redwood/DialogTheme.js';
import { F as FocusTrap } from './FocusTrap-9a390118.js';
import { L as Layer } from './Layer-c8a9c60e.js';
import './TopLayerHost-d50e5624.js';
import { M as Modal } from './Modal-006abcf9.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { g as globalImages } from './ImageVars.css-21c6b0f7.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { u as useId } from './useId-1bde71ab.js';
import './mergeInterpolations-f71c7446.js';
import './_curry3-110d1c8b.js';
import './_curry1-b1d25277.js';
import './_curry2-3213c7b9.js';
import './_isObject-6f2350ae.js';
import './vanilla-extract-dynamic.esm-4479b60e.js';
import './UNSAFE_WindowOverlay/themes/WindowOverlayStyles.css.js';
import './WindowOverlayStyles.styles.css';
import './UNSAFE_WindowOverlay/themes/WindowOverlayContract.css.js';
import './useUser-6e653d84.js';
import './UNSAFE_WindowOverlay/themes/redwood/WindowOverlayTheme.js';
import './UNSAFE_WindowOverlay/themes/redwood/WindowOverlayBaseTheme.css.js';
import './UNSAFE_WindowOverlay/themes/redwood/WindowOverlayBaseTheme.styles.css';
import './UNSAFE_WindowOverlay/themes/redwood/WindowOverlayVariants.css.js';
import './vanilla-extract-recipes-createRuntimeFn.esm-8c3e2104.js';
import './UNSAFE_Dialog/themes/DialogStyles.css.js';
import './DialogStyles.styles.css';
import './UNSAFE_Dialog/themes/redwood/DialogBaseTheme.css.js';
import './ImageVars.styles.css';
import './UNSAFE_Dialog/themes/redwood/DialogBaseTheme.styles.css';
import './UNSAFE_Dialog/themes/redwood/DialogVariants.css.js';
import './tabbableUtils-d03bccb0.js';
import './head-6177894e.js';
import './_arity-703022b9.js';
import './_isArray-7f64b525.js';
import './_isString-c97680cd.js';
import './FocusTracker-5f9c7635.js';
import 'preact';
import './PRIVATE_FocusTracker/themes/FocusTrackerStyles.css.js';
import './FocusTrackerStyles.styles.css';
import './useThemeInterpolations-f5a207dc.js';
import './useColorScheme-daa151e2.js';
import './useScale-578a7ec2.js';
import './theme-651551cd.js';
import './Theme-99a9e50d.js';
import './Common/themes/redwood/theme.js';
import './Common/themes/themeContract.css.js';
import './UNSAFE_Modal/themes/ModalStyles.css.js';
import './ModalStyles.styles.css';
import './logger-c710d0a3.js';

const StyledDialogHeader = ({ header, id }) => {
    const { styles } = useComponentTheme(DialogRedwoodTheme);
    const content = typeof header === 'string' ? (jsx("h1", { id: id, className: styles.dialogTitleStyle, children: header })) : (header);
    return jsx("div", { className: styles.dialogHeaderStyle, children: content });
};
const StyledDialog = forwardRef(({ children, header, footer, labelId, role, ariaLabelledBy, ariaDescribedBy, onKeyDown, testId }, ref) => {
    const { classes: styleClasses, styles } = useComponentTheme(DialogRedwoodTheme);
    const testIdProps = useTestId(testId);
    return (jsx("div", { ref: ref, className: classNames([styleClasses, globalImages]), style: { opacity: 0 }, onKeyDown: onKeyDown, role: role, "aria-labelledby": ariaLabelledBy, "aria-describedby": ariaDescribedBy, ...testIdProps, children: jsxs("div", { className: styles.dialogContainerStyle, children: [jsx(StyledDialogHeader, { id: labelId, header: header }), jsx("div", { className: styles.dialogContentStyle, children: jsx("div", { className: styles.dialogBodyStyle, children: children }) }), jsx("div", { className: styles.dialogFooterStyle, children: footer })] }) }));
});
// main Dialog component
/**
 * WAI-ARIA-compliant dialog component. A dialog is a floating window that typically contains
 * a title bar and a content area.
 * The dialog is generally placed on top of the rest of the page content. It can be either modal
 * (only the content in the dialog can be interacted with) or non-modal (it's still possible
 * to interact with content outside of the dialog).
 */
const Dialog = ({ autoFocusRef, children, isOpen, header = '', footer, modality = 'modal', placement = 'center', offset = 0, onClose, onTransitionEnd, role = 'dialog', 'aria-labelledby': ariaLabelledBy, 'aria-describedby': ariaDescribedBy, testId }) => {
    const [state, setState] = useState(isOpen ? 'opening' : 'unmounted');
    const uniqueId = useId();
    const labelId = ariaLabelledBy ? ariaLabelledBy : uniqueId;
    useEffect(() => {
        // Ignore state update on initial render
        if (state === 'unmounted' && !isOpen) {
            return;
        }
        setState(isOpen ? 'opening' : 'closing');
    }, [isOpen, state]);
    const { nodeRef } = useAnimation(state, {
        animationStates,
        isAnimatedOnMount: true,
        onAnimationEnd: ({ animationState }) => {
            if (animationState === 'closing') {
                setState('unmounted');
                onTransitionEnd?.(false);
            }
            else {
                onTransitionEnd?.(true);
            }
        }
    });
    const handleKeyDown = (event) => {
        switch (event.code) {
            case 'Escape': {
                onClose?.({ reason: 'escapeKey' });
                break;
            }
        }
    };
    const renderDialog = () => {
        return (jsx(WindowOverlay, { placement: placement, offset: offset, children: jsx(FocusTrap, { autoFocusRef: autoFocusRef, children: jsx(StyledDialog, { ref: nodeRef, labelId: labelId, header: header, footer: footer, role: role, ariaLabelledBy: labelId, ariaDescribedBy: ariaDescribedBy, onKeyDown: handleKeyDown, testId: testId, children: children }) }) }));
    };
    if (modality === 'modal') {
        return jsx(Modal, { isOpen: state !== 'unmounted', children: renderDialog() });
    }
    else {
        return state !== 'unmounted' ? jsx(Layer, { children: renderDialog() }) : null;
    }
};
const animationStates = {
    opening: {
        from: {
            scaleX: 0,
            scaleY: 0,
            transformOrigin: 'center',
            opacity: 0
        },
        to: {
            scaleX: 1,
            scaleY: 1,
            opacity: 1
        },
        options: {
            duration: 200
        }
    },
    closing: {
        from: {
            scaleX: 1,
            scaleY: 1,
            transformOrigin: 'center',
            opacity: 1
        },
        to: {
            scaleX: 0,
            scaleY: 0,
            opacity: 0
        },
        options: {
            duration: 200
        }
    }
};

export { Dialog };
//# sourceMappingURL=UNSAFE_Dialog.js.map
