/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { useState, useRef, useEffect, useCallback } from 'preact/hooks';
import { L as Layer } from './Layer-c8a9c60e.js';
import 'preact/compat';
import './TopLayerHost-d50e5624.js';
import { M as Modal } from './Modal-006abcf9.js';
import { F as FocusTrap } from './FocusTrap-9a390118.js';
import { g as getActiveElement, b as focusWithin, f as focusOn } from './tabbableUtils-d03bccb0.js';
import { n as normalizePosition } from './positionUtils-c728c841.js';
import { g as getClientHints } from './clientHints-ee1c8473.js';
import { u as useOutsideClick } from './useOutsideClick-113d9979.js';
import { u as useAnimation } from './useAnimation-7d188187.js';
import { u as useUser } from './useUser-6e653d84.js';
import { u as useTestId } from './useTestId-5050f73e.js';
import { u as useSwipe } from './useSwipe-e77cb2c5.js';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { c as colorInterpolations } from './colors-8cd7269e.js';
import { m as mergeInterpolations } from './mergeInterpolations-f71c7446.js';
import { DrawerPopupRedwoodTheme } from './UNSAFE_DrawerPopup/themes/redwood/DrawerPopupTheme.js';
import './useThemeInterpolations-f5a207dc.js';
import './useColorScheme-daa151e2.js';
import './useScale-578a7ec2.js';
import './theme-651551cd.js';
import './Theme-99a9e50d.js';
import 'preact';
import './Common/themes/redwood/theme.js';
import './Common/themes/themeContract.css.js';
import './UNSAFE_Modal/themes/ModalStyles.css.js';
import './ModalStyles.styles.css';
import './vanilla-extract-recipes-createRuntimeFn.esm-8c3e2104.js';
import './FocusTracker-5f9c7635.js';
import './classNames-2e96f1d4.js';
import './PRIVATE_FocusTracker/themes/FocusTrackerStyles.css.js';
import './FocusTrackerStyles.styles.css';
import './head-6177894e.js';
import './_arity-703022b9.js';
import './_curry3-110d1c8b.js';
import './_curry1-b1d25277.js';
import './_curry2-3213c7b9.js';
import './_isArray-7f64b525.js';
import './_isString-c97680cd.js';
import './arrayUtils-bda0b35d.js';
import './logger-c710d0a3.js';
import './_isObject-6f2350ae.js';
import './UNSAFE_DrawerPopup/themes/redwood/DrawerPopupBaseTheme.css.js';
import './UNSAFE_DrawerPopup/themes/redwood/DrawerPopupBaseTheme.styles.css';
import './UNSAFE_DrawerPopup/themes/DrawerPopupStyles.css.js';
import './DrawerPopupStyles.styles.css';
import './UNSAFE_DrawerPopup/themes/redwood/DrawerPopupVariants.css.js';
import './DrawerPopupVariants.styles.css';

// Constants
const displayModefullWidthThreshold = 600;
const zero = '0px';
const duration = 250;
const getElementHeight = (element) => {
    return Math.round(element.offsetHeight);
};
const getElementWidth = (element) => {
    return Math.round(element.getBoundingClientRect().width);
};
/**
 * A drawer popup adds a single slide-in side content alongside some primary content to an application window.
 *
 * This drawer always overlays the page and can be placed at the 'start', 'end' or 'bottom' edge.
 */
const DrawerPopup = ({ children, isOpen = false, autoFocusRef, placement = 'start', modality = 'modal', onClose, onTransitionEnd, 'aria-label': ariaLabel, 'aria-labelledby': ariaLabelledBy, 'aria-describedby': ariaDescribedBy, testId, ...props }) => {
    // Initialize states
    const [status, setStatus] = useState(isOpen ? 'initial' : 'unmounted');
    const [visibility, setVisibility] = useState('hidden');
    const [isOutsideClickDisabled, setIsOutsideClickDisabled] = useState(!isOpen);
    const [displayModeHorizontal, setDisplayModeHorizontal] = useState('overlay');
    const rootRef = useRef(null);
    const launcherRef = useRef(null);
    const isBottomPlacement = placement === 'bottom';
    // Animation config
    const animationStates = {
        opening: () => {
            if (isBottomPlacement) {
                return {
                    from: {
                        translateY: getTranslateY()
                    },
                    to: {
                        translateY: zero
                    },
                    options: {
                        duration: duration
                    }
                };
            }
            else {
                return {
                    from: {
                        translateX: getTranslateX()
                    },
                    to: {
                        translateX: zero
                    },
                    options: {
                        duration: duration
                    }
                };
            }
        },
        closing: () => {
            if (isBottomPlacement) {
                return {
                    from: {
                        translateY: zero
                    },
                    to: {
                        translateY: getTranslateY()
                    },
                    options: {
                        duration: duration
                    }
                };
            }
            else {
                return {
                    from: {
                        translateX: zero
                    },
                    to: {
                        translateX: getTranslateX()
                    },
                    options: {
                        duration: duration
                    }
                };
            }
        }
    };
    // testId support
    const testIdProps = useTestId(testId);
    // Normalizing position
    const { direction } = useUser();
    const getNormalizedPlacement = () => {
        return normalizePosition(placement, direction);
    };
    const getTranslateX = () => {
        return `${getNormalizedPlacement() === 'right' ? '' : '-'}${getElementWidth(rootRef.current)}px`;
    };
    const getTranslateY = () => {
        return `${getElementHeight(rootRef.current)}px`;
    };
    // Animation
    const { nodeRef } = useAnimation(status, {
        animationStates,
        isAnimatedOnMount: true,
        onAnimationEnd: () => {
            if (status === 'opening') {
                onTransitionEnd?.(true);
            }
            else if (!isOpen) {
                nodeRef(null);
                setVisibility('hidden');
                setStatus('unmounted');
                onTransitionEnd?.(false);
            }
        }
    });
    useEffect(() => {
        // 1. Ignore status update on initial render
        if (status === 'unmounted' && !isOpen) {
            return;
        }
        // 2. Mount visually hidden component
        // Component gets mounted, but will be visually hidden to avoid flickering
        // Flickering happens because it gets rendered before animation hook repositions
        // it in next render cycle
        if (status === 'unmounted' && isOpen) {
            setStatus('initial');
            return;
        }
        // 3. Setup animation
        // Component has been mounted.
        // We will kick out animation as node is already present in DOM.
        // To avoid flickering, we still keep it visually hidden until the next render cycle.
        if (status === 'initial' && isOpen) {
            nodeRef(rootRef.current);
            setIsOutsideClickDisabled(false);
            setStatus('opening');
            return;
        }
        // 3. Unhide component. It becomes visible.
        if (status === 'opening' && isOpen) {
            setVisibility('visible');
            return;
        }
        // 4. Closing
        if (!isOpen) {
            setIsOutsideClickDisabled(true);
            setStatus('closing');
        }
    }, [isOpen, status, nodeRef]);
    // Launcher handler
    useEffect(() => {
        if (status === 'opening') {
            // Remember launcher
            const activeElement = getActiveElement();
            launcherRef.current = activeElement === document.body ? document.body : activeElement;
            // Register F6 key handler to enter the Popup
            launcherRef.current?.addEventListener('keydown', launcherKeyDownCallback);
        }
        if (status === 'closing') {
            return () => {
                // Deregister F6 key handler
                launcherRef.current?.removeEventListener('keydown', launcherKeyDownCallback);
            };
        }
        return;
    }, [status]);
    const launcherKeyDownCallback = (event) => {
        // Moves focus from launcher (must have focus) back to the Drawer
        if (launcherRef.current === getActiveElement() && event.code === 'F6') {
            // Prevent default F6 handlers.
            // F6 is a standard Chrome address bar shortcut on Windows.
            event.preventDefault();
            event.stopPropagation();
            // Focus on first tabbable in a container or container itself if there is none
            focusWithin(rootRef.current);
        }
    };
    const handleKeyDown = useCallback((event) => {
        const launcherEl = launcherRef.current;
        switch (event.code) {
            case 'Escape': {
                onClose?.({ reason: 'escapeKey' });
                break;
            }
            case 'F6':
                // Prevent default F6 handlers.
                // F6 is a standard Chrome address bar shortcut on Windows.
                event.preventDefault();
                event.stopPropagation();
                // Focus launcher
                if (launcherEl) {
                    focusOn(launcherEl);
                }
                break;
        }
    }, [onClose]);
    // OutsideClick handler
    const handleOutsideClick = useCallback(() => {
        // Trigger onClose event with outside click detail
        onClose?.({ reason: 'outsideClick' });
    }, [onClose]);
    useOutsideClick({
        isDisabled: isOutsideClickDisabled,
        ref: rootRef,
        handler: handleOutsideClick
    });
    // Drawer placement
    const getDrawerPlacement = () => {
        if (isBottomPlacement) {
            return placement;
        }
        else {
            if (direction === 'rtl') {
                return placement === 'start' ? 'right' : 'left';
            }
            else {
                return placement === 'start' ? 'left' : 'right';
            }
        }
    };
    const clientHints = getClientHints();
    const getViewportWidth = useCallback(() => {
        if (clientHints.platform === 'ios') {
            // On ios window.innerWidth is not recommended way of measuring the viewport
            return document.documentElement.clientWidth;
        }
        return window.innerWidth;
    }, [clientHints.platform]);
    // Window resize handler
    // Handles display mode: Overlay <> Full overlay (full width side drawers)
    const handleResize = useCallback(() => {
        const viewportWidth = getViewportWidth();
        if (displayModeHorizontal === 'overlay' && viewportWidth < displayModefullWidthThreshold) {
            setDisplayModeHorizontal('fullOverlay');
        }
        if (displayModeHorizontal === 'fullOverlay' && viewportWidth > displayModefullWidthThreshold) {
            setDisplayModeHorizontal('overlay');
        }
    }, [getViewportWidth, displayModeHorizontal]);
    useEffect(() => {
        const root = rootRef.current;
        if (root && status === 'opening') {
            const resizeObserver = new ResizeObserver(() => {
                handleResize();
            });
            resizeObserver.observe(document.body);
            return () => {
                resizeObserver.unobserve(document.body);
            };
        }
        return undefined;
    }, [status, handleResize]);
    // OutsideClick handler
    const swipeHandler = useCallback((event) => {
        const swipeDir = event.direction;
        if ((direction === 'ltr' && placement === 'start' && swipeDir === 'left') ||
            (placement === 'end' && swipeDir === 'right') ||
            (direction === 'rtl' && placement === 'start' && swipeDir === 'right') ||
            (placement === 'end' && swipeDir === 'left') ||
            (placement === 'bottom' && swipeDir === 'down')) {
            onClose?.({ reason: 'swipe' });
        }
    }, [direction, placement, onClose]);
    const swipeOptions = {
        threshold: 50,
        maximumTime: 750,
        tolerance: 40
    };
    const { swipeProps } = useSwipe(swipeHandler, swipeOptions);
    // Styles
    const { classes } = useComponentTheme(DrawerPopupRedwoodTheme, {
        placement: getDrawerPlacement(),
        visibility,
        displayModeHorizontal: ['start', 'end'].indexOf(placement) > -1 ? displayModeHorizontal : undefined
    });
    const interpolations = [...Object.values(colorInterpolations)];
    const styleInterpolations = mergeInterpolations(interpolations);
    const { ...styles } = styleInterpolations(props);
    const renderDrawer = () => {
        return (jsx("div", { ref: rootRef, className: classes, style: styles, tabIndex: -1, role: "dialog", onKeyDown: handleKeyDown, "aria-label": ariaLabel, "aria-labelledby": ariaLabelledBy, "aria-describedby": ariaDescribedBy, ...swipeProps, ...testIdProps, children: jsx(FocusTrap, { autoFocusRef: autoFocusRef, children: children }) }));
    };
    const isMounted = status !== 'unmounted';
    if (isMounted) {
        // Modal vs. modelss upper wrapper
        if (modality === 'modal') {
            return jsx(Modal, { isOpen: isMounted, children: renderDrawer() });
        }
        else {
            return jsx(Layer, { children: renderDrawer() });
        }
    }
    return null;
};

export { DrawerPopup };
//# sourceMappingURL=UNSAFE_DrawerPopup.js.map
