/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx, jsxs } from 'preact/jsx-runtime';
import { useCallback, useContext, useState } from 'preact/hooks';
import { createContext, toChildArray } from 'preact';
import { u as useComponentTheme } from './useComponentTheme-7202d4c1.js';
import { u as useTestId } from './useTestId-5050f73e.js';
import { TrainRedwoodTheme } from './UNSAFE_Train/themes/redwood/TrainTheme.js';
import { u as usePress } from './usePress-6e2cde1b.js';
import { u as useInteractionStyle } from './useInteractionStyle-43b76a9a.js';
import { c as classNames } from './classNames-2e96f1d4.js';
import { m as mergeProps } from './mergeProps-dfdba053.js';
import { S as SvgSuccessS } from './SuccessS-fd0eaab1.js';
import { S as SvgErrorS } from './ErrorS-a4c64c74.js';
import { S as SvgInformationS } from './InformationS-219d8de8.js';
import { S as SvgWarningS } from './WarningS-542fff95.js';
import { u as useActive } from './useActive-ac391e31.js';
import { u as useTranslationBundle } from './useTranslationBundle-95f6039e.js';
import { H as HiddenAccessible } from './HiddenAccessible-653c0211.js';
import './logger-c710d0a3.js';
import './TopLayerHost-d50e5624.js';
import './Common/themes/redwood/theme.js';
import './Common/themes/themeContract.css.js';
import 'preact/compat';
import './UNSAFE_Train/themes/TrainStyles.css.js';
import './TrainStyles.styles.css';
import './UNSAFE_Train/themes/redwood/TrainBaseTheme.css.js';
import './UNSAFE_Train/themes/redwood/TrainBaseTheme.styles.css';
import './UNSAFE_Train/themes/redwood/TrainVariants.css.js';
import './TrainVariants.styles.css';
import './vanilla-extract-recipes-createRuntimeFn.esm-8c3e2104.js';
import './clientHints-ee1c8473.js';
import './useHover-693303bd.js';
import './useToggle-7589399a.js';
import './Icon-715e5a9e.js';
import './size-aa934024.js';
import './utils-4aa3c8ad.js';
import './colorUtils-30dc7d3f.js';
import './_curry1-b1d25277.js';
import './useTooltip-cb4418ec.js';
import './useTooltipControlled-fe816388.js';
import './useId-1bde71ab.js';
import './Floating-9ce291ba.js';
import './useFloating-cec1aeb7.js';
import './useUser-6e653d84.js';
import './positionUtils-c728c841.js';
import './refUtils-54b2b0a5.js';
import './useOutsideClick-113d9979.js';
import './arrayUtils-bda0b35d.js';
import './UNSAFE_Floating/themes/redwood/FloatingTheme.js';
import './UNSAFE_Floating/themes/FloatingStyles.css.js';
import './FloatingStyles.styles.css';
import './UNSAFE_Floating/themes/redwood/FloatingBaseTheme.css.js';
import './UNSAFE_Floating/themes/redwood/FloatingBaseTheme.styles.css';
import './UNSAFE_Floating/themes/redwood/FloatingVariants.css.js';
import './vanilla-extract-dynamic.esm-4479b60e.js';
import './UNSAFE_Floating/themes/FloatingContract.css.js';
import './Layer-c8a9c60e.js';
import './useThemeInterpolations-f5a207dc.js';
import './useColorScheme-daa151e2.js';
import './useScale-578a7ec2.js';
import './theme-651551cd.js';
import './Theme-99a9e50d.js';
import './mergeInterpolations-f71c7446.js';
import './_curry3-110d1c8b.js';
import './_curry2-3213c7b9.js';
import './_isObject-6f2350ae.js';
import './useFocus-1c5b1863.js';
import './useTouch-c052c62f.js';
import './useAnimation-7d188187.js';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentTheme.js';
import './hooks/UNSAFE_useTooltip/themes/TooltipContentStyles.css.js';
import './TooltipContentStyles.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentBaseTheme.css.js';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentBaseTheme.styles.css';
import './hooks/UNSAFE_useTooltip/themes/redwood/TooltipContentVariants.css.js';
import './TooltipContentVariants.styles.css';
import './EnvironmentProvider-12bc1f90.js';
import './LayerManager-587f5086.js';
import './UNSAFE_Icon/themes/IconStyle.css.js';
import './IconStyle.styles.css';
import './HiddenAccessibleStyles.styles.css';

const Context = createContext(null);

/**
The JET Train component serves as a visual navigator enabling users to traverse through different 'steps'. Each 'step' is represented by the Step sub-component which can manifest its own state - 'visited', 'unvisited', or 'disabled', and is capable of communicating various message types including 'error', 'confirmation', 'warning', or 'info'.
 */
function Train({ onSelect, selectedStep, children, testId }) {
    const { classes, styles } = useComponentTheme(TrainRedwoodTheme);
    const testIdProps = useTestId(testId);
    const selectedStepIndex = children.findIndex((step) => step.props.id === selectedStep);
    const onStepSelect = useCallback((event, stepId) => {
        onSelect?.({
            event: event,
            toStep: stepId
        });
    }, [onSelect]);
    const trainContext = {
        onSelect: onStepSelect,
        selectedStep: selectedStep,
        selectedStepIndex: selectedStepIndex,
        numberOfSteps: children.length,
        index: 0
    };
    const childrenArray = toChildArray(children).filter(Boolean);
    return (jsx("div", { class: classes, ...testIdProps, children: childrenArray.map((step, index) => (jsx(Context.Provider, { value: { ...trainContext, index }, children: jsx("div", { class: styles.baseListStyle, children: step }) }))) }));
}

const Connector = ({ isConnected }) => {
    const { styles, variantClasses } = useComponentTheme(TrainRedwoodTheme, {
        connectorConnected: isConnected ? 'isConnected' : 'notConnected'
    });
    return jsx("span", { class: classNames([styles.connectorBaseStyle, variantClasses]) });
};

const messageTypeIcons = {
    confirmation: SvgSuccessS,
    error: SvgErrorS,
    fatal: SvgErrorS,
    info: SvgInformationS,
    warning: SvgWarningS
};
/**
 * UserChosenIcon Component for rendering the severity based icon in Message
 */
function UserChosenIcon({ messageType, isDisabled = false }) {
    const IconComponent = messageTypeIcons[messageType];
    const { styles, variantClasses } = useComponentTheme(TrainRedwoodTheme, {
        messageType: messageType,
        stepIconDisabled: isDisabled ? 'isDisabled' : 'notDisabled'
    });
    return (jsx("div", { class: classNames([styles.messageTypeIconBaseStyle, variantClasses]), role: "presentation", children: jsx(IconComponent, {}) }));
}

/**
 * Step icon
 */
const StepIcon = ({ isDisabled, isVisited, messageType }) => {
    const { index, selectedStepIndex } = useContext(Context);
    const isCurrent = selectedStepIndex === index;
    const { interactionProps, applyPseudoHoverStyle, applyHoverStyle } = useInteractionStyle(isDisabled || isCurrent);
    const { isActive, activeProps } = useActive({ isDisabled: isDisabled || isCurrent });
    console.log('isActive', isActive);
    const mergedProps = mergeProps(interactionProps, activeProps);
    const { styles, variantClasses } = useComponentTheme(TrainRedwoodTheme, {
        stepIconDisabled: isDisabled ? 'isDisabled' : 'notDisabled',
        stepIconCurrent: isCurrent ? 'isCurrent' : 'notCurrent',
        stepIconVisited: isVisited ? 'isVisited' : 'notVisited',
        stepIconNeedsEventsHover: applyHoverStyle ? 'isNeedsEventsHover' : 'notNeedsEventsHover',
        stepIconPseudoHover: applyPseudoHoverStyle ? 'isPseudoHover' : 'notPseudoHover',
        stepIconActive: isActive ? 'isActive' : 'notActive'
    });
    const iconStyles = () => {
        if (messageType) {
            return jsx(UserChosenIcon, { messageType: messageType, isDisabled: isDisabled });
        }
        else {
            return (jsx("span", { ...mergedProps, class: classNames([styles.stepIconBaseStyle, variantClasses]), children: index + 1 }));
        }
    };
    return jsx("span", { class: styles.stepIconContainerStyle, children: iconStyles() });
};

/**
 * Step label component
 */
const StepLabel = ({ label, isDisabled, isVisited, messageType }) => {
    const { numberOfSteps, index, selectedStepIndex } = useContext(Context);
    const translations = useTranslationBundle('@oracle/oraclejet-preact');
    const isCurrent = selectedStepIndex === index;
    const { interactionProps, applyPseudoHoverStyle, applyHoverStyle } = useInteractionStyle(isDisabled || isCurrent);
    const currentStepStatus = isCurrent
        ? translations.train_current()
        : isVisited
            ? translations.train_visited()
            : translations.train_not_visited();
    const currentStep = index + 1;
    const statusTextForCurrentStep = messageType
        ? `${translations.train_status({
            currentStep: currentStep.toString(),
            numberOfSteps: numberOfSteps.toString()
        })}. ${currentStepStatus}, ${translations.train_message_type()}: ${messageType}`
        : `${translations.train_status({
            currentStep: currentStep.toString(),
            numberOfSteps: numberOfSteps.toString()
        })}. ${currentStepStatus}`;
    const { styles, variantClasses } = useComponentTheme(TrainRedwoodTheme, {
        stepLabelDisabled: isDisabled ? 'isDisabled' : 'notDisabled',
        stepLabelCurrent: isCurrent ? 'isCurrent' : 'notCurrent',
        stepLabelVisited: isVisited ? 'isVisited' : 'notVisited',
        stepLabelNeedsEventsHover: applyHoverStyle ? 'isNeedsEventsHover' : 'notNeedsEventsHover',
        stepLabelPseudoHover: applyPseudoHoverStyle ? 'isPseudoHover' : 'notPseudoHover'
    });
    const handleClick = (event) => {
        event.preventDefault();
    };
    return (jsx("span", { class: styles.stepLabelWrapperStyle, children: jsxs("a", { ...interactionProps, href: "#", class: classNames([styles.stepLabelBaseStyle, variantClasses]), onClick: handleClick, children: [label, jsx(HiddenAccessible, { children: statusTextForCurrentStep })] }) }));
};

/**
 * Step component
 */
const Step = ({ id, isDisabled, isVisited, messageType, label }) => {
    const { onSelect, /* selectedStep ,*/ numberOfSteps, selectedStepIndex, index } = useContext(Context) || // get the context object or...
        (() => {
            // if there's no context object, throw an error
            throw new Error('Step component must be rendered within a Train component.');
        })();
    const isCurrent = selectedStepIndex === index;
    const [isActive, setIsActive] = useState(false);
    const { interactionProps, applyPseudoHoverStyle, applyHoverStyle } = useInteractionStyle(isDisabled || isCurrent);
    const isConnectorConnected = selectedStepIndex > index;
    const onSelectHandler = useCallback((event) => {
        if (!isDisabled && !isCurrent) {
            onSelect(event, id);
        }
    }, [isDisabled, isCurrent, onSelect, id]);
    const { pressProps } = usePress(onSelectHandler, {});
    const isLast = index !== numberOfSteps - 1;
    const onMouseUpHandler = useCallback(() => {
        setIsActive(false);
    }, []);
    const onMouseDownHandler = useCallback(() => {
        setIsActive(true);
    }, []);
    const { styles, variantClasses } = useComponentTheme(TrainRedwoodTheme, {
        stepDisabled: isDisabled ? 'isDisabled' : 'notDisabled',
        needsEventsHover: applyHoverStyle ? 'isNeedsEventsHover' : 'notNeedsEventsHover',
        pseudoHover: applyPseudoHoverStyle ? 'isPseudoHover' : 'notPseudoHover',
        stepActive: isActive && !isCurrent ? 'isActive' : 'notActive'
    });
    const mergedProps = mergeProps(pressProps, interactionProps, {
        onMouseUp: onMouseUpHandler,
        onMouseDown: onMouseDownHandler
    });
    return (jsxs("span", { class: styles.stepContainerStyle, children: [isLast && jsx(Connector, { isConnected: isConnectorConnected }), jsxs("span", { ...mergedProps, class: classNames([styles.stepBaseStyle, variantClasses]), children: [jsx(StepIcon, { isVisited: isVisited, isDisabled: isDisabled, messageType: messageType }), jsx(StepLabel, { label: label, isVisited: isVisited, isDisabled: isDisabled, messageType: messageType })] })] }));
};

export { Step, Train };
//# sourceMappingURL=UNSAFE_Train.js.map
