/* @oracle/oraclejet-preact: 16.0.1 */
import { jsxs, jsx } from 'preact/jsx-runtime';
import { styles } from './hooks/UNSAFE_useChartDataCursor/themes/DataCursorStyles.css.js';

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
function getDataCursorCoords(xScale, x, plotAreaSpace, isHoriz, dataCursorStyle) {
    const lineColor = dataCursorStyle?.lineColor;
    const lineStyle = dataCursorStyle?.lineStyle;
    const y1 = isHoriz ? xScale.transform(x) : plotAreaSpace.y + plotAreaSpace.height;
    const x1 = isHoriz ? plotAreaSpace.x : xScale.transform(x);
    const y2 = isHoriz ? xScale.transform(x) : plotAreaSpace.y;
    const x2 = isHoriz ? plotAreaSpace.x + plotAreaSpace.width : xScale.transform(x);
    const lineWidth = dataCursorStyle?.lineWidth || 1;
    return {
        mainLineProps: {
            x1,
            y1,
            x2,
            y2,
            style: {
                stroke: lineColor,
                strokeDashArray: lineStyle === 'dotted' ? 3 : lineStyle === 'dashed' ? 6 : undefined,
                strokeWidth: lineWidth
            }
        },
        contrastLineProps: {
            x1: isHoriz ? x1 : x1 + lineWidth,
            y1: isHoriz ? y1 + lineWidth : y1,
            x2: isHoriz ? x2 : x2 + lineWidth,
            y2: isHoriz ? y2 + lineWidth : y2
        }
    };
}

function useChartDataCursor({ findNearest, dataCursorPosition, onDataCursorPositionChange, isDataCursorEnabled, isHoriz, plotAreaSpace, yScale, xScale, dataCursorStyle, focusedItemInfo, getDataItemPos }) {
    let dataCursorContent = null;
    const x = dataCursorPosition?.x;
    const isSnap = dataCursorStyle?.behavior === 'snap';
    const updateDataCursor = () => {
        const { seriesIndex, groupIndex } = focusedItemInfo;
        const elemBounds = getDataItemPos(seriesIndex, groupIndex);
        if (!elemBounds) {
            return;
        }
        const px = elemBounds?.x + elemBounds.width / 2;
        const py = elemBounds?.y + elemBounds.height / 2;
        let X = xScale.invert(isHoriz ? py : px);
        let Y = yScale.invert(isHoriz ? px : py);
        const neareastItemInfo = findNearest(X, Y);
        if (isSnap) {
            X = xScale.invert(neareastItemInfo.xCoord);
            Y = yScale.invert(neareastItemInfo.yCoord);
        }
        onDataCursorPositionChange?.({ x: X, y: Y });
    };
    const onPointerMove = (event) => {
        const px = event.offsetX;
        const py = event.offsetY;
        if (px < plotAreaSpace.x ||
            py < plotAreaSpace.y ||
            px > plotAreaSpace.x + plotAreaSpace.width ||
            py > plotAreaSpace.y + plotAreaSpace.height) {
            dataCursorContent = null;
            onDataCursorPositionChange?.({ x: undefined, y: undefined });
            return;
        }
        let X = xScale.invert(isHoriz ? py : px);
        let Y = yScale.invert(isHoriz ? px : py);
        const neareastItemInfo = findNearest(X, Y);
        if (isSnap) {
            X = xScale.invert(neareastItemInfo.xCoord);
            Y = yScale.invert(neareastItemInfo.yCoord);
        }
        onDataCursorPositionChange?.({ x: X, y: Y });
    };
    const onPointerLeave = () => {
        onDataCursorPositionChange?.({ x: undefined, y: undefined });
    };
    const onKeyDown = (event) => {
        if (event.key === 'Tab') {
            onDataCursorPositionChange?.({ x: undefined, y: undefined });
            return;
        }
        updateDataCursor();
    };
    const onKeyUp = (event) => {
        if (event.key === 'Tab' && focusedItemInfo.isCurrent) {
            return;
        }
        updateDataCursor();
    };
    if (x != undefined && isDataCursorEnabled) {
        const { mainLineProps, contrastLineProps } = getDataCursorCoords(xScale, x, plotAreaSpace, isHoriz, dataCursorStyle);
        dataCursorContent = (jsxs("g", { class: styles.dataCursor, children: [jsx("line", { ...mainLineProps, class: styles.dataCursorLine }), jsx("line", { ...contrastLineProps, class: styles.contrastLine })] }));
    }
    return {
        dataCursorContent,
        dataCursorProps: isDataCursorEnabled
            ? {
                onPointerMove,
                onPointerLeave,
                onKeyDown,
                onKeyUp
            }
            : {}
    };
}

export { useChartDataCursor as u };
//# sourceMappingURL=useChartDataCursor-cbef76df.js.map
