/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx } from 'preact/jsx-runtime';
import { L as Layer } from './Layer-c8a9c60e.js';
import 'preact/compat';
import './TopLayerHost-d50e5624.js';
import { F as Floating } from './Floating-9ce291ba.js';
import { useRef, useState } from 'preact/hooks';
import { styles } from './hooks/PRIVATE_useDatatip/themes/useDatatip.css.js';
import { u as useId } from './useId-1bde71ab.js';

/**
 * The default offset for datatip mainAxis.
 */
const DEFAULT_OFFSET = 8;
/**
 * The hook to support custom datatip in visualization components.
 */
function useDatatip({ content, placement = 'top-end-corner', offset, borderColor, anchor }) {
    const uniqueIdRef = useRef(useId());
    const [coords, setCoords] = useState({ x: 0, y: 0 });
    const offsetValue = offset || { mainAxis: DEFAULT_OFFSET, crossAxis: -1 * DEFAULT_OFFSET };
    const [isVisible, setIsVisible] = useState(false);
    const targetRef = useRef(null);
    const toggleTimeoutRef = useRef();
    const onPointerMove = (event) => {
        setCoords({ x: event.clientX, y: event.clientY });
        setIsVisible(true);
    };
    const onFocus = (event) => {
        if (event.eventPhase === Event.AT_TARGET || event.eventPhase === Event.BUBBLING_PHASE) {
            targetRef.current = event.target;
            setIsVisible(true);
        }
    };
    const onPointerEnter = (event) => {
        if (event.eventPhase === Event.AT_TARGET || event.eventPhase === Event.BUBBLING_PHASE) {
            targetRef.current = event.currentTarget;
            setIsVisible(true);
        }
    };
    const onBlur = () => {
        setIsVisible(false);
    };
    const onContentPointerEnter = () => {
        clearTimeout(toggleTimeoutRef.current);
    };
    const onPointerLeave = () => {
        toggleTimeoutRef.current = setTimeout(() => {
            setIsVisible(false);
        }, 20);
    };
    const onKeyUp = (event) => {
        if (event.code === 'Escape') {
            event.preventDefault();
            setIsVisible(false);
        }
        else if (event.code === 'ArrowUp' ||
            event.code === 'ArrowDown' ||
            event.code === 'ArrowRight' ||
            event.code === 'ArrowLeft' ||
            event.code === 'Tab') {
            setIsVisible(true);
        }
    };
    const anchorRef = anchor === 'element' ? targetRef : { current: coords };
    const datatipContent = isVisible && content != null ? (jsx(Layer, { children: jsx(Floating, { anchorRef: anchorRef, placement: placement, offsetValue: offsetValue, flipOptions: { crossAxis: false, mainAxis: false }, shiftOptions: { crossAxis: true, mainAxis: true }, children: jsx("div", { role: "tooltip", id: uniqueIdRef.current, className: styles.dvtDatatipStyle, style: { borderColor }, onPointerEnter: onContentPointerEnter, onPointerLeave: onPointerLeave, children: content }) }) })) : null;
    return {
        datatipContent,
        datatipProps: {
            'aria-describedby': uniqueIdRef.current,
            onKeyUp,
            onPointerLeave,
            onPointerEnter,
            onPointerMove,
            onFocus,
            onBlur
        }
    };
}

export { useDatatip as u };
//# sourceMappingURL=useDatatip-1db9113a.js.map
