/* @oracle/oraclejet-preact: 16.0.1 */
import { useCallback } from 'preact/hooks';
import { k as keyExtractor } from './collectionUtils-ed8b8b4c.js';

const useItemAction = (currentKey, dataState, onItemAction, itemSelector, allowSpacebar) => {
    const onClick = useCallback((event) => {
        const key = keyExtractor(event.target, itemSelector);
        if (handleItemAction(key, dataState, onItemAction)) {
            event.stopPropagation();
        }
    }, [dataState, onItemAction, itemSelector]);
    const onKeyDown = useCallback((event) => {
        if (event.key === 'Enter') {
            handleItemAction(currentKey, dataState, onItemAction);
        }
        else if (allowSpacebar && event.key === ' ') {
            handleItemAction(currentKey, dataState, onItemAction);
            event.preventDefault();
        }
    }, [dataState, onItemAction, currentKey, allowSpacebar]);
    return !dataState || !onItemAction
        ? {}
        : {
            onClick,
            onKeyDown
        };
};
const handleItemAction = (key, dataState, onItemAction) => {
    if (key !== undefined) {
        const context = findItemContext(key, dataState);
        context && onItemAction({ context });
        return context !== undefined;
    }
    return false;
};
// returns a ListItemContext based on key
const findItemContext = (key, dataState) => {
    const index = dataState?.data?.findIndex((d) => d.metadata.key === key) ?? -1;
    if (index > -1) {
        const context = {
            index: index + (dataState?.offset || 0),
            data: dataState?.data[index].data,
            metadata: dataState?.data[index].metadata
        };
        return context;
    }
    return undefined;
};

export { useItemAction as u };
//# sourceMappingURL=useItemAction-62c293eb.js.map
