/* @oracle/oraclejet-preact: 16.0.1 */
import { jsx, jsxs } from 'preact/jsx-runtime';
import { c as classNames } from './classNames-2e96f1d4.js';
import { S as SvgSymbol } from './SvgSymbol-9a3a7510.js';
import { styles } from './UNSAFE_Legend/themes/LegendStyles.css.js';
import { T as Text } from './Text-ec0e2d47.js';
import { forwardRef } from 'preact/compat';
import { e as getItemProps, f as getGridCellIndices, h as getItemInfo, i as isEqualItem, g as getRandomId } from './LegendUtils-5b720c38.js';
import { useState, useRef, useLayoutEffect, useMemo } from 'preact/hooks';
import { u as useUser } from './useUser-6e653d84.js';
import { u as useTooltipControlled } from './useTooltipControlled-fe816388.js';
import { u as useTextDimensions } from './useTextDimensions-16f92049.js';

function ImageMarker(props) {
    const { imageMarkerStyle } = styles;
    return jsx("img", { class: imageMarkerStyle, src: props.source });
}

const LegendSymbol = ({ 
// The default color is not in the redwood pallete.
// TODO: replace default legend color with UX approved pallete color when available.
markerColor = '#a6acb1', lineColor = '#a6acb1', markerShape = 'square', isHidden = false, lineStyle = 'none', ...props }) => {
    const { legendSymbolBaseStyle } = styles;
    const dimensionsNotSet = props.width == null && props.height == null;
    const setAspectRatioNone = !dimensionsNotSet && (markerShape === 'rectangle' || markerShape === 'ellipse');
    const defaultLineWidth = lineStyle != 'none' && markerShape != 'none' ? 2 : 3;
    return (jsx("div", { className: legendSymbolBaseStyle, style: { width: props.width, height: props.height }, children: props.source ? (jsx(ImageMarker, { source: props.source })) : (jsx(SvgSymbol, { setAspectRatioNone: setAspectRatioNone, markerShape: isHidden ? 'square' : markerShape, lineStyle: isHidden ? 'none' : lineStyle, lineColor: lineColor, lineLength: props.lineLength, lineWidth: props.lineWidth != null ? props.lineWidth : defaultLineWidth, markerColor: isHidden ? 'transparent' : markerColor, borderColor: isHidden ? markerColor || lineColor : props.borderColor, setAbsolutePos: true })) }));
};

/**
 * Returns the legend text component.
 */
const LegendText = ({ text, type = 'label', id, align = 'start', ...props }) => {
    const { legendTextBaseStyle, legendTextTitle, legendTitleStartAlign, legendTitleCenterAlign, legendTitleEndAlign } = styles;
    const isTitle = type === 'title';
    return (jsx("div", { class: classNames([
            legendTextBaseStyle,
            isTitle ? legendTextTitle : '',
            isTitle && align === 'start' ? legendTitleStartAlign : '',
            isTitle && align === 'center' ? legendTitleCenterAlign : '',
            isTitle && align === 'end' ? legendTitleEndAlign : ''
        ]), style: {
            ...props
        }, children: jsx(Text, { size: "inherit", variant: "inherit", weight: "inherit", truncation: 'ellipsis', id: id, children: text }) }));
};

const BaseLegendItem = forwardRef(({ id, text, sectionIdx, itemIdx, isHighlighted, symbolHeight, symbolWidth, isFocused, isCurrent, isHidden, ...props }, ref) => {
    const textStyles = {
        fontFamily: props.textFontFamily,
        fontSize: props.textFontSize,
        color: props.textColor,
        fontStyle: props.textFontStyle,
        fontWeight: props.textFontWeight,
        textDecoration: props.textTextDecoration
    };
    const symbolProps = {
        lineStyle: props.lineStyle,
        lineWidth: props.lineWidth,
        markerShape: props.markerShape,
        markerColor: props.markerColor,
        source: props.source,
        borderColor: props.borderColor,
        lineColor: props.lineColor
    };
    const isNotDimmed = isHighlighted || isHighlighted == null;
    const { legendItemBaseStyle, legendItemIsCurrent, legendItemOpacity, legendItemFocusRing } = styles;
    return (jsxs("div", { class: classNames([
            legendItemBaseStyle,
            isCurrent && isHighlighted == null ? legendItemIsCurrent : '',
            !isNotDimmed ? legendItemOpacity : '',
            isFocused ? legendItemFocusRing : '',
            props.class
        ]), style: {
            gridRow: props.gridRow,
            gridColumn: props.gridCol,
            minWidth: props.minWidth !== undefined ? `${props.minWidth}px` : undefined
        }, id: id, ref: ref, "data-oj-section": sectionIdx, "data-oj-item": itemIdx, "aria-labelledby": props.labelledBy, role: isHidden != null ? 'menuitemcheckbox' : 'img', "aria-checked": isHidden != null ? !isHidden : undefined, "aria-label": props['aria-label'] || text, children: [jsx(LegendSymbol, { ...symbolProps, isHidden: isHidden, lineLength: symbolWidth, width: symbolWidth != null ? `${symbolWidth}px` : undefined, height: symbolHeight != null ? `${symbolHeight}px` : undefined }), jsx(LegendText, { ...textStyles, text: text, type: "label" })] }));
});

/**
 * The controlled legend reponsible only for rendering the legend items.
 */
const BaseLegend = ({ orientation = 'horizontal', highlightedIds, hiddenIds, focusedItemInfo, hoveredItemInfo, sectionIdx, activeId, isReadOnly, items, symbolHeight, symbolWidth, ...props }) => {
    const isHoriz = orientation === 'horizontal';
    const columTemplates = isHoriz ? `repeat(${props.itemsPerRow}, ${props.itemWidth})` : undefined;
    const { baseLegendStyles, baseLegendHorizontal, baseLegendVertical, baseLegendVerticalItem, baseLegendPaddingBottom, baseLegendPaddingTop, baseLegendPaddingEnd } = styles;
    return (jsx("div", { className: classNames([
            baseLegendStyles,
            isHoriz ? baseLegendHorizontal : baseLegendVertical
        ]), style: {
            gridTemplateColumns: columTemplates,
            // set maxWidth to 100% on second render for text truncation to work.
            maxWidth: '100%'
        }, children: items.map((item, index) => {
            const itemProps = getItemProps(item, index, sectionIdx, focusedItemInfo, isReadOnly, hoveredItemInfo, hiddenIds, highlightedIds);
            const isFocusedItem = focusedItemInfo.sectionIdx === sectionIdx && focusedItemInfo.itemIdx === index;
            const { gridRow, gridCol } = getGridCellIndices(items.length, index, props.itemsPerRow, props.itemsPerCol);
            const needsBottomSpace = gridRow != null && gridRow != props.itemsPerCol;
            const needsTopSpace = gridRow != null && gridRow != 1;
            return (jsx(BaseLegendItem, { ref: isFocusedItem ? props.focusedItemRef : undefined, itemIdx: index, sectionIdx: sectionIdx, symbolHeight: symbolHeight, symbolWidth: symbolWidth, ...props, ...itemProps, gridRow: gridRow, gridCol: gridCol, minWidth: props.itemWidth, id: itemProps.isCurrent ? activeId : '', class: classNames([
                    isHoriz ? '' : baseLegendVerticalItem,
                    needsTopSpace ? baseLegendPaddingTop : '',
                    needsBottomSpace ? baseLegendPaddingBottom : '',
                    baseLegendPaddingEnd
                ]) }));
        }) }));
};

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
function useEvents(isInteractive, isHideShow, getDetailFromInfo, getPrevItemInfo, getNextItemInfo, onItemAction, onInput) {
    const [focusedItemInfo, setfocusedItemInfo] = useState({
        itemIdx: 0,
        sectionIdx: 0
    });
    const [hoveredItemInfo, sethoveredItemInfo] = useState();
    const activeId = useRef();
    const { direction } = useUser();
    const isRtl = direction === 'rtl';
    const cancelEvent = (event) => {
        event.preventDefault();
        event.stopPropagation();
    };
    const hoverHandler = (event) => {
        const itemInfo = getItemInfo(event.target);
        if (!isEqualItem(itemInfo, hoveredItemInfo)) {
            sethoveredItemInfo(itemInfo && { ...itemInfo, isCurrent: true });
            activeId.current = getRandomId();
            if (isInteractive) {
                focusedItemInfo ? setfocusedItemInfo({ ...focusedItemInfo, isCurrent: false }) : '';
                onInput?.(getDetailFromInfo(itemInfo));
            }
        }
    };
    const pointerLeaveHandler = () => {
        sethoveredItemInfo(undefined);
        activeId.current = undefined;
        if (isInteractive) {
            onInput?.(getDetailFromInfo());
        }
    };
    if (!isInteractive) {
        return {
            focusedItemInfo,
            hoveredItemInfo,
            // include pointer leave and pointer move for datatip
            onPointerLeave: pointerLeaveHandler,
            onPointerMove: hoverHandler
        };
    }
    const updatefocusedItemInfo = (itemInfo) => {
        onInput?.(getDetailFromInfo(itemInfo));
        activeId.current = getRandomId();
        setfocusedItemInfo(itemInfo);
    };
    const handleKeyboardInput = (item) => {
        if (!isEqualItem(item, focusedItemInfo)) {
            item.isCurrent = true;
            item.isFocusVisible = true;
            if (hoveredItemInfo) {
                sethoveredItemInfo({ ...hoveredItemInfo, isCurrent: false });
            }
            updatefocusedItemInfo(item);
        }
    };
    const triggerOnAction = () => {
        const { sectionId, itemId } = getDetailFromInfo(focusedItemInfo);
        if (itemId != null) {
            onItemAction?.({ sectionId, itemId });
            activeId.current = getRandomId();
        }
    };
    const pointerUpHandler = (event) => {
        const itemInfo = getItemInfo(event.target);
        if (itemInfo != null) {
            setfocusedItemInfo(itemInfo);
            const { sectionId, itemId } = getDetailFromInfo(itemInfo);
            if (itemId != null) {
                onItemAction?.({ sectionId, itemId });
                activeId.current = getRandomId();
            }
        }
    };
    const keyDownHandler = (event) => {
        const key = event.key;
        switch (key) {
            case 'Tab':
                return;
            case 'ArrowDown': {
                const nextItemInfo = getNextItemInfo(focusedItemInfo);
                handleKeyboardInput(nextItemInfo);
                break;
            }
            case 'ArrowUp': {
                const nextItemInfo = getPrevItemInfo(focusedItemInfo);
                handleKeyboardInput(nextItemInfo);
                break;
            }
            case 'ArrowRight': {
                const nextItemInfo = isRtl
                    ? getPrevItemInfo(focusedItemInfo)
                    : getNextItemInfo(focusedItemInfo);
                handleKeyboardInput(nextItemInfo);
                break;
            }
            case 'ArrowLeft': {
                const nextItemInfo = isRtl
                    ? getNextItemInfo(focusedItemInfo)
                    : getPrevItemInfo(focusedItemInfo);
                handleKeyboardInput(nextItemInfo);
                break;
            }
        }
        cancelEvent(event);
    };
    const keyUpHandler = (event) => {
        const key = event.code;
        switch (key) {
            case 'Space': {
                if (isHideShow) {
                    triggerOnAction();
                }
                break;
            }
            case 'Enter': {
                triggerOnAction();
                break;
            }
            case 'Home': {
                updatefocusedItemInfo({
                    itemIdx: 0,
                    sectionIdx: 0,
                    isFocusVisible: true,
                    isCurrent: true
                });
                break;
            }
            case 'End': {
                updatefocusedItemInfo({
                    itemIdx: -1,
                    sectionIdx: -1,
                    isCurrent: true,
                    isFocusVisible: true
                });
                break;
            }
            case 'Tab': {
                updatefocusedItemInfo({
                    ...focusedItemInfo,
                    isCurrent: true,
                    isFocusVisible: true
                });
                break;
            }
        }
    };
    const blurHandler = () => {
        const itemInfo = { ...focusedItemInfo, isCurrent: false, isFocusVisible: false };
        if (hoveredItemInfo != null || focusedItemInfo.isCurrent) {
            onInput?.(getDetailFromInfo(undefined));
        }
        setfocusedItemInfo(itemInfo);
    };
    return {
        focusedItemInfo,
        hoveredItemInfo,
        'aria-activedescendant': activeId.current,
        onPointerUp: pointerUpHandler,
        onPointerMove: hoverHandler,
        onKeyUp: keyUpHandler,
        onKeyDown: keyDownHandler,
        onPointerLeave: pointerLeaveHandler,
        onBlur: blurHandler
    };
}

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
const DEFAULT_OFFSET = 8;
/**
 * Calculate the datatip offset for datatip when keyboard focus is active.
 * @param focusedItemRef The ref for currentItem node.
 * @param rootRef The ref for root of the legend.
 * @param currentItem The ItemInfo of the current keyboard focused item.
 * @returns
 */
const calculateOffset = (focusedItemRef, rootDimsRef, currentItem, isRtl) => {
    let crossAxis = 0;
    let mainAxisOffset = DEFAULT_OFFSET;
    if (focusedItemRef.current && currentItem.isCurrent && rootDimsRef.current) {
        const rootDim = rootDimsRef.current;
        const itemDim = focusedItemRef.current.getBoundingClientRect();
        crossAxis = itemDim.x + itemDim.width / 2 - (rootDim.x + rootDim.width / 2);
        crossAxis = isRtl ? -1 * crossAxis : crossAxis;
        mainAxisOffset += rootDim.y - itemDim.y;
        mainAxisOffset = Math.min(DEFAULT_OFFSET, Math.max(mainAxisOffset, itemDim.height - rootDim.height + DEFAULT_OFFSET));
    }
    return { crossAxis, mainAxis: mainAxisOffset };
};
const useLegendTooltip = ({ text, rootDimsRef, focusedItemRef, focusedItemInfo }) => {
    const { direction } = useUser();
    const isRtl = direction === 'rtl';
    const [offset, setOffset] = useState(calculateOffset(focusedItemRef, rootDimsRef, focusedItemInfo, isRtl));
    const [isOpen, setIsOpen] = useState(false);
    useLayoutEffect(() => {
        setOffset(calculateOffset(focusedItemRef, rootDimsRef, focusedItemInfo, isRtl));
    }, [focusedItemInfo, focusedItemRef, isRtl, rootDimsRef]);
    const anchor = focusedItemInfo?.isCurrent ? 'element' : 'pointer';
    const { tooltipContent, tooltipProps } = useTooltipControlled({
        text: text,
        position: 'top',
        isDisabled: false,
        variant: 'datatip',
        anchor: { x: anchor, y: anchor },
        offset: offset,
        isOpen,
        onToggle: ({ value }) => setIsOpen(value)
    });
    return { tooltipContent, tooltipProps };
};

/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
/**
 * Returns maximum of the width among the items in a section of legend also return the width of the entire section.
 * @param items Legend items array
 * @param unitLength Base  element
 * @param getTextDimensions Text width measurement function
 * @param legendFontStylesObj Legend text font style properties object
 * @param symbolWidth Width of legend symbol
 *
 * @returns
 */
const getMaxWidth = (items, unitLength, getTextDimensions, legendFontStylesObj, symbolWidth) => {
    let maxItemWidth = 0;
    let horizontalSectionWidth = 0;
    let itemSymbolWidth = 0;
    if (symbolWidth) {
        itemSymbolWidth = unitLength * 1.5 + symbolWidth;
    }
    else {
        //itemSymbolWidth = marginLeft + marginRight + width below expression is simplified
        itemSymbolWidth = 4 * unitLength;
    }
    items?.forEach((item) => {
        const legendItem = item;
        const { width } = getTextDimensions(legendItem.text, legendFontStylesObj);
        //itemTextWidth = paddingLeft + paddingRight + width
        const itemTextWidth = unitLength * 2.5 + width;
        //horizontalSectionWidth = total width of the section
        horizontalSectionWidth += itemSymbolWidth + itemTextWidth;
        maxItemWidth = Math.max(maxItemWidth, Math.ceil(itemSymbolWidth + itemTextWidth));
    });
    return { maxItemWidth, horizontalSectionWidth };
};
function useLegendLayout({ resolvedVars, sections, items, rootRef, rootDimsRef, orientation, legendFontStylesObj = {}, symbolWidth }) {
    const { textMeasureContent, getTextDimensions } = useTextDimensions();
    const orientationRef = useRef(orientation);
    const hasOrientationChanged = orientationRef.current !== orientation;
    const [needsRerender, setRerender] = useState(false);
    useLayoutEffect(() => {
        if (hasOrientationChanged) {
            //rerender the Legend
            orientationRef.current = orientation;
            setRerender(!needsRerender);
        }
    }, [hasOrientationChanged, needsRerender, orientation]);
    const layoutInfo = useMemo(() => {
        if (!rootRef.current ||
            !Object.keys(resolvedVars).length ||
            !getTextDimensions ||
            hasOrientationChanged ||
            orientation === 'vertical') {
            return;
        }
        const unitLength = parseFloat(resolvedVars['unit']);
        // recalculate rootDims everytime items change.
        rootDimsRef.current = rootRef?.current?.getBoundingClientRect();
        let maxWidth = 0;
        let isWrapped = false;
        if (sections) {
            sections.forEach((section) => {
                const { maxItemWidth, horizontalSectionWidth } = getMaxWidth(section.items, unitLength, getTextDimensions, legendFontStylesObj, symbolWidth);
                if (!isWrapped) {
                    isWrapped = horizontalSectionWidth > rootDimsRef.current?.width;
                }
                maxWidth = Math.max(maxWidth, maxItemWidth);
            });
        }
        if (items) {
            const { maxItemWidth, horizontalSectionWidth } = getMaxWidth(items, unitLength, getTextDimensions, legendFontStylesObj, symbolWidth);
            maxWidth = maxItemWidth;
            isWrapped = horizontalSectionWidth > rootDimsRef.current?.width;
        }
        const legendLayoutInfo = {
            maxItemWidth: Math.min(maxWidth, rootDimsRef.current?.width),
            itemsPerRow: Math.max(1, Math.floor(rootDimsRef.current?.width / maxWidth)),
            isWrapped
        };
        return legendLayoutInfo;
    }, [
        sections,
        items,
        rootDimsRef,
        rootRef,
        legendFontStylesObj,
        resolvedVars,
        symbolWidth,
        getTextDimensions,
        orientation,
        hasOrientationChanged
    ]);
    return {
        textMeasureContent,
        layoutInfo
    };
}

export { BaseLegend as B, LegendText as L, useEvents as a, useLegendTooltip as b, useLegendLayout as u };
//# sourceMappingURL=useLegendLayout-16fdac28.js.map
