/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
import { ScaleLog, ScaleLinear } from '../utils/PRIVATE_chartUtils';
import { DataAxisInfoProps } from './axis.types';
export declare class DataAxisInfo {
    private dataMin;
    private dataMax;
    private viewportMin?;
    private viewportMax?;
    private min?;
    private max?;
    private isLog;
    private majorTickCount;
    private minorTickCount;
    private majorIncrement?;
    private minorIncrement?;
    private minMajorIncrement?;
    private isZeroBaseline;
    constructor({ scale, baselineScaling, dataMax, dataMin, min, max, viewportMin, viewportMax, ...options }: DataAxisInfoProps);
    /**
     * Set the dataMin and dataMax.
     * @param dataMin The minimum of the data.
     * @param dataMax The maximum of the data.
     */
    setDataRange(dataMin: number, dataMax: number): void;
    setViewportRange(viewportMin?: number, viewportMax?: number, min?: number, max?: number): void;
    setDefaultMin(scaleUnit: number): void;
    setDefaultMax(scaleUnit: number): void;
    calcAxisExtents(): void;
    /**
     * Determines the number of major and minor tick counts and increments for the axis if values were not given.
     * The default minor tick count is 2.
     * @param {number} scaleUnit The scale unit of the axis.
     * @private
     */
    calcMajorMinorSteps(scaleUnit: number): void;
    calcAxisStep(minValue: number, maxValue: number): number;
    /**
     * Aesthetically choose a step depending of the spread value.
     * @param spread The spread of the data values.
     * @returns The step for the axis.
     */
    getDefaultAxisStep(spread: number): number;
    linearToActual(value: number): number;
    actualToLinear(value?: number): number | undefined;
    getScale(range: [number, number]): ScaleLinear | ScaleLog;
    getFirstTick(): number;
    getAxisViewport(): {
        viewportMin: number;
        viewportMax: number;
    };
    /**
     * Returns the min and max extent of the axis.
     */
    getAxisExtent(): {
        min: number;
        max: number;
    };
    /**
     * Returns the major ticks.
     */
    getTicks(): number[];
    getMinorTicks(): number[];
}
