/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
import { ScaleLinear } from '../utils/PRIVATE_chartUtils';
import { AxisPosition, Dimension, Group, GroupAxisProps } from './axis.types';
import { useTextDimensions } from '../hooks/PRIVATE_useTextDimensions';
type GetTextDimensions = NonNullable<ReturnType<typeof useTextDimensions>['getTextDimensions']>;
/**
 * Retuns the info about group axis.
 * @param groups The groups array.
 * @param xAxis The xAxis props.
 * @param offset The start and end offset for the chart.
 * @returns
 */
export declare function getGroupAxisInfo(groups: Group[], xAxis: GroupAxisProps | undefined, offset: number): {
    min: number;
    max: number;
    viewportMin: number;
    viewportMax: number;
    viewportStartGroup: string | undefined;
    viewportEndGroup: string | undefined;
    startIndex: number;
    endIndex: number;
};
/**
 * The function that returns the x axis properties.
 * @param groupAxisIno The GroupAxisInfo.
 * @param groupAxis The groupAxis props.
 * @param groups The chart groups.
 * @param position The position of axis in the given space.
 * @param availSpace The available space.
 * @param isRtl Whether the reading direction is right to left.
 * @param getTextDimensions Function for text measurement.
 * @returns
 */
export declare function getGroupAxisProps(groups: Group[], position: AxisPosition, availSpace: Dimension, isRtl: boolean, viewportMin: number, viewportMax: number, startIndex: number, endIndex: number, isDomainFlipped: boolean, groupAxis?: GroupAxisProps, axisFontSize?: string, getTextDimensions?: GetTextDimensions): {
    title: string | undefined;
    titleProps: {} | undefined;
    tickLabels: {
        index: number;
        label: string;
        labelProps: import("../hooks/PRIVATE_useTextDimensions").TextDimensions;
    }[];
    tickLabelStyle: {
        textAnchor: string;
        dominantBaseline: string;
        color?: import("csstype").Property.Color | undefined;
        fontSize: string | undefined;
        fontWeight?: string | undefined;
        fontStyle?: string | undefined;
        fill: import("csstype").Property.Color | undefined;
    };
    axisDim: number;
    scale: ScaleLinear;
    groupWidth: number;
};
export {};
