/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
import { AxisPosition, Dimension, Group, TimeAxisProps } from './axis.types';
import { ChartItem, ChartSeries } from '../UNSAFE_Chart';
import { useTextDimensions } from '../hooks/PRIVATE_useTextDimensions';
type GetTextDimensions = NonNullable<ReturnType<typeof useTextDimensions>['getTextDimensions']>;
export declare function getTimeAxisProps(groups: Group[], position: AxisPosition, availSpace: Dimension, viewportMin: number, viewportMax: number, averageInterval: number, isRtl: boolean, xAxis?: TimeAxisProps, axisFontSize?: string, getTextDimensions?: GetTextDimensions): {
    title: string | undefined;
    titleProps: {} | undefined;
    axisDim: number;
    scale: {
        transform: (value: number) => number;
        range: () => number[];
        domain: () => number[];
        invert: (value: number) => number;
    };
    tickLabels: any[];
    groupWidth: number;
    tickLabelStyle: {
        textAnchor: string;
        dominantBaseline: string;
        color?: import("csstype").Property.Color | undefined;
        fontSize: string | undefined;
        fontWeight?: string | undefined;
        fontStyle?: string | undefined;
        fill: import("csstype").Property.Color | undefined;
    };
};
/**
 * Calculates the granularity of the time axis.
 * @param step The step of the axis.
 * @param timeRange The time range of the axis.
 * @returns
 */
export declare function calculateGranularity(step: number, timeRange: number): number;
/**
 * Returns the step for mixed frequency time range.
 * @param timeRange
 * @returns
 */
export declare function getMixedFrequencyStep(timeRange: number): number;
export declare function formatLabelWithFormatter(date: Date, prevDate: Date | undefined, granularity: number, formatter?: (date: number) => string): string;
export declare function formatAxisLabel(date: Date, prevDate: Date | undefined, bOneLabel: boolean, isVert: boolean, locale: string, granularity: number, label1Formatter?: (date: number) => string, label2Formatter?: (date: number) => string): (string | null)[];
/**
 * Returns the formatted date string.
 * @param date
 * @param locale
 * @param showDay
 * @param showMonth
 * @param showYear
 * @returns
 */
export declare function formatDate(date: Date, locale: string, showDay: boolean, showMonth: boolean, showYear: boolean): string;
/**
 * Returns the formatted time string.
 * @param date
 * @param locale
 * @param showMinute
 * @param showSecond
 * @returns
 */
export declare function formatTime(date: Date, locale: string, showMinute: boolean, showSecond: boolean): string;
export declare function getLabelPos(start: number, end: number, step: number): number[];
export declare function generateLabels(times: number[], granularity: number, getCoordFromValue: (time: number) => number, label1Formatter?: (time: number) => string, label2Formatter?: (time: number) => string): {
    labelInfos1: {
        text: string;
        coord: number;
    }[];
    labelInfos2: {
        label2: string;
        coord: number;
    }[];
    coords1: number[];
    coords2: number[];
    isOneLevel: boolean;
};
/**
 * Adds the time by one year, e.g. 2014 January 15 -> 2015 January 15 -> ...
 */
export declare function addOneYear(time: number): number;
/**
 * Adds the time by one month, e.g. January 15 -> February 15 -> March 15 -> ...
 */
export declare function addOneMonth(time: number): number;
/**
 * Returns the corresponding index for the given time in time axis.
 * @param time The time stamp.
 * @param averageInterval The average interval of the time axis.
 * @param groups The groups array.
 * @returns
 */
export declare function timeToIndex(time: number, averageInterval: number, groups: Group[]): number;
/**
 * Returns the time stamp for given index.
 * @param index The index.
 * @param averageInterval The average interval of the axis.
 * @param groups The groups array.
 * @returns
 */
export declare function indexToTime(index: number, averageInterval: number, groups: Group[]): number;
export declare function getTime(uxTime: string): number;
export declare function getCoordFromValue(value: number, minValue: number, maxValue: number, startCoord: number, endCoord: number, averageInterval: number, groups: Group[], timeAxisType: 'skipGaps' | 'mixedFrequency' | 'enabled'): number;
export declare function getValueFromCoord(coord: number, startCoord: number, endCoord: number, minValue: number, maxValue: number, averageInterval: number, groups: Group[], timeAxisType: 'skipGaps' | 'enabled' | 'mixedFrequency'): number;
export declare function isoToNumber(dateString?: string): number | undefined;
export declare function getAverageInterval(groups: Group[], start: number, end: number): number;
/**
 * Returns the time axis info.
 * @param groups The groups array.
 * @param xAxis The xAxis props.
 * @param offset The start and end offset for the axis.
 * @param timeAxisRange The start and end of the time range.
 * @returns
 */
export declare function getTimeAxisInfo(groups: Group[], xAxis: TimeAxisProps | undefined, offset: number, timeAxisRange: {
    start: number;
    end: number;
}): {
    viewportMin: number;
    viewportMax: number;
    viewportStartGroup: string | undefined;
    viewportEndGroup: string | undefined;
    startIndex: number;
    endIndex: number;
    min: number;
    max: number;
    averageInterval: number;
};
/**
 * Returns the maximum of time stamps for the given dataset
 * and time axis type.
 * @param groups The array of groups.
 * @param series The array of series.
 * @param getDataItem The function that returns item for given seriesIndex and groupIndex.
 * @param timeAxisType The time axis type.
 * @returns
 */
export declare function getStartAndEndTime<K extends string | number, D extends ChartItem<K>>(groups: Group[], series: ChartSeries<K, D>[], getDataItem: (seriesIndex: number, groupIndex: number) => D | undefined, timeAxisType?: 'enabled' | 'mixedFrequency' | 'skipGaps'): {
    start: number;
    end: number;
};
export {};
