/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
import type { useTextDimensions } from '../../hooks/PRIVATE_useTextDimensions';
import { AxisPosition, DataAxisProps, Dimension } from '../axis.types';
type GetTextDimensions = NonNullable<ReturnType<typeof useTextDimensions>['getTextDimensions']>;
/**
 * Returns resolved axis position.
 * @param axis
 * @param orientation
 * @param isRtl
 */
export declare function getAxesPosition(yAxis: DataAxisProps | undefined, isHoriz: boolean, isRtl: boolean): {
    xAxisPosition: AxisPosition;
    yAxisPosition: AxisPosition;
};
/**
 * Returns the scale for the axis.
 * @param availSpace
 * @param axisViewportMin
 * @param axisViewportMax
 * @param position
 * @returns
 */
export declare function getAxisRange(availSpace: Dimension, position: 'left' | 'right' | 'top' | 'bottom', isRtl: boolean): number[];
/**
 * Returns the maximum possible size of the axis.
 * @param availSpace The available space.
 * @param isHoriz If the axis is horizontal.
 * @param axisSize The ratio of axis to the chart.
 * @returns
 */
export declare function getAxisMaxSize(availSpace: Dimension, isHoriz: boolean, axisSize?: number): number;
/**
 * Returns the width of a group for given availableSpace, axisposition and axis extents.
 * @param availSpace The available space to render the axis
 * @param position The position of the axis
 * @param min The minimum value of axis coord
 * @param max The maximum value of axis coord
 * @returns
 */
export declare function getGroupWidth(availSpace: {
    x: number;
    y: number;
    width: number;
    height: number;
}, position: 'left' | 'right' | 'top' | 'bottom', min: number, max: number): number;
/**
 * Returns the num of labels to skip safely.
 * @param groupWidth
 * @param tickLabelStyle
 * @param getTextDimensions
 * @returns
 */
export declare function getSafeSkips(groupWidth: number, tickLabelStyle: Partial<CSSStyleDeclaration>, getTextDimensions?: GetTextDimensions): number;
export {};
