/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
import { TextDimensions, useTextDimensions } from '../../hooks/PRIVATE_useTextDimensions';
import { AxisPosition, Dimension } from '../axis.types';
type GetTextDimensions = NonNullable<ReturnType<typeof useTextDimensions>['getTextDimensions']>;
/**
 * Calculates the number of skips between axis labels and the dimensions of unskipped labels.
 * @param labels The labels to render.
 * @param getLabelDims The function that returns dimension of given label.
 * @param isHoriz Whether the axis is horizontal.
 * @param numOfSafeSkips The no of labels to skip safely.
 * @returns
 */
export declare function skipLabelsUniform(labels: string[], getLabelDims: (label: string, index: number) => Dimension, isHoriz: boolean, numOfSafeSkips?: number): {
    numOfSkips: number;
    dims: TextDimensions[];
};
/**
 * Greedily skip labels. We will skip next label if it overlaps with the previous one.
 * This might not result in uniformly spaced labels.
 * @param labels
 * @param getLabelDims
 * @param isHoriz
 * @returns
 */
export declare function skipLabelsGreedy(labels: string[], getLabelDims: (label: string, index: number) => Dimension, isHoriz: boolean): {
    tickLabels: {
        label: string;
        index: number;
        labelProps: Dimension;
    }[];
    maxWidth: number;
    maxHeight: number;
};
/**
 * Retuns the max dimension of the rendered labels after skipping overlapping labels.
 * @param labels An array of labels.
 * @param getLabelDims The function called to get dimension of the given label.
 * @param isHoriz If the overlap be measured in horizontal direction.
 * @param numOfSafeSkips Predetermined number labels to safely skip. This will optimize label
 * measurement as we don't have to measure all the labels.
 * @returns
 */
export declare function getMaxTextDimensions(labels: string[], getLabelDims: (label: string, index: number) => TextDimensions, isHoriz: boolean, numOfSafeSkips?: number): {
    maxHeight: number;
    maxWidth: number;
    tickLabels: {
        index: number;
        label: string;
        labelProps: TextDimensions;
    }[];
    numOfSkips: number;
};
export declare function updateLabelsDims(maxWidth: number, maxHeight: number, renderedLabelsInfo: any[], getAxisPos: (index: number) => number, availSpace: Dimension, position: AxisPosition, isRtl: boolean): void;
/**
 * Updates the given available space after labels are rendered.
 * @param availSpace The avaialable space.
 * @param position The position of the axis.
 * @param maxWidth The maximum width of rendered labels.
 * @param maxHeight The maxium height of the rendered labels.
 */
export declare function updateAvailableSpace(availSpace: Dimension, position: AxisPosition, maxWidth: number, maxHeight: number): void;
/**
 * Returns the enclosing rect for the label of given dimension.
 * @param dims The dimension
 * @param axisPosition The position of the axis.
 * @returns
 */
export declare function getEnclosingRectDims(dims: Dimension, axisPosition: AxisPosition): {
    x: number;
    y: number;
    width: number;
    height: number;
};
/**
 * Retuns the unaligned label dimensions that can be used by skipLabels.
 * @param coord
 * @param label
 * @param labelStyle
 * @param isHoriz
 * @param availSpace
 * @param getTextDimensions
 * @returns
 */
export declare function getLabelDimensions(coord: number, label: string, labelStyle: Partial<CSSStyleDeclaration>, isHoriz: boolean, availSpace: Dimension, getTextDimensions?: GetTextDimensions): {
    width: number;
    height: number;
    x: number;
    y: number;
};
export {};
