/**
 * @license
 * Copyright (c) %FIRST_YEAR% %CURRENT_YEAR%, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 * @ignore
 */
import { ComponentProps } from 'preact';
import { Dispatch, MutableRef } from 'preact/hooks';
import { DateFieldAction } from './dateReducer';
import { DateFieldState } from './useDateField';
import { InputDateMask } from './InputDateMask';
type InputDateMaskProps = ComponentProps<typeof InputDateMask>;
type Props = {
    dateResetRef: MutableRef<boolean>;
    direction: 'ltr' | 'rtl';
    dispatch: Dispatch<DateFieldAction>;
    isDisabled?: boolean;
    isFocused?: boolean;
    isInputFocused?: boolean;
    onCommit: InputDateMaskProps['onCommit'];
    onInput: InputDateMaskProps['onInput'];
    state: DateFieldState;
};
/**
 * A hook that creates various handlers and manages effects for a DateField.
 */
declare const useDateFieldHandlers: ({ dateResetRef, direction, dispatch, isDisabled, isFocused, isInputFocused, onCommit, onInput, state }: Props) => {
    dateClearedRef: MutableRef<boolean>;
    dateFieldHandlers: {
        onClick?: undefined;
        onDoubleClick?: undefined;
        onKeyDown?: undefined;
        onPointerDown?: undefined;
    } | {
        onClick: (e: Event) => void;
        onDoubleClick: () => void;
        onKeyDown: (event: KeyboardEvent) => void;
        onPointerDown: () => void;
    };
    segmentHandlers: {
        onChange?: undefined;
    } | {
        onChange: (action: DateFieldAction) => void;
    };
    tabIndex: number;
};
export { useDateFieldHandlers };
