import { ListProps } from './List.types';
/**
 * Props specific to non-virtualized ListView
 */
type NonVirtualizedListProps<K, D> = {
    /**
     * The data that is used to render each item in ListView.  If the value is null, then ListView will show nothing
     * if hasMore is false, or loading indicator if hasMore is true.
     */
    data: D[] | null;
    /**
     * A callback function to extract the key which is used to identify the item.
     */
    getRowKey: (data: D) => K;
    /**
     * Whether there are more data available to load.  By default this returns false.
     */
    hasMore?: boolean;
    /**
     * An optional callback function which is invoked when user scrolls to the end of the list.
     */
    onLoadMore?: () => void;
};
/**
 * Props for the ListView Component
 */
export type Props<K, D> = Pick<ListProps<K, D>, 'aria-label' | 'aria-labelledby' | 'children' | 'currentKey' | 'currentItemVariant' | 'gridlines' | 'onCurrentKeyChange' | 'onItemAction' | 'onSelectionChange' | 'promotedSection' | 'selectedKeys' | 'selectionMode' | 'viewportConfig' | 'testId' | 'contextMenuRenderer'> & NonVirtualizedListProps<K, D>;
export declare function ListView<K extends string | number, D>(props: Props<K, D>): import("preact").JSX.Element;
export {};
