import { type Ref } from 'preact/hooks';
type UseResizeObserverOptions = {
    /**
     * The ref to the target DOM element to observe.
     */
    ref: Ref<Element>;
    /**
     * A callback function to be called when the size of the observed element changes.
     * If the provided callback triggers the consuming component to re-render, then
     * it's important to pass the same instance of the callback function during each render
     * to prevent infinite re-render loops. One way to achieve this is by using useCallback.
     */
    callback: (entry: ResizeObserverEntry) => void;
    /**
     * Configuration options for the ResizeObserver.
     * If the consuming component re-renders due to resize changes, then
     * it's important to pass the same instance of the options object during each render
     * to prevent infinite re-render loops. One way to achieve this is by using useMemo.
     */
    resizeObserverOptions?: ResizeObserverOptions;
};
/**
 * A hook for observing and responding to changes to the size of a DOM element.
 * It utilizes a ResizeObserver to observe changes to the size of the specified element
 * and accepts a callback function to respond to these changes.
 */
declare const useResizeObserver: ({ ref, callback, resizeObserverOptions }: UseResizeObserverOptions) => void;
export { useResizeObserver };
