import { CalendarDate, DateGranularity, DatePlaceholders, IsoDay, IsoMonth } from "../../UNSAFE_InputDateMask/types";
import { BCP47Locale, DateISOStr } from "../../UNSAFE_IntlDateTime";
declare const MAX_ISO_YEAR = 9999;
declare const MIN_ISO_YEAR = 0;
declare const MIN_ISO_DAY = 1;
declare const MIN_ISO_MONTH = 1;
declare const checkNoCalendarExtension: (locale: BCP47Locale) => void;
/**
 * This function formats a test date using Intl.DateTimeFormat for the given locale.
 * It then uses Intl.DateTimeFormat.formatToParts to get the parts of the date in a
 * locale specific order, substitutes translated placeholders, and filters out
 * unwanted parts of the date. If granularity = month, then the placeholder for day
 * is removed. The shape of the return value matches what is returned by formatToParts.
 *
 * @param locale The BCP47Locale
 * @param granularity Specifies whether you want placeholders for month, year OR day, month, year
 * @param monthPlaceholder The translated month placeholder
 * @param dayPlaceholder The translated day placeholder
 * @param yearPlaceholder The translated year placeholder
 */
declare const getDatePlaceholderMasks: (locale: BCP47Locale, granularity: 'month' | 'day', monthPlaceholder: string, dayPlaceholder: string, yearPlaceholder: string) => DatePlaceholders;
/**
 * Return the number of days in a given month and year. Pass '*' if you don't know a value.
 * If neither month or year are known, then return the maximum number of days in any month in any year.
 * If only month is known, return the maximum number of days for that month in any year.
 * For Gregorian calendar:
 *   getDaysInMonth(2, *) => returns 29
 *   getDaysInMonth(*, *) => returns 31
 *   getDaysInMonth(*, 2023) => returns 31
 */
declare const getDaysInMonth: (month: IsoMonth | '*', year: number | '*') => 28 | 31 | 30 | 29;
/**
 * Return the number of months in a given year. Pass '*' if you don't know the year.
 * In a Gregorian calendar the number of months is always 12. We'll have to revisit this
 * when multiple calendars are supported; in some cases the number of months can vary by year.
 */
declare const getMonthsInYear: (year: number | '*') => number;
/**
 * Throw an error if year is not a valid ISO year (0-9999).
 * The allowed range is too large to express using Typescript.
 */
declare const checkIsValidIsoYear: (year: number) => void;
/**
 * Construct and return an IS0 date string from year, month, and day.
 */
declare const getIsoDateStr: (year: number, month: IsoMonth, day: IsoDay) => string;
/**
 * Parse a DateIsoStr and return an equivalent CalendarDate.
 */
declare const getCalendarDateFromIso: (str: DateISOStr) => CalendarDate | undefined;
/**
 * Return the calendar date for year, month, and day formatted using the specified date style
 * in the given locale.
 */
declare const getFormattedYearMonthDay: (locale: BCP47Locale, year: number, month: IsoMonth, day: IsoDay, style: NonNullable<Intl.DateTimeFormatOptions['dateStyle']>) => string;
/**
 * Format and return the calendar date as a string. If the date is not a complete
 * date, i.e. any of its segments are missing, return an empty string.
 */
declare const formatCalendarDateAsString: (locale: BCP47Locale, calendarDate: CalendarDate, style: NonNullable<Intl.DateTimeFormatOptions['dateStyle']>) => string;
/**
 * Format an example DateISOStr in short format.
 */
declare const formatIsoDateStrAsExample: (locale: BCP47Locale, str: DateISOStr, masks?: DatePlaceholders) => string;
/**
 * Format a CalendarDate in short format.
 */
declare const formatShortCalendarDate: (cd: CalendarDate, granularity: DateGranularity, masks: DatePlaceholders) => string;
/**
 * Format a CalendarDate in full format.
 */
declare const formatFullCalendarDate: (locale: BCP47Locale, cd: CalendarDate, granularity: DateGranularity) => string;
declare const isValidCalendarDate: (value: any, granularity?: DateGranularity) => boolean;
declare const isCompleteCalendarDate: (value: CalendarDate) => boolean;
declare const getDatePlaceholdersFromPattern: (pattern: string) => DatePlaceholders;
export { MIN_ISO_DAY, MIN_ISO_MONTH, MIN_ISO_YEAR, MAX_ISO_YEAR, checkIsValidIsoYear, checkNoCalendarExtension, formatCalendarDateAsString, formatShortCalendarDate, formatFullCalendarDate, formatIsoDateStrAsExample, getCalendarDateFromIso, getDatePlaceholderMasks, getFormattedYearMonthDay, getDaysInMonth, getDatePlaceholdersFromPattern, getIsoDateStr, getMonthsInYear, isCompleteCalendarDate, isValidCalendarDate };
